@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: d879bf216775cacbecf1f8a8d35ace3e58d7e592 $
 */
 
 #ifndef BUILTIN_EFUNS_H
@@ -11,7 +10,6 @@
 #define TYPEP(ID,NAME,TYPE) PMOD_EXPORT void ID(INT32 args);
 
 #include "callback.h"
-#include "block_alloc_h.h"
 
 /* Weak flags for arrays, multisets and mappings. 1 is avoided for
  * compatibility reasons. */
@@ -72,6 +70,7 @@ PMOD_EXPORT void f_string_to_unicode(INT32 args);
 PMOD_EXPORT void f_unicode_to_string(INT32 args);
 PMOD_EXPORT void f_string_to_utf8(INT32 args);
 PMOD_EXPORT void f_utf8_to_string(INT32 args);
+void f___get_first_arg_type(INT32 args);
 PMOD_EXPORT void f_all_constants(INT32 args);
 PMOD_EXPORT void f_allocate(INT32 args);
 void f_this_object(INT32 args);
@@ -84,6 +83,7 @@ PMOD_EXPORT void f_crypt(INT32 args);
 PMOD_EXPORT void f_destruct(INT32 args);
 PMOD_EXPORT void f_indices(INT32 args);
 PMOD_EXPORT void f_values(INT32 args);
+PMOD_EXPORT void f_types(INT32 args);
 PMOD_EXPORT void f_next_object(INT32 args);
 PMOD_EXPORT void f_object_program(INT32 args);
 int find_longest_prefix(char *str,
@@ -165,6 +165,7 @@ PMOD_EXPORT void f_inherit_list(INT32 args);
 PMOD_EXPORT void f_function_defined(INT32 args);
 void init_builtin_efuns(void);
 void exit_builtin_efuns(void);
+unsigned int rec_size_svalue( struct svalue *s, struct mapping **m );
 
 /* From iterators.cmod. */
 PMOD_EXPORT void f_get_iterator(INT32 args);
@@ -207,13 +208,15 @@ struct pike_list_node
   struct pike_list_node *prev;
   struct svalue val;
 };
-BLOCK_ALLOC_FILL_PAGES(pike_list_node, 4);
+void count_memory_in_pike_list_nodes(size_t * n, size_t * s);
+void free_all_pike_list_node_blocks(void);
 PMOD_EXPORT void free_list_node(struct pike_list_node *node);
 PMOD_EXPORT void unlink_list_node(struct pike_list_node *n);
 PMOD_EXPORT void prepend_list_node(struct pike_list_node *node,
 				   struct pike_list_node *new_node);
 PMOD_EXPORT void append_list_node(struct pike_list_node *node,
 				  struct pike_list_node *new_node);
+PMOD_EXPORT struct object *get_setter(struct object *o, int f);
 void init_builtin(void);
 void exit_builtin(void);
 
