@@ -1,7 +1,6 @@
 /*
- * Created by
- * $Id: 78d9754d0c725869f0fb204fe8621e624e6dd726 $
- * on Thu May  1 02:28:46 2014
+ * Created by make_ci.pike
+ * on Thu Dec 31 21:22:56 2015
  *
  * Table used for looking up the case of
  * Unicode characters.
@@ -94,7 +93,7 @@
 { 0x00023b, CIM_CASEBITOFF, 0x0001, },
 { 0x00023d, CIM_UPPERDELTA, -0x00a3, },
 { 0x00023e, CIM_UPPERDELTA, 0x2a28, },
-{ 0x00023f, CIM_NONE, 0x0000, },
+{ 0x00023f, CIM_LOWERDELTA, -0x2a3f, },
 { 0x000241, CIM_CASEBITOFF, 0x0001, },
 { 0x000243, CIM_UPPERDELTA, -0x00c3, },
 { 0x000244, CIM_UPPERDELTA, 0x0045, },
@@ -102,7 +101,7 @@
 { 0x000246, CIM_CASEBIT, 0x0001, },
 { 0x000250, CIM_LOWERDELTA, -0x2a1f, },
 { 0x000251, CIM_LOWERDELTA, -0x2a1c, },
-{ 0x000252, CIM_NONE, 0x0000, },
+{ 0x000252, CIM_LOWERDELTA, -0x2a1e, },
 { 0x000253, CIM_LOWERDELTA, 0x00d2, },
 { 0x000254, CIM_LOWERDELTA, 0x00ce, },
 { 0x000255, CIM_NONE, 0x0000, },
@@ -111,16 +110,22 @@
 { 0x000259, CIM_LOWERDELTA, 0x00ca, },
 { 0x00025a, CIM_NONE, 0x0000, },
 { 0x00025b, CIM_LOWERDELTA, 0x00cb, },
-{ 0x00025c, CIM_NONE, 0x0000, },
+{ 0x00025c, CIM_LONGLOWERDELTA, -0x254f, },
+{ 0x00025d, CIM_NONE, 0x0000, },
 { 0x000260, CIM_LOWERDELTA, 0x00cd, },
-{ 0x000261, CIM_NONE, 0x0000, },
+{ 0x000261, CIM_LONGLOWERDELTA, -0x254b, },
+{ 0x000262, CIM_NONE, 0x0000, },
 { 0x000263, CIM_LOWERDELTA, 0x00cf, },
 { 0x000264, CIM_NONE, 0x0000, },
+{ 0x000265, CIM_LONGLOWERDELTA, -0x2528, },
+{ 0x000266, CIM_LONGLOWERDELTA, -0x2544, },
+{ 0x000267, CIM_NONE, 0x0000, },
 { 0x000268, CIM_LOWERDELTA, 0x00d1, },
 { 0x000269, CIM_LOWERDELTA, 0x00d3, },
 { 0x00026a, CIM_NONE, 0x0000, },
 { 0x00026b, CIM_LOWERDELTA, -0x29f7, },
-{ 0x00026c, CIM_NONE, 0x0000, },
+{ 0x00026c, CIM_LONGLOWERDELTA, -0x2541, },
+{ 0x00026d, CIM_NONE, 0x0000, },
 { 0x00026f, CIM_LOWERDELTA, 0x00d3, },
 { 0x000270, CIM_NONE, 0x0000, },
 { 0x000271, CIM_LOWERDELTA, -0x29fd, },
@@ -134,6 +139,7 @@
 { 0x000281, CIM_NONE, 0x0000, },
 { 0x000283, CIM_LOWERDELTA, 0x00da, },
 { 0x000284, CIM_NONE, 0x0000, },
+{ 0x000287, CIM_LONGLOWERDELTA, -0x252a, },
 { 0x000288, CIM_LOWERDELTA, 0x00da, },
 { 0x000289, CIM_LOWERDELTA, 0x0045, },
 { 0x00028a, CIM_LOWERDELTA, 0x00d9, },
@@ -141,6 +147,9 @@
 { 0x00028d, CIM_NONE, 0x0000, },
 { 0x000292, CIM_LOWERDELTA, 0x00db, },
 { 0x000293, CIM_NONE, 0x0000, },
+{ 0x00029d, CIM_LONGLOWERDELTA, -0x2515, },
+{ 0x00029e, CIM_LONGLOWERDELTA, -0x2512, },
+{ 0x00029f, CIM_NONE, 0x0000, },
 { 0x000345, CIM_LOWERDELTA, -0x0054, },
 { 0x000346, CIM_NONE, 0x0000, },
 { 0x000370, CIM_CASEBIT, 0x0001, },
@@ -149,6 +158,8 @@
 { 0x000378, CIM_NONE, 0x0000, },
 { 0x00037b, CIM_LOWERDELTA, -0x0082, },
 { 0x00037e, CIM_NONE, 0x0000, },
+{ 0x00037f, CIM_UPPERDELTA, 0x0074, },
+{ 0x000380, CIM_NONE, 0x0000, },
 { 0x000386, CIM_UPPERDELTA, 0x0026, },
 { 0x000387, CIM_NONE, 0x0000, },
 { 0x000388, CIM_UPPERDELTA, 0x0025, },
@@ -181,7 +192,7 @@
 { 0x0003f0, CIM_LOWERDELTA, 0x0056, },
 { 0x0003f1, CIM_LOWERDELTA, 0x0050, },
 { 0x0003f2, CIM_LOWERDELTA, -0x0007, },
-{ 0x0003f3, CIM_NONE, 0x0000, },
+{ 0x0003f3, CIM_LOWERDELTA, 0x0074, },
 { 0x0003f4, CIM_UPPERDELTA, -0x003c, },
 { 0x0003f5, CIM_LOWERDELTA, 0x0060, },
 { 0x0003f6, CIM_NONE, 0x0000, },
@@ -203,13 +214,22 @@
 { 0x0004c1, CIM_CASEBITOFF, 0x0001, },
 { 0x0004cf, CIM_LOWERDELTA, 0x000f, },
 { 0x0004d0, CIM_CASEBIT, 0x0001, },
-{ 0x000524, CIM_NONE, 0x0000, },
+{ 0x000530, CIM_NONE, 0x0000, },
 { 0x000531, CIM_UPPERDELTA, 0x0030, },
 { 0x000557, CIM_NONE, 0x0000, },
 { 0x000561, CIM_LOWERDELTA, 0x0030, },
 { 0x000587, CIM_NONE, 0x0000, },
 { 0x0010a0, CIM_UPPERDELTA, 0x1c60, },
 { 0x0010c6, CIM_NONE, 0x0000, },
+{ 0x0010c7, CIM_UPPERDELTA, 0x1c60, },
+{ 0x0010c8, CIM_NONE, 0x0000, },
+{ 0x0010cd, CIM_UPPERDELTA, 0x1c60, },
+{ 0x0010ce, CIM_NONE, 0x0000, },
+{ 0x0013a0, CIM_LONGUPPERDELTA, 0x17d1, },
+{ 0x0013f0, CIM_CASEBIT, 0x0008, },
+{ 0x0013f6, CIM_NONE, 0x0000, },
+{ 0x0013f8, CIM_CASEBIT, 0x0008, },
+{ 0x0013fe, CIM_NONE, 0x0000, },
 { 0x001d79, CIM_LONGLOWERDELTA, -0x0a04, },
 { 0x001d7a, CIM_NONE, 0x0000, },
 { 0x001d7d, CIM_LOWERDELTA, -0x0ee6, },
@@ -329,21 +349,29 @@
 { 0x002c6d, CIM_UPPERDELTA, -0x2a1c, },
 { 0x002c6e, CIM_UPPERDELTA, -0x29fd, },
 { 0x002c6f, CIM_UPPERDELTA, -0x2a1f, },
-{ 0x002c70, CIM_NONE, 0x0000, },
+{ 0x002c70, CIM_UPPERDELTA, -0x2a1e, },
+{ 0x002c71, CIM_NONE, 0x0000, },
 { 0x002c72, CIM_CASEBIT, 0x0001, },
 { 0x002c74, CIM_NONE, 0x0000, },
 { 0x002c75, CIM_CASEBITOFF, 0x0001, },
 { 0x002c77, CIM_NONE, 0x0000, },
+{ 0x002c7e, CIM_UPPERDELTA, -0x2a3f, },
 { 0x002c80, CIM_CASEBIT, 0x0001, },
 { 0x002ce4, CIM_NONE, 0x0000, },
+{ 0x002ceb, CIM_CASEBITOFF, 0x0001, },
+{ 0x002cef, CIM_NONE, 0x0000, },
+{ 0x002cf2, CIM_CASEBIT, 0x0001, },
+{ 0x002cf4, CIM_NONE, 0x0000, },
 { 0x002d00, CIM_LOWERDELTA, 0x1c60, },
 { 0x002d26, CIM_NONE, 0x0000, },
+{ 0x002d27, CIM_LOWERDELTA, 0x1c60, },
+{ 0x002d28, CIM_NONE, 0x0000, },
+{ 0x002d2d, CIM_LOWERDELTA, 0x1c60, },
+{ 0x002d2e, CIM_NONE, 0x0000, },
 { 0x00a640, CIM_CASEBIT, 0x0001, },
-{ 0x00a660, CIM_NONE, 0x0000, },
-{ 0x00a662, CIM_CASEBIT, 0x0001, },
 { 0x00a66e, CIM_NONE, 0x0000, },
 { 0x00a680, CIM_CASEBIT, 0x0001, },
-{ 0x00a698, CIM_NONE, 0x0000, },
+{ 0x00a69c, CIM_NONE, 0x0000, },
 { 0x00a722, CIM_CASEBIT, 0x0001, },
 { 0x00a730, CIM_NONE, 0x0000, },
 { 0x00a732, CIM_CASEBIT, 0x0001, },
@@ -353,7 +381,26 @@
 { 0x00a77e, CIM_CASEBIT, 0x0001, },
 { 0x00a788, CIM_NONE, 0x0000, },
 { 0x00a78b, CIM_CASEBITOFF, 0x0001, },
-{ 0x00a78d, CIM_NONE, 0x0000, },
+{ 0x00a78d, CIM_LONGUPPERDELTA, -0x2528, },
+{ 0x00a78e, CIM_NONE, 0x0000, },
+{ 0x00a790, CIM_CASEBIT, 0x0001, },
+{ 0x00a794, CIM_NONE, 0x0000, },
+{ 0x00a796, CIM_CASEBIT, 0x0001, },
+{ 0x00a7aa, CIM_LONGUPPERDELTA, -0x2544, },
+{ 0x00a7ab, CIM_LONGUPPERDELTA, -0x254f, },
+{ 0x00a7ac, CIM_LONGUPPERDELTA, -0x254b, },
+{ 0x00a7ad, CIM_LONGUPPERDELTA, -0x2541, },
+{ 0x00a7ae, CIM_NONE, 0x0000, },
+{ 0x00a7b0, CIM_LONGUPPERDELTA, -0x2512, },
+{ 0x00a7b1, CIM_LONGUPPERDELTA, -0x252a, },
+{ 0x00a7b2, CIM_LONGUPPERDELTA, -0x2515, },
+{ 0x00a7b3, CIM_UPPERDELTA, 0x03a0, },
+{ 0x00a7b4, CIM_CASEBIT, 0x0001, },
+{ 0x00a7b8, CIM_NONE, 0x0000, },
+{ 0x00ab53, CIM_LOWERDELTA, 0x03a0, },
+{ 0x00ab54, CIM_NONE, 0x0000, },
+{ 0x00ab70, CIM_LONGLOWERDELTA, 0x17d1, },
+{ 0x00abc0, CIM_NONE, 0x0000, },
 { 0x00ff21, CIM_CASEBITOFF, 0x0020, },
 { 0x00ff3b, CIM_NONE, 0x0000, },
 { 0x00ff41, CIM_CASEBITOFF, 0x0020, },
@@ -361,6 +408,12 @@
 { 0x010400, CIM_UPPERDELTA, 0x0028, },
 { 0x010428, CIM_LOWERDELTA, 0x0028, },
 { 0x010450, CIM_NONE, 0x0000, },
+{ 0x010c80, CIM_CASEBIT, 0x0040, },
+{ 0x010cb3, CIM_NONE, 0x0000, },
+{ 0x010cc0, CIM_CASEBIT, 0x0040, },
+{ 0x010cf3, CIM_NONE, 0x0000, },
+{ 0x0118a0, CIM_CASEBITOFF, 0x0020, },
+{ 0x0118e0, CIM_NONE, 0x0000, },
 
 
 #define CASE_INFO_SHIFT0_HIGH 0x000b
