@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 52efcaea8bc4a892095ead2653c561e0d2e9a8e4 $
 */
 
 /*
@@ -15,7 +14,8 @@
 #include "builtin_functions.h"
 
 enum ia32_reg {P_REG_EAX = 0, P_REG_EBX = 3, P_REG_ECX = 1, P_REG_EDX = 2,
-               P_REG_NONE = 4 };
+               P_REG_NONE = 4, P_REG_ESP = 4, P_REG_EBP = 5, P_REG_ESI = 6,
+               P_REG_EDI = 7, };
 
 #define REG_BITMASK ((1 << P_REG_NONE) - 1)
 
@@ -76,6 +76,13 @@ static int alloc_regs = 0, valid_regs = 0;
     PUSH_ADDR (&(ADDR));						\
   } while (0)
 
+#define MOV_REG_TO_REG(FROM, TO) do {				\
+    CHECK_VALID_REG(FROM);					\
+    MAKE_VALID_REG(TO);						\
+    add_to_program (0x89);	/* Move r32 to r/m32. */	\
+    add_to_program (0xc0 | ((FROM)<<3) | (TO));			\
+  } while(0)
+
 #define MOV_REG_TO_ABSADDR(REG, ADDR) do {				\
     CHECK_VALID_REG (REG);						\
     /* movl %reg,addr */						\
@@ -202,6 +209,37 @@ static int alloc_regs = 0, valid_regs = 0;
     }									\
   } while (0)
 
+#define MOV_RELSTACK_TO_REG(OFFSET, REG) do {				\
+    INT32 off_ = (OFFSET);						\
+    MAKE_VALID_REG (REG);						\
+    /* movl offset(%esp), %reg */					\
+    add_to_program (0x8b); /* Move r/m32 to r32. */			\
+    if (off_ < -128 || off_ > 127) {					\
+      add_to_program (0x84 | ((REG) << 3));				\
+      add_to_program (0x24);						\
+      PUSH_INT (off_);							\
+    }									\
+    else if (off_) {							\
+      add_to_program (0x44 | ((REG) << 3));				\
+      add_to_program (0x24);						\
+      add_to_program (off_);						\
+    }									\
+    else {								\
+      add_to_program (0x04 | ((REG) << 3));				\
+      add_to_program (0x24);						\
+    }									\
+  } while (0)
+
+#define PUSH_REG(REG) do {			\
+    CHECK_VALID_REG (REG);			\
+    add_to_program (0x50 + (REG));		\
+  } while (0)
+
+#define POP_REG(REG) do {			\
+    MAKE_VALID_REG (REG);			\
+    add_to_program (0x58 + (REG));		\
+  } while (0)
+
 #define ADD_VAL_TO_REG(VAL, REG) do {					\
     INT32 val_ = (VAL);							\
     CHECK_VALID_REG (REG);						\
@@ -297,6 +335,45 @@ static int alloc_regs = 0, valid_regs = 0;
     }									\
   } while (0)
 
+#define CMP_REG_IMM32(REG, IMM) do {		\
+    if ((REG) == P_REG_EAX) {			\
+      add_to_program(0x3d);			\
+    } else {					\
+      add_to_program(0x81);			\
+      add_to_program(0xf8 | (REG));		\
+    }						\
+    PUSH_INT((IMM));				\
+  } while (0)
+
+#define CMP_REG_REG(REG1, REG2) do {			\
+    add_to_program(0x39);				\
+    add_to_program(0xc0 | ((REG1)<<3) | (REG2));	\
+  } while (0)
+
+#define JNE(SKIP) do {					\
+    int skip__ = (SKIP);				\
+    if ((skip__ >= -0x80) && (skip__ <= 0x7f)) {	\
+      add_to_program(0x75);				\
+      add_to_program(skip__);				\
+    } else {						\
+      Pike_fatal("Not supported yet.\n");		\
+    }							\
+  } while(0)
+
+#define JE(SKIP) do {					\
+    int skip__ = (SKIP);				\
+    if ((skip__ >= -0x80) && (skip__ <= 0x7f)) {	\
+      add_to_program(0x74);				\
+      add_to_program(skip__);				\
+    } else {						\
+      Pike_fatal("Not supported yet.\n");		\
+    }							\
+  } while(0)
+
+#define RET() do {					\
+    add_to_program(0xc3);				\
+  } while (0);
+
 #define CALL_RELATIVE(X) do{						\
   struct program *p_=Pike_compiler->new_program;			\
   add_to_program(0xe8);							\
@@ -309,6 +386,36 @@ static int alloc_regs = 0, valid_regs = 0;
     ((INT32)(X)) - (INT32)(p_->program + p_->num_program);              \
 }while(0)
 
+#ifdef INS_ENTRY
+void ia32_ins_entry(void)
+{
+#ifdef REGISTER_DEBUG
+  /* All registers are valid here... */
+  valid_regs = 0xff;
+#endif
+  /* Push all registers that the ABI requires to be preserved.
+   *
+   * cf Sys V ABI Intel386 Architecture Supplement 4th ed 3-11:
+   *   "Registers %ebp, %ebx, %edi, %esi, and %esp ``belong'' to the cal-
+   *    ling function."
+   *
+   * also cf Fig 3-16.
+   */
+  PUSH_REG(P_REG_EBP);
+  MOV_REG_TO_REG(P_REG_ESP, P_REG_EBP);
+  /* FIXME: Add local storage here if needed. */
+  PUSH_REG(P_REG_EDI);
+  PUSH_REG(P_REG_ESI);
+  PUSH_REG(P_REG_EBX);
+  /* Funcall arguments (4 * 4 bytes). */
+  ADD_VAL_TO_REG(-0x10, P_REG_ESP);
+
+#ifdef REGISTER_DEBUG
+  valid_regs = 0;
+#endif
+}
+#endif
+
 static void update_arg1(INT32 value)
 {
   MOV_VAL_TO_RELSTACK (value, 0);
@@ -325,13 +432,13 @@ static void update_arg2(INT32 value)
 static int cpu_vendor = PIKE_CPU_VENDOR_UNKNOWN;
 
 static enum ia32_reg next_reg;
-static enum ia32_reg sp_reg, fp_reg, mark_sp_reg;
+static enum ia32_reg pi_reg, sp_reg, fp_reg, mark_sp_reg;
 ptrdiff_t ia32_prev_stored_pc; /* PROG_PC at the last point Pike_fp->pc was updated. */
 
 void ia32_flush_code_generator(void)
 {
   next_reg = P_REG_EAX;
-  sp_reg = fp_reg = mark_sp_reg = P_REG_NONE;
+  pi_reg = sp_reg = fp_reg = mark_sp_reg = P_REG_NONE;
   CLEAR_REGS();
   ia32_prev_stored_pc = -1;
 }
@@ -369,8 +476,11 @@ static enum ia32_reg alloc_reg (int avoid_regs)
     if (sp_reg == reg)			{sp_reg = P_REG_NONE; DEALLOC_REG (reg);}
     else if (fp_reg == reg)		{fp_reg = P_REG_NONE; DEALLOC_REG (reg);}
     else if (mark_sp_reg == reg)	{mark_sp_reg = P_REG_NONE; DEALLOC_REG (reg);}
+    else if (pi_reg == reg)		{pi_reg = P_REG_NONE; DEALLOC_REG (reg);}
   }
 
+  if (reg == pi_reg)		        {pi_reg = P_REG_NONE; DEALLOC_REG (reg);}
+
 #ifdef REGISTER_DEBUG
   if ((1 << reg) & alloc_regs) Pike_fatal ("Clobbering allocated register.\n");
   alloc_regs &= avoid_regs;
@@ -386,28 +496,37 @@ static enum ia32_reg alloc_reg (int avoid_regs)
       REG = alloc_reg (avoid_regs);					\
       /* Update the round robin pointer here so that we disregard */	\
       /* the direct calls to alloc_reg for temporary registers. */	\
-      next_reg = (REG + 1) % P_REG_NONE;				\
+      next_reg = (REG + 1) % P_REG_NONE;					\
       {SET;}								\
     }									\
     else								\
       ALLOC_REG (REG);							\
   }
 
+DEF_LOAD_REG (pi_reg, {
+    MOV_ABSADDR_TO_REG (Pike_interpreter_pointer, pi_reg);
+  });
 DEF_LOAD_REG (sp_reg, {
-    MOV_ABSADDR_TO_REG (Pike_interpreter.stack_pointer, sp_reg);
+    load_pi_reg(avoid_regs|(1 << sp_reg));
+    MOV_RELADDR_TO_REG (OFFSETOF(Pike_interpreter_struct, stack_pointer),
+			pi_reg, sp_reg);
   });
 DEF_LOAD_REG (fp_reg, {
-    MOV_ABSADDR_TO_REG (Pike_interpreter.frame_pointer, fp_reg);
+    load_pi_reg(avoid_regs|(1 << fp_reg));
+    MOV_RELADDR_TO_REG (OFFSETOF(Pike_interpreter_struct, frame_pointer),
+			pi_reg, fp_reg);
   });
 DEF_LOAD_REG (mark_sp_reg, {
-    MOV_ABSADDR_TO_REG (Pike_interpreter.mark_stack_pointer, mark_sp_reg);
+    load_pi_reg(avoid_regs|(1 << mark_sp_reg));
+    MOV_RELADDR_TO_REG (OFFSETOF(Pike_interpreter_struct, mark_stack_pointer),
+			pi_reg, mark_sp_reg);
   });
 
 static void ia32_call_c_function(void *addr)
 {
   CALL_RELATIVE(addr);
   next_reg = P_REG_EAX;
-  sp_reg = fp_reg = mark_sp_reg = P_REG_NONE;
+  pi_reg = sp_reg = fp_reg = mark_sp_reg = P_REG_NONE;
   CLEAR_REGS();
 }
 
@@ -416,16 +535,19 @@ static void ia32_call_c_function(void *addr)
 static void ia32_push_constant(struct svalue *tmp)
 {
   int e;
-  if(tmp->type <= MAX_REF_TYPE)
+  if(REFCOUNTED_TYPE(TYPEOF(*tmp)))
     ADD_VAL_TO_ABSADDR (1, tmp->u.refs);
 
-  load_sp_reg (0);
+  load_pi_reg (0);
+  load_sp_reg (1<<pi_reg);
 
   for(e=0;e<(int)sizeof(*tmp)/4;e++)
     MOV_VAL_TO_RELADDR (((INT32 *)tmp)[e], e*4, sp_reg);
 
   ADD_VAL_TO_REG (sizeof(*tmp), sp_reg);
-  MOV_REG_TO_ABSADDR (sp_reg, Pike_interpreter.stack_pointer);
+  MOV_REG_TO_RELADDR (sp_reg,
+		      OFFSETOF(Pike_interpreter_struct, stack_pointer),
+		      pi_reg);
 }
 
 /* The given register holds the address of the svalue we wish to push. */
@@ -458,10 +580,10 @@ static void ia32_push_svalue (enum ia32_reg svalue_ptr_reg)
   add_to_program(0x66);		/* Switch to 16 bit operand mode. */
   add_to_program(0x83);		/* cmp $xx,svalue_ptr_reg */
   add_to_program(0xf8 | svalue_ptr_reg);
-  add_to_program(MAX_REF_TYPE);
+  add_to_program(MIN_REF_TYPE);
   DEALLOC_REG (svalue_ptr_reg);
 
-  add_to_program(0x77); /* ja bork */
+  add_to_program(0x7c); /* jl bork */
   add_to_program(0x02);
 
   CHECK_VALID_REG (tmp_reg);
@@ -471,7 +593,10 @@ static void ia32_push_svalue (enum ia32_reg svalue_ptr_reg)
 
   /* bork: */
   ADD_VAL_TO_REG (sizeof(struct svalue), sp_reg);
-  MOV_REG_TO_ABSADDR (sp_reg, Pike_interpreter.stack_pointer);
+  load_pi_reg(1 << sp_reg);
+  MOV_REG_TO_RELADDR (sp_reg,
+		      OFFSETOF(Pike_interpreter_struct, stack_pointer),
+		      pi_reg);
 }
 
 /* A register holding &Pike_fp->locals[arg] is returned. */
@@ -496,11 +621,12 @@ static void ia32_local_lvalue(INT32 arg)
   struct svalue tmp[2];
   enum ia32_reg addr_reg = ia32_get_local_addr(arg);
 
-  MEMSET(tmp, 0, sizeof(tmp));
-  tmp[0].type=T_SVALUE_PTR;
-  tmp[1].type=T_VOID;
+  memset(tmp, 0, sizeof(tmp));
+  SET_SVAL_TYPE(tmp[0], T_SVALUE_PTR);
+  SET_SVAL_TYPE(tmp[1], T_VOID);
 
-  load_sp_reg (1 << addr_reg);
+  load_pi_reg ((1 << addr_reg)|(1 << sp_reg));
+  load_sp_reg ((1 << addr_reg)|(1 << pi_reg));
 
   for(e=0;e<sizeof(tmp)/4;e++)
   {
@@ -514,7 +640,9 @@ static void ia32_local_lvalue(INT32 arg)
   DEALLOC_REG (addr_reg);
 
   ADD_VAL_TO_REG (sizeof(struct svalue)*2, sp_reg);
-  MOV_REG_TO_ABSADDR (sp_reg, Pike_interpreter.stack_pointer);
+  MOV_REG_TO_RELADDR (sp_reg,
+		      OFFSETOF(Pike_interpreter_struct, stack_pointer),
+		      pi_reg);
 }
 
 static void ia32_push_global (INT32 arg)
@@ -539,27 +667,32 @@ static void ia32_push_global (INT32 arg)
   /* Could do an add directly to memory here, but this isn't much
    * slower and it's not unlikely that sp_reg will be used later
    * on. */
-  load_sp_reg (0);
+  load_pi_reg (0);
+  load_sp_reg (1 << pi_reg);
   ADD_VAL_TO_REG (sizeof (struct svalue), sp_reg);
-  MOV_REG_TO_ABSADDR (sp_reg, Pike_interpreter.stack_pointer);
+  MOV_REG_TO_RELADDR (sp_reg,
+		      OFFSETOF(Pike_interpreter_struct, stack_pointer),
+		      pi_reg);
 }
 
 static void ia32_mark(void)
 {
-  load_mark_sp_reg (1 << sp_reg);
-  load_sp_reg (1 << mark_sp_reg);
+  load_mark_sp_reg ((1 << sp_reg)|(1 << pi_reg));
+  load_sp_reg ((1 << mark_sp_reg)|(1 << pi_reg));
 
   MOV_REG_TO_RELADDR (sp_reg, 0, mark_sp_reg);
   ADD_VAL_TO_REG (sizeof (struct svalue *), mark_sp_reg);
-  MOV_REG_TO_ABSADDR (mark_sp_reg, Pike_interpreter.mark_stack_pointer);
+
+  load_pi_reg (0);
+  MOV_REG_TO_RELADDR (mark_sp_reg,
+		      OFFSETOF(Pike_interpreter_struct, mark_stack_pointer),
+		      pi_reg);
 }
 
 static void ia32_push_int(INT32 x)
 {
   struct svalue tmp;
-  tmp.type=PIKE_T_INT;
-  tmp.subtype=0;
-  tmp.u.integer=x;
+  SET_SVAL(tmp, PIKE_T_INT, 0, integer, x);
   ia32_push_constant(&tmp);
 }
 
@@ -577,7 +710,7 @@ static void ia32_push_string (INT32 x, int subtype)
   MOV_RELADDR_TO_REG (x * sizeof (struct pike_string *), tmp_reg, tmp_reg);
   /* tmp_reg is now Pike_fp->context->prog->strings[x] */
 
-  load_sp_reg (1 << tmp_reg);
+  load_sp_reg ((1 << tmp_reg)|(1 << pi_reg));
 
   for (e = 0; e < sizeof (tmp) / 4; e++) {
     if (((INT32 *) &tmp) + e == (INT32 *) &tmp.u.string)
@@ -589,8 +722,11 @@ static void ia32_push_string (INT32 x, int subtype)
   ADD_VAL_TO_RELADDR (1, 0, tmp_reg);
   DEALLOC_REG (tmp_reg);
 
+  load_pi_reg (0);
   ADD_VAL_TO_REG (sizeof (struct svalue), sp_reg);
-  MOV_REG_TO_ABSADDR (sp_reg, Pike_interpreter.stack_pointer);
+  MOV_REG_TO_RELADDR (sp_reg,
+		      OFFSETOF(Pike_interpreter_struct, stack_pointer),
+		      pi_reg);
 }
 
 void ia32_update_pc(void)
@@ -681,6 +817,7 @@ static void ins_debug_instr_prologue (PIKE_INSTR_T instr, INT32 arg1, INT32 arg2
 void ins_f_byte(unsigned int b)
 {
   void *addr;
+  INT32 rel_addr = 0;
 
   b-=F_OFFSET;
 #ifdef PIKE_DEBUG
@@ -747,16 +884,84 @@ void ins_f_byte(unsigned int b)
 	addr = (void *)f_get_iterator;
       }
       break;
+
+#ifdef OPCODE_INLINE_RETURN
+  case F_CATCH - F_OFFSET:
+    {
+      /* Special argument for the F_CATCH instruction. */
+      addr = inter_return_opcode_F_CATCH;
+      /* FIXME: Is there really no easier way to get at EIP? */
+      add_to_program(0xe8);
+      PUSH_INT(0);
+      POP_REG(P_REG_EAX);	/* EIP ==> EAX */
+      ADD_VAL_TO_REG(0x7fffffff, P_REG_EAX);
+      rel_addr = PIKE_PC;
+      MOV_REG_TO_RELSTACK(P_REG_EAX, 0);
+    }
+    break;
+#endif
   }
 #endif /* !DEBUG_MALLOC */
 
   ia32_call_c_function(addr);
 
+#ifdef INS_ENTRY
+  if (instrs[b].flags & I_RETURN) {
+    INT32 skip;
+#ifdef REGISTER_DEBUG
+    INT32 orig_valid_regs = valid_regs;
+    valid_regs = 0xff;
+#endif
+    if ((b + F_OFFSET) == F_RETURN_IF_TRUE) {
+      /* Kludge. We must check if the ret addr is
+       * orig_addr + JUMP_EPILOGUE_SIZE. */
+      /* There's no easy way to get at EIP directly,
+       * so we assume that the function we called
+       * above exited with a RET, in which case EIP
+       * is at ESP[-4]...
+       */
+      MOV_RELSTACK_TO_REG(-4, P_REG_ESI);
+      ADD_VAL_TO_REG(JUMP_EPILOGUE_SIZE, P_REG_ESI);
+    }
+    CMP_REG_IMM32(P_REG_EAX, -1);
+    JNE(0);
+    skip = (INT32)PIKE_PC;
+    ADD_VAL_TO_REG(0x10, P_REG_ESP);	/* Funcall arguments (4 * 4 bytes). */
+    POP_REG(P_REG_EBX);
+    POP_REG(P_REG_ESI);
+    POP_REG(P_REG_EDI);
+    POP_REG(P_REG_EBP);
+    RET();
+    Pike_compiler->new_program->program[skip-1] = ((INT32)PIKE_PC) - skip;
+    if ((b + F_OFFSET) == F_RETURN_IF_TRUE) {
+      /* Kludge. We must check if the ret addr is
+       * orig_addr + JUMP_EPILOGUE_SIZE. */
+      CMP_REG_REG(P_REG_EAX, P_REG_ESI);
+      JE(0x02);
+      add_to_program (0xff);	/* jmp *%eax */
+      add_to_program (0xe0);
+#ifdef REGISTER_DEBUG
+      valid_regs = orig_valid_regs;
+#endif
+      return;
+    }
+#ifdef REGISTER_DEBUG
+    valid_regs = orig_valid_regs;
+#endif
+  }
+#endif
+
 #ifdef OPCODE_RETURN_JUMPADDR
   if (instrs[b].flags & I_JUMP) {
     /* This is the code that JUMP_EPILOGUE_SIZE compensates for. */
     add_to_program (0xff);	/* jmp *%eax */
     add_to_program (0xe0);
+
+#ifdef OPCODE_INLINE_RETURN
+    if (b + F_OFFSET == F_CATCH) {
+      upd_pointer(rel_addr - 4, PIKE_PC + 6 - rel_addr);
+    }
+#endif
   }
 #endif
 }
@@ -834,8 +1039,8 @@ void ins_f_byte_with_arg(unsigned int a, INT32 b)
        *
        * /grubba 2003-12-11
        */
-      if((Pike_compiler->new_program->constants[b].sval.type > MAX_REF_TYPE) &&
-	 !Pike_compiler->new_program->constants[b].sval.subtype)
+      if(!REFCOUNTED_TYPE(TYPEOF(Pike_compiler->new_program->constants[b].sval)) &&
+	 !SUBTYPEOF(Pike_compiler->new_program->constants[b].sval))
       {
 	ins_debug_instr_prologue (a - F_OFFSET, b, 0);
 	ia32_push_constant(& Pike_compiler->new_program->constants[b].sval);
@@ -1030,12 +1235,12 @@ void ia32_flush_instruction_cache(void *start, size_t len)
     addr = (char *)(((size_t)start) & ~(ia32_clflush_size-1));
     end_addr = addr + len;
     while (addr < end_addr) {
-#ifdef USE_CL_IA32_ASM_STYLE
+#ifdef CL_IA32_ASM_STYLE
       __asm {
 	mov eax, addr;
 	clflush [eax];
       }
-#else  /* USE_GCC_IA32_ASM_STYLE */
+#else  /* GCC_IA32_ASM_STYLE */
 #if 1
       __asm__ __volatile__("clflush %0" :: "m" (*addr));
 #else
@@ -1050,79 +1255,14 @@ void ia32_flush_instruction_cache(void *start, size_t len)
   }
 }
 
-static void ia32_get_cpuid(int oper, INT32 *cpuid_ptr)
-{
-  static int cpuid_supported = 0;
-  if (!cpuid_supported) {
-    int fbits=0;
-#ifdef USE_CL_IA32_ASM_STYLE
-    __asm {
-      pushf
-      pop  eax
-      mov  ecx, eax
-      xor  eax, 00200000h
-      push eax
-      popf
-      pushf
-      pop  eax
-      xor  ecx, eax
-      mov  fbits, ecx
-    };
-#else  /* USE_GCC_IA32_ASM_STYLE */
-    /* Note: gcc swaps the argument order... */
-    __asm__("pushf\n\t"
-	    "pop  %%eax\n\t"
-	    "movl %%eax, %%ecx\n\t"
-	    "xorl $0x00200000, %%eax\n\t"
-	    "push %%eax\n\t"
-	    "popf\n\t"
-	    "pushf\n\t"
-	    "pop  %%eax\n\t"
-	    "xorl %%eax, %%ecx\n\t"
-	    "movl %%ecx, %0"
-	    : "=m" (fbits)
-	    :
-	    : "cc", "eax", "ecx");
-#endif
-    if (fbits & 0x00200000) {
-      cpuid_supported = 1;
-    } else {
-      cpuid_supported = -1;
-    }
-  }
-
-  if (cpuid_supported > 0) {
-#ifdef USE_CL_IA32_ASM_STYLE
-    __asm {
-      mov eax, oper;
-      mov edi, cpuid_ptr;
-      cpuid;
-      mov [edi], eax;
-      mov [edi+4], ebx;
-      mov [edi+8], edx;
-      mov [edi+12], ecx;
-    };
-#else  /* USE_GCC_IA32_ASM_STYLE */
-    __asm__ __volatile__("cpuid"
-			 : "=a" (cpuid_ptr[0]),
-			   "=b" (cpuid_ptr[1]),
-			   "=d" (cpuid_ptr[2]),
-			   "=c" (cpuid_ptr[3])
-			 : "0" (oper));
-#endif
-  } else {
-    cpuid_ptr[0] = cpuid_ptr[1] = cpuid_ptr[2] = cpuid_ptr[3] = 0;
-  }
-}
-
 void ia32_init_interpreter_state(void)
 {
   /* Note: One extra zero for nul-termination. */
   INT32 cpuid[5] = { 0, 0, 0, 0, 0 };
 
-  /* fprintf(stderr, "Calling ia32_get_cpuid()...\n"); */
+  /* fprintf(stderr, "Calling x86_get_cpuid()...\n"); */
 
-  ia32_get_cpuid(0, cpuid);
+  x86_get_cpuid(0, cpuid);
 
   /* fprintf(stderr, "CPUID: %d, \"%.12s\"\n", cpuid[0], (char *)(cpuid+1)); */
 
@@ -1142,7 +1282,7 @@ void ia32_init_interpreter_state(void)
       int feature_flags_edx;
       int feature_flags_ecx;
     } cpu_info;
-    ia32_get_cpuid(1, (INT32 *) &cpu_info);
+    x86_get_cpuid(1, (INT32 *) &cpu_info);
 #if 0
     fprintf(stderr,
 	    "features: 0x%08x\n"
@@ -1157,4 +1297,8 @@ void ia32_init_interpreter_state(void)
       ia32_clflush_size = cpu_info.clflush_size * 8;
     }
   }
+
+#ifdef OPCODE_INLINE_RETURN
+  instrs[F_CATCH - F_OFFSET].address = inter_return_opcode_F_CATCH;
+#endif
 }
