@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 8165183dc2d23c9d30b19d071ca79848e2518be5 $
 */
 
 #define PPC_INSTR_B_FORM(OPCD,BO,BI,BD,AA,LK)			\
@@ -88,29 +87,29 @@ void ppc32_flush_code_generator_state(void);
 #define PPC_CODEGEN_MARK_SP_NEEDSSTORE 16
 #define PPC_CODEGEN_PC_ISSET 32
 
-#define LOAD_FP_REG() do {				\
-    if(!(ppc32_codegen_state & PPC_CODEGEN_FP_ISSET)) {	\
-      /* lwz pike_fp,frame_pointer(pike_interpreter) */	\
-      LWZ(PPC_REG_PIKE_FP, PPC_REG_PIKE_INTERP, 	\
-	  OFFSETOF(Pike_interpreter, frame_pointer));	\
-      ppc32_codegen_state |= PPC_CODEGEN_FP_ISSET;	\
-    }							\
+#define LOAD_FP_REG() do {					\
+    if(!(ppc32_codegen_state & PPC_CODEGEN_FP_ISSET)) {		\
+      /* lwz pike_fp,frame_pointer(pike_interpreter) */		\
+      LWZ(PPC_REG_PIKE_FP, PPC_REG_PIKE_INTERP,			\
+	  OFFSETOF(Pike_interpreter_struct, frame_pointer));	\
+      ppc32_codegen_state |= PPC_CODEGEN_FP_ISSET;		\
+    }								\
   } while(0)
 
-#define LOAD_SP_REG() do {				\
-    if(!(ppc32_codegen_state & PPC_CODEGEN_SP_ISSET)) {	\
-      /* lwz pike_sp,stack_pointer(pike_interpreter) */	\
-      LWZ(PPC_REG_PIKE_SP, PPC_REG_PIKE_INTERP,		\
-	  OFFSETOF(Pike_interpreter, stack_pointer));	\
-      ppc32_codegen_state |= PPC_CODEGEN_SP_ISSET;	\
-    }							\
+#define LOAD_SP_REG() do {					\
+    if(!(ppc32_codegen_state & PPC_CODEGEN_SP_ISSET)) {		\
+      /* lwz pike_sp,stack_pointer(pike_interpreter) */		\
+      LWZ(PPC_REG_PIKE_SP, PPC_REG_PIKE_INTERP,			\
+	  OFFSETOF(Pike_interpreter_struct, stack_pointer));	\
+      ppc32_codegen_state |= PPC_CODEGEN_SP_ISSET;		\
+    }								\
   } while(0)
 
 #define LOAD_MARK_SP_REG() do {						\
     if(!(ppc32_codegen_state & PPC_CODEGEN_MARK_SP_ISSET)) {		\
       /* lwz pike_mark_sp,mark_stack_pointer(pike_interpreter) */	\
       LWZ(PPC_REG_PIKE_MARK_SP, PPC_REG_PIKE_INTERP,			\
-	  OFFSETOF(Pike_interpreter, mark_stack_pointer));		\
+	  OFFSETOF(Pike_interpreter_struct, mark_stack_pointer));	\
       ppc32_codegen_state |= PPC_CODEGEN_MARK_SP_ISSET;			\
     }									\
   } while(0)
@@ -238,7 +237,7 @@ void ppc32_decode_program(struct program *p);
 			"	mr "PPC_REGNAME(29)",%1\n"		    \
 			"	bctr"					    \
 			:						    \
-			: "r" (pc), "r" (&Pike_interpreter)		    \
+			: "r" (pc), "r" (Pike_interpreter_pointer)	    \
 			: "ctr", "lr", "cc", "memory", "r29", "r0",	    \
 			  "r3", "r4", "r5", "r6", "r7", "r8", "r9",	    \
 			  "r10", "r11", "r12")
