@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 2feefdde76930db0d6d8203070aff32fe325ed66 $
 */
 
 /*
@@ -35,10 +34,10 @@
     SET_REG64(11, (func_+32768)&~0xffff);	\
     if ((func_ &= 0xffff) > 32767)		\
       func_ -= 65536;				\
-    /* ld r0,%lo(func)(r11) */			\
-    LD(0, 11, func_);				\
-    /* ld r2,%lo(func+8)(r11) */		\
-    LD(2, 11, func_+8);				\
+    /* ldu r0,%lo(func)(r11) */			\
+    LDU(0, 11, func_);				\
+    /* ld r2,%8(r11) */				\
+    LD(2, 11, 8);				\
     /* mtlr r0 */				\
     MTSPR(0, PPC_SPREG_LR);			\
     /* blrl */					\
@@ -53,12 +52,12 @@ void ppc64_flush_code_generator_state()
   if(ppc64_codegen_state & PPC_CODEGEN_SP_NEEDSSTORE) {
     /* stw pike_sp,stack_pointer(pike_interpreter) */
     STD(PPC_REG_PIKE_SP, PPC_REG_PIKE_INTERP,
-	OFFSETOF(Pike_interpreter, stack_pointer));
+	OFFSETOF(Pike_interpreter_struct, stack_pointer));
   }
   if(ppc64_codegen_state & PPC_CODEGEN_MARK_SP_NEEDSSTORE) {
     /* stw pike_mark_sp,mark_stack_pointer(pike_interpreter) */
     STD(PPC_REG_PIKE_MARK_SP, PPC_REG_PIKE_INTERP,
-	OFFSETOF(Pike_interpreter, mark_stack_pointer));
+	OFFSETOF(Pike_interpreter_struct, mark_stack_pointer));
   }
   ppc64_codegen_state = 0;
   last_prog_id=-1;
@@ -115,23 +114,23 @@ void ppc64_push_svalue(int reg, INT32 offs, int force_reftype)
   STD(0, PPC_REG_PIKE_SP, 0);
   if(!force_reftype) {
 #if PIKE_BYTEORDER == 1234
-    /* rldicl r0,r0,0,48 */
-    RLDICL(0, 0, 0, 48);
+    /* rldicl r0,r0,61,51 */
+    RLDICL(0, 0, 61, 51);
 #else
-    /* rldicl r0,r0,16,48 */
-    RLDICL(0, 0, 16, 48);
+    /* rldicl r0,r0,13,51 */
+    RLDICL(0, 0, 13, 51);
 #endif
   }
   /* std r11,refs(pike_sp) */
   STD(11, PPC_REG_PIKE_SP, OFFSETOF(svalue,u.refs));
   if(!force_reftype) {
-    /* cmplwi r0,MAX_REF_TYPE */
-    CMPLI(0, 0, MAX_REF_TYPE);
+    /* cmplwi r0,MIN_REF_TYPE>>3 */
+    CMPLI(0, 0, MIN_REF_TYPE>>3);
   }
   INCR_SP_REG(sizeof(struct svalue));
   if(!force_reftype) {
-    /* bgt bork */
-    BC(12, 1, 4);
+    /* bne bork */
+    BC(4, 2, 4);
   }
   /* lwz r0,0(r11) */
   LWZ(0, 11, 0);
@@ -151,7 +150,7 @@ void ppc64_push_constant(INT32 arg)
    * Note: The constants table may contain UNDEFINED in case of being
    *       called through decode_value() in PORTABLE_BYTECODE mode.
    */
-  if((sval->type > MAX_REF_TYPE) && !sval->subtype) {
+  if(!REFCOUNTED_TYPE(TYPEOF(*sval)) && !SUBTYPEOF(*sval)) {
     int e;
     INT64 last=0;
 
@@ -187,7 +186,7 @@ void ppc64_push_constant(INT32 arg)
       offs -= 65536;
   }
 
-  ppc64_push_svalue(PPC_REG_ARG1, offs, (sval->type <= MAX_REF_TYPE));
+  ppc64_push_svalue(PPC_REG_ARG1, offs, !!REFCOUNTED_TYPE(TYPEOF(*sval)));
 }
 
 void ppc64_push_local(INT32 arg)
@@ -252,7 +251,7 @@ void ppc64_push_global(INT32 arg)
   } else {
     /* ld r3,stack_pointer(pike_interpreter) */ 
     LD(PPC_REG_ARG1, PPC_REG_PIKE_INTERP,
-       OFFSETOF(Pike_interpreter, stack_pointer));
+       OFFSETOF(Pike_interpreter_struct, stack_pointer));
   }
   if(arg) {
     /* addi r5,r5,arg */
@@ -358,13 +357,13 @@ static void ppc64_escape_catch(void)
   LOAD_FP_REG();
   /* ld r3,catch_ctx(pike_interpreter) */
   LD(PPC_REG_ARG1, PPC_REG_PIKE_INTERP,
-     OFFSETOF(Pike_interpreter, catch_ctx));
+     OFFSETOF(Pike_interpreter_struct, catch_ctx));
   /* ld r4,recovery.previous(r3) */
   LD(PPC_REG_ARG2, PPC_REG_ARG1,
      OFFSETOF(catch_context, recovery.previous));
   /* std r4,recoveries(pike_interpreter) */
   STD(PPC_REG_ARG2, PPC_REG_PIKE_INTERP,
-      OFFSETOF(Pike_interpreter, recoveries));
+      OFFSETOF(Pike_interpreter_struct, recoveries));
   /* ld r4,save_expendible(r3) */
   LD(PPC_REG_ARG2, PPC_REG_ARG1,
      OFFSETOF(catch_context, save_expendible));
@@ -376,7 +375,7 @@ static void ppc64_escape_catch(void)
      OFFSETOF(catch_context, prev));
   /* std r4,catch_ctx(pike_interpreter) */
   STD(PPC_REG_ARG2, PPC_REG_PIKE_INTERP,
-      OFFSETOF(Pike_interpreter, catch_ctx));
+      OFFSETOF(Pike_interpreter_struct, catch_ctx));
 
   FLUSH_CODE_GENERATOR_STATE();
   ADD_CALL(really_free_catch_context);
@@ -901,11 +900,12 @@ void ppc64_disassemble_code(void *addr, size_t bytes)
       case 4:
 	h = (xo^98)%26;
 	instr_name = (xo == opxo_31_100[h]? opname_31_100[h]:NULL);
-	if(instr & (1<<20))
+	if(instr & (1<<20)) {
 	  if(xo == 144)
 	    instr_name = "Fmtocrf";
 	  else if(xo == 19)
 	    instr_name = "Fmfocrf";
+	}
 	break;
       case 5:
 	h = (xo^67)%99;
@@ -1071,7 +1071,7 @@ void ppc64_disassemble_code(void *addr, size_t bytes)
 	  if((xo & 479)==274)
 	    fprintf(stderr, "%s r%d,%d\n", instr_name,
 		    (instr>>11)&31, (instr>>21)&1);
-	  else (xo & 32)
+	  else if(xo & 32)
 	    fprintf(stderr, "%s r%d\n", instr_name, (instr>>11)&31);
 	  else if(xo == 595)
 	    fprintf(stderr, "%s r%d,%d\n", instr_name,
