@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 40d72e38f38fd4dd240a60d9b53f652bf074c41a $
 */
 
 #define NO_PIKE_SHORTHAND
@@ -23,6 +22,7 @@
 #include "module_support.h"
 #include "threads.h"
 #include "gc.h"
+#include "pike_types.h"
 
 /* __attribute__ only applies to function declarations, not
    definitions, so we disable them here. */
@@ -32,7 +32,7 @@
 /* #define ERROR_DEBUG */
 
 PMOD_EXPORT const char msg_fatal_error[] =
-  "%s:%d: Fatal error:\n";
+  "%s:%ld: Fatal error:\n";
 #ifdef PIKE_DEBUG
 PMOD_EXPORT const char msg_unsetjmp_nosync_1[] =
   "UNSETJMP out of sync! (last SETJMP at %s)!\n";
@@ -204,7 +204,7 @@ PMOD_EXPORT DECLSPEC(noreturn) void pike_throw(void) ATTRIBUTE((noreturn))
 
 #if defined(DEBUG_MALLOC) && defined(PIKE_DEBUG)
   /* This will tell us where the value was caught (I hope) */
-  if(throw_value.type <= MAX_REF_TYPE)
+  if(REFCOUNTED_TYPE(TYPEOF(throw_value)))
   {
     debug_malloc_update_location(throw_value.u.refs,
 				 Pike_interpreter.recoveries->file);
@@ -238,7 +238,7 @@ PMOD_EXPORT void push_error(const char *description)
 }
 
 PMOD_EXPORT struct svalue throw_value = SVALUE_INIT_FREE;
-int throw_severity;
+PMOD_EXPORT int throw_severity;
 static const char *in_error;
 
 PMOD_EXPORT DECLSPEC(noreturn) void low_error(const char *buf) ATTRIBUTE((noreturn))
@@ -251,117 +251,6 @@ PMOD_EXPORT DECLSPEC(noreturn) void low_error(const char *buf) ATTRIBUTE((noretu
   pike_throw();  /* Hope someone is catching, or we will be out of balls. */
 }
 
-PMOD_EXPORT void Pike_vsnprintf(char *str, size_t size,
-				const char *fmt, va_list args)
-{
-  size--;
-
-  do {
-    if(fmt[0]=='%') {
-
-      fmt++;
-
-      switch( fmt++[0] ) {
-
-      case 'O':
-	{
-	  dynamic_buffer buf;
-	  dynbuf_string s;
-	  char *ostr;
-	  struct svalue t = va_arg(args, struct svalue);
-	  init_buf(&buf);
-	  describe_svalue(&t,0,0);
-	  s=complex_free_buf(&buf);
-	  ostr=s.str;
-
-	  while( --size>0 && (s.len--)>0 )
-	    str++[0]=s.str++[0];
-
-	  free(ostr);
-	}
-	break;
-
-      case 'S':
-	{
-	  dynamic_buffer buf;
-	  dynbuf_string s;
-	  char *ostr;
-	  struct svalue t;
-
-	  t.type = PIKE_T_STRING;
-	  t.u.string = va_arg(args, struct pike_string *);
-
-	  init_buf(&buf);
-	  describe_svalue(&t,0,0);
-	  s=complex_free_buf(&buf);
-	  ostr=s.str;
-
-	  while( --size>0 && (s.len--)>0 )
-	    str++[0]=s.str++[0];
-
-	  free(ostr);
-	}
-	break;
-
-      case 's':
-	{
-	  char *from = va_arg(args, char *);
-	  while( --size>0 && from[0]!=0 )
-	    str++[0]=from++[0];
-	}
-	break;
-
-      case 'c':
-	{
-	  int c = (char)va_arg(args, int);
-	  sprintf(str, "%c", c);
-	  str++;
-	  size--;
-	}
-	break;
-
-      case 'd':
-	{
-	  char buf[12];
-	  int pos=0;
-	  sprintf(buf, "%d", va_arg(args, int));
-	  while( --size>0 && buf[pos]!=0 )
-	    str++[0]=buf[pos++];
-	}
-	break;
-
-      case 'x':
-	{
-	  char buf[12];
-	  int pos=0;
-	  sprintf(buf, "%x", va_arg(args, int));
-	  while( --size>0 && buf[pos]!=0 )
-	    str++[0]=buf[pos++];
-	}
-	break;
-
-      case '%':
-	str++[0]='%';
-	size--;
-	break;
-
-      default:
-	Pike_fatal("Unknown Pike_vsnprintf formatting char '%c'.\n",
-		   (fmt-1)[0]);
-      }
-    }
-    else {
-      str++[0]=fmt++[0];
-      size--;
-    }
-
-  } while(fmt[0] && size>0);
-
-  str[0]=0;
-  va_end(args);
-}
-
-
 PMOD_EXPORT void va_make_error (const char *fmt, va_list args)
 {
   struct string_builder s;
@@ -418,6 +307,7 @@ PMOD_EXPORT void make_error (const char *fmt, ...)
   va_end (args);
 }
 
+/* coverity[+kill] */
 PMOD_EXPORT DECLSPEC(noreturn) void Pike_error(const char *fmt,...) ATTRIBUTE((noreturn))
 {
   va_list args;
@@ -481,12 +371,12 @@ PMOD_EXPORT DECLSPEC(noreturn) void new_error(const char *name,
     push_int(0);
 
   for (i=-args; i; i++) {
-    if (oldsp[i].type <= PIKE_T_FLOAT) {
+    if (TYPEOF(oldsp[i]) <= MAX_TYPE) {
       push_svalue(oldsp + i);
     } else {
       char buffer[50];
       sprintf(buffer, "<Svalue:0x%04x:0x%04x:%p>",
-	      oldsp[i].type, oldsp[i].subtype, oldsp[i].u.ptr);
+	      TYPEOF(oldsp[i]), SUBTYPEOF(oldsp[i]), oldsp[i].u.ptr);
       push_text(buffer);
     }
   }
@@ -590,7 +480,7 @@ PMOD_EXPORT DECLSPEC(noreturn) void debug_va_fatal(const char *fmt, va_list args
   /* Prevent double fatal. */
   if (in_fatal)
   {
-    if (fmt) (void)VFPRINTF(stderr, fmt, args);
+    if (fmt) (void)vfprintf(stderr, fmt, args);
     do_abort();
   }
 
@@ -601,7 +491,7 @@ PMOD_EXPORT DECLSPEC(noreturn) void debug_va_fatal(const char *fmt, va_list args
     if (fmt) {
       va_list a;
       va_copy (a, args);
-      (void)VFPRINTF(stderr, fmt, a);
+      (void)vfprintf(stderr, fmt, a);
       va_end (a);
     }
 #endif
@@ -609,7 +499,7 @@ PMOD_EXPORT DECLSPEC(noreturn) void debug_va_fatal(const char *fmt, va_list args
   }
 #endif
 
-  if (fmt) (void)VFPRINTF(stderr, fmt, args);
+  if (fmt) (void)vfprintf(stderr, fmt, args);
 
   if(Pike_in_gc)
     fprintf(stderr,"Pike was in GC stage %d when this fatal occurred.\n",Pike_in_gc);
@@ -626,19 +516,23 @@ PMOD_EXPORT DECLSPEC(noreturn) void debug_va_fatal(const char *fmt, va_list args
      * evaluator stuff while we let the master describe the backtrace
      * below. Doing it the naughty way without going through
      * init_threads_disable etc to avoid hanging on runaway locks. */
+#ifdef PIKE_THREADS
     threads_disabled++;
-    MEMSET (&evaluator_callbacks, 0, sizeof (evaluator_callbacks));
+#endif
+    memset (&evaluator_callbacks, 0, sizeof (evaluator_callbacks));
     if (SETJMP (jmp))
       fprintf(stderr,"Got exception when trying to describe backtrace.\n");
     else {
       jmp.severity = THROW_EXIT; /* Don't want normal exit code to run here. */
       push_error("Backtrace at time of fatal:\n");
       APPLY_MASTER("describe_backtrace",1);
-      if(Pike_sp[-1].type==PIKE_T_STRING)
+      if(TYPEOF(Pike_sp[-1]) == PIKE_T_STRING)
 	write_to_stderr(Pike_sp[-1].u.string->str, Pike_sp[-1].u.string->len);
     }
     UNSETJMP (jmp);
+#ifdef PIKE_THREADS
     threads_disabled--;
+#endif
     evaluator_callbacks = saved_eval_cbs;
   }else{
     fprintf(stderr,"No stack - no backtrace.\n");
@@ -647,6 +541,7 @@ PMOD_EXPORT DECLSPEC(noreturn) void debug_va_fatal(const char *fmt, va_list args
   do_abort();
 }
 
+/* coverity[+kill] */
 PMOD_EXPORT DECLSPEC(noreturn) void debug_fatal(const char *fmt, ...) ATTRIBUTE((noreturn))
 {
   va_list args;
@@ -730,16 +625,17 @@ PMOD_EXPORT DECLSPEC(noreturn) void debug_fatal(const char *fmt, ...) ATTRIBUTE(
  */
 static void f_error_cast(INT32 args)
 {
-  char *s;
-  get_all_args("error->cast",args,"%s",&s);
-  if(!strncmp(s,"array",5))
+  if(Pike_sp[-args].u.string == literal_array_string)
   {
-    pop_n_elems(args);
+    pop_stack();
     apply_current (generic_err_message_fun, 0);
     apply_current (generic_err_backtrace_fun, 0);
     f_aggregate(2);
-  }else{
-    SIMPLE_BAD_ARG_ERROR("error->cast", 1, "the value \"array\"");
+  }
+  else
+  {
+    pop_stack();
+    push_undefined();
   }
 }
 
@@ -764,7 +660,7 @@ static void f_error_cast(INT32 args)
 static void f_error_index(INT32 args)
 {
   INT_TYPE ind;
-  get_all_args("error->`[]",args,"%i",&ind);
+  get_all_args("`[]",args,"%i",&ind);
 
   switch(ind)
   {
@@ -777,7 +673,7 @@ static void f_error_index(INT32 args)
       apply_current (generic_err_backtrace_fun, 0);
       break;
     default:
-      index_error("error->`[]", Pike_sp-args, args, NULL, Pike_sp-args,
+      index_error("`[]", Pike_sp-args, args, NULL, Pike_sp-args,
 		  "Index %"PRINTPIKEINT"d is out of range 0..1.\n", ind);
       break;
   }
@@ -858,7 +754,7 @@ static void f_error__sprintf(INT32 args)
 {
   int mode = 0;
 
-  if(args>0 && Pike_sp[-args].type == PIKE_T_INT)
+  if(args>0 && TYPEOF(Pike_sp[-args]) == PIKE_T_INT)
     mode = Pike_sp[-args].u.integer;
   pop_n_elems(args);
 
@@ -881,14 +777,14 @@ static void f_error__sprintf(INT32 args)
 						  SEE_PRIVATE);
     if (i != -1)
       push_svalue (&PROG_FROM_INT (p, i)->
-		   constants[ID_FROM_INT (p, i)->func.offset].sval);
+		   constants[ID_FROM_INT (p, i)->func.const_info.offset].sval);
     else {
       ref_push_program (p);
       SAFE_APPLY_MASTER ("describe_program", 1);
     }
   }
 
-  push_constant_text("(%O)");
+  push_text("(%O)");
   if(GENERIC_ERROR_THIS->error_message)
     ref_push_string(GENERIC_ERROR_THIS->error_message);
   else
@@ -909,7 +805,7 @@ static void f_error__is_type(INT32 args)
   MAKE_CONST_STRING(array_string, "array");
   if (args < 0) SIMPLE_TOO_FEW_ARGS_ERROR("_is_type", 1);
   if (args > 1) SIMPLE_WRONG_NUM_ARGS_ERROR("_is_type", 1);
-  if (Pike_sp[-args].type != PIKE_T_STRING)
+  if (TYPEOF(Pike_sp[-args]) != PIKE_T_STRING)
     SIMPLE_ARG_TYPE_ERROR("_is_type", 1, "string");
   ret = Pike_sp[-args].u.string == array_string;
   pop_n_elems(args);
@@ -982,17 +878,12 @@ static void f_error_create(INT32 args)
     if (X) { \
       assign_svalue_no_free( & ERROR_STRUCT(STRUCT,o)->X, X); \
     } else { \
-      ERROR_STRUCT(STRUCT, o)->X.type = PIKE_T_INT; \
-      ERROR_STRUCT(STRUCT, o)->X.subtype = NUMBER_UNDEFINED; \
-      ERROR_STRUCT(STRUCT, o)->X.u.integer = 0; \
+      SET_SVAL(ERROR_STRUCT(STRUCT, o)->X, PIKE_T_INT, NUMBER_UNDEFINED, \
+	       integer, 0); \
     } \
   } while (0)
 
 
-#define ERROR_COPY_REF(STRUCT,X) \
-  add_ref( ERROR_STRUCT(STRUCT,o)->X=X )
-
-
 /* This prepares the passed object o, which is assumed to inherit
  * generic_error_program, and throws it:
  *
@@ -1010,8 +901,7 @@ PMOD_EXPORT DECLSPEC(noreturn) void generic_error_va(
   struct object *o, const char *func, const struct svalue *base_sp, int args,
   const char *fmt, va_list *fmt_args)
 {
-  struct generic_error_struct *err =
-    (struct generic_error_struct *) get_storage (o, generic_error_program);
+  struct generic_error_struct *err = get_storage (o, generic_error_program);
 
 #ifdef PIKE_DEBUG
   if (!err)
@@ -1057,7 +947,7 @@ PMOD_EXPORT DECLSPEC(noreturn) void generic_error_va(
     f_add(2);
   }
 
-  if(Pike_sp[-1].type!=PIKE_T_ARRAY)
+  if(TYPEOF(Pike_sp[-1]) != PIKE_T_ARRAY)
     Pike_fatal("f_backtrace failed to generate a backtrace!\n");
 
   if (err->error_backtrace) free_array(err->error_backtrace);
@@ -1066,9 +956,7 @@ PMOD_EXPORT DECLSPEC(noreturn) void generic_error_va(
   dmalloc_touch_svalue(Pike_sp);
 
   free_svalue(& throw_value);
-  throw_value.type=PIKE_T_OBJECT;
-  throw_value.subtype = 0;
-  throw_value.u.object=o;
+  SET_SVAL(throw_value, PIKE_T_OBJECT, 0, object, o);
   throw_severity = THROW_ERROR;
   in_error=0;
 
@@ -1130,7 +1018,8 @@ PMOD_EXPORT DECLSPEC(noreturn) void bad_arg_error(
   INIT_ERROR(bad_argument);
   ERROR_COPY(bad_argument, which_argument);
   if (expected_type)
-    ERROR_STRUCT(bad_argument,o)->expected_type=make_shared_string(expected_type);
+    ERROR_STRUCT(bad_argument,o)->expected_type =
+      make_shared_string(expected_type);
   else
     ERROR_STRUCT(bad_argument,o)->expected_type = NULL;
   ERROR_COPY_SVALUE(bad_argument, got_value);
@@ -1150,9 +1039,8 @@ PMOD_EXPORT DECLSPEC(noreturn) void math_error(
   {
     ERROR_COPY_SVALUE(math, number);
   }else{
-    ERROR_STRUCT(math,o)->number.type=PIKE_T_INT;
-    ERROR_STRUCT(math,o)->number.subtype=NUMBER_UNDEFINED;
-    ERROR_STRUCT(math,o)->number.u.integer=0;
+    SET_SVAL(ERROR_STRUCT(math,o)->number, PIKE_T_INT, NUMBER_UNDEFINED,
+	     integer, 0);
   }
   ERROR_DONE(generic);
 }
@@ -1198,7 +1086,7 @@ PMOD_EXPORT void wrong_number_of_args_error(const char *name, int args, int expe
 }
 
 #ifdef PIKE_DEBUG
-static void gc_check_throw_value(struct callback *foo, void *bar, void *gazonk)
+static void gc_check_throw_value(struct callback *UNUSED(foo), void *UNUSED(bar), void *UNUSED(gazonk))
 {
   gc_mark_external_svalues(&throw_value,1," in the throw value");
 }
