@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 84009926a74582f2f273454d1810454a72f07421 $
 */
 
 #ifndef TESTING
@@ -157,7 +156,7 @@ static void grow_fds_to_close(void)
   if(!fds_to_close)
     Pike_fatal("Out of memory in fd_control::grow_fds_to_close()\n"
           "Tried to allocate %d fd_datum structs\n", fds_to_close_size);
-  MEMSET( fds_to_close+(fds_to_close_size/2), 0, fds_to_close_size*sizeof(int)/2 );
+  memset( fds_to_close+(fds_to_close_size/2), 0, fds_to_close_size*sizeof(int)/2 );
 }
 
 void do_close_on_exec(void)
@@ -225,6 +224,22 @@ PMOD_EXPORT int set_close_on_exec(int fd, int which)
 #endif /* !HAVE_BROKEN_F_SETFD */
 }
 
+#ifndef HAVE_ACCEPT4
+int accept4(int fd, struct sockaddr *addr, ACCEPT_SIZE_T *addrlen, int flags)
+{
+  fd = fd_accept(fd, addr, addrlen);
+  if (!flags || (fd < 0)) return fd;
+  if (((flags & SOCK_NONBLOCK) && (set_nonblocking(fd, 1) < 0)) ||
+      ((flags & SOCK_CLOEXEC) && (set_close_on_exec(fd, 1) < 0))) {
+    int e = errno;
+    fd_close(fd);
+    errno = e;
+    return -1;
+  }
+  return fd;
+}
+#endif /* !HAVE_ACCEPT4 */
+
 #endif /* !TESTING */
 
 #ifdef TESTING
