@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 7a8b9cf9fc93d321096264cf3c296313b29f3dff $
 */
 
 #ifndef GLOBAL_H
@@ -45,21 +44,24 @@
 #define WIN32	100	/* WinNT 1.0 */
 #endif
 
-/* We want WinNT 5.0 API's if available. */
-#if !defined(_WIN32_WINDOWS) || (_WIN32_WINDOWS < 0x500)
+/* We want WinNT 5.0+ API's if available.
+ *
+ * We avoid the WinNT 6.0+ API's for now.
+ */
+#if !defined(_WIN32_WINDOWS) || (_WIN32_WINDOWS < 0x5ff)
 #undef _WIN32_WINDOWS
-#define _WIN32_WINDOWS 0x0500
+#define _WIN32_WINDOWS 0x05ff
 #endif
 
-#if !defined(_WIN32_WINNT) || (_WIN32_WINNT < 0x500)
+#if !defined(_WIN32_WINNT) || (_WIN32_WINNT < 0x5ff)
 #undef _WIN32_WINNT
-#define _WIN32_WINNT 0x0500
+#define _WIN32_WINNT 0x05ff
 #endif
 
 /* In later versions of the WIN32 SDKs, we also need to define this one. */
-#if !defined(NTDDI_VERSION) || (NTDDI_VERSION < 0x05000000)
+#if !defined(NTDDI_VERSION) || (NTDDI_VERSION < 0x05ffffff)
 #undef NTDDI_VERSION
-#define NTDDI_VERSION 0x05000000
+#define NTDDI_VERSION 0x05ffffff
 #endif
 
 #endif /* __NT__ */
@@ -205,6 +207,14 @@ struct timeval;
 #define DECLSPEC(X)
 #endif /* HAVE_DECLSPEC */
 
+#ifdef HAS_BUILTIN_EXPECT
+# define UNLIKELY(X) __builtin_expect( (X), 0 )
+# define LIKELY(X) __builtin_expect( (X), 1 )
+#else
+# define UNLIKELY(X) X
+# define LIKELY(X) X
+#endif
+
 #ifndef HAVE_WORKING_REALLOC_NULL
 #define realloc(PTR, SZ)	pike_realloc(PTR,SZ)
 #endif
@@ -273,12 +283,6 @@ void *alloca();
 # endif
 #endif
 
-#ifdef __NT__
-/* We are running NT */
-#undef FD_SETSIZE
-#define FD_SETSIZE MAX_OPEN_FILEDESCRIPTORS
-#endif
-
 #ifdef HAVE_DEVICES_TIMER_H
 /* On AmigaOS, struct timeval is defined in a variety of places
    and a variety of ways.  Making sure <devices/timer.h> is included
@@ -484,33 +488,52 @@ typedef signed INT32 p_wchar2;
 
 typedef struct p_wchar_p
 {
-  p_wchar0 *ptr;
+  void *ptr;
   int shift;
 } PCHARP;
 
-#ifndef CONFIGURE_TEST
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7)
-#define RCSID(X) \
- static const char rcsid[] __attribute__ ((unused)) =X
-#elif __GNUC__ == 2
-#define RCSID(X) \
- static const char rcsid[] = X; \
- static const void *use_rcsid=(&use_rcsid, (void *)&rcsid)
-#else
-#define RCSID(X) \
- static const char rcsid[] = X
-#endif
-#else
+/* Pointless in git. */
 #define RCSID(X)
-#endif
 
 #ifdef PIKE_DEBUG
+
 #define DO_IF_DEBUG(X) X
+#define DO_IF_DEBUG_ELSE(DEBUG, NO_DEBUG) DEBUG
+
 #undef NDEBUG
-#else
+
+/* Set of macros to simplify passing __FILE__ and __LINE__ to
+ * functions only in debug mode. */
+#define DLOC			__FILE__, __LINE__
+#define COMMA_DLOC		, __FILE__, __LINE__
+#define DLOC_DECL		const char *dloc_file, int dloc_line
+#define COMMA_DLOC_DECL		, const char *dloc_file, int dloc_line
+#define DLOC_ARGS		dloc_file, dloc_line
+#define DLOC_PF(PRE, SUF)	PRE "%s:%d" SUF
+#define DLOC_ARGS_OPT		dloc_file, dloc_line
+#define COMMA_DLOC_ARGS_OPT	, dloc_file, dloc_line
+#define USE_DLOC_ARGS()		((void)(DLOC_ARGS_OPT))
+#define DLOC_ENABLED
+
+#else  /* !PIKE_DEBUG */
+
 #define DO_IF_DEBUG(X)
+#define DO_IF_DEBUG_ELSE(DEBUG, NO_DEBUG) NO_DEBUG
 #define NDEBUG
-#endif
+
+#define DLOC
+#define COMMA_DLOC
+#define DLOC_DECL
+#define COMMA_DLOC_DECL
+#define DLOC_ARGS		__FILE__, __LINE__
+#define DLOC_PF(PRE, SUF)
+#define DLOC_ARGS_OPT
+#define COMMA_DLOC_ARGS_OPT
+#define USE_DLOC_ARGS()
+
+#endif	/* !PIKE_DEBUG */
+
+#include <assert.h>
 
 #if defined (PIKE_DEBUG) || defined (DO_PIKE_CLEANUP)
 #define DO_IF_DEBUG_OR_CLEANUP(X) X
@@ -532,6 +555,34 @@ typedef struct p_wchar_p
 #endif
 #endif
 
+/* Suppress compiler warnings for unused parameters if possible. The mangling of
+   argument name is required to catch when an unused argument later is used without
+   removing the annotation. */
+#ifndef PIKE_UNUSED_ATTRIBUTE
+# ifdef __GNUC__
+#  define PIKE_UNUSED_ATTRIBUTE  ATTRIBUTE((unused))
+# else
+#  define PIKE_UNUSED_ATTRIBUTE
+# endif
+#endif
+#ifndef PIKE_UNUSED
+# define PIKE_UNUSED(x)  PIKE_CONCAT(x,_UNUSED) PIKE_UNUSED_ATTRIBUTE
+#endif
+#ifndef UNUSED
+# define UNUSED(x)  PIKE_UNUSED(x)
+#endif
+#ifdef PIKE_DEBUG
+# define DEBUGUSED(x) x
+#else
+# define DEBUGUSED(x) PIKE_UNUSED(x)
+#endif
+#ifdef DEBUG_MALLOC
+# define DMALLOCUSED(x) x
+#else
+# define DMALLOCUSED(x) PIKE_UNUSED(x)
+#endif
+
+
 /* PMOD_EXPORT exports a function / variable vfsh. */
 #ifndef PMOD_EXPORT
 # if defined (__NT__) && defined (USE_DLL)
@@ -543,7 +594,17 @@ typedef struct p_wchar_p
  * themselves, unless they are compiled statically. */
 #   define PMOD_EXPORT __declspec(dllexport)
 #  endif
-# else
+# elif defined(__clang__) && defined(MAC_OS_X_VERSION_MIN_REQUIRED)
+/* According to Clang source the protected behavior is ELF-specific and not
+   applicable to OS X. */
+#  define PMOD_EXPORT    __attribute__ ((visibility("default")))
+# elif __GNUC__ >= 4
+#  ifdef DYNAMIC_MODULE
+#    define PMOD_EXPORT  __attribute__ ((visibility("default")))
+#  else
+#    define PMOD_EXPORT  __attribute__ ((visibility("protected")))
+#  endif
+# else 
 #  define PMOD_EXPORT
 # endif
 #endif
@@ -552,12 +613,8 @@ typedef struct p_wchar_p
 #define PMOD_PROTO
 #endif
 
-#if defined(USE_VALGRIND) && !defined(__CHECKER__)
-#define __CHECKER__
-#endif
-
 #ifndef DO_PIKE_CLEANUP
-#if defined(PURIFY) || defined(__CHECKER__) || defined(DEBUG_MALLOC) || defined(USE_VALGRIND)
+#if defined(PURIFY) || defined(__CHECKER__) || defined(DEBUG_MALLOC)
 #define DO_PIKE_CLEANUP
 #endif
 #endif
@@ -609,10 +666,10 @@ struct iovec {
 #endif
 
 #ifdef MALLOC_DECL_MISSING
-char *malloc PROT((int));
-char *realloc PROT((char *,int));
-void free PROT((char *));
-char *calloc PROT((int,int));
+void *malloc PROT((int));
+void *realloc PROT((void *,int));
+void free PROT((void *));
+void *calloc PROT((int,int));
 #endif
 
 #ifdef GETPEERNAME_DECL_MISSING
@@ -644,14 +701,7 @@ char *crypt(const char *, const char *);
 /* Compatibility... */
 #define USE_PIKE_TYPE	2
 
-#ifdef PIKE_RUN_UNLOCKED
-#define DO_IF_RUN_UNLOCKED(X) X
-#else
-#define DO_IF_RUN_UNLOCKED(X)
-#endif
-
 /* Used in more than one place, better put it here */
-
 #if defined(PROFILING)
 #define DO_IF_PROFILING(X) X
 #else
