@@ -1,9 +1,8 @@
-#line 213 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 204 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 OPCODE0(F_UNDEFINED, "push UNDEFINED", I_UPDATE_SP)
 
 
 
-
 OPCODE0(F_CONST0, "push 0", I_UPDATE_SP)
 
 
@@ -38,27 +37,58 @@ OPCODE1(F_NUMBER, "push int", I_UPDATE_SP)
 
 
 OPCODE2(F_NUMBER64, "push 64-bit int", I_UPDATE_SP)
-#line 262 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
+
 OPCODE1(F_NEG_NUMBER, "push -int", I_UPDATE_SP)
 
 
 
 OPCODE1(F_CONSTANT, "constant", I_UPDATE_SP)
-#line 275 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
 OPCODE2(F_REARRANGE, "rearrange", 0)
-#line 284 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
 OPCODE1_TAIL(F_MARK_AND_STRING, "mark & string", I_UPDATE_SP|I_UPDATE_M_SP)OPCODE1(F_STRING, "string", I_UPDATE_SP)
-#line 297 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
 OPCODE1(F_ARROW_STRING, "->string", I_UPDATE_SP)
 
 
 
 
 
-
-
 OPCODE1(F_LOOKUP_LFUN, "->lfun", 0)
-#line 338 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 317 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_LFUN, "local function", I_UPDATE_SP)
 
 
@@ -66,13 +96,60 @@ OPCODE1(F_LFUN, "local function", I_UPDATE_SP)
 
 
 OPCODE2(F_TRAMPOLINE, "trampoline", I_UPDATE_SP)
-#line 367 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 344 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
+
 OPCODE1_TAIL(F_MARK_AND_GLOBAL, "mark & global", I_UPDATE_SP|I_UPDATE_M_SP)OPCODE1(F_GLOBAL, "global", I_UPDATE_SP)
-#line 377 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
+OPCODE1(F_PRIVATE_GLOBAL, "global <private>", I_UPDATE_SP)
+
+
+
+
+
+
+
+
+
+OPCODE2(F_PRIVATE_IF_DIRECT_GLOBAL, "global <private if direct>", I_UPDATE_SP)
+#line 387 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE2(F_ASSIGN_PRIVATE_IF_DIRECT_GLOBAL, "assign global <private if direct>", 0)
+#line 405 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE2(F_ASSIGN_PRIVATE_TYPED_GLOBAL_AND_POP, "assign global <private,typed> and pop", I_UPDATE_SP)
+#line 417 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
+OPCODE2(F_ASSIGN_PRIVATE_TYPED_GLOBAL, "assign global <private,typed>", 0)
+#line 465 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+OPCODE2(F_PRIVATE_TYPED_GLOBAL, "global <private,typed>", I_UPDATE_SP)
+#line 492 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
+
 OPCODE2_TAIL(F_MARK_AND_EXTERNAL, "mark & external", I_UPDATE_SP|I_UPDATE_M_SP)OPCODE2(F_EXTERNAL, "external", I_UPDATE_SP)
-#line 409 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 525 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE2(F_EXTERNAL_LVALUE, "& external", I_UPDATE_SP)
-#line 431 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 546 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_MARK_AND_LOCAL, "mark & local", I_UPDATE_SP|I_UPDATE_M_SP)
 
 
@@ -113,65 +190,138 @@ OPCODE1(F_LOCAL_LVALUE, "& local", I_UPDATE_SP)
 
 
 
-
 OPCODE2(F_LEXICAL_LOCAL, "lexical local", I_UPDATE_SP)
-#line 483 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 597 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE2(F_LEXICAL_LOCAL_LVALUE, "&lexical local", I_UPDATE_SP)
-#line 496 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 609 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_ARRAY_LVALUE, "[ lvalues ]", I_UPDATE_SP)
-#line 506 "/Users/hww3/pikebuild/src/interpret_functions.h"
-OPCODE1(F_CLEAR_2_LOCAL, "clear 2 local", 0)
-#line 516 "/Users/hww3/pikebuild/src/interpret_functions.h"
-OPCODE1(F_CLEAR_4_LOCAL, "clear 4 local", 0)
-#line 527 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
+OPCODE2(F_CLEAR_N_LOCAL, "clear n local", 0)
+
+
+
+
+
+
+
+
+
 OPCODE1(F_CLEAR_LOCAL, "clear local", 0)
 
 
 
 
+OPCODE2(F_ADD_LOCALS_AND_POP, "local += local", 0)
+#line 680 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE2(F_ADD_LOCAL_INT_AND_POP, "local += number", 0)
+#line 698 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE2(F_ADD_LOCAL_INT, "local += number local", 0)
+#line 717 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 
 
 OPCODE1(F_INC_LOCAL, "++local", I_UPDATE_SP)
-#line 551 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 733 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_POST_INC_LOCAL, "local++", I_UPDATE_SP)
-#line 570 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 749 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_INC_LOCAL_AND_POP, "++local and pop", 0)
-#line 587 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 765 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_DEC_LOCAL, "--local", I_UPDATE_SP)
-#line 604 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 781 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_POST_DEC_LOCAL, "local--", I_UPDATE_SP)
-#line 623 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 797 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_DEC_LOCAL_AND_POP, "--local and pop", 0)
-#line 644 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 813 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
+
+
+
 OPCODE0(F_LTOSVAL, "lvalue to svalue", I_UPDATE_SP)
-#line 675 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 850 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 OPCODE0(F_LTOSVAL2_AND_FREE, "ltosval2 and free", I_UPDATE_SP)
-#line 700 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 867 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
+
+
+
 OPCODE0(F_LTOSVAL3_AND_FREE, "ltosval3 and free", I_UPDATE_SP)
-#line 733 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 898 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
+
+
+
 OPCODE0(F_LTOSVAL_AND_FREE, "ltosval and free", I_UPDATE_SP)
-#line 753 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 920 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_ADD_TO, "+=", I_UPDATE_SP)
-#line 822 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 985 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_ADD_TO_AND_POP, "+= and pop", I_UPDATE_SP)
-#line 889 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1048 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_GLOBAL_LVALUE, "& global", I_UPDATE_SP)
 
 
 
 
 OPCODE0(F_INC, "++x", I_UPDATE_SP)
-#line 914 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1071 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_DEC, "--x", I_UPDATE_SP)
-#line 934 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1089 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_DEC_AND_POP, "x-- and pop", I_UPDATE_SP)
-#line 953 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1106 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_INC_AND_POP, "x++ and pop", I_UPDATE_SP)
-#line 972 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1123 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_POST_INC, "x++", I_UPDATE_SP)
-#line 995 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1144 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_POST_DEC, "x--", I_UPDATE_SP)
-#line 1018 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1165 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_ASSIGN_LOCAL, "assign local", 0)
 
 
@@ -184,6 +334,22 @@ OPCODE0(F_ASSIGN, "assign", I_UPDATE_SP)
 
 
 
+OPCODE1(F_ASSIGN_INDICES, "assign[]", I_UPDATE_SP)
+
+
+
+
+
+
+
+
+OPCODE1(F_ASSIGN_ALL_INDICES, "assign[*]", I_UPDATE_SP)
+
+
+
+
+
+
 OPCODE2(F_APPLY_ASSIGN_LOCAL_AND_POP, "apply, assign local and pop", I_UPDATE_SP|I_UPDATE_M_SP)
 
 
@@ -192,6 +358,7 @@ OPCODE2(F_APPLY_ASSIGN_LOCAL_AND_POP, "apply, assign local and pop", I_UPDATE_SP
 
 
 
+
 OPCODE2(F_APPLY_ASSIGN_LOCAL, "apply, assign local", I_UPDATE_ALL)
 
 
@@ -209,6 +376,11 @@ OPCODE1(F_ASSIGN_LOCAL_AND_POP, "assign local and pop", I_UPDATE_SP)
 
 
 
+OPCODE2(F_ASSIGN_LOCAL_NUMBER_AND_POP, "assign local number and pop", 0)
+
+
+
+
 OPCODE1(F_ASSIGN_GLOBAL, "assign global", 0)
 
 
@@ -216,7 +388,37 @@ OPCODE1(F_ASSIGN_GLOBAL, "assign global", 0)
 
 
 OPCODE1(F_ASSIGN_GLOBAL_AND_POP, "assign global and pop", I_UPDATE_SP)
-#line 1071 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+OPCODE1(F_ASSIGN_PRIVATE_GLOBAL_AND_POP, "assign private global and pop", I_UPDATE_SP)
+#line 1249 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE1(F_ASSIGN_PRIVATE_GLOBAL, "assign private global", I_UPDATE_SP)
+
+
+
+
+
+
+
+
+
+OPCODE2(F_ASSIGN_GLOBAL_NUMBER_AND_POP, "assign global number and pop", 0)
+
+
+
+
+
+
+
+
+
+
 OPCODE0(F_POP_VALUE, "pop", I_UPDATE_SP)
 
 
@@ -226,7 +428,9 @@ OPCODE1(F_POP_N_ELEMS, "pop_n_elems", I_UPDATE_SP)
 
 
 OPCODE0_TAIL(F_MARK2, "mark mark", I_UPDATE_M_SP)OPCODE0_TAIL(F_SYNCH_MARK, "synch mark", I_UPDATE_M_SP)OPCODE0(F_MARK, "mark", I_UPDATE_M_SP)
-#line 1092 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1291 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_MARK_X, "mark Pike_sp-X", I_UPDATE_M_SP)
 
 
@@ -243,7 +447,9 @@ OPCODE0(F_POP_TO_MARK, "pop to mark", I_UPDATE_SP|I_UPDATE_M_SP)
 
 
 OPCODE0_TAIL(F_CLEANUP_SYNCH_MARK, "cleanup synch mark", I_UPDATE_SP|I_UPDATE_M_SP)OPCODE0(F_POP_SYNCH_MARK, "pop synch mark", I_UPDATE_SP|I_UPDATE_M_SP)
-#line 1127 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1326 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_CLEAR_STRING_SUBTYPE, "clear string subtype", 0)
 
 
@@ -254,15 +460,54 @@ OPCODE0_BRANCH(F_BRANCH, "branch", 0)
 
 
 OPCODE2_BRANCH(F_BRANCH_IF_NOT_LOCAL_ARROW, "branch if !local->x", 0)OPCODE0_TAILBRANCH(F_BRANCH_WHEN_ZERO, "branch if zero", I_UPDATE_SP)
-#line 1161 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1357 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE0_BRANCH(F_QUICK_BRANCH_WHEN_ZERO, "(Q) branch if zero", I_UPDATE_SP)
+
+
+
+
+
+
+
+
+
+OPCODE0_BRANCH(F_QUICK_BRANCH_WHEN_NON_ZERO, "(Q) branch if not zero", I_UPDATE_SP)
+
+
+
+
+
+
+
+
+
 OPCODE0_BRANCH(F_BRANCH_WHEN_NON_ZERO, "branch if not zero", I_UPDATE_SP)
-#line 1172 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
 OPCODE1_BRANCH(F_BRANCH_IF_TYPE_IS_NOT, "branch if type is !=", I_UPDATE_SP)
-#line 1200 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1414 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1_BRANCH(F_BRANCH_IF_LOCAL, "branch if local", 0)
-#line 1210 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
 OPCODE1_BRANCH(F_BRANCH_IF_NOT_LOCAL, "branch if !local", 0)
-#line 1231 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1444 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 OPCODE0_BRANCH(F_BRANCH_WHEN_EQ, "branch if ==", I_UPDATE_SP)
 
 OPCODE0_BRANCH(F_BRANCH_WHEN_NE, "branch if !=", I_UPDATE_SP)
@@ -276,19 +521,55 @@ OPCODE0_BRANCH(F_BRANCH_WHEN_GT, "branch if >", I_UPDATE_SP)
 OPCODE0_BRANCH(F_BRANCH_WHEN_GE, "branch if >=", I_UPDATE_SP)
 
 OPCODE0_BRANCH(F_BRANCH_AND_POP_WHEN_ZERO, "branch & pop if zero", 0)
-#line 1249 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
 OPCODE0_BRANCH(F_BRANCH_AND_POP_WHEN_NON_ZERO, "branch & pop if !zero", 0)
-#line 1260 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
 OPCODE0_BRANCH(F_LAND, "&&", I_UPDATE_SP)
-#line 1273 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
 OPCODE0_BRANCH(F_LOR, "||", I_UPDATE_SP)
-#line 1284 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
 OPCODE0_BRANCH(F_EQ_OR, "==||", I_UPDATE_SP)
-#line 1297 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1501 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0_BRANCH(F_EQ_AND, "==&&", I_UPDATE_SP)
-#line 1317 "/Users/hww3/pikebuild/src/interpret_functions.h"
-OPCODE0_PTRJUMP(F_CATCH, "catch", I_UPDATE_ALL)
-#line 1434 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1522 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+OPCODE0_PTRJUMP(F_CATCH, "catch", I_UPDATE_ALL|I_RETURN)
+#line 1637 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_ESCAPE_CATCH, "escape catch", 0)
 
 
@@ -299,11 +580,15 @@ OPCODE0(F_EXIT_CATCH, "exit catch", I_UPDATE_SP)
 
 
 OPCODE1_JUMP(F_SWITCH, "switch", I_UPDATE_ALL)
-#line 1458 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1661 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1_JUMP(F_SWITCH_ON_INDEX, "switch on index", I_UPDATE_ALL)
-#line 1477 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1680 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE2_JUMP(F_SWITCH_ON_LOCAL, "switch on local", 0)
-#line 1530 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1726 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 OPCODE0_BRANCH(F_INC_LOOP, "++Loop", 0)
 
 OPCODE0_BRANCH(F_DEC_LOOP, "--Loop", 0)
@@ -311,22 +596,35 @@ OPCODE0_BRANCH(F_DEC_LOOP, "--Loop", 0)
 OPCODE0_BRANCH(F_INC_NEQ_LOOP, "++Loop!=", 0)
 
 OPCODE0_BRANCH(F_DEC_NEQ_LOOP, "--Loop!=", 0)
-#line 1546 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1742 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 OPCODE0_BRANCH(F_LOOP, "loop", I_UPDATE_SP)
-#line 1559 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1752 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0_BRANCH(F_FOREACH, "foreach", 0)
-#line 1580 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1772 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_MAKE_ITERATOR, "get_iterator", 0)
 
 
 
 
 OPCODE0_BRANCH(F_FOREACH_START, "foreach start", 0)
-#line 1599 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1790 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
 OPCODE0_BRANCH(F_FOREACH_LOOP, "foreach loop", 0)
-#line 1614 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1805 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
 OPCODE1_RETURN(F_RETURN_LOCAL, "return local", I_UPDATE_SP|I_UPDATE_FP)
-#line 1633 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1825 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
 OPCODE0_RETURN(F_RETURN_IF_TRUE, "return if true", I_UPDATE_SP|I_UPDATE_FP)
 
 
@@ -352,15 +650,20 @@ OPCODE0_RETURN(F_DUMB_RETURN, "dumb return", I_UPDATE_FP)
 
 
 OPCODE0(F_NEGATE, "unary minus", 0)
-#line 1681 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1872 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0_ALIAS(F_COMPL, "~", 0, o_compl)
 
 OPCODE0(F_NOT, "!", 0)
-#line 1712 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1900 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
 OPCODE0_ALIAS(F_LSH, "<<", I_UPDATE_SP, o_lsh)
 
 OPCODE0_ALIAS(F_RSH, ">>", I_UPDATE_SP, o_rsh)
-#line 1722 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1913 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 OPCODE0(F_EQ, "==", I_UPDATE_SP)
 
 OPCODE0(F_NE, "!=", I_UPDATE_SP)
@@ -380,9 +683,15 @@ OPCODE0(F_ADD, "+", I_UPDATE_SP)
 
 
 OPCODE0(F_ADD_INTS, "int+int", I_UPDATE_SP)
-#line 1752 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1937 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
 OPCODE0(F_ADD_FLOATS, "float+float", I_UPDATE_SP)
-#line 1764 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 1949 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
 OPCODE0_ALIAS(F_SUBTRACT, "-", I_UPDATE_SP, o_subtract)
 
 OPCODE0_ALIAS(F_AND, "&", I_UPDATE_SP, o_and)
@@ -397,16 +706,71 @@ OPCODE0_ALIAS(F_DIVIDE, "/", I_UPDATE_SP, o_divide)
 
 OPCODE0_ALIAS(F_MOD, "%", I_UPDATE_SP, o_mod)
 
+OPCODE1(F_SUBTRACT_INT, "- int", 0)
+
+
+
+
+OPCODE1(F_AND_INT, "& int", 0)
+
+
+
+
+OPCODE1(F_OR_INT, "| int", 0)
+
+
+
+
+OPCODE1(F_XOR_INT, "^ int", 0)
+
+
+
+
+OPCODE1(F_MULTIPLY_INT, "* int", 0)
+
+
+
+
+OPCODE1(F_DIVIDE_INT, "/ int", 0)
+
+
+
+
+OPCODE1(F_MOD_INT, "% int", 0)
+
+
+
+
+OPCODE1(F_RSH_INT, ">> int", 0)
+
+
+
+
+OPCODE1(F_LSH_INT, "<< int", 0)
+
+
+
+
 OPCODE1(F_ADD_INT, "add integer", 0)
-#line 1787 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2015 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_ADD_NEG_INT, "add -integer", 0)
-#line 1802 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2027 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_PUSH_ARRAY, "@", I_UPDATE_SP)
-#line 1833 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2060 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_APPEND_ARRAY, "append array", I_UPDATE_SP|I_UPDATE_M_SP)
 
 
 
+OPCODE0(F_APPEND_MAPPING, "append mapping", I_UPDATE_SP|I_UPDATE_M_SP)
+
+
+
 OPCODE2(F_LOCAL_LOCAL_INDEX, "local[local]", I_UPDATE_SP)
 
 
@@ -416,15 +780,37 @@ OPCODE2(F_LOCAL_LOCAL_INDEX, "local[local]", I_UPDATE_SP)
 
 
 OPCODE1(F_LOCAL_INDEX, "local index", 0)
-#line 1855 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
 OPCODE2(F_GLOBAL_LOCAL_INDEX, "global[local]", I_UPDATE_SP)
-#line 1867 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2098 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE2(F_LOCAL_ARROW, "local->x", I_UPDATE_SP)
-#line 1877 "/Users/hww3/pikebuild/src/interpret_functions.h"
+
+
+
+
+
+
+
+
+
 OPCODE1(F_ARROW, "->x", 0)
-#line 1889 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2119 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_STRING_INDEX, "string index", 0)
-#line 1901 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2130 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_POS_INT_INDEX, "int index", 0)
 
 
@@ -464,7 +850,10 @@ OPCODE0_ALIAS(F_CAST_TO_INT, "cast_to_int", 0, o_cast_to_int)
 OPCODE0_ALIAS(F_CAST_TO_STRING, "cast_to_string", 0, o_cast_to_string)
 
 OPCODE0(F_SOFT_CAST, "soft cast", I_UPDATE_SP)
-#line 1959 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2187 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
 OPCODE1_ALIAS(F_RANGE, "range", I_UPDATE_SP, o_range2)
 
 OPCODE0(F_COPY_VALUE, "copy_value", 0)
@@ -476,34 +865,74 @@ OPCODE0(F_COPY_VALUE, "copy_value", 0)
 
 
 OPCODE0(F_INDIRECT, "indirect", I_UPDATE_SP)
-#line 1991 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2220 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_SIZEOF, "sizeof", 0)
 
 
 
 
 
+OPCODE0(F_SIZEOF_STRING, "sizeof(string)", 0)
+
+
+
+
+
 OPCODE1(F_SIZEOF_LOCAL, "sizeof local", I_UPDATE_SP)
 
 
 
-OPCODE2_ALIAS(F_SSCANF, "sscanf", I_UPDATE_SP, o_sscanf)
-#line 2097 "/Users/hww3/pikebuild/src/interpret_functions.h"
-OPCODE1_JUMP(F_CALL_LFUN, "call lfun", I_UPDATE_ALL)OPCODE1_JUMP(F_CALL_LFUN_AND_POP, "call lfun" " & pop", I_UPDATE_ALL)OPCODE1_RETURN(F_CALL_LFUN_AND_RETURN, "call lfun" " & return", I_UPDATE_ALL)OPCODE1_JUMP(F_MARK_CALL_LFUN, "mark, " "call lfun", I_UPDATE_ALL)OPCODE1_JUMP(F_MARK_CALL_LFUN_AND_POP, "mark, " "call lfun" " & pop", I_UPDATE_ALL)OPCODE1_RETURN(F_MARK_CALL_LFUN_AND_RETURN, "mark, " "call lfun" " & return", I_UPDATE_ALL)
+OPCODE1(F_SIZEOF_LOCAL_STRING, "sizeof local string", I_UPDATE_SP)
 
 
 
+OPCODE2_ALIAS(F_SSCANF, "sscanf", I_UPDATE_SP, o_sscanf)
+#line 2343 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+OPCODE1_JUMP(F_CALL_LFUN, "call lfun", I_UPDATE_ALL)
+#line 2357 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE2_JUMP(F_CALL_LFUN_N, "call lfun <n>", I_UPDATE_ALL)
+#line 2373 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE1_JUMP(F_CALL_LFUN_AND_POP, "call lfun & pop", I_UPDATE_ALL)
+#line 2390 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE1_RETURN(F_CALL_LFUN_AND_RETURN, "call lfun & return", I_UPDATE_ALL)
+#line 2404 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE1_JUMP(F_MARK_CALL_LFUN, "mark, call lfun", I_UPDATE_ALL)
+#line 2416 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE1_JUMP(F_MARK_CALL_LFUN_AND_POP, "mark, call lfun & pop", I_UPDATE_ALL)
+#line 2432 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE1_RETURN(F_MARK_CALL_LFUN_AND_RETURN, "mark, call lfun & return", I_UPDATE_ALL)
+#line 2447 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1_JUMP(F_APPLY, "apply", I_UPDATE_ALL)OPCODE1_JUMP(F_APPLY_AND_POP, "apply" " & pop", I_UPDATE_ALL)OPCODE1_RETURN(F_APPLY_AND_RETURN, "apply" " & return", I_UPDATE_ALL)OPCODE1_JUMP(F_MARK_APPLY, "mark, " "apply", I_UPDATE_ALL)OPCODE1_JUMP(F_MARK_APPLY_AND_POP, "mark, " "apply" " & pop", I_UPDATE_ALL)OPCODE1_RETURN(F_MARK_APPLY_AND_RETURN, "mark, " "apply" " & return", I_UPDATE_ALL)
 
 
 OPCODE0_JUMP(F_CALL_FUNCTION, "call function", I_UPDATE_ALL)OPCODE0_JUMP(F_CALL_FUNCTION_AND_POP, "call function" " & pop", I_UPDATE_ALL)OPCODE0_RETURN(F_CALL_FUNCTION_AND_RETURN, "call function" " & return", I_UPDATE_ALL)
 
 OPCODE1_JUMP(F_CALL_OTHER, "call other", I_UPDATE_ALL)
-#line 2164 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2510 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1_JUMP(F_CALL_OTHER_AND_POP, "call other & pop", I_UPDATE_ALL)
-#line 2225 "/Users/hww3/pikebuild/src/interpret_functions.h"
-OPCODE1_JUMP(F_CALL_OTHER_AND_RETURN, "call other & return", I_UPDATE_ALL)
-#line 2337 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2572 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE1_RETURN(F_CALL_OTHER_AND_RETURN, "call other & return", I_UPDATE_ALL)
+#line 2700 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 OPCODE1(F_CALL_BUILTIN, "call builtin", I_UPDATE_ALL)
 
 
@@ -545,6 +974,15 @@ OPCODE1(F_CALL_BUILTIN1, "call builtin 1", I_UPDATE_ALL)
 
 
 
+OPCODE2(F_CALL_BUILTIN_N, "call builtin N", I_UPDATE_ALL)
+
+
+
+
+OPCODE2(F_APPLY_N, "apply N", I_UPDATE_ALL)
+
+
+
 OPCODE1(F_CALL_BUILTIN1_AND_POP, "call builtin1 & pop", I_UPDATE_ALL)
 
 
@@ -552,11 +990,16 @@ OPCODE1(F_CALL_BUILTIN1_AND_POP, "call builtin1 & pop", I_UPDATE_ALL)
 
 
 OPCODE1(F_LTOSVAL_CALL_BUILTIN_AND_ASSIGN, "ltosval, call builtin & assign", I_UPDATE_ALL)
-#line 2432 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2800 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_LTOSVAL_CALL_BUILTIN_AND_ASSIGN_POP, "ltosval, call builtin, assign & pop", I_UPDATE_ALL)
-#line 2560 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2916 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
 OPCODE1_PTRJUMP(F_COND_RECUR, "recur if not overloaded", I_UPDATE_ALL)OPCODE0_TAILPTRJUMP(F_RECUR, "recur", I_UPDATE_ALL)
-#line 2618 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 2967 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+
 OPCODE0_PTRJUMP(F_RECUR_AND_POP, "recur & pop", I_UPDATE_ALL)
 
 
@@ -565,11 +1008,25 @@ OPCODE0_PTRJUMP(F_RECUR_AND_POP, "recur & pop", I_UPDATE_ALL)
 
 
 OPCODE0_PTRJUMP(F_TAIL_RECUR, "tail recursion", I_UPDATE_ALL)
-#line 2670 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 3001 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE1(F_THIS_OBJECT, "this_object", I_UPDATE_SP)
-#line 2688 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 3019 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE0(F_UNDEFINEDP, "undefinedp", 0)
+#line 3034 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE0(F_DESTRUCTEDP, "destructedp", 0)
+#line 3051 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_ZERO_TYPE, "zero_type", 0)
-#line 2706 "/Users/hww3/pikebuild/src/interpret_functions.h"
+#line 3068 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
 OPCODE0(F_SWAP, "swap", 0)
 
 
@@ -579,3 +1036,24 @@ OPCODE0(F_DUP, "dup", I_UPDATE_SP)
 
 
 OPCODE2(F_THIS, "this", I_UPDATE_SP)
+#line 3098 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE2(F_MAGIC_TYPES, "::_types", I_UPDATE_SP)
+
+
+
+OPCODE2(F_INIT_FRAME, "init_frame", 0)
+
+
+
+
+OPCODE1(F_PROTECT_STACK, "protect_stack", 0)
+
+
+
+OPCODE2(F_FILL_STACK, "fill_stack", I_UPDATE_SP)
+#line 3122 "/home/zino/hack/pike8-rel/pike/src/interpret_functions.h"
+
+
+OPCODE1(F_MARK_AT, "mark_at", I_UPDATE_SP)
