@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 388b54ad4234057af723fa23ef64b507929c4e5e $
 */
 
 #include "global.h"
@@ -21,7 +20,7 @@
 #include "gc.h"
 #include "stralloc.h"
 #include "pike_security.h"
-#include "block_alloc.h"
+#include "block_allocator.h"
 #include "opcodes.h"
 #include "stuff.h"
 
@@ -43,39 +42,41 @@ static struct mapping *gc_mark_mapping_pos = 0;
 #define MAPPING_DATA_SIZE(HSIZE, KEYPAIRS) \
    PTR_TO_INT(MD_KEYPAIRS(0, HSIZE) + KEYPAIRS)
 
-#undef EXIT_BLOCK
-#define EXIT_BLOCK(m)	do{						\
-DO_IF_DEBUG(								\
-  if(m->refs) {								\
-    DO_IF_DMALLOC(describe_something(m, T_MAPPING, 0,2,0, NULL));	\
-    Pike_fatal("really free mapping on mapping with %d refs.\n", m->refs); \
-  }									\
-)									\
-									\
-  FREE_PROT(m);								\
-									\
-  unlink_mapping_data(m->data);						\
-									\
-  DOUBLEUNLINK(first_mapping, m);					\
-									\
-  GC_FREE(m);                                                           \
-}while(0)
+static struct block_allocator mapping_allocator = BA_INIT_PAGES(sizeof(struct mapping), 2);
+void count_memory_in_mappings(size_t * num, size_t * size) {
+    struct mapping *m;
+    double datasize = 0.0;
+    ba_count_all(&mapping_allocator, num, size);
+    for(m=first_mapping;m;m=m->next) {
+	datasize+=MAPPING_DATA_SIZE(m->data->hashsize, m->data->num_keypairs) / (double) m->data->refs;
+    }
+    *size += (size_t) datasize;
+}
 
+void really_free_mapping(struct mapping * m) {
+#ifdef PIKE_DEBUG
+  if (m->refs) {
+# ifdef DEBUG_MALLOC
+    describe_something(m, T_MAPPING, 0,2,0, NULL);
+# endif
+    Pike_fatal("really free mapping on mapping with %d refs.\n", m->refs);
+  }
+#endif
+  FREE_PROT(m);
+  unlink_mapping_data(m->data);
+  DOUBLEUNLINK(first_mapping, m);
+  GC_FREE(m);
+  ba_free(&mapping_allocator, m);
+}
 
-#undef COUNT_OTHER
+ATTRIBUTE((malloc))
+static struct mapping * alloc_mapping(void) {
+    return ba_alloc(&mapping_allocator);
+}
 
-#define COUNT_OTHER() do{				\
-  struct mapping *m;					\
-  double datasize = 0.0;				\
-  for(m=first_mapping;m;m=m->next)			\
-  {							\
-    datasize+=MAPPING_DATA_SIZE(m->data->hashsize, m->data->num_keypairs) / \
-      (double) m->data->refs;						\
-  }							\
-  size += (size_t) datasize;				\
-}while(0)
-
-BLOCK_ALLOC_FILL_PAGES(mapping, 2)
+void free_all_mapping_blocks(void) {
+    ba_destroy(&mapping_allocator);
+}
 
 #ifndef PIKE_MAPPING_KEYPAIR_LOOP
 #define IF_ELSE_KEYPAIR_LOOP(X, Y)	Y
@@ -93,7 +94,7 @@ BLOCK_ALLOC_FILL_PAGES(mapping, 2)
       INT32 e;						\
       /* Move the last keypair to the new hole. */	\
       *k = *(md->free_list);				\
-      h_ = k->hval % md->hashsize;			\
+      h_ = k->hval & ( md->hashsize - 1);		\
       prev_ = md->hash + h_;				\
       DO_IF_DEBUG(					\
 	if (!*prev_) {					\
@@ -113,6 +114,19 @@ BLOCK_ALLOC_FILL_PAGES(mapping, 2)
   } while(0)
 #endif /* !PIKE_MAPPING_KEYPAIR_LOOP */
 
+void mapping_free_keypair(struct mapping_data *md, struct keypair *k)
+{
+  FREE_KEYPAIR(md, k);
+}
+
+static INLINE int check_type_contains(TYPE_FIELD types, const struct svalue * s) {
+    return (TYPEOF(*s) == PIKE_T_OBJECT || types & (BIT_OBJECT|(1 << TYPEOF(*s))));
+}
+
+static INLINE int check_type_overlaps(TYPE_FIELD t1, TYPE_FIELD t2) {
+    return (!t1 && !t2) || t1 & t2 || (t1|t2) & BIT_OBJECT;
+}
+
 #ifdef PIKE_DEBUG
 
 /** This function checks that the type field isn't lacking any bits.
@@ -130,14 +144,14 @@ static void check_mapping_type_fields(const struct mapping *m)
   md = m->data;
   NEW_MAPPING_LOOP(md)
     {
-      if (k->val.type > MAX_TYPE)
+      if (TYPEOF(k->val) > MAX_TYPE)
 	Pike_fatal("Invalid mapping keypair value type: %s\n",
-		   get_name_of_type(k->val.type));
-      val_types |= 1 << k->val.type;
-      if (k->ind.type > MAX_TYPE)
+		   get_name_of_type(TYPEOF(k->val)));
+      val_types |= 1 << TYPEOF(k->val);
+      if (TYPEOF(k->ind) > MAX_TYPE)
 	Pike_fatal("Invalid maping keypair index type: %s\n",
-		   get_name_of_type(k->ind.type));
-      ind_types |= 1 << k->ind.type;
+		   get_name_of_type(TYPEOF(k->ind)));
+      ind_types |= 1 << TYPEOF(k->ind);
     }
 
   if(val_types & ~(m->data->val_types))
@@ -149,16 +163,16 @@ static void check_mapping_type_fields(const struct mapping *m)
 #endif
 
 static struct mapping_data empty_data =
-  { PIKE_CONSTANT_MEMOBJ_INIT(1), 1, 0,0,0,0,0,0, 0,
+  { PIKE_CONSTANT_MEMOBJ_INIT(1, T_MAPPING_DATA), 1, 0,0,0,0,0,0, 0,
     IF_ELSE_KEYPAIR_LOOP((struct keypair *)&empty_data.hash, 0), {0}};
 static struct mapping_data weak_ind_empty_data =
-  { PIKE_CONSTANT_MEMOBJ_INIT(1), 1, 0,0,0,0,0,0, MAPPING_WEAK_INDICES,
+  { PIKE_CONSTANT_MEMOBJ_INIT(1, T_MAPPING_DATA), 1, 0,0,0,0,0,0, MAPPING_WEAK_INDICES,
     IF_ELSE_KEYPAIR_LOOP((struct keypair *)&weak_ind_empty_data.hash, 0), {0}};
 static struct mapping_data weak_val_empty_data =
-  { PIKE_CONSTANT_MEMOBJ_INIT(1), 1, 0,0,0,0,0,0, MAPPING_WEAK_VALUES,
+  { PIKE_CONSTANT_MEMOBJ_INIT(1, T_MAPPING_DATA), 1, 0,0,0,0,0,0, MAPPING_WEAK_VALUES,
     IF_ELSE_KEYPAIR_LOOP((struct keypair *)&weak_val_empty_data.hash, 0), {0}};
 static struct mapping_data weak_both_empty_data =
-  { PIKE_CONSTANT_MEMOBJ_INIT(1), 1, 0,0,0,0,0,0, MAPPING_WEAK,
+  { PIKE_CONSTANT_MEMOBJ_INIT(1, T_MAPPING_DATA), 1, 0,0,0,0,0,0, MAPPING_WEAK,
     IF_ELSE_KEYPAIR_LOOP((struct keypair *)&weak_both_empty_data.hash, 0), {0}};
 
 /** This function allocates the hash table and svalue space for a mapping
@@ -181,17 +195,17 @@ static void init_mapping(struct mapping *m,
 #endif
   if(size)
   {
-    hashsize=find_good_hash_size(size / AVG_LINK_LENGTH + 1);
+    hashsize=find_next_power(size / AVG_LINK_LENGTH + 1);
+
+    if (size < hashsize) size = hashsize;
 
     e=MAPPING_DATA_SIZE(hashsize, size);
 
-    md=(struct mapping_data *)xalloc(e);
+    md=xcalloc(1,e);
 
     m->data=md;
     md->hashsize=hashsize;
-    
-    MEMSET((char *)md->hash, 0, sizeof(struct keypair *) * md->hashsize);
-    
+
     md->free_list=MD_KEYPAIRS(md, hashsize);
 #ifndef PIKE_MAPPING_KEYPAIR_LOOP
     for(e=1;e<size;e++)
@@ -200,17 +214,17 @@ static void init_mapping(struct mapping *m,
       mark_free_svalue (&md->free_list[e-1].ind);
       mark_free_svalue (&md->free_list[e-1].val);
     }
-    md->free_list[e-1].next=0;
+    /* md->free_list[e-1].next=0; */
     mark_free_svalue (&md->free_list[e-1].ind);
     mark_free_svalue (&md->free_list[e-1].val);
 #endif /* !PIKE_MAPPING_KEYPAIR_LOOP */
-    md->ind_types = 0;
-    md->val_types = 0;
+    /* md->ind_types = 0; */
+    /* md->val_types = 0; */
     md->flags = flags;
-    md->size = 0;
-    md->refs=0;
-    md->valrefs=0;
-    md->hardlinks=0;
+    /* md->size = 0; */
+    /* md->refs=0; */
+    /* md->valrefs=0; */
+    /* md->hardlinks=0; */
     md->num_keypairs=size;
   }else{
     switch (flags & MAPPING_WEAK) {
@@ -264,7 +278,23 @@ PMOD_EXPORT void really_free_mapping_data(struct mapping_data *md)
 #ifdef PIKE_DEBUG
   if (md->refs) {
     Pike_fatal("really_free_mapping_data(): md has non-zero refs: %d\n",
-	  md->refs);
+	       md->refs);
+  }
+
+  if (!md->size) {
+    /* Paranoia and keep gcc happy. */
+    if (md == &empty_data) {
+      Pike_fatal("really_free_mapping_data(): md is empty_data!\n");
+    }
+    if (md == &weak_ind_empty_data) {
+      Pike_fatal("really_free_mapping_data(): md is weak_ind_empty_data!\n");
+    }
+    if (md == &weak_val_empty_data) {
+      Pike_fatal("really_free_mapping_data(): md is weak_val_empty_data!\n");
+    }
+    if (md == &weak_both_empty_data) {
+      Pike_fatal("really_free_mapping_data(): md is weak_both_empty_data!\n");
+    }
   }
 #endif /* PIKE_DEBUG */
 
@@ -274,7 +304,7 @@ PMOD_EXPORT void really_free_mapping_data(struct mapping_data *md)
     free_svalue(& k->ind);
   }
 
-  free((char *) md);
+  free(md);
   GC_FREE_BLOCK(md);
 }
 
@@ -288,6 +318,7 @@ PMOD_EXPORT void do_free_mapping(struct mapping *m)
  * order in each hash chain. This is to prevent mappings from becoming
  * inefficient just after being rehashed.
  */
+/* Evil: Steal the svalues from the old md. */
 static void mapping_rehash_backwards_evil(struct mapping_data *md,
 					  struct keypair *from)
 {
@@ -304,10 +335,55 @@ static void mapping_rehash_backwards_evil(struct mapping_data *md,
   }
   k->next = prev;
 
-  from = k;
+  prev = k;
 
   /* Rehash and reverse the hash chain. */
-  while (from) {
+  while ((from = prev)) {
+    /* Reverse */
+    prev = from->next;
+    from->next = next;
+    next = from;
+
+    if (md->flags & MAPPING_WEAK) {
+
+      switch(md->flags & MAPPING_WEAK) {
+      default:
+	Pike_fatal("Instable mapping data flags.\n");
+      case MAPPING_WEAK_INDICES:
+	if (!REFCOUNTED_TYPE(TYPEOF(from->ind)) ||
+	    (*from->ind.u.refs > 1)) {
+	  goto keep_keypair;
+	}
+	break;
+      case MAPPING_WEAK_VALUES:
+	if (!REFCOUNTED_TYPE(TYPEOF(from->val)) ||
+	    (*from->val.u.refs > 1)) {
+	  goto keep_keypair;
+	}
+	break;
+      case MAPPING_WEAK:
+	/* NB: Compat: Unreference counted values are counted
+	 *             as multi-referenced here.
+	 */
+	if ((!REFCOUNTED_TYPE(TYPEOF(from->ind)) ||
+	     (*from->ind.u.refs > 1)) &&
+	    (!REFCOUNTED_TYPE(TYPEOF(from->val)) ||
+	     (*from->val.u.refs > 1))) {
+	  goto keep_keypair;
+	}
+	break;
+      }
+
+      /* Free.
+       * Note that we don't need to free or unlink the keypair,
+       * since that will be done by the caller anyway. */
+      free_svalue(&from->ind);
+      free_svalue(&from->val);
+
+      continue;
+    }
+  keep_keypair:
+
     /* unlink */
     k=md->free_list;
 #ifndef PIKE_MAPPING_KEYPAIR_LOOP
@@ -324,23 +400,18 @@ static void mapping_rehash_backwards_evil(struct mapping_data *md,
 
     /* link */
     h=k->hval;
-    h%=md->hashsize;
+    h&=md->hashsize - 1;
     k->next=md->hash[h];
     md->hash[h]=k;
 
     /* update */
-    md->ind_types |= 1<< (k->ind.type);
-    md->val_types |= 1<< (k->val.type);
+    md->ind_types |= 1<< (TYPEOF(k->ind));
+    md->val_types |= 1<< (TYPEOF(k->val));
     md->size++;
-
-    /* Reverse */
-    prev = from->next;
-    from->next = next;
-    next = from;
-    from = prev;
   }
 }
 
+/* Good: Copy the svalues from the old md. */
 static void mapping_rehash_backwards_good(struct mapping_data *md,
 					  struct keypair *from)
 {
@@ -357,10 +428,56 @@ static void mapping_rehash_backwards_good(struct mapping_data *md,
   }
   k->next = prev;
 
-  from = k;
+  prev = k;
 
   /* Rehash and reverse the hash chain. */
-  while (from) {
+  while ((from = prev)) {
+    /* Reverse */
+    prev = from->next;
+    from->next = next;
+    next = from;
+
+    if (md->flags & MAPPING_WEAK) {
+
+      switch(md->flags & MAPPING_WEAK) {
+      default:
+	Pike_fatal("Instable mapping data flags.\n");
+      case MAPPING_WEAK_INDICES:
+	if (REFCOUNTED_TYPE(TYPEOF(from->ind)) &&
+	    (*from->ind.u.refs > 1)) {
+	  goto keep_keypair;
+	}
+	break;
+      case MAPPING_WEAK_VALUES:
+	if (REFCOUNTED_TYPE(TYPEOF(from->val)) &&
+	    (*from->val.u.refs > 1)) {
+	  goto keep_keypair;
+	}
+	break;
+      case MAPPING_WEAK:
+	/* NB: Compat: Unreference counted values are counted
+	 *             as multi-referenced here.
+	 */
+	if ((!REFCOUNTED_TYPE(TYPEOF(from->ind)) ||
+	     (*from->ind.u.refs > 1)) &&
+	    (!REFCOUNTED_TYPE(TYPEOF(from->val)) ||
+	     (*from->val.u.refs > 1))) {
+	  goto keep_keypair;
+	}
+	break;
+      }
+
+      /* Skip copying of this keypair.
+       *
+       * NB: We can't mess with the original md here,
+       *     since it might be in use by an iterator
+       *     or similar.
+       */
+
+      continue;
+    }
+  keep_keypair:
+
     /* unlink */
     k=md->free_list;
 #ifndef PIKE_MAPPING_KEYPAIR_LOOP
@@ -379,20 +496,14 @@ static void mapping_rehash_backwards_good(struct mapping_data *md,
 
     /* link */
     h=k->hval;
-    h%=md->hashsize;
+    h&=md->hashsize - 1;
     k->next=md->hash[h];
     md->hash[h]=k;
 
     /* update */
-    md->ind_types |= 1<< (k->ind.type);
-    md->val_types |= 1<< (k->val.type);
+    md->ind_types |= 1<< (TYPEOF(k->ind));
+    md->val_types |= 1<< (TYPEOF(k->val));
     md->size++;
-
-    /* Reverse */
-    prev = from->next;
-    from->next = next;
-    next = from;
-    from = prev;
   }
 }
 
@@ -421,7 +532,10 @@ static struct mapping *rehash(struct mapping *m, int new_size)
   if(d_flag>1)  check_mapping(m);
 #endif
 
-  if (md->hashsize == new_size) return m;
+  /* FIXME: The special case below seems suspect.
+   *	/grubba 2011-09-04
+   */
+  if ((md->hashsize == new_size) && (md->refs == 1)) return m;
 
   init_mapping(m, new_size, md->flags);
   debug_malloc_touch(m);
@@ -431,22 +545,31 @@ static struct mapping *rehash(struct mapping *m, int new_size)
   if(md->refs>1)
   {
     /* good */
+    /* More than one reference to the md ==> We need to
+     * keep it afterwards.
+     */
     for(e=0;e<md->hashsize;e++)
       mapping_rehash_backwards_good(new_md, md->hash[e]);
 
     unlink_mapping_data(md);
   }else{
     /* evil */
+    /* We have the only reference to the md,
+     * so we can just copy the svalues without
+     * bothering about type checking.
+     */
     for(e=0;e<md->hashsize;e++)
       mapping_rehash_backwards_evil(new_md, md->hash[e]);
 
-    free((char *)md);
+    free(md);
     GC_FREE_BLOCK(md);
   }
 
 #ifdef PIKE_DEBUG
-  if(m->data->size != tmp)
-    Pike_fatal("Rehash failed, size not same any more.\n");
+  if((m->data->size != tmp) &&
+     ((m->data->size > tmp) || !(m->data->flags & MAPPING_WEAK)))
+    Pike_fatal("Rehash failed, size not same any more (%ld != %ld).\n",
+	       (long)m->data->size, (long)tmp);
 #endif
 #ifdef MAPPING_SIZE_DEBUG
   m->debug_size = m->data->size;
@@ -483,7 +606,7 @@ struct mapping_data *copy_mapping_data(struct mapping_data *md)
   size=MAPPING_DATA_SIZE(md->hashsize, md->num_keypairs);
 
   nmd=(struct mapping_data *)xalloc(size);
-  MEMCPY(nmd, md, size);
+  memcpy(nmd, md, size);
   off=((char *)nmd) - ((char *)md);
 
   RELOC(nmd->free_list);
@@ -534,9 +657,9 @@ struct mapping_data *copy_mapping_data(struct mapping_data *md)
   add_ref(md);						        \
   if(md->hashsize)						\
   {								\
-    h=h2 % md->hashsize;					\
+    h=h2 & (md->hashsize - 1);					\
     DO_IF_DEBUG( if(d_flag > 1) check_mapping_type_fields(m); ) \
-    if(md->ind_types & ((1 << key->type) | BIT_OBJECT))		\
+    if(check_type_contains(md->ind_types, key))			\
     {								\
       for(prev= md->hash + h;(k=*prev);prev=&k->next)		\
       {								\
@@ -557,11 +680,11 @@ struct mapping_data *copy_mapping_data(struct mapping_data *md)
   add_ref(md);							\
   if(md->hashsize)						\
   {								\
-    h=h2 % md->hashsize;					\
+    h=h2 & (md->hashsize-1);					\
     DO_IF_DEBUG( if(d_flag > 1) check_mapping_type_fields(m); ) \
-    if(md->ind_types & ((1 << key->type) | BIT_OBJECT))		\
+    if(check_type_contains(md->ind_types, key))			\
     {								\
-      k2=omd->hash[h2 % omd->hashsize];			        \
+      k2=omd->hash[h2 & (omd->hashsize - 1)];			        \
       prev= md->hash + h;					\
       for(;(k=*prev) && k2;(prev=&k->next),(k2=k2->next))	\
         if(!(h2 == k->hval && is_identical(&k2->ind, &k->ind)))	\
@@ -622,7 +745,7 @@ struct mapping_data *copy_mapping_data(struct mapping_data *md)
 #define PROPAGATE() do {			\
    if(md->refs==1)				\
    {						\
-     h=h2 % md->hashsize;                       \
+     h=h2 & (md->hashsize - 1);                 \
      *prev=k->next;				\
      k->next=md->hash[h];			\
      md->hash[h]=k;				\
@@ -662,8 +785,8 @@ PMOD_EXPORT void mapping_fix_type_field(struct mapping *m)
 
   NEW_MAPPING_LOOP(m->data)
     {
-      val_types |= 1 << k->val.type;
-      ind_types |= 1 << k->ind.type;
+      val_types |= 1 << TYPEOF(k->val);
+      ind_types |= 1 << TYPEOF(k->ind);
     }
 
 #ifdef PIKE_DEBUG
@@ -753,8 +876,8 @@ PMOD_EXPORT void low_mapping_insert(struct mapping *m,
   if(!overwrite) return;
   PREPARE_FOR_DATA_CHANGE2();
   PROPAGATE(); /* propagate after preparing */
-  md->val_types |= 1 << val->type;
-  if (overwrite == 2 && key->type == T_OBJECT)
+  md->val_types |= 1 << TYPEOF(*val);
+  if (overwrite == 2 && TYPEOF(*key) == T_OBJECT)
     /* Should replace the index too. It's only for objects that it's
      * possible to tell the difference. */
     assign_svalue (&k->ind, key);
@@ -784,7 +907,7 @@ PMOD_EXPORT void low_mapping_insert(struct mapping *m,
     rehash(m, md->size * 2 + 2);
     md=m->data;
   }
-  h=h2 % md->hashsize;
+  h=h2 & ( md->hashsize - 1);
 
   /* no need to lock here since we are not calling is_eq - Hubbe */
 
@@ -796,8 +919,8 @@ PMOD_EXPORT void low_mapping_insert(struct mapping *m,
 #endif /* !PIKE_MAPPING_KEYPAIR_LOOP */
   k->next=md->hash[h];
   md->hash[h]=k;
-  md->ind_types |= 1 << key->type;
-  md->val_types |= 1 << val->type;
+  md->ind_types |= 1 << TYPEOF(*key);
+  md->val_types |= 1 << TYPEOF(*val);
   assign_svalue_no_free(& k->ind, key);
   assign_svalue_no_free(& k->val, val);
   k->hval = h2;
@@ -874,7 +997,7 @@ PMOD_EXPORT union anything *mapping_get_item_ptr(struct mapping *m,
     check_mapping(m);
 #endif
   free_mapping_data(md);
-  if(k->val.type == t)
+  if(TYPEOF(k->val) == t)
   {
     PREPARE_FOR_DATA_CHANGE2();
     PROPAGATE(); /* prepare then propagate */
@@ -908,7 +1031,7 @@ PMOD_EXPORT union anything *mapping_get_item_ptr(struct mapping *m,
     rehash(m, md->size * 2 + 2);
     md=m->data;
   }
-  h=h2 % md->hashsize;
+  h=h2 & ( md->hashsize - 1);
 
   k=md->free_list;
 #ifndef PIKE_MAPPING_KEYPAIR_LOOP
@@ -919,11 +1042,9 @@ PMOD_EXPORT union anything *mapping_get_item_ptr(struct mapping *m,
   k->next=md->hash[h];
   md->hash[h]=k;
   assign_svalue_no_free(& k->ind, key);
-  k->val.type=T_INT;
-  k->val.subtype=NUMBER_NUMBER;
-  k->val.u.integer=0;
+  SET_SVAL(k->val, T_INT, NUMBER_NUMBER, integer, 0);
   k->hval = h2;
-  md->ind_types |= 1 << key->type;
+  md->ind_types |= 1 << TYPEOF(*key);
   md->val_types |= BIT_INT;
   md->size++;
 #ifdef MAPPING_SIZE_DEBUG
@@ -971,9 +1092,7 @@ PMOD_EXPORT void map_delete_no_free(struct mapping *m,
   free_mapping_data(md);
   if(to)
   {
-    to->type=T_INT;
-    to->subtype=NUMBER_UNDEFINED;
-    to->u.integer=0;
+    SET_SVAL(*to, T_INT, NUMBER_UNDEFINED, integer, 0);
   }
   return;
 
@@ -1049,7 +1168,7 @@ PMOD_EXPORT void check_mapping_for_destruct(struct mapping *m)
       {
 	check_destructed(& k->val);
 	
-	if((k->ind.type == T_OBJECT || k->ind.type == T_FUNCTION) &&
+	if((TYPEOF(k->ind) == T_OBJECT || TYPEOF(k->ind) == T_FUNCTION) &&
 	   !k->ind.u.object->prog)
 	{
 	  debug_malloc_touch(m);
@@ -1072,8 +1191,8 @@ PMOD_EXPORT void check_mapping_for_destruct(struct mapping *m)
 #endif
 	  debug_malloc_touch(md);
 	}else{
-	  val_types |= 1 << k->val.type;
-	  ind_types |= 1 << k->ind.type;
+	  val_types |= 1 << TYPEOF(k->val);
+	  ind_types |= 1 << TYPEOF(k->ind);
 	  prev=&k->next;
 	}
       }
@@ -1124,8 +1243,7 @@ PMOD_EXPORT struct svalue *low_mapping_string_lookup(struct mapping *m,
                                                      struct pike_string *p)
 {
   struct svalue tmp;
-  tmp.type=T_STRING;
-  tmp.u.string=p;
+  SET_SVAL(tmp, T_STRING, 0, string, p);
   return low_mapping_lookup(m, &tmp);
 }
 
@@ -1134,9 +1252,7 @@ PMOD_EXPORT void mapping_string_insert(struct mapping *m,
                                        const struct svalue *val)
 {
   struct svalue tmp;
-  tmp.type=T_STRING;
-  tmp.subtype=0;
-  tmp.u.string=p;
+  SET_SVAL(tmp, T_STRING, 0, string, p);
   mapping_insert(m, &tmp, val);
 }
 
@@ -1145,8 +1261,7 @@ PMOD_EXPORT void mapping_string_insert_string(struct mapping *m,
 				  struct pike_string *val)
 {
   struct svalue tmp;
-  tmp.type=T_STRING;
-  tmp.u.string=val;
+  SET_SVAL(tmp, T_STRING, 0, string, val);
   mapping_string_insert(m, p, &tmp);
 }
 
@@ -1175,11 +1290,10 @@ PMOD_EXPORT struct svalue *mapping_mapping_lookup(struct mapping *m,
     Pike_fatal("Zero refs in mapping->data\n");
 #endif
 
-  if(!s || !s->type==T_MAPPING)
+  if(!s || TYPEOF(*s) != T_MAPPING)
   {
     if(!create) return 0;
-    tmp.u.mapping=allocate_mapping(5);
-    tmp.type=T_MAPPING;
+    SET_SVAL(tmp, T_MAPPING, 0, mapping, allocate_mapping(5));
     mapping_insert(m, key1, &tmp);
     debug_malloc_touch(m);
     debug_malloc_touch(tmp.u.mapping);
@@ -1193,9 +1307,7 @@ PMOD_EXPORT struct svalue *mapping_mapping_lookup(struct mapping *m,
   if(s) return s;
   if(!create) return 0;
 
-  tmp.type=T_INT;
-  tmp.subtype=NUMBER_UNDEFINED;
-  tmp.u.integer=0;
+  SET_SVAL(tmp, T_INT, NUMBER_UNDEFINED, integer, 0);
 
   mapping_insert(m2, key2, &tmp);
   debug_malloc_touch(m2);
@@ -1210,10 +1322,8 @@ PMOD_EXPORT struct svalue *mapping_mapping_string_lookup(struct mapping *m,
 				      int create)
 {
   struct svalue k1,k2;
-  k1.type=T_STRING;
-  k1.u.string=key1;
-  k2.type=T_STRING;
-  k2.u.string=key2;
+  SET_SVAL(k1, T_STRING, 0, string, key1);
+  SET_SVAL(k2, T_STRING, 0, string, key2);
   return mapping_mapping_lookup(m,&k1,&k2,create);
 }
 
@@ -1233,12 +1343,10 @@ PMOD_EXPORT void mapping_index_no_free(struct svalue *dest,
     /* Note: There is code that counts on storing UNDEFINED in mapping
      * values (using e.g. low_mapping_lookup to get them), so we have
      * to fix the subtype here rather than in mapping_insert. */
-    if(p->type==T_INT)
-      dest->subtype=NUMBER_NUMBER;
+    if(TYPEOF(*p) == T_INT)
+      SET_SVAL_SUBTYPE(*dest, NUMBER_NUMBER);
   }else{
-    dest->type=T_INT;
-    dest->u.integer=0;
-    dest->subtype=NUMBER_UNDEFINED;
+    SET_SVAL(*dest, T_INT, NUMBER_UNDEFINED, integer, 0);
   }
 }
 
@@ -1321,9 +1429,8 @@ PMOD_EXPORT struct array *mapping_to_array(struct mapping *m)
       struct array *b=allocate_array(2);
       assign_svalue(b->item+0, & k->ind);
       assign_svalue(b->item+1, & k->val);
-      b->type_field = (1 << k->ind.type) | (1 << k->val.type);
-      s->u.array=b;
-      s->type=T_ARRAY;
+      b->type_field = (1 << TYPEOF(k->ind)) | (1 << TYPEOF(k->val));
+      SET_SVAL(*s, T_ARRAY, 0, array, b);
       s++;
     }
   a->type_field = BIT_ARRAY;
@@ -1352,7 +1459,7 @@ PMOD_EXPORT void mapping_replace(struct mapping *m,struct svalue *from, struct s
 	{
 	  PREPARE_FOR_DATA_CHANGE();
 	  assign_svalue(& k->val, to);
-	  md->val_types|=1<<to->type;
+	  md->val_types |= 1<<TYPEOF(*to);
 	}
       }
     free_mapping_data(md);
@@ -1382,27 +1489,6 @@ PMOD_EXPORT struct mapping *mkmapping(struct array *ind, struct array *val)
   return m;
 }
 
-#if 0
-PMOD_EXPORT struct mapping *copy_mapping(struct mapping *m)
-{
-  INT32 e;
-  struct mapping *n;
-  struct keypair *k;
-  struct mapping_data *md;
-
-  md=m->data;
-  n=allocate_mapping(MAP_SLOTS(md->size));
-
-  md->valrefs++;
-  add_ref(md);
-  NEW_MAPPING_LOOP(md) mapping_insert(n, &k->ind, &k->val);
-  md->valrefs--;
-  free_mapping_data(md);
-  
-  return n;
-}
-#else
-
 /* deferred mapping copy! */
 PMOD_EXPORT struct mapping *copy_mapping(struct mapping *m)
 {
@@ -1427,8 +1513,6 @@ PMOD_EXPORT struct mapping *copy_mapping(struct mapping *m)
   return n;
 }
 
-#endif
-
 /* copy_mapping() for when destructive operations are ok.
  *
  * Note: It destructive operations on the resulting mapping *will*
@@ -1470,7 +1554,7 @@ static struct mapping *subtract_mappings(struct mapping *a, struct mapping *b)
     res = allocate_mapping(a_md->size);
     SET_ONERROR(err, do_free_mapping, res);
     NEW_MAPPING_LOOP(a_md) {
-      size_t h = k->hval % b_md->hashsize;
+      size_t h = k->hval & ( b_md->hashsize - 1);
       struct keypair *k2;
       for (k2 = b_md->hash[h]; k2; k2 = k2->next) {
 	if ((k2->hval == k->hval) && is_eq(&k2->ind, &k->ind)) {
@@ -1515,7 +1599,7 @@ static struct mapping *and_mappings(struct mapping *a, struct mapping *b)
 
   /* Remove elements in res that aren't in a. */
   NEW_MAPPING_LOOP(b_md) {
-    size_t h = k->hval % a_md->hashsize;
+    size_t h = k->hval & ( a_md->hashsize - 1);
     struct keypair *k2;
     for (k2 = a_md->hash[h]; k2; k2 = k2->next) {
       if ((k2->hval == k->hval) && is_eq(&k2->ind, &k->ind)) {
@@ -1558,7 +1642,7 @@ static struct mapping *or_mappings(struct mapping *a, struct mapping *b)
 
     /* Add elements in a that aren't in b. */
     NEW_MAPPING_LOOP(a_md) {
-      size_t h = k->hval % b_md->hashsize;
+      size_t h = k->hval & ( b_md->hashsize - 1);
       struct keypair *k2;
       for (k2 = b_md->hash[h]; k2; k2 = k2->next) {
 	if ((k2->hval == k->hval) && is_eq(&k2->ind, &k->ind)) {
@@ -1615,7 +1699,7 @@ static struct mapping *xor_mappings(struct mapping *a, struct mapping *b)
 
   /* Add elements in a that aren't in b, and remove those that are. */
   NEW_MAPPING_LOOP(a_md) {
-    size_t h = k->hval % b_md->hashsize;
+    size_t h = k->hval & ( b_md->hashsize - 1);
     struct keypair *k2;
     for (k2 = b_md->hash[h]; k2; k2 = k2->next) {
       if ((k2->hval == k->hval) && is_eq(&k2->ind, &k->ind)) {
@@ -1674,7 +1758,7 @@ PMOD_EXPORT struct mapping *merge_mappings(struct mapping *a, struct mapping *b,
     zipper=get_set_order(ai);
     order_array(ai, zipper);
     order_array(av, zipper);
-    free((char *)zipper);
+    free(zipper);
   }
 
   bi=mapping_indices(b);
@@ -1688,7 +1772,7 @@ PMOD_EXPORT struct mapping *merge_mappings(struct mapping *a, struct mapping *b,
     zipper=get_set_order(bi);
     order_array(bi, zipper);
     order_array(bv, zipper);
-    free((char *)zipper);
+    free(zipper);
   }
 
   zipper=merge(ai,bi,op);
@@ -1703,7 +1787,7 @@ PMOD_EXPORT struct mapping *merge_mappings(struct mapping *a, struct mapping *b,
   UNSET_ONERROR(r2); free_array(bv);
   UNSET_ONERROR(r1); free_array(av);
 
-  free((char *)zipper);
+  free(zipper);
 
   m=mkmapping(ci, cv);
   free_array(ci);
@@ -1734,7 +1818,7 @@ PMOD_EXPORT struct mapping *merge_mapping_array_ordered(struct mapping *a,
     zipper=get_set_order(ai);
     order_array(ai, zipper);
     order_array(av, zipper);
-    free((char *)zipper);
+    free(zipper);
   }
 
   switch (op) /* no elements from b may be selected */
@@ -1756,7 +1840,7 @@ PMOD_EXPORT struct mapping *merge_mapping_array_ordered(struct mapping *a,
   UNSET_ONERROR(r2); free_array(av);
   UNSET_ONERROR(r1); free_array(ai);
   
-  free((char *)zipper);
+  free(zipper);
 
   m=mkmapping(ci, cv);
   free_array(ci);
@@ -1788,6 +1872,49 @@ PMOD_EXPORT struct mapping *merge_mapping_array_unordered(struct mapping *a,
   return m;
 }
 
+void o_append_mapping( INT32 args )
+{
+  struct svalue *lval = Pike_sp - args;
+  struct svalue *val = lval + 2;
+  int lvalue_type;
+
+#ifdef PIKE_DEBUG
+  if (args < 3) {
+    Pike_fatal("Too few arguments to o_append_mapping(): %d\n", args);
+  }
+#endif
+  args -= 3;
+  /* Note: val should always be a zero here! */
+  lvalue_type = lvalue_to_svalue_no_free(val, lval);
+
+  if ((TYPEOF(*val) == T_MAPPING) && (lvalue_type != PIKE_T_GET_SET)) {
+    struct mapping *m = val->u.mapping;
+    if( m->refs == 2 )
+    {
+      if ((TYPEOF(*lval) == T_OBJECT) &&
+	  lval->u.object->prog &&
+	  ((FIND_LFUN(lval->u.object->prog, LFUN_ASSIGN_INDEX) >= 0) ||
+	   (FIND_LFUN(lval->u.object->prog, LFUN_ASSIGN_ARROW) >= 0))) {
+	/* There's a function controlling assignments in this object,
+	 * so we can't alter the mapping in place.
+	 */
+      } else {
+	int i;
+	/* fprintf( stderr, "map_refs==2\n" ); */
+	for( i=0; i<args; i+=2 )
+	  low_mapping_insert( m, Pike_sp-(i+2), Pike_sp-(i+1), 2 );
+	stack_pop_n_elems_keep_top(2+args);
+	return;
+      }
+    }
+  }
+
+  f_aggregate_mapping(args);
+  f_add(2);
+  assign_lvalue(lval, val);
+  stack_pop_2_elems_keep_top();
+}
+
 /* NOTE: May perform destructive operations on either of the arguments
  *       if it has only a single reference.
  */
@@ -1879,6 +2006,9 @@ PMOD_EXPORT int mapping_equal_p(struct mapping *a, struct mapping *b, struct pro
 
   if(m_sizeof(a) != m_sizeof(b)) return 0;
 
+  if (!check_type_overlaps(a->data->ind_types, b->data->ind_types) ||
+      !check_type_overlaps(a->data->val_types, b->data->val_types)) return 0;
+
   curr.pointer_a = a;
   curr.pointer_b = b;
   curr.next = p;
@@ -2090,9 +2220,7 @@ node *make_node_from_mapping(struct mapping *m)
     if(!m->data->size)
       return mkefuncallnode("aggregate_mapping",0);
 
-    s.type=T_MAPPING;
-    s.subtype=0;
-    s.u.mapping=m;
+    SET_SVAL(s, T_MAPPING, 0, mapping, m);
     return mkconstantsvaluenode(&s);
   }
 }
@@ -2160,10 +2288,8 @@ PMOD_EXPORT struct mapping *copy_mapping_recursively(struct mapping *m,
 
   if (p) {
     struct svalue aa, bb;
-    aa.type = T_MAPPING;
-    aa.u.mapping = m;
-    bb.type = T_MAPPING;
-    bb.u.mapping = ret;
+    SET_SVAL(aa, T_MAPPING, 0, mapping, m);
+    SET_SVAL(bb, T_MAPPING, 0, mapping, ret);
     mapping_insert(p, &aa, &bb);
   }
 
@@ -2226,9 +2352,7 @@ PMOD_EXPORT void mapping_search_no_free(struct svalue *to,
       
       if(!k)
       {
-	to->type=T_INT;
-	to->subtype=NUMBER_UNDEFINED;
-	to->u.integer=0;
+	SET_SVAL(*to, T_INT, NUMBER_UNDEFINED, integer, 0);
 	return;
       }
       k=k->next;
@@ -2269,9 +2393,7 @@ PMOD_EXPORT void mapping_search_no_free(struct svalue *to,
     free_mapping_data(md);
   }
 
-  to->type=T_INT;
-  to->subtype=NUMBER_UNDEFINED;
-  to->u.integer=0;
+  SET_SVAL(*to, T_INT, NUMBER_UNDEFINED, integer, 0);
 }
 
 #ifdef PIKE_DEBUG
@@ -2350,6 +2472,9 @@ void check_mapping(const struct mapping *m)
   if(md->size > md->num_keypairs)
     Pike_fatal("Pretty mean hashtable there buster!\n");
 
+  if(md->hashsize & (md->hashsize - 1))
+    Pike_fatal("Invalid hashtable size: 0x%08lx\n", (long)md->hashsize);
+
   if(md->hashsize > md->num_keypairs)
     Pike_fatal("Pretty mean hashtable there buster %d > %d (2)!\n",md->hashsize,md->num_keypairs);
 
@@ -2364,16 +2489,16 @@ void check_mapping(const struct mapping *m)
     {
       num++;
 
-      if (k->ind.type > MAX_TYPE)
+      if (TYPEOF(k->ind) > MAX_TYPE)
 	Pike_fatal("Invalid maping keypair index type: %s\n",
-		   get_name_of_type(k->ind.type));
-      if(! ( (1 << k->ind.type) & (md->ind_types) ))
+		   get_name_of_type(TYPEOF(k->ind)));
+      if(! ( (1 << TYPEOF(k->ind)) & (md->ind_types) ))
 	Pike_fatal("Mapping indices type field lies.\n");
 
-      if (k->val.type > MAX_TYPE)
+      if (TYPEOF(k->val) > MAX_TYPE)
 	Pike_fatal("Invalid mapping keypair value type: %s\n",
-		   get_name_of_type(k->val.type));
-      if(! ( (1 << k->val.type) & (md->val_types) ))
+		   get_name_of_type(TYPEOF(k->val)));
+      if(! ( (1 << TYPEOF(k->val)) & (md->val_types) ))
 	Pike_fatal("Mapping values type field lies.\n");
 
       check_svalue(& k->ind);
@@ -2400,9 +2525,10 @@ void check_all_mappings(void)
 #endif
 
 static void visit_mapping_data (struct mapping_data *md, int action,
-				struct mapping *m)
+				void *extra)
 {
-  switch (action) {
+  visit_enter(md, T_MAPPING_DATA, extra);
+  switch (action & VISIT_MODE_MASK) {
 #ifdef PIKE_DEBUG
     default:
       Pike_fatal ("Unknown visit action %d.\n", action);
@@ -2415,7 +2541,8 @@ static void visit_mapping_data (struct mapping_data *md, int action,
       break;
   }
 
-  if ((md->ind_types | md->val_types) &
+  if (!(action & VISIT_NO_REFS) &&
+      (md->ind_types | md->val_types) &
       (action & VISIT_COMPLEX_ONLY ? BIT_COMPLEX : BIT_REF_TYPES)) {
     int ind_ref_type =
       md->flags & MAPPING_WEAK_INDICES ? REF_TYPE_WEAK : REF_TYPE_NORMAL;
@@ -2424,15 +2551,17 @@ static void visit_mapping_data (struct mapping_data *md, int action,
     INT32 e;
     struct keypair *k;
     NEW_MAPPING_LOOP (md) {
-      visit_svalue (&k->ind, ind_ref_type);
-      visit_svalue (&k->val, val_ref_type);
+      visit_svalue (&k->ind, ind_ref_type, extra);
+      visit_svalue (&k->val, val_ref_type, extra);
     }
   }
+  visit_leave(md, T_MAPPING_DATA, extra);
 }
 
-PMOD_EXPORT void visit_mapping (struct mapping *m, int action)
+PMOD_EXPORT void visit_mapping (struct mapping *m, int action, void *extra)
 {
-  switch (action) {
+  visit_enter(m, T_MAPPING, extra);
+  switch (action & VISIT_MODE_MASK) {
 #ifdef PIKE_DEBUG
     default:
       Pike_fatal ("Unknown visit action %d.\n", action);
@@ -2446,7 +2575,8 @@ PMOD_EXPORT void visit_mapping (struct mapping *m, int action)
   }
 
   visit_ref (m->data, REF_TYPE_INTERNAL,
-	     (visit_thing_fn *) &visit_mapping_data, m);
+	     (visit_thing_fn *) &visit_mapping_data, extra);
+  visit_leave(m, T_MAPPING, extra);
 }
 
 #ifdef MAPPING_SIZE_DEBUG
@@ -2464,7 +2594,7 @@ PMOD_EXPORT void visit_mapping (struct mapping *m, int action)
       DO_IF_DEBUG(Pike_fatal("Didn't expect an svalue zapping now.\n")); \
     }									\
     RECURSE_FN(&k->val, 1);						\
-    VAL_TYPES |= 1 << k->val.type;					\
+    VAL_TYPES |= 1 << TYPEOF(k->val);					\
   }									\
 } while (0)
 
@@ -2484,7 +2614,7 @@ PMOD_EXPORT void visit_mapping (struct mapping *m, int action)
       if (remove) {							\
 	/* Find and unlink k. */					\
 	unsigned INT32 h_;						\
-	h_ = k->hval % md->hashsize;					\
+	h_ = k->hval & ( md->hashsize - 1);				\
 	prev_ = md->hash + h_;						\
 	DO_IF_DEBUG(							\
 		    if (!*prev_) {					\
@@ -2509,8 +2639,8 @@ PMOD_EXPORT void visit_mapping (struct mapping *m, int action)
 	    M->debug_size--;						\
 	);								\
       } else {								\
-	VAL_TYPES |= 1 << k->val.type;					\
-	IND_TYPES |= 1 << k->ind.type;					\
+	VAL_TYPES |= 1 << TYPEOF(k->val);				\
+	IND_TYPES |= 1 << TYPEOF(k->ind);				\
 	k++;								\
       }									\
     }									\
@@ -2542,8 +2672,8 @@ PMOD_EXPORT void visit_mapping (struct mapping *m, int action)
 	    M->debug_size--;						\
 	);								\
       }else{								\
-	VAL_TYPES |= 1 << k->val.type;					\
-	IND_TYPES |= 1 << k->ind.type;					\
+	VAL_TYPES |= 1 << TYPEOF(k->val);				\
+	IND_TYPES |= 1 << TYPEOF(k->ind);				\
 	prev=&k->next;							\
       }									\
     }									\
