@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: f5564cccfff47e975292d450ea115919e1e2194d $
 */
 
 #ifndef MAPPING_H
@@ -10,7 +9,6 @@
 
 #include "svalue.h"
 #include "dmalloc.h"
-#include "block_alloc_h.h"
 
 /* Compatible with PIKE_WEAK_INDICES and PIKE_WEAK_VALUES. */
 #define MAPPING_WEAK_INDICES	2
@@ -65,9 +63,11 @@ extern struct mapping *gc_internal_mapping;
 
 #define MD_KEYPAIRS(MD, HSIZE) \
    ( (struct keypair *)							\
-     DO_ALIGN( PTR_TO_INT(((struct mapping_data *)(MD))->hash + HSIZE),	\
+     DO_ALIGN( PTR_TO_INT(MD) + OFFSETOF(mapping_data, hash) + HSIZE * sizeof(struct keypair *),	\
 	       ALIGNOF(struct keypair)) )
 
+
+
 #ifndef PIKE_MAPPING_KEYPAIR_LOOP
 #define NEW_MAPPING_LOOP(md) \
   for((e=0) DO_IF_DMALLOC( ?0:(debug_malloc_touch(md)) ) ;e<(md)->hashsize;e++) for(k=(md)->hash[e];k;k=k->next)
@@ -88,7 +88,7 @@ extern struct mapping *gc_internal_mapping;
 #endif /* PIKE_MAPPING_KEYPAIR_LOOP */
 
 
-void really_free_mapping(struct mapping *md);
+PMOD_EXPORT void really_free_mapping(struct mapping *md);
 
 #define inl_free_mapping(M) do{						\
     struct mapping *m_=(M);						\
@@ -134,12 +134,14 @@ void really_free_mapping(struct mapping *md);
 }while(0)
 
 /* Prototypes begin here */
-BLOCK_ALLOC_FILL_PAGES(mapping, 2);
+void really_free_mapping(struct mapping * m);
+void count_memory_in_mappings(size_t * num, size_t * size);
 
 
 
 
 
+void mapping_free_keypair(struct mapping_data *md, struct keypair *k);
 
 PMOD_EXPORT struct mapping *debug_allocate_mapping(int size);
 
@@ -373,7 +375,7 @@ PMOD_EXPORT INT32 mapping_generation(struct mapping *m);
 void check_mapping(const struct mapping *m);
 void check_all_mappings(void);
 #endif
-PMOD_EXPORT void visit_mapping (struct mapping *m, int action);
+PMOD_EXPORT void visit_mapping (struct mapping *m, int action, void *extra);
 void gc_mark_mapping_as_referenced(struct mapping *m);
 void real_gc_cycle_check_mapping(struct mapping *m, int weak);
 unsigned gc_touch_all_mappings(void);
@@ -386,13 +388,16 @@ void simple_describe_mapping(struct mapping *m);
 void debug_dump_mapping(struct mapping *m);
 int mapping_is_constant(struct mapping *m,
 			struct processing *p);
+void free_all_mapping_blocks(void);
+
+void o_append_mapping( INT32 args );
 /* Prototypes end here */
 
 #define allocate_mapping(X) dmalloc_touch(struct mapping *,debug_allocate_mapping(X))
 
-#define visit_mapping_ref(M, REF_TYPE)				\
+#define visit_mapping_ref(M, REF_TYPE, EXTRA)			\
   visit_ref (pass_mapping (M), (REF_TYPE),			\
-	     (visit_thing_fn *) &visit_mapping, NULL)
+	     (visit_thing_fn *) &visit_mapping, (EXTRA))
 #define gc_cycle_check_mapping(X, WEAK) \
   gc_cycle_enqueue((gc_cycle_check_cb *) real_gc_cycle_check_mapping, (X), (WEAK))
 
