@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: c7bfb9e315e1e12f1077b40d92672efe004ac47e $
 */
 
 /* Hohum. Here we go. This is try number four for a more optimized
@@ -15,7 +14,20 @@
 #define PARSE_FAILED ("HTTP/1.0 500 Internal Server Error\r\nContent-Type: text/html\r\n\r\nRequest parsing failed.\r\n")
 
 #include "global.h"
-	  
+/*! @module HTTPAccept
+ *!
+ *! High performance webserver optimized for somewhat static content.
+ *!
+ *! HTTPAccept is a less capable WWW-server than the
+ *! @[Protocols.HTTP.Server] server, but for some applications it can be
+ *! preferable. It is significantly more optimized, for most uses, and
+ *! can handle a very high number of requests per second on even
+ *! low end machines.
+ */
+
+/*! @class Loop
+ */
+
 #include "bignum.h"
 #include "backend.h"
 #include "machine.h"
@@ -83,37 +95,18 @@ static MUTEX_T arg_lock;
 static int next_free_arg, num_args;
 static struct args *free_arg_list[100];
 
-int aap_total_bytes;
-void *debug_aap_malloc(int nbytes)
-{
-  int *data;
-  aap_total_bytes += nbytes;
-  data = malloc( nbytes+16 );
-  data[0] = nbytes;
-  return ((char *)data)+16; /* keep it aligned.. */
-}
-
-void debug_aap_free( void *what )
-{
-  int *data = (void *)(((char *)what-16));
-  aap_total_bytes -= data[0];
-  fprintf( stderr, "%d Kbytes\n", aap_total_bytes/1024 );
-  free( data );
-}
-
-
 void free_args( struct args *arg )
 {
   num_args--;
 
-  if( arg->res.data ) aap_free( arg->res.data );
+  free( arg->res.data );
   if( arg->fd )       fd_close( arg->fd );
 
   mt_lock( &arg_lock );
   if( next_free_arg < 100 )
     free_arg_list[ next_free_arg++ ] = arg;
   else
-    aap_free(arg);
+    free(arg);
   mt_unlock( &arg_lock );
 }
 
@@ -125,7 +118,7 @@ struct args *new_args(void)
   if( next_free_arg )
     res = free_arg_list[--next_free_arg];
   else
-    res = aap_malloc( sizeof( struct args ) );
+    res = malloc( sizeof( struct args ) );
   mt_unlock( &arg_lock );
   return res;
 }
@@ -305,7 +298,7 @@ void aap_handle_connection(struct args *arg)
     arg->res.data=0;
   }
   else 
-    p = buffer = aap_malloc(8192);
+    p = buffer = malloc(8192);
 
   if(arg->res.leftovers && arg->res.leftovers_len)
   {
@@ -316,7 +309,7 @@ void aap_handle_connection(struct args *arg)
       return;
     }
     buffer_len = arg->res.leftovers_len;
-    MEMCPY(buffer, arg->res.leftovers, arg->res.leftovers_len);
+    memcpy(buffer, arg->res.leftovers, arg->res.leftovers_len);
     pos = arg->res.leftovers_len;
     arg->res.leftovers=0;
     if((tmp = my_memmem("\r\n\r\n", 4, buffer, pos)))
@@ -453,7 +446,7 @@ static void low_accept_loop(struct args *arg)
   ACCEPT_SIZE_T len = sizeof(arg->from);
   while(1)
   {
-    MEMCPY(arg2, arg, sizeof(struct args));
+    memcpy(arg2, arg, sizeof(struct args));
     arg2->fd = fd_accept(arg->fd, (struct sockaddr *)&arg2->from, &len);
     if(arg2->fd != -1)
     {
@@ -478,14 +471,14 @@ static void low_accept_loop(struct args *arg)
 	    e = e->next;
 	    t->next = 0;
 	    free_string(t->data);
-	    aap_free(t->url);
-	    aap_free(t);
+	    free(t->url);
+	    free(t);
 	  }
 	}
 	while(arg->log->log_head)
 	{
 	  struct log_entry *l = arg->log->log_head->next;
-	  aap_free(arg->log->log_head);
+	  free(arg->log->log_head);
 	  arg->log->log_head = l;
 	}
 
@@ -498,7 +491,7 @@ static void low_accept_loop(struct args *arg)
 	  else
 	    first_cache = c->next;
 	  c->gone = 1;
-	  aap_free(c);
+	  free(c);
 	}
 
 
@@ -508,11 +501,11 @@ static void low_accept_loop(struct args *arg)
 	{
 	  if(n)    n->next = l->next;
 	  else     aap_first_log = l->next;
-	  aap_free(l);
+	  free(l);
 	}
 	mt_unlock_interpreter();
-	aap_free(arg2);
-	aap_free(arg);
+	free(arg2);
+	free(arg);
 	return; /* No more accept here.. */
       }
     }
@@ -520,10 +513,8 @@ static void low_accept_loop(struct args *arg)
 }
 
 
-static void finished_p(struct callback *foo, void *b, void *c)
+static void finished_p(struct callback *UNUSED(foo), void *UNUSED(b), void *UNUSED(c))
 {
-  extern void f_low_aap_reqo__init( struct c_request_object * );
-
   aap_clean_cache();
 
   while(request)
@@ -539,7 +530,7 @@ static void finished_p(struct callback *foo, void *b, void *c)
 
     o = clone_object( request_program, 0 ); /* see requestobject.c */
     obj = (struct c_request_object *)get_storage(o, c_request_program );
-    MEMSET(obj, 0, sizeof(struct c_request_object));
+    memset(obj, 0, sizeof(struct c_request_object));
     obj->request = arg;
     obj->done_headers   = allocate_mapping( 20 );
     obj->misc_variables = allocate_mapping( 40 );
@@ -549,31 +540,36 @@ static void finished_p(struct callback *foo, void *b, void *c)
     push_object( o );
     assign_svalue_no_free(sp++, &arg->args);
 
-/*     { */
-/*       JMP_BUF recovery; */
-
-/*       free_svalue(& throw_value); */
-/*       mark_free_svalue (&throw_value); */
-
-/*       if(SETJMP(recovery)) */
-/*       { */
-/*       } */
-/*       else */
-/*       { */
-        apply_svalue(&arg->cb, 2);
-/*       } */
-/*     } */
+    apply_svalue(&arg->cb, 2);
     pop_stack();
   }
 }
 
+/*! @decl void create( Stdio.Port port, RequestProgram program, @
+ *!                    function(RequestProgram:void) request_handler, @
+ *!                    int cache_size, @
+ *!                    bool keep_log, int timeout )
+ *!
+ *! Create a new @[HTTPAccept].
+ *!
+ *! This will start a new thread that will listen for requests on the
+ *! port, parse them and pass on requests, instanced from the
+ *! @[program] class (which has to inherit @[RequestProgram] to the
+ *! @[request_handler] callback function.
+ *!
+ *! @[cache_size] is the maximum size of the cache, in bytes.
+ *! @[keep_log] indicates if a log of all requests should be kept.
+ *! @[timeout] if non-zero indicates a maximum time the server will wait for requests.
+ *!
+*/
 static void f_accept_with_http_parse(INT32 nargs)
 {
-/* From socket.c */
+  /* From socket.c */
   struct port 
   {
     struct fd_callback_box box;
     int my_errno;
+    unsigned int immediate_cnt;
     struct svalue accept_callback;
     struct svalue id;
   };
@@ -585,18 +581,16 @@ static void f_accept_with_http_parse(INT32 nargs)
   struct args *args = LTHIS; 
   get_all_args("accept_http_loop", nargs, "%o%*%*%*%i%i%i", &port, &program,
 	       &fun, &cb, &ms, &dolog, &to);
-  MEMSET(args, 0, sizeof(struct args));
+  memset(args, 0, sizeof(struct args));
   if(dolog)
   {
-    struct log *log = aap_malloc(sizeof(struct log));
-    MEMSET(log, 0, sizeof(struct log));
+    struct log *log = calloc(1, sizeof(struct log));
     mt_init(&log->log_lock);
     args->log = log;
     log->next = aap_first_log;
     aap_first_log = log;
   }
-  c = aap_malloc(sizeof(struct cache));
-  MEMSET(c, 0, sizeof(struct cache));
+  c = calloc(1, sizeof(struct cache));
   mt_init(&c->mutex);
   c->next = first_cache;
   first_cache = c;
@@ -629,49 +623,179 @@ static void f_accept_with_http_parse(INT32 nargs)
   }
 }
 
+/*! @decl array(LogEntry) log_as_array()
+ *!
+ *! Return the current log as an array of LogEntry objects.
+ */
+
+/*! @decl int log_as_commonlog_to_file(Stdio.Stream fd)
+ *!
+ *! Write the current log to the specified file in a somewhat common
+ *! commonlog format.
+ *!
+ *! Will return the number of bytes written.
+ */
+
+/*! @decl bool logp()
+ *! Returns true if logging is enabled
+ */
+
+/*! @decl int log_size()
+ *!
+ *! Returns the number of entries waiting to be logged.
+ */
+
+/*! @decl mapping(string:int) cache_status()
+ *!
+ *! Returns information about the cache.
+ *! @mapping result
+ *! @member int hits
+ *!   The number of hits since start
+ *! @member int misses
+ *!   The number of misses since start
+ *! @member int stale
+ *!   The number of misses that were stale hits, and not used
+ *! @member int size
+ *!   The total current size
+ *! @member int entries
+ *!   The number of entries in the cache
+ *! @member int max_size
+ *!   The maximum size of the cache
+ *!
+ *! @member int sent_bytes
+ *!  The number of bytes sent since the last call to cache_status
+ *!
+ *! @member int received_bytes
+ *!  The number of bytes received since the last call to cache_status
+ *!
+ *! @member int num_requests
+ *!  The number of requests received since the last call to cache_status
+ *! @endmapping
+ */
 static void f_cache_status(INT32 args)
 {
   struct cache *c = LTHIS->cache;
   pop_n_elems(args);
-  push_constant_text("hits"); 
+  push_text("hits"); 
   push_int64(c->hits);
-  push_constant_text("misses"); 
+  push_text("misses"); 
   push_int64(c->misses);
-  push_constant_text("stale"); 
+  push_text("stale"); 
   push_int64(c->stale);
-  push_constant_text("size"); 
+  push_text("size"); 
   push_int64(c->size);
-  push_constant_text("entries"); 
+  push_text("entries"); 
   push_int64(c->entries);
-  push_constant_text("max_size"); 
+  push_text("max_size"); 
   push_int64(c->max_size);
 
   /* Relative from last call */
-  push_constant_text("sent_bytes"); 
+  push_text("sent_bytes"); 
   push_int(c->sent_data);        c->sent_data=0;
-  push_constant_text("num_request"); 
+  push_text("num_request"); 
   push_int(c->num_requests);    c->num_requests=0;
-  push_constant_text("received_bytes"); 
+  push_text("received_bytes"); 
   push_int(c->received_data);c->received_data=0;
   f_aggregate_mapping( 18 );
 }
+/*! @endclass
+ */
 
+/*! @class LogEntry
+ *!
+ *! @decl int time
+ *!
+ *! @decl int sent_bytes
+ *!
+ *! @decl int received_bytes
+ *!
+ *! @decl int reply
+ *!
+ *! @decl string raw
+ *!
+ *! @decl string url
+ *!
+ *! @decl string method
+ *!
+ *! @decl string protocol
+ *!
+ *! @decl string from
+ *!
+ *! @endclass
+ */
+
+/*! @class RequestProgram
+ *!
+ *! @decl string prot
+ *!   The protocol part of the request. As an example "HTTP/1.1"
+ *!
+ *! @decl int time
+ *!   The time_t when the request arrived to the server
+ *!
+ *! @decl string raw_url
+ *!   The raw URL received, the part after the method and before the protocol.
+ *!
+ *! @decl string not_query
+ *!   The part of the URL before the first '?'.
+ *!
+ *! @decl string query
+ *!   The part of the URL after the first '?'
+ *!
+ *! @decl string rest_query
+ *!   The part of the URL after the first '?' that does not seem to be query variables.
+ *!
+ *! @decl mapping(string:string) variables;
+ *!   Parsed query variables
+ *!
+ *! @decl string client
+ *!   The user agent
+ *!
+ *! @decl string referer
+ *!   The referer header
+ *!
+ *! @decl string since
+ *!   The get-if-not-modified, if set.
+ *!
+ *! @decl string data
+ *!   Any payload that arrived with the request
+ *!
+ *! @decl mapping(string:array(string)) headers;
+ *!   All received headers
+ *!
+ *! @decl multiset(string) pragma;
+ *!   Tokenized pragma headers
+ *!
+ *! @decl Stdio.NonblockingStream my_fd
+ *!   The filedescriptor for this request.
+ *!
+ *! @decl string remoteaddr
+ *!   The remote address
+ *!
+ *! @decl string method
+ *!   The method (GET, PUT etc)
+ *!
+ *! @decl string raw
+ *!   The full request
+ *!
+ *! @decl void output(string data)
+ *!  Send @[data] directly to the remote side.
+ *!
+ *! @decl void reply( string data )
+ *! @decl void reply( string headers, Stdio.File fd, int len )
+ *! @decl void reply( int(0..0) ignore, Stdio.File fd, int len )
+ *!   Send a reply to the remote side.
+ *!   In the first case the @[data] will be sent.
+ *!   In the second case the @[headers] will be sent, then @[len] bytes from @[fd].
+ *!   In the last case @[len] bytes from @[fd] will be sent.
+ *!
+ *! @decl void reply_with_cache( string data, int(1..) stay_time )
+ *!  Send @[data] as the reply, and keep it as a cache entry to
+ *!  requests to this URL for @[stay_time] seconds.
+ *!
+ *! @endclass
+ */
 #endif /* _REENTRANT */
 
-void f_aap_add_filesystem( INT32 args )
-{
-  INT_TYPE nosyms = 0;
-  struct pike_string *basedir, *mountpoint;
-  struct array *noparse;
-
-  if(args == 4)
-    get_all_args( "add_filesystem", args, 
-                  "%s%s%a%i", &basedir, &mountpoint, &noparse, &nosyms );
-  else
-    get_all_args( "add_filesystem", args, 
-                  "%s%s%a", &basedir, &mountpoint, &noparse );
-}
-
 
 PIKE_MODULE_INIT
 {
@@ -691,11 +815,7 @@ PIKE_MODULE_INIT
 #endif
 
   start_new_program();
-#ifdef ADD_STORAGE
   ADD_STORAGE(struct args);
-#else
-  add_storage(sizeof(struct args));
-#endif
   add_function("create", f_accept_with_http_parse, 
 	       "function(object,program,function,mixed,int,int,int:void)", 0);
   add_function("cache_status", f_cache_status,"function(void:mapping)", 0);
@@ -706,26 +826,11 @@ PIKE_MODULE_INIT
   add_function("log_size", f_aap_log_size, "function(void:int)", 0);
   add_function("logp", f_aap_log_exists, "function(void:int)", 0);
 
-#if 0
-  add_function( "add_filesystem", f_aap_add_filesystem, 
-                "function(string,string,array(string),int|void:void)", 0);
-  add_function( "add_contenttype", f_aap_add_contenttype, 
-                "function(strign,string:void)", 0);
-#ifdef FS_STATS
-  add_function( "filesystem_stats", f_filesystem_stats,
-                "function(void:mapping)", 0);
-#endif
-#endif
-
   add_program_constant("Loop", accept_loop_program = end_program(), 0);
 
   start_new_program();
 #define OFFSET(X) (offset + OFFSETOF(log_object,X))
-#ifdef ADD_STORAGE
   offset=ADD_STORAGE(struct log_object);
-#else
-  offset=add_storage(sizeof(struct log_object));
-#endif
   map_variable("time", "int", 0, OFFSET(time), T_INT);
   map_variable("sent_bytes", "int", 0, OFFSET(sent_bytes), T_INT);
   map_variable("reply", "int", 0, OFFSET(reply), T_INT);
@@ -735,16 +840,11 @@ PIKE_MODULE_INIT
   map_variable("method", "string", 0, OFFSET(method), T_STRING);
   map_variable("protocol", "string", 0, OFFSET(protocol), T_STRING);
   map_variable("from", "string", 0, OFFSET(from), T_STRING);
-  add_program_constant("logentry", (aap_log_object_program=end_program()), 0);
+  add_program_constant("LogEntry", (aap_log_object_program=end_program()), 0);
 
 
   start_new_program();
-#ifdef ADD_STORAGE
   ADD_STORAGE(struct c_request_object);
-#else
-  add_storage(sizeof(struct c_request_object));
-#endif
-
   add_function("`[]", f_aap_index_op, "function(string:mixed)",0);
   add_function("`->", f_aap_index_op, "function(string:mixed)",0);
 
@@ -763,9 +863,9 @@ PIKE_MODULE_INIT
   set_init_callback( aap_init_request_object );
   set_exit_callback( aap_exit_request_object );
   add_program_constant("prog", (c_request_program = end_program()), 0);
+  add_program_constant("RequestProgram", c_request_program, 0 );
 #else
-  if(!TEST_COMPAT(7,6))
-    HIDE_MODULE();
+  HIDE_MODULE();
 #endif /* _REENTRANT */
 }
 
@@ -810,7 +910,7 @@ PIKE_MODULE_EXIT
       while(log_head)
       {
 	struct log_entry *l = log_head->next;
-	aap_free(log_head);
+	free(log_head);
 	log_head = l;
       }
       log->next = NULL;
@@ -836,8 +936,8 @@ PIKE_MODULE_EXIT
 	e = e->next;
 	t->next = 0;
         free_string(t->data);
-	aap_free(t->url);
-	aap_free(t);
+	free(t->url);
+	free(t);
       }
       first_cache->htable[i]=0;
     }
@@ -862,3 +962,4 @@ PIKE_MODULE_EXIT
   free_program(accept_loop_program);
 #endif /* _REENTRANT */
 }
+/*! @endmodule */
