@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 8cf2d28935c99789add2e43efddbf92cc3713a96 $
 */
 
 #include "global.h"
@@ -95,14 +94,14 @@ static INLINE int getrgb(struct image *img,
    if (max<3 || args-args_start<3) return 0;
 
    for (i=0; i<3; i++)
-      if (sp[-args+i+args_start].type!=T_INT)
+      if (TYPEOF(sp[-args+i+args_start]) != T_INT)
          Pike_error("Illegal r,g,b argument to %s\n",name);
    img->rgb.r=(unsigned char)sp[-args+args_start].u.integer;
    img->rgb.g=(unsigned char)sp[1-args+args_start].u.integer;
    img->rgb.b=(unsigned char)sp[2-args+args_start].u.integer;
 
    if (max > 3 && args-args_start>=4) {
-      if (sp[3-args+args_start].type!=T_INT) {
+      if (TYPEOF(sp[3-args+args_start]) != T_INT) {
          Pike_error("Illegal alpha argument to %s\n",name);
       }
       img->alpha=sp[3-args+args_start].u.integer;
@@ -121,7 +120,7 @@ void img_clear(rgb_group *dest, rgb_group rgb, ptrdiff_t size)
   if(!size) return;
   THREADS_ALLOW();
   if( ( rgb.r == rgb.g && rgb.r == rgb.b ) )
-    MEMSET(dest, rgb.r, size*sizeof(rgb_group) );
+    memset(dest, rgb.r, size*sizeof(rgb_group) );
   else if(size)
   {
     int increment = 1;
@@ -130,11 +129,11 @@ void img_clear(rgb_group *dest, rgb_group rgb, ptrdiff_t size)
     size -= 1;
     while (size>increment) 
     {
-      MEMCPY(dest,from,increment*sizeof(rgb_group));
+      memcpy(dest,from,increment*sizeof(rgb_group));
       size-=increment,dest+=increment;
       if (increment<1024) increment *= 2;
     }
-    if(size>0) MEMCPY(dest,from,size*sizeof(rgb_group));
+    if(size>0) memcpy(dest,from,size*sizeof(rgb_group));
   }
   THREADS_DISALLOW();
 }
@@ -163,7 +162,7 @@ void img_box_nocheck(INT32 x1,INT32 y1,INT32 x2,INT32 y2)
 	 if(!length)
 	   break;	/* Break to the while(0). */
 	 for(x=0; x<length; x++)  *(foo+x) = rgb;
-	 while(--y)  MEMCPY((foo+=xs), from, length*sizeof(rgb_group)); 
+	 while(--y)  memcpy((foo+=xs), from, length*sizeof(rgb_group)); 
        } while(0);
        THREADS_DISALLOW();
      }
@@ -190,11 +189,11 @@ CHRONO("image_blit begin");
 
    THREADS_ALLOW();
    if(!moddest && !modsrc)
-     MEMCPY(dest,src,sizeof(rgb_group)*width*lines);
+     memcpy(dest,src,sizeof(rgb_group)*width*lines);
    else
      while (lines--)
      {
-       MEMCPY(dest,src,sizeof(rgb_group)*width);
+       memcpy(dest,src,sizeof(rgb_group)*width);
        dest+=moddest;
        src+=modsrc;
      }
@@ -216,14 +215,14 @@ void img_crop(struct image *dest,
    if (x1>x2) tmp=x1, x1=x2, x2=tmp;
    if (y1>y2) tmp=y1, y1=y2, y2=tmp;
 
-   new=xalloc( (x2-x1+1)*(y2-y1+1)*sizeof(rgb_group)+1 );
+   new=xalloc( (x2-x1+1)*(y2-y1+1)*sizeof(rgb_group)+RGB_VEC_PAD );
 
    if (x1==0 && y1==0 &&
        img->xsize-1==x2 && img->ysize-1==y2)
    {
       *dest=*img;
       THREADS_ALLOW();
-      MEMCPY(new,img->img,(x2-x1+1)*(y2-y1+1)*sizeof(rgb_group));
+      memcpy(new,img->img,(x2-x1+1)*(y2-y1+1)*sizeof(rgb_group));
       THREADS_DISALLOW();
       dest->img=new;
       return;
@@ -260,9 +259,9 @@ void img_crop(struct image *dest,
 void img_clone(struct image *newimg,struct image *img)
 {
    if (newimg->img) free(newimg->img);
-   newimg->img=xalloc(sizeof(rgb_group)*img->xsize*img->ysize+1);
+   newimg->img=xalloc(sizeof(rgb_group)*img->xsize*img->ysize+RGB_VEC_PAD);
    THREADS_ALLOW();
-   MEMCPY(newimg->img,img->img,sizeof(rgb_group)*img->xsize*img->ysize);
+   memcpy(newimg->img,img->img,sizeof(rgb_group)*img->xsize*img->ysize);
    THREADS_DISALLOW();
    newimg->xsize=img->xsize;
    newimg->ysize=img->ysize;
@@ -291,10 +290,10 @@ void image_paste(INT32 args)
    INT32 x1,y1,x2,y2,blitwidth,blitheight;
 
    if (args<1
-       || sp[-args].type!=T_OBJECT
-       || !(img=(struct image*)get_storage(sp[-args].u.object,image_program)))
-      bad_arg_error("image->paste",sp-args,args,1,"",sp+1-1-args,
-		"Bad argument 1 to image->paste()\n");
+       || TYPEOF(sp[-args]) != T_OBJECT
+       || !(img=get_storage(sp[-args].u.object,image_program)))
+     bad_arg_error("paste",sp-args,args,1,"",sp+1-1-args,
+                   "Bad argument 1 to paste.\n");
    if (!THIS->img) return;
 
    if (!img->img) return;
@@ -302,10 +301,10 @@ void image_paste(INT32 args)
    if (args>1)
    {
       if (args<3 
-	  || sp[1-args].type!=T_INT
-	  || sp[2-args].type!=T_INT)
-         bad_arg_error("image->paste",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->paste()\n");
+	  || TYPEOF(sp[1-args]) != T_INT
+	  || TYPEOF(sp[2-args]) != T_INT)
+        bad_arg_error("paste",sp-args,args,0,"",sp-args,
+                      "Bad arguments to paste.\n");
       x1=sp[1-args].u.integer;
       y1=sp[2-args].u.integer;
    }
@@ -369,22 +368,22 @@ void image_paste_alpha(INT32 args)
    INT32 x1,y1;
 
    if (args<2
-       || sp[-args].type!=T_OBJECT
+       || TYPEOF(sp[-args]) != T_OBJECT
        || !sp[-args].u.object
-       || !(img=(struct image*)get_storage(sp[-args].u.object,image_program))
-       || sp[1-args].type!=T_INT)
-      bad_arg_error("image->paste_alpha",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->paste_alpha()\n");
+       || !(img=get_storage(sp[-args].u.object,image_program))
+       || TYPEOF(sp[1-args]) != T_INT)
+      bad_arg_error("paste_alpha",sp-args,args,0,"",sp-args,
+                    "Bad arguments to paste_alpha.\n");
    if (!THIS->img) return;
    if (!img->img) return;
    THIS->alpha=(unsigned char)(sp[1-args].u.integer);
    
    if (args>=4)
    {
-      if (sp[2-args].type!=T_INT
-	  || sp[3-args].type!=T_INT)
-         bad_arg_error("image->paste_alpha",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->paste_alpha()\n");
+      if (TYPEOF(sp[2-args]) != T_INT
+	  || TYPEOF(sp[3-args]) != T_INT)
+        bad_arg_error("paste_alpha",sp-args,args,0,"",sp-args,
+                      "Bad arguments to paste_alpha.\n");
       x1=sp[2-args].u.integer;
       y1=sp[3-args].u.integer;
    }
@@ -459,14 +458,14 @@ CHRONO("image_paste_mask init");
 
    if (args<2)
       Pike_error("illegal number of arguments to image->paste_mask()\n");
-   if (sp[-args].type!=T_OBJECT
-       || !(img=(struct image*)get_storage(sp[-args].u.object,image_program)))
-      bad_arg_error("image->paste_mask",sp-args,args,1,"",sp+1-1-args,
-		"Bad argument 1 to image->paste_mask()\n");
-   if (sp[1-args].type!=T_OBJECT
-       || !(mask=(struct image*)get_storage(sp[1-args].u.object,image_program)))
-      bad_arg_error("image->paste_mask",sp-args,args,2,"",sp+2-1-args,
-		"Bad argument 2 to image->paste_mask()\n");
+   if (TYPEOF(sp[-args]) != T_OBJECT
+       || !(img=get_storage(sp[-args].u.object,image_program)))
+      bad_arg_error("paste_mask",sp-args,args,1,"",sp+1-1-args,
+                    "Bad argument 1 to paste_mask.\n");
+   if (TYPEOF(sp[1-args]) != T_OBJECT
+       || !(mask=get_storage(sp[1-args].u.object,image_program)))
+      bad_arg_error("paste_mask",sp-args,args,2,"",sp+2-1-args,
+                    "Bad argument 2 to paste_mask.\n");
    if (!THIS->img) return;
 
    if (!mask->img) return;
@@ -474,8 +473,8 @@ CHRONO("image_paste_mask init");
    
    if (args>=4)
    {
-      if (sp[2-args].type!=T_INT
-	  || sp[3-args].type!=T_INT)
+      if (TYPEOF(sp[2-args]) != T_INT
+	  || TYPEOF(sp[3-args]) != T_INT)
          Pike_error("illegal coordinate arguments to image->paste_mask()\n");
       x1=sp[2-args].u.integer;
       y1=sp[3-args].u.integer;
@@ -564,11 +563,11 @@ void image_paste_alpha_color(INT32 args)
 
    if (args<1)
       SIMPLE_TOO_FEW_ARGS_ERROR("image->paste_alpha_color",1);
-   if (sp[-args].type!=T_OBJECT
+   if (TYPEOF(sp[-args]) != T_OBJECT
        || !sp[-args].u.object
-       || !(mask=(struct image*)get_storage(sp[-args].u.object,image_program)))
-      bad_arg_error("image->paste_alpha_color",sp-args,args,1,"",sp+1-1-args,
-		"Bad argument 1 to image->paste_alpha_color()\n");
+       || !(mask=get_storage(sp[-args].u.object,image_program)))
+     bad_arg_error("paste_alpha_color",sp-args,args,1,"",sp+1-1-args,
+                   "Bad argument 1 to paste_alpha_color.\n");
    if (!THIS->img) return;
    if (!mask->img) return;
 
@@ -576,8 +575,8 @@ void image_paste_alpha_color(INT32 args)
       arg=1+getrgb(THIS,1,args,3,"image->paste_alpha_color()\n");
    if (args>arg+1) 
    {
-      if (sp[arg-args].type!=T_INT
-	  || sp[1+arg-args].type!=T_INT)
+      if (TYPEOF(sp[arg-args]) != T_INT
+	  || TYPEOF(sp[1+arg-args]) != T_INT)
          Pike_error("illegal coordinate arguments to image->paste_alpha_color()\n");
       x1=sp[arg-args].u.integer;
       y1=sp[1+arg-args].u.integer;
