@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: b53951f49a835b403dab04142cf0256e5ff65b8b $
 */
 
 #include "global.h"
@@ -16,6 +15,7 @@
 #include "operators.h"
 #include "threads.h"
 #include "module_support.h"
+#include "pike_types.h"
 
 #include "image.h"
 #include "colortable.h"
@@ -25,7 +25,7 @@
 
 /*! @module Image
  *!
- *! @module _XPM
+ *! @module XPM
  */
 
 extern struct program *image_program;
@@ -83,9 +83,7 @@ static rgba_group decode_color( struct buffer *s )
   } 
   if(s->len==4&&(!strncmp(s->str,"None",4)||!strncmp(s->str,"none",4)))
   {
-#ifdef HIDE_WARNINGS
-      res.r = res.g = res.b = 0;
-#endif
+    res.r = res.g = res.b = 0;
     res.alpha = 0;
     return res;
   }
@@ -100,14 +98,15 @@ static rgba_group decode_color( struct buffer *s )
   push_svalue( parse_color );
   push_string(make_shared_binary_string(s->str,s->len));
   f_index( 2 );
-  if(sp[-1].type != T_OBJECT) {
+  if(TYPEOF(sp[-1]) != T_OBJECT) {
     push_int(0);
     stack_swap();
   } else {
-    push_constant_text( "array" );
-    apply( sp[-2].u.object, "cast", 1 );
+    ref_push_type_value(array_type_string);
+    stack_swap();
+    f_cast();
   }
-  if(sp[-1].type == T_ARRAY && sp[-1].u.array->size == 3)
+  if(TYPEOF(sp[-1]) == T_ARRAY && sp[-1].u.array->size == 3)
   {
     res.r = sp[-1].u.array->item[0].u.integer;
     res.g = sp[-1].u.array->item[1].u.integer;
@@ -263,23 +262,23 @@ void f__xpm_write_rows( INT32 args )
   fprintf(stderr, ")\n");
 #endif /* 0 */
 
-  iimg = (struct image *)get_storage( img, image_program );
-  ialpha = (struct image *)get_storage( alpha, image_program );
+  iimg = get_storage( img, image_program );
+  ialpha = get_storage( alpha, image_program );
   if(!iimg || !ialpha)
-    Pike_error("Sluta pilla p interna saker..\n");
+    Pike_error("Expected images as arguments\n");
 
   if (pixels->size < iimg->ysize + colors->size) {
     SIMPLE_ARG_ERROR("_xpm_write_rows", 5, "pixel array is too short.");
   }
 
   for(y = 0; y < iimg->ysize + colors->size + 1; y++) {
-    if ((pixels->item[y].type != T_STRING) ||
+    if ((TYPEOF(pixels->item[y]) != T_STRING) ||
 	(pixels->item[y].u.string->size_shift)) {
       SIMPLE_ARG_ERROR("_xpm_write_rows", 5,
 		       "Pixel array contains elements other than 8bit strings.");
     }
     if (y < colors->size) {
-      if ((colors->item[y].type != T_STRING) ||
+      if ((TYPEOF(colors->item[y]) != T_STRING) ||
 	  (pixels->item[y].u.string->size_shift)) {
 	SIMPLE_ARG_ERROR("_xpm_write_rows", 5,
 			 "Color array contains elements other than 8bit strings.");
@@ -323,8 +322,7 @@ void f__xpm_write_rows( INT32 args )
    {
      rgba_group **p_colors;
      int i;
-     p_colors = (rgba_group **)xalloc(sizeof(rgba_group *)*65536);
-     MEMSET(p_colors, 0, sizeof(rgba_group *)*65536);
+     p_colors = xcalloc(sizeof(rgba_group *), 65536);
      for(i=0; i<colors->size; i++)
      {
        struct pike_string *c = colors->item[i].u.string;
@@ -332,13 +330,12 @@ void f__xpm_write_rows( INT32 args )
        unsigned short id = extract_short((unsigned char *)c->str);
        if(!p_colors[id])
        {
-         p_colors[id] = (rgba_group *)xalloc(sizeof(rgba_group)*128);
-         MEMSET(p_colors[id],0,sizeof(rgba_group)*128);
+         p_colors[id] = xcalloc(sizeof(rgba_group), 128);
        }
        if(ind > 127) 
        {
-         p_colors[id] = (rgba_group *)realloc(p_colors[id],sizeof(rgba_group)*256);
-         MEMSET(p_colors[id]+128, 0, sizeof(rgba_group)*128);
+         p_colors[id] = realloc(p_colors[id],sizeof(rgba_group)*256);
+         memset(p_colors[id]+128, 0, sizeof(rgba_group)*128);
        }
        p_colors[id][ind]=parse_color_line( c, bpc );
      }
@@ -445,7 +442,7 @@ void f__xpm_trim_rows( INT32 args )
   {
     int len,start;
     struct pike_string *s = a->item[i].u.string;
-    if(a->item[i].type != T_STRING)
+    if(TYPEOF(a->item[i]) != T_STRING)
       Pike_error("Array must be array(string).\n");
     if(s->len > 4)
     {
