@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 932378d5405b1c95307a0c237d683a11b6ad1834 $
 */
 
 /*
@@ -12,6 +11,19 @@
 **!	This submodule keep the ILBM encode/decode capabilities
 **!	of the <ref>Image</ref> module.
 **!
+**! ILBM is a subtype of the Interchange File Format (IFF) used for
+**! storing picture data. ILBM stands for InterLeaved BitMap which
+**! refers to the way the pictures are stored. The image data is
+**! stored as a varying number of bitplanes, each storing one bit of
+**! data for each pixel in the image. The format supports horizontal
+**! and vertical RLE (Run Length Encoding) compression.
+**!
+**! The format also natively supports images using the somewhat
+**! esoteric image modes of the original Commodore Amiga, such as HAM
+**! (Hold And Modify)
+**!
+**! The format was commonly used for images on the Amiga.
+**!
 **! see also: Image, Image.Image, Image.Colortable
 */
 #include "global.h"
@@ -120,9 +132,9 @@ static void image_ilbm___decode(INT32 args)
    map_delete(m, &string_[string_CAMG]);
    map_delete(m, &string_[string_BODY]);
 
-   if(sp[-5].type != T_STRING)
+   if(TYPEOF(sp[-5]) != T_STRING)
      Pike_error("Missing BMHD chunk\n");
-   if(sp[-2].type != T_STRING)
+   if(TYPEOF(sp[-2]) != T_STRING)
      Pike_error("Missing BODY chunk\n");
 
    /* Extract image size from BMHD */
@@ -134,12 +146,8 @@ static void image_ilbm___decode(INT32 args)
 
    free_svalue(sp-7);
 
-   sp[-7].u.integer = (s[0]<<8)|s[1];
-   sp[-7].type = T_INT;
-   sp[-7].subtype = NUMBER_NUMBER;
-   sp[-6].u.integer = (s[2]<<8)|s[3];
-   sp[-6].type = T_INT;
-   sp[-6].subtype = NUMBER_NUMBER;
+   SET_SVAL(sp[-7], T_INT, NUMBER_NUMBER, integer, (s[0]<<8)|s[1]);
+   SET_SVAL(sp[-6], T_INT, NUMBER_NUMBER, integer, (s[2]<<8)|s[3]);
 
    f_aggregate(7);
 }
@@ -302,7 +310,7 @@ static void parse_body(struct BMHD *bmhd, unsigned char *body, ptrdiff_t blen,
     if(alpha != NULL)
       switch(bmhd->masking) {
       case mskNone:
-	memset((char *)adest, ~0, bmhd->w*sizeof(*adest));
+	memset(adest, ~0, bmhd->w*sizeof(*adest));
 	adest += bmhd->w;
 	break;
       case mskHasMask:
@@ -441,7 +449,7 @@ static void image_ilbm__decode(INT32 args)
   int n = 0;
   INT32 camg = 0;
 
-  if(args>0 && sp[-args].type == T_STRING) {
+  if(args>0 && TYPEOF(sp[-args]) == T_STRING) {
     image_ilbm___decode(args);
     args = 1;
   }
@@ -452,8 +460,10 @@ static void image_ilbm__decode(INT32 args)
     pop_n_elems(args-1);
 
   if(arr->size < 6 ||
-     ITEM(arr)[2].type != T_STRING || ITEM(arr)[2].u.string->size_shift != 0 ||
-     ITEM(arr)[5].type != T_STRING || ITEM(arr)[5].u.string->size_shift != 0)
+     TYPEOF(ITEM(arr)[2]) != T_STRING ||
+     ITEM(arr)[2].u.string->size_shift != 0 ||
+     TYPEOF(ITEM(arr)[5]) != T_STRING ||
+     ITEM(arr)[5].u.string->size_shift != 0)
     Pike_error("Image.ILBM._decode: illegal argument\n");
 
   parse_bmhd(&bmhd, STR0(ITEM(arr)[2].u.string), ITEM(arr)[2].u.string->len);
@@ -462,7 +472,7 @@ static void image_ilbm__decode(INT32 args)
   push_int(bmhd.w);
   push_int(bmhd.h);
   o=clone_object(image_program,2);
-  img=(struct image*)get_storage(o,image_program);
+  img=get_storage(o,image_program);
   push_object(o);
   n++;
 
@@ -471,12 +481,13 @@ static void image_ilbm__decode(INT32 args)
     push_int(bmhd.w);
     push_int(bmhd.h);
     o=clone_object(image_program,2);
-    alpha=(struct image*)get_storage(o,image_program);
+    alpha=get_storage(o,image_program);
     push_object(o);
     n++;
   }
 
-  if(ITEM(arr)[4].type == T_STRING && ITEM(arr)[4].u.string->size_shift == 0 &&
+  if(TYPEOF(ITEM(arr)[4]) == T_STRING &&
+     ITEM(arr)[4].u.string->size_shift == 0 &&
      ITEM(arr)[4].u.string->len>=4) {
     unsigned char *camgp = STR0(ITEM(arr)[4].u.string);
     camg = (camgp[0]<<24)|(camgp[1]<<16)|(camgp[2]<<8)|camgp[3];
@@ -493,7 +504,8 @@ static void image_ilbm__decode(INT32 args)
     n++;
   }
 
-  if(ITEM(arr)[3].type == T_STRING && ITEM(arr)[3].u.string->size_shift == 0) {
+  if(TYPEOF(ITEM(arr)[3]) == T_STRING &&
+     ITEM(arr)[3].u.string->size_shift == 0) {
     unsigned char *pal = STR0(ITEM(arr)[3].u.string);
     INT32 col, mcol = 1<<bmhd.nPlanes;
     ptrdiff_t ncol = ITEM(arr)[3].u.string->len/3;
@@ -522,7 +534,7 @@ static void image_ilbm__decode(INT32 args)
     }
     f_aggregate(DO_NOT_WARN((INT32)ncol));
     push_object(clone_object(image_colortable_program,1));
-    ctable=(struct neo_colortable*)get_storage(sp[-1].u.object,
+    ctable=get_storage(sp[-1].u.object,
 					       image_colortable_program);
     n++;
   }
@@ -559,12 +571,12 @@ void img_ilbm_decode(INT32 args)
    if (!args)
       Pike_error("Image.ILBM.decode: too few argument\n");
 
-   if (sp[-args].type != T_MAPPING) {
+   if (TYPEOF(sp[-args]) != T_MAPPING) {
      image_ilbm__decode(args);
      args = 1;
    }
      
-   if (sp[-args].type != T_MAPPING)
+   if (TYPEOF(sp[-args]) != T_MAPPING)
      Pike_error("Image.ILBM.decode: illegal argument\n");
 
    if(args>1)
@@ -572,7 +584,7 @@ void img_ilbm_decode(INT32 args)
 
    sv = simple_mapping_string_lookup(sp[-args].u.mapping, "image");
 
-   if(sv == NULL || sv->type != T_OBJECT)
+   if(sv == NULL || TYPEOF(*sv) != T_OBJECT)
      Pike_error("Image.ILBM.decode: illegal argument\n");
 
    ref_push_object(sv->u.object);
@@ -692,13 +704,13 @@ static void chunky2planar(INT32 *src, int w,
 }
 
 static struct pike_string *make_body(struct BMHD *bmhd,
-				     struct image *img, struct image *alpha,
+				     struct image *img, struct image *UNUSED(alpha),
 				     struct neo_colortable *ctable)
 {
   unsigned int x, y;
   int rbyt = ((bmhd->w+15)&~15)>>3;
   int eplanes = (bmhd->masking == mskHasMask? bmhd->nPlanes+1:bmhd->nPlanes);
-  unsigned char *line = alloca(rbyt*eplanes);
+  unsigned char *line = xcalloc(rbyt, eplanes);
   INT32 *cptr, *cline = alloca((rbyt<<3)*sizeof(INT32));
   rgb_group *src = img->img;
   struct string_builder bldr;
@@ -711,7 +723,6 @@ static struct pike_string *make_body(struct BMHD *bmhd,
     ctfunc = image_colortable_index_32bit_function(ctable);
   }
 
-  memset(line, 0, rbyt*eplanes);
   init_string_builder(&bldr, 0);
   for(y=0; y<bmhd->h; y++) {
     if(ctfunc != NULL) {
@@ -733,6 +744,7 @@ static struct pike_string *make_body(struct BMHD *bmhd,
   }
   if(ctable != NULL)
     image_colortable_free_dither(&dith);
+  free(line);
   return finish_string_builder(&bldr);
 }
 
@@ -751,18 +763,18 @@ static void image_ilbm_encode(INT32 args)
   get_all_args("encode", args, (args>1 && !UNSAFE_IS_ZERO(&sp[1-args])? "%o%m":"%o"),
 	       &imgo, &optm);
 
-  if((img=(struct image*)get_storage(imgo, image_program))==NULL)
+  if((img=get_storage(imgo, image_program))==NULL)
      Pike_error("Image.ILBM.encode: illegal argument 1\n");
 
   if(optm != NULL) {
     struct svalue *s;
     if((s = simple_mapping_string_lookup(optm, "alpha"))!=NULL && !UNSAFE_IS_ZERO(s))
-      if(s->type != T_OBJECT ||
-	 (alpha=(struct image*)get_storage(s->u.object, image_program))==NULL)
+      if(TYPEOF(*s) != T_OBJECT ||
+	 (alpha=get_storage(s->u.object, image_program))==NULL)
 	Pike_error("Image.ILBM.encode: option (arg 2) \"alpha\" has illegal type\n");
     if((s=simple_mapping_string_lookup(optm, "palette"))!=NULL && !UNSAFE_IS_ZERO(s))
-      if(s->type != T_OBJECT ||
-	 (ct=(struct neo_colortable*)
+      if(TYPEOF(*s) != T_OBJECT ||
+	 (ct=
 	  get_storage(s->u.object, image_colortable_program))==NULL)
 	Pike_error("Image.ILBM.encode: option (arg 2) \"palette\" has illegal type\n");
   }
