@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: e37046d27cbd0a7092d7ce5780dcdf6a0ad62a9d $
 */
 
 #include "global.h"
@@ -91,12 +90,11 @@ static unsigned char psd_read_uchar( struct buffer *from )
   return res;
 }
 
-static int psd_read_char( struct buffer *from )
+static int psd_read_schar(struct buffer *from)
 {
-  return (char)psd_read_uchar( from );
+  return (signed char)psd_read_uchar(from);
 }
 
-
 static char *psd_read_data( struct buffer * from, size_t len )
 {
   char *res;
@@ -206,15 +204,15 @@ static void decode_layers_and_masks( struct psd_image *dst,
   
   if( count < 0 )
   {
-    count = -count;
+    if (DO_INT16_NEG_OVERFLOW(count, &count))
+        Pike_error("overflow\n");
     first_alpha_is_magic = 1;
   } else if(count == 0)
     return;
   while( count-- )
   {
     unsigned int cnt;
-    struct layer *l=
-      layer = (struct layer *)xcalloc( sizeof( struct layer ), 1);
+    struct layer *l = layer = xcalloc( sizeof( struct layer ), 1);
     layer->next = dst->first_layer;
     if(dst->first_layer) dst->first_layer->prev = layer;
     dst->first_layer = layer;
@@ -278,15 +276,13 @@ static void decode_layers_and_masks( struct psd_image *dst,
 static struct buffer
 packbitsdecode(struct buffer src, 
                struct buffer dst, 
-               int nbytes)
+               size_t nbytes)
 {
   int n;
 
   while( nbytes--  )
   {
-    n = psd_read_uchar( &src );
-    if(n >= 128)
-      n -= 256;
+    n = psd_read_schar( &src );
     if (n > 0)
     {
       ++n;
@@ -331,7 +327,7 @@ static void f_decode_packbits_encoded(INT32 args)
   struct pike_string *dest;
   int compression = -1;
   struct buffer b, ob, d;
-  if(sp[-args].type != T_STRING)
+  if(TYPEOF(sp[-args]) != T_STRING)
     Pike_error("Internal argument error.\n");
 
   get_all_args("decode_packbits_encoded", args,
@@ -343,13 +339,14 @@ static void f_decode_packbits_encoded(INT32 args)
   b.str = (unsigned char *)src->str;
   b.len = src->len;
   if (compression < 0) {
-    compression = (src->str[0] << 8) | src->str[1];
-    b.str += 2;
-    b.len -= 2;
+    compression = psd_read_ushort(&b);
   }
 
   pop_n_elems(args-1);
 
+  if (nelems < 0 || b.len < (size_t)nelems*2)
+    Pike_error("Not enough space for %d short integers.\n", nelems);
+
   ob = b;
   ob.str += nelems*2;
   ob.len -= nelems*2;
@@ -357,11 +354,10 @@ static void f_decode_packbits_encoded(INT32 args)
   switch(compression)
   {
    case 1:
-     dest = begin_shared_string( width * nelems );
-     d.str = (unsigned char *)dest->str; d.len = width*nelems;
-/*      while(nelems--) */
-     /*ob =*/ 
-     packbitsdecode( ob, d, width*nelems );
+     d.len = width * nelems;
+     dest = begin_shared_string( d.len );
+     d.str = (unsigned char *)dest->str;
+     packbitsdecode( ob, d, d.len );
      push_string( end_shared_string( dest ) );
      break;
    case 0:
@@ -397,7 +393,7 @@ static void f_decode_image_channel( INT32 args )
   source = (unsigned char *)s->str;
   push_int( w ); push_int( h );
   io = clone_object( image_program, 2 );
-  dst = ((struct image *)get_storage(io,image_program))->img;
+  dst = ((struct image*)get_storage(io,image_program))->img;
   for(y=0; y<w*h; y++)
   {
     dst->r = dst->g = dst->b = *(source++);
@@ -439,7 +435,7 @@ static void f_decode_image_data( INT32 args )
   source4 = source+w*h*3;
   push_int( w ); push_int( h );
   io = clone_object( image_program, 2 );
-  dst = ((struct image *)get_storage(io,image_program))->img;
+  dst = ((struct image*)get_storage(io,image_program))->img;
   for(y=0; y<w*h; y++)
   {
     switch( d )
@@ -503,7 +499,7 @@ static struct psd_image low_psd_decode( struct buffer *b )
   int *s = (int *)b->str;
   struct psd_image i;
   ONERROR err;
-  MEMSET(&i, 0, sizeof(i));
+  memset(&i, 0, sizeof(i));
   SET_ONERROR( err, free_image, &i );
   i.num_channels = psd_read_ushort( b );
   i.rows = psd_read_uint( b );
@@ -618,17 +614,17 @@ static void decode_resources( struct buffer *b )
       case 0x03f0: /* caption */
 	{
 	  struct buffer b = psd_read_pstring( &data );
-	  push_constant_text( "caption" );
+	  push_text( "caption" );
 	  push_buffer( &b );
 	}
 	break;
 
       case 0x0400: /* layer state info */
-	push_constant_text( "active_layer" );
+	push_text( "active_layer" );
 	push_int( psd_read_short( &data ) );
 	break;
       case 0x0408: /* guides */
-	push_constant_text( "guides" );
+	push_text( "guides" );
 	{
 	  int i,num_guides;
 	  short magic1, magic2, magic3, magic4, magic5, magic6; /*from gimp.*/
@@ -650,8 +646,8 @@ static void decode_resources( struct buffer *b )
 	      p = (ptrdiff_t)((((double)p) * (magic4>>8)) / ((double)(magic4&255)));
 	    else
 	      p = (ptrdiff_t)((((double)p) * (magic6>>8)) / ((double)(magic6&255)));
-	    push_constant_text( "pos" );      push_int64( p );
-	    push_constant_text( "vertical" ); push_int( !h );
+	    push_text( "pos" );      push_int64( p );
+	    push_text( "vertical" ); push_int( !h );
 	    f_aggregate_mapping( 4 );
 	  }
 	  f_aggregate( num_guides );
@@ -660,20 +656,20 @@ static void decode_resources( struct buffer *b )
       case 0x040b: /* URL */
 	{
 	  struct buffer b = psd_read_pstring( &data );
-	  push_constant_text( "url" );
+	  push_text( "url" );
 	  push_buffer( &b );
 	}
 	break;
       case 0x03ed: /* res. info. */
-	push_constant_text( "resinfo" );
+	push_text( "resinfo" );
 
-	push_constant_text( "hres" );       push_int(psd_read_int( &data ) );
-	push_constant_text( "hres_unit" );  push_int(psd_read_short( &data ) );
-	push_constant_text( "width_unit" ); push_int(psd_read_short( &data ) );
+	push_text( "hres" );       push_int(psd_read_int( &data ) );
+	push_text( "hres_unit" );  push_int(psd_read_short( &data ) );
+	push_text( "width_unit" ); push_int(psd_read_short( &data ) );
 
-	push_constant_text( "vres" );       push_int(psd_read_int( &data ) );
-	push_constant_text( "vres_unit" );  push_int(psd_read_short( &data ) );
-	push_constant_text( "height_unit" );push_int(psd_read_short( &data ) );
+	push_text( "vres" );       push_int(psd_read_int( &data ) );
+	push_text( "vres_unit" );  push_int(psd_read_short( &data ) );
+	push_text( "height_unit" );push_int(psd_read_short( &data ) );
 
 	f_aggregate_mapping( 12 );
 	break;
@@ -747,7 +743,7 @@ static void image_f_psd___decode( INT32 args )
 {
   struct pike_string *s;
   struct buffer b;
-  get_all_args( "Image.PSD.___decode", args, "%S", &s );
+  get_all_args( "___decode", args, "%S", &s );
   if(args > 1)
     pop_n_elems( args-1 );
   if(s->len < 26+4+4+4 ) /* header+color mode+image res+layers */
@@ -782,7 +778,7 @@ static void f_apply_cmap( INT32 args )
   get_all_args( "apply_cmap", args, "%o%S", &io, &cmap );
   if(cmap->len < 256*3)
     Pike_error("Invalid colormap resource\n");
-  if(!(i = (struct image *)get_storage( io, image_program )))
+  if(!(i = get_storage( io, image_program )))
     Pike_error("Invalid image object\n");
   n = i->xsize * i->ysize;
   d = i->img;
@@ -800,7 +796,7 @@ static void f_apply_cmap( INT32 args )
 }
 
 static struct program *image_encoding_psd_program=NULL;
-void init_image_psd()
+void init_image_psd(void)
 {
   ADD_FUNCTION( "___decode", image_f_psd___decode, tFunc(tStr,tMapping), 0);
   ADD_FUNCTION( "___decode_image_channel", f_decode_image_channel,tFunction,0);
@@ -828,7 +824,7 @@ void init_image_psd()
 }
 
 
-void exit_image_psd()
+void exit_image_psd(void)
 {
 #define STRING(X) free_string(PIKE_CONCAT(s_, X))
 #include "psd_constant_strings.h"
