@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 9cd47487ebcbcff10851edb8ae7752d47d64c185 $
 */
 
 /*
@@ -38,7 +37,7 @@
 #include "builtin_functions.h"
 #include "operators.h"
 #include "encodings.h"
-
+#include "pike_types.h"
 
 #define sp Pike_sp
 
@@ -129,8 +128,7 @@ void img_xwd__decode(INT32 args,int header_only,int skipcmap)
 
    ONERROR uwp;
 
-   if (args<1 ||
-       sp[-args].type!=T_STRING)
+   if (args<1 || TYPEOF(sp[-args]) != T_STRING)
       Pike_error("Image.XWD._decode(): Illegal arguments\n");
    
    s=sp[-args].u.string;
@@ -235,7 +233,7 @@ void img_xwd__decode(INT32 args,int header_only,int skipcmap)
 
    n+=25;
 
-   push_text("type");
+   ref_push_string(literal_type_string);
    push_text("image/x-xwd");
    n++;
 
