@@ -2,11 +2,10 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: ea6b87946c8ba8bac373c68b2f1e01771322565f $
 */
 
 /*
- * RFC1521 functionality for Pike
+ * @rfc{1521@} functionality for Pike
  *
  * Marcus Comstedt 1996-1999
  */
@@ -121,19 +120,25 @@ PIKE_MODULE_INIT
     rfc822ctype[(int)"<>@,;:\\/?"[i]] = CT_SPECIAL;
 
   /* Add global functions */
-  add_function_constant( "decode_base64", f_decode_base64,
-			 "function(string:string)", OPT_TRY_OPTIMIZE );
-  add_function_constant( "encode_base64", f_encode_base64,
-			 "function(string,void|int:string)",OPT_TRY_OPTIMIZE );
+
+  /* Really tFunc(tStr7, tStr8), but cut down on warnings for now. */
+  ADD_FUNCTION2( "decode_base64", f_decode_base64,
+                 tFunc(tStr, tStr8), 0, OPT_TRY_OPTIMIZE );
+
+  ADD_FUNCTION2( "encode_base64", f_encode_base64,
+                 tFunc(tStr tOr(tVoid,tInt),tStr7), 0, OPT_TRY_OPTIMIZE );
+
   add_function_constant( "decode_qp", f_decode_qp,
 			 "function(string:string)", OPT_TRY_OPTIMIZE );
-  add_function_constant( "encode_qp", f_encode_qp,
-			 "function(string,void|int:string)",OPT_TRY_OPTIMIZE );
+
+  ADD_FUNCTION2( "encode_qp", f_encode_qp,
+                 tFunc(tStr tOr(tVoid,tInt),tStr7), 0, OPT_TRY_OPTIMIZE );
+
   add_function_constant( "decode_uue", f_decode_uue,
 			 "function(string:string)", OPT_TRY_OPTIMIZE );
-  add_function_constant( "encode_uue", f_encode_uue,
-			 "function(string,void|string:string)",
-			 OPT_TRY_OPTIMIZE);
+
+  ADD_FUNCTION2( "encode_uue", f_encode_uue,
+                 tFunc(tStr tOr(tVoid,tStr),tStr7), 0, OPT_TRY_OPTIMIZE );
 
   add_integer_constant("TOKENIZE_KEEP_ESCAPES", TOKENIZE_KEEP_ESCAPES, 0);
 
@@ -172,7 +177,7 @@ static void f_decode_base64( INT32 args )
 {
   if(args != 1)
     Pike_error( "Wrong number of arguments to MIME.decode_base64()\n" );
-  else if (sp[-1].type != T_STRING)
+  else if (TYPEOF(sp[-1]) != T_STRING)
     Pike_error( "Wrong type of argument to MIME.decode_base64()\n" );
   else if (sp[-1].u.string->size_shift != 0)
     Pike_error( "Char out of range for MIME.decode_base64()\n" );
@@ -187,7 +192,7 @@ static void f_decode_base64( INT32 args )
     SIGNED char *src;
     ptrdiff_t cnt;
     INT32 d = 1;
-    int pads = 0;
+    int pads = 3;
 
     init_string_builder( &buf, 0 );
 
@@ -203,14 +208,27 @@ static void f_decode_base64( INT32 args )
 	  d=1;
 	}
       } else if (*src=='=') {
-	/* A pad character has been encountered.
-	   Increase pad count, and remove unused bits from d. */
-	pads++;
-	d>>=2;
+	/* A pad character has been encountered. */
+        break;
       }
 
     /* If data size not an even multiple of 3 bytes, output remaining data */
+    if (d & 0x3f000000) {
+      /* NOT_REACHED, but here for symmetry. */
+      pads = 0;
+    } else if (d & 0xfc0000) {
+      pads = 1;
+      /* Remove unused bits from d. */
+      d >>= 2;
+    } else if (d & 0x3f000) {
+      pads = 2;
+      /* Remove unused bits from d. */
+      d >>= 4;
+    }
     switch(pads) {
+    case 0:
+      /* NOT_REACHED, but here for symmetry. */
+      string_builder_putchar( &buf, (d>>16)&0xff );
     case 1:
       string_builder_putchar( &buf, (d>>8)&0xff );
     case 2:
@@ -271,7 +289,7 @@ static void f_encode_base64( INT32 args )
 {
   if(args != 1 && args != 2)
     Pike_error( "Wrong number of arguments to MIME.encode_base64()\n" );
-  else if(sp[-args].type != T_STRING)
+  else if(TYPEOF(sp[-args]) != T_STRING)
     Pike_error( "Wrong type of argument to MIME.encode_base64()\n" );
   else if (sp[-args].u.string->size_shift != 0)
     Pike_error( "Char out of range for MIME.encode_base64()\n" );
@@ -284,7 +302,7 @@ static void f_encode_base64( INT32 args )
     ptrdiff_t groups = (sp[-args].u.string->len+2)/3;
     ptrdiff_t last = (sp[-args].u.string->len-1)%3+1;
 
-    int insert_crlf = !(args == 2 && sp[-1].type == T_INT &&
+    int insert_crlf = !(args == 2 && TYPEOF(sp[-1]) == T_INT &&
 			sp[-1].u.integer != 0);
 
     /* We need 4 bytes for each 24 bit group, and 2 bytes for each linebreak */
@@ -337,7 +355,7 @@ static void f_decode_qp( INT32 args )
 {
   if(args != 1)
     Pike_error( "Wrong number of arguments to MIME.decode_qp()\n" );
-  else if(sp[-1].type != T_STRING)
+  else if(TYPEOF(sp[-1]) != T_STRING)
     Pike_error( "Wrong type of argument to MIME.decode_qp()\n" );
   else if (sp[-1].u.string->size_shift != 0)
     Pike_error( "Char out of range for MIME.decode_qp()\n" );
@@ -403,7 +421,7 @@ static void f_encode_qp( INT32 args )
 {
   if (args != 1 && args != 2)
     Pike_error( "Wrong number of arguments to MIME.encode_qp()\n" );
-  else if (sp[-args].type != T_STRING)
+  else if (TYPEOF(sp[-args]) != T_STRING)
     Pike_error( "Wrong type of argument to MIME.encode_qp()\n" );
   else if (sp[-args].u.string->size_shift != 0)
     Pike_error( "Char out of range for MIME.encode_qp()\n" );
@@ -417,7 +435,7 @@ static void f_encode_qp( INT32 args )
     unsigned char *src = (unsigned char *)sp[-args].u.string->str;
     ptrdiff_t cnt;
     int col = 0;
-    int insert_crlf = !(args == 2 && sp[-1].type == T_INT &&
+    int insert_crlf = !(args == 2 && TYPEOF(sp[-1]) == T_INT &&
 			sp[-1].u.integer != 0);
 
     init_string_builder( &buf, 0 );
@@ -463,7 +481,7 @@ static void f_decode_uue( INT32 args )
 {
   if (args != 1)
     Pike_error( "Wrong number of arguments to MIME.decode_uue()\n" );
-  else if(sp[-1].type != T_STRING)
+  else if(TYPEOF(sp[-1]) != T_STRING)
     Pike_error( "Wrong type of argument to MIME.decode_uue()\n" );
   else if (sp[-1].u.string->size_shift != 0)
     Pike_error( "Char out of range for MIME.decode_uue()\n" );
@@ -611,12 +629,12 @@ static void f_encode_uue( INT32 args )
 {
   if (args != 1 && args != 2)
     Pike_error( "Wrong number of arguments to MIME.encode_uue()\n" );
-  else if (sp[-args].type != T_STRING ||
-	   (args == 2 && sp[-1].type != T_VOID && sp[-1].type != T_STRING &&
-	    sp[-1].type != T_INT))
+  else if (TYPEOF(sp[-args]) != T_STRING ||
+	   (args == 2 && TYPEOF(sp[-1]) != T_VOID &&
+	    TYPEOF(sp[-1]) != T_STRING && TYPEOF(sp[-1]) != T_INT))
     Pike_error( "Wrong type of argument to MIME.encode_uue()\n" );
   else if (sp[-args].u.string->size_shift != 0 ||
-	   (args == 2 && sp[-1].type == T_STRING &&
+	   (args == 2 && TYPEOF(sp[-1]) == T_STRING &&
 	    sp[-1].u.string->size_shift != 0))
     Pike_error( "Char out of range for MIME.encode_uue()\n" );
   else {
@@ -633,7 +651,7 @@ static void f_encode_uue( INT32 args )
     ptrdiff_t last= (sp[-args].u.string->len - 1)%3 + 1;
 
     /* Get the filename if provided */
-    if (args == 2 && sp[-1].type == T_STRING)
+    if (args == 2 && TYPEOF(sp[-1]) == T_STRING)
       filename = sp[-1].u.string->str;
 
     /* Allocate the space we need.  This included space for the actual
@@ -734,7 +752,7 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
       if (l>0) {
 	/* Yup.  It's an encoded word, so it must be an atom.  */
 	if(mode)
-	  push_constant_text("encoded-word");
+	  push_text("encoded-word");
 	push_string( make_shared_binary_string( (char *)src, l ) );
 	if(mode)
 	  f_aggregate(2);
@@ -748,7 +766,7 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
     case CT_RPAR:
       /* Individual special character, push as a char (= int) */
       if(mode)
-	push_constant_text("special");
+	push_text("special");
       push_int( *src++ );
       if(mode)
 	f_aggregate(2);
@@ -763,7 +781,7 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
 	  break;
 
       if(mode)
-	push_constant_text("word");
+	push_text("word");
       push_string( make_shared_binary_string( (char *)src, l ) );
       if(mode)
 	f_aggregate(2);
@@ -786,7 +804,7 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
 
       /* Push the resulting string */
       if(mode)
-	push_constant_text("word");
+	push_text("word");
 
       if (e) {
 	/* l is the distance to the ending ", and e is the number of	\
@@ -824,7 +842,7 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
       if (l >= cnt) {
 	/* No ]; seems that this was no domain literal after all... */
 	if(mode)
-	  push_constant_text("special");
+	  push_text("special");
 	push_int( *src++ );
 	if(mode)
 	  f_aggregate(2);
@@ -843,7 +861,7 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
 
       /* Push the resulting string */
       if(mode)
-	push_constant_text("domain-literal");
+	push_text("domain-literal");
       push_string( end_shared_string( str ) );
       if(mode)
 	f_aggregate(2);
@@ -871,7 +889,7 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
 	  }
 
       if(mode) {
-	push_constant_text("comment");
+	push_text("comment");
 
 	str = begin_shared_string( l-e-1 );
 
@@ -912,7 +930,7 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
 
 /*! @decl array(string|int) tokenize(string header, int|void flags)
  *!
- *! A structured header field, as specified by RFC822, is constructed from
+ *! A structured header field, as specified by @rfc{822@}, is constructed from
  *! a sequence of lexical elements.
  *!
  *! @param header
@@ -955,10 +973,10 @@ static void low_tokenize( const char *funname, INT32 args, int mode )
  *! domain-literal @tt{[127.0.0.1]@} from the quoted-string
  *! @tt{"[127.0.0.1]"@}. Hopefully this won't cause any problems.
  *! Domain-literals are used seldom, if at all, anyway...
- *! 
- *! The set of special-characters is the one specified in RFC1521
+ *!
+ *! The set of special-characters is the one specified in @rfc{1521@}
  *! (i.e. @expr{"<", ">", "@@", ",", ";", ":", "\", "/", "?", "="@}),
- *! and not the set specified in RFC822.
+ *! and not the set specified in @rfc{822@}.
  *!
  *! @seealso
  *!   @[MIME.quote()], @[tokenize_labled()],
@@ -1083,7 +1101,7 @@ static void f_quote( INT32 args )
 
   if (args != 1)
     Pike_error( "Wrong number of arguments to MIME.quote()\n" );
-  else if (sp[-1].type != T_ARRAY)
+  else if (TYPEOF(sp[-1]) != T_ARRAY)
     Pike_error( "Wrong type of argument to MIME.quote()\n" );
 
   /* Quote array in sp[-1].u.array.  Once again we'll rely on a
@@ -1093,13 +1111,13 @@ static void f_quote( INT32 args )
 
   for (cnt=sp[-1].u.array->size, item=sp[-1].u.array->item; cnt--; item++) {
 
-    if (item->type == T_INT) {
+    if (TYPEOF(*item) == T_INT) {
 
       /* Single special character */
       string_builder_putchar( &buf, item->u.integer );
       prev_atom = 0;
 
-    } else if (item->type != T_STRING) {
+    } else if (TYPEOF(*item) != T_STRING) {
 
       /* Neither int or string.  Too bad... */
       free_string_builder( &buf );
@@ -1170,7 +1188,7 @@ static void f_quote_labled( INT32 args )
 
   if (args != 1)
     Pike_error( "Wrong number of arguments to MIME.quote_labled()\n" );
-  else if (sp[-1].type != T_ARRAY)
+  else if (TYPEOF(sp[-1]) != T_ARRAY)
     Pike_error( "Wrong type of argument to MIME.quote_labled()\n" );
 
   /* Quote array in sp[-1].u.array.  Once again we'll rely on a
@@ -1180,15 +1198,15 @@ static void f_quote_labled( INT32 args )
 
   for (cnt=sp[-1].u.array->size, item=sp[-1].u.array->item; cnt--; item++) {
 
-    if (item->type != T_ARRAY || item->u.array->size<2 ||
-	item->u.array->item[0].type != T_STRING) {
+    if (TYPEOF(*item) != T_ARRAY || item->u.array->size<2 ||
+	TYPEOF(item->u.array->item[0]) != T_STRING) {
       free_string_builder( &buf );
       Pike_error( "Wrong type of argument to MIME.quote_labled()\n" );
     }
 
     if (c_compare_string( item->u.array->item[0].u.string, "special", 7 )) {
 
-      if(item->u.array->item[1].type != T_INT) {
+      if(TYPEOF(item->u.array->item[1]) != T_INT) {
 	free_string_builder( &buf );
 	Pike_error( "Wrong type of argument to MIME.quote_labled()\n" );
       }
@@ -1197,7 +1215,7 @@ static void f_quote_labled( INT32 args )
       string_builder_putchar( &buf, item->u.array->item[1].u.integer );
       prev_atom = 0;
 
-    } else if(item->u.array->item[1].type != T_STRING) {
+    } else if(TYPEOF(item->u.array->item[1]) != T_STRING) {
 
       /* All the remaining lexical items require item[1] to be a string */
       free_string_builder( &buf );
