@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 74177bb151cbb484e8625e240fc81b20a9a4f22d $
 */
 
 /* All this code is pretty useless if we don't have a msql library...*/
@@ -835,7 +834,7 @@ static void do_list_fields (INT32 args)
     
     push_text("name");
 		push_text(field->name);
-		push_text("type");
+		ref_push_string(literal_type_string);
 		push_text(decode_msql_type(field->type));
 		push_text("length");
 		push_int(field->length);
