@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 5c545cc8a5965f6aa0ef95a10b317bf6d1a50797 $
 */
 
 /*
@@ -69,11 +68,9 @@ int odbc_driver_is_old_freetds(HDBC odbc_conn)
     int is_tds = 0;
     push_text(buf);
     f_lower_case(1);
-    push_constant_text("libtdsodbc");
-    f_has_value(2);
-    is_tds = !SAFE_IS_ZERO(&Pike_sp[-1]);
+    is_tds = strstr(Pike_sp[-1].u.string->str, "libtdsodbc") ? 1 : 0;
     pop_stack();
-    
+
     if(is_tds) {
       if(SQLGetInfo(odbc_conn, SQL_DRIVER_VER, buf, sizeof(buf), &len) == SQL_SUCCESS) {
 	double version = atof(buf);
@@ -92,7 +89,6 @@ int odbc_driver_is_old_freetds(HDBC odbc_conn)
  * Helper functions
  */
 
-#ifdef SQL_WCHAR
 struct pike_string *make_shared_binary_sqlwchar(SQLWCHAR *str,
 						size_t len)
 {
@@ -105,7 +101,6 @@ void push_sqlwchar(SQLWCHAR *str, size_t len)
 {
   push_string(make_shared_binary_sqlwchar(str, len));
 }
-#endif /* SQL_WCHAR */
 
 void odbc_error(const char *fun, const char *msg,
 		struct precompiled_odbc *odbc, SQLHSTMT hstmt,
@@ -125,26 +120,15 @@ void odbc_error(const char *fun, const char *msg,
 		RETCODE code, void (*clean)(void *), void *clean_arg)
 {
   RETCODE _code;
-#ifdef SQL_WCHAR
   SQLWCHAR errcode[256];
   SQLWCHAR errmsg[SQL_MAX_MESSAGE_LENGTH];
-#else
-  unsigned char errcode[256];
-  unsigned char errmsg[SQL_MAX_MESSAGE_LENGTH];
-#endif
   SWORD errmsg_len = 0;
-  SDWORD native_error;
-  HDBC hdbc = odbc->hdbc;
+  SQLINTEGER native_error;
+  HDBC hdbc = odbc?odbc->hdbc:SQL_NULL_HDBC;
 
   ODBC_ALLOW();
-  _code =
-#ifdef SQL_WCHAR
-    SQLErrorW
-#else
-    SQLError
-#endif
-    (odbc_henv, hdbc, hstmt, errcode, &native_error,
-     errmsg, (SQL_MAX_MESSAGE_LENGTH-1), &errmsg_len);
+  _code = SQLErrorW(odbc_henv, hdbc, hstmt, errcode, &native_error,
+		    errmsg, (SQL_MAX_MESSAGE_LENGTH-1), &errmsg_len);
   errmsg[errmsg_len] = '\0';
   ODBC_DISALLOW();
 
@@ -152,11 +136,7 @@ void odbc_error(const char *fun, const char *msg,
     if (odbc->last_error) {
       free_string(odbc->last_error);
     }
-#ifdef SQL_WCHAR
     odbc->last_error = make_shared_binary_sqlwchar(errmsg, errmsg_len);
-#else
-    odbc->last_error = make_shared_binary_string((char *)errmsg, errmsg_len);
-#endif
   }
 
   if (clean) {
@@ -165,15 +145,9 @@ void odbc_error(const char *fun, const char *msg,
   switch(_code) {
   case SQL_SUCCESS:
   case SQL_SUCCESS_WITH_INFO:
-#ifdef SQL_WCHAR
     Pike_error("%s(): %s:\n"
 	  "%d:%ls:%ls\n",
 	  fun, msg, code, errcode, errmsg);
-#else
-    Pike_error("%s(): %s:\n"
-	  "%d:%s:%s\n",
-	  fun, msg, code, errcode, errmsg);
-#endif
     break;
   case SQL_ERROR:
     Pike_error("%s(): %s:\n"
@@ -239,7 +213,7 @@ static void clean_last_error(void)
  *!   @[Sql.odbc], @[Sql.dsn]
  */
 
-static void init_odbc_struct(struct object *o)
+static void init_odbc_struct(struct object *UNUSED(o))
 {
   HDBC hdbc = SQL_NULL_HDBC;
   RETCODE code;
@@ -257,7 +231,7 @@ static void init_odbc_struct(struct object *o)
 		   code, NULL, NULL);
 }
 
-static void exit_odbc_struct(struct object *o)
+static void exit_odbc_struct(struct object *UNUSED(o))
 {
   SQLHDBC hdbc = PIKE_ODBC->hdbc;
 
@@ -321,7 +295,7 @@ static void f_create(INT32 args)
 		 BIT_MAPPING|BIT_INT|BIT_VOID, 0);
 
 #define GET_ARG(VAR, ARG) do {					\
-    if ((args > (ARG)) && (sp[(ARG)-args].type == T_STRING)) {	\
+    if ((args > (ARG)) && (TYPEOF(sp[(ARG)-args]) == T_STRING)) {	\
       VAR = sp[(ARG)-args].u.string;				\
       if (VAR->size_shift) {					\
 	SIMPLE_ARG_TYPE_ERROR("create", (ARG), "string(8bit)");	\
@@ -330,6 +304,16 @@ static void f_create(INT32 args)
   } while(0)
 
   GET_ARG(pwd, 3);
+  if (pwd) {
+    add_ref(pwd);
+    pop_n_elems(args-3);
+    /* NB: Potential but very unlikely leak of pwd here
+     *     if "CENSORED" fails to allocate.
+     */
+    push_text("CENSORED");
+    push_string(pwd);	/* Let the stack hold the reference. */
+    args = 5;
+  }
   GET_ARG(user, 2);
   GET_ARG(database, 1);
   GET_ARG(server, 0);
@@ -345,7 +329,7 @@ static void f_create(INT32 args)
 
   if (!database || !database->len) {
     if (!server || !server->len) {
-      push_constant_text("default");
+      push_text("default");
       database = sp[-1].u.string;
       args++;
     } else {
@@ -404,7 +388,7 @@ static void f_create_dsn(INT32 args)
   SQLCHAR outconnectionstring[1024]; /* Smallest allowed buffer = 1024 */
   SQLSMALLINT stringlength2;
 
-  get_all_args("odbc->create_dsn", args, "%S", &connectstring);
+  get_all_args("create_dsn", args, "%S", &connectstring);
 
   if (!connectstring->len) {
     Pike_error("odbc->create_dsn connection string empty.\n");
@@ -447,7 +431,7 @@ static void f_affected_rows(INT32 args)
   push_int64(PIKE_ODBC->affected_rows);
 }
 
-static void f_select_db(INT32 args)
+static void f_select_db(INT32 UNUSED(args))
 {
   /**********************************************/
 }
@@ -468,7 +452,7 @@ static void f_big_query(INT32 args)
   struct svalue *save_sp = sp + 1 - args;
 #endif /* PIKE_DEBUG */
 
-  get_all_args("odbc->big_query", args, "%W", &q);
+  get_all_args("big_query", args, "%W", &q);
 
   add_ref(q);
   SET_ONERROR(ebuf, odbc_free_string, q);
@@ -479,7 +463,12 @@ static void f_big_query(INT32 args)
 
   /* Allocate the statement (result) object */
   ref_push_object(fp->current_object);
-  push_object(clone_object(odbc_result_program, 1));
+  apply_current(odbc_result_fun_num, 1);
+
+  if (TYPEOF(Pike_sp[-1]) != T_OBJECT) {
+    Pike_error("odbc->big_typed_query(): Unexpected return value from "
+	  "odbc_result().\n");
+  }
 
   UNSET_ONERROR(ebuf);
 
@@ -491,7 +480,7 @@ static void f_big_query(INT32 args)
   push_string(q);
   apply(sp[-2].u.object, "execute", 1);
   
-  if (sp[-1].type != T_INT) {
+  if (TYPEOF(sp[-1]) != T_INT) {
     Pike_error("odbc->big_query(): Unexpected return value from "
 	  "odbc_result->execute().\n");
   }
@@ -523,6 +512,74 @@ static void f_big_query(INT32 args)
 #endif /* PIKE_DEBUG */
 }
 
+static void f_big_typed_query(INT32 args)
+{
+  ONERROR ebuf;
+  struct pike_string *q = NULL;
+#ifdef PIKE_DEBUG
+  struct svalue *save_sp = sp + 1 - args;
+#endif /* PIKE_DEBUG */
+
+  get_all_args("big_typed_query", args, "%W", &q);
+
+  add_ref(q);
+  SET_ONERROR(ebuf, odbc_free_string, q);
+
+  pop_n_elems(args);
+
+  clean_last_error();
+
+  /* Allocate the statement (result) object */
+  ref_push_object(fp->current_object);
+  apply_current(odbc_typed_result_fun_num, 1);
+
+  if (TYPEOF(Pike_sp[-1]) != T_OBJECT) {
+    Pike_error("odbc->big_typed_query(): Unexpected return value from "
+	  "odbc_typed_result().\n");
+  }
+
+  UNSET_ONERROR(ebuf);
+
+  /* Potential return value is now in place */
+
+  PIKE_ODBC->affected_rows = 0;
+
+  /* Do the actual query */
+  push_string(q);
+  apply(sp[-2].u.object, "execute", 1);
+
+  if (TYPEOF(sp[-1]) != T_INT) {
+    Pike_error("odbc->big_typed_query(): Unexpected return value from "
+	  "odbc_result->execute().\n");
+  }
+
+  if (!sp[-1].u.integer) {
+    pop_n_elems(2);	/* Zap the result object too */
+
+#ifdef ENABLE_IMPLICIT_COMMIT
+    /* This breaks with Free TDS. */
+    {
+      HDBC hdbc = PIKE_ODBC->hdbc;
+      RETCODE code;
+      ODBC_ALLOW();
+      code = SQLTransact(odbc_henv, hdbc, SQL_COMMIT);
+      ODBC_DISALLOW();
+      odbc_check_error("odbc->big_query", "Couldn't commit query",
+		       code, NULL, NULL);
+    }
+#endif /* ENABLE_IMPLICIT_COMMIT */
+
+    push_int(0);
+  } else {
+    pop_stack();	/* Keep the result object */
+  }
+#ifdef PIKE_DEBUG
+  if (sp != save_sp) {
+    Pike_fatal("Stack error in odbc->big_typed_query().\n");
+  }
+#endif /* PIKE_DEBUG */
+}
+
 static void f_list_tables(INT32 args)
 {
 #ifdef PIKE_DEBUG
@@ -532,7 +589,7 @@ static void f_list_tables(INT32 args)
   struct pike_string *pattern = NULL;
 
   if (args) {
-    if ((Pike_sp[-args].type != T_STRING) ||
+    if ((TYPEOF(Pike_sp[-args]) != T_STRING) ||
 	(Pike_sp[-args].u.string->size_shift)) {
       Pike_error("odbc->list_tables(): "
 		 "Bad argument 1. Expected 8-bit string.\n");
@@ -564,7 +621,7 @@ static void f_list_tables(INT32 args)
     apply(sp[-1].u.object, "list_tables", 0);
   }
   
-  if (sp[-1].type != T_INT) {
+  if (TYPEOF(sp[-1]) != T_INT) {
     Pike_error("odbc->list_tables(): Unexpected return value from "
 	       "odbc_result->list_tables().\n");
   }
@@ -583,22 +640,22 @@ static void f_list_tables(INT32 args)
 #endif /* PIKE_DEBUG */
 }
 
-static void f_create_db(INT32 args)
+static void f_create_db(INT32 UNUSED(args))
 {
   /**************************************************/
 }
 
-static void f_drop_db(INT32 args)
+static void f_drop_db(INT32 UNUSED(args))
 {
   /**************************************************/
 }
 
-static void f_shutdown(INT32 args)
+static void f_shutdown(INT32 UNUSED(args))
 {
   /**************************************************/
 }
 
-static void f_reload(INT32 args)
+static void f_reload(INT32 UNUSED(args))
 {
   /**************************************************/
 }
@@ -612,13 +669,8 @@ static void f_reload(INT32 args)
  */
 static void f_list_dbs(INT32 args)
 {
-#ifdef SQL_WCHAR
   static SQLWCHAR buf[SQL_MAX_DSN_LENGTH+1];
   static SQLWCHAR descr[256];
-#else
-  static UCHAR buf[SQL_MAX_DSN_LENGTH+1];
-  static UCHAR descr[256];
-#endif
   SQLSMALLINT buf_len = 0;
   SQLSMALLINT descr_len = 0;
   int cnt = 0;
@@ -627,35 +679,19 @@ static void f_list_dbs(INT32 args)
   pop_n_elems(args);
 
   ODBC_ALLOW();
-  ret =
-#ifdef SQL_WCHAR
-    SQLDataSourcesW
-#else
-    SQLDataSources
-#endif
-    (odbc_henv, SQL_FETCH_FIRST,
-     buf, SQL_MAX_DSN_LENGTH, &buf_len,
-     descr, 255, &descr_len);
+  ret = SQLDataSourcesW(odbc_henv, SQL_FETCH_FIRST,
+			buf, SQL_MAX_DSN_LENGTH, &buf_len,
+			descr, 255, &descr_len);
   ODBC_DISALLOW();
 
   while ((ret == SQL_SUCCESS) || (ret == SQL_SUCCESS_WITH_INFO)) {
-#ifdef SQL_WCHAR
     push_sqlwchar(buf, buf_len);
-#else
-    push_string(make_shared_binary_string(buf, buf_len));
-#endif
     cnt++;
 
     ODBC_ALLOW();
-    ret =
-#ifdef SQL_WCHAR
-      SQLDataSourcesW
-#else
-      SQLDataSources
-#endif
-      (odbc_henv, SQL_FETCH_NEXT,
-       buf, SQL_MAX_DSN_LENGTH, &buf_len,
-       descr, 255, &descr_len);
+    ret = SQLDataSourcesW(odbc_henv, SQL_FETCH_NEXT,
+			  buf, SQL_MAX_DSN_LENGTH, &buf_len,
+			  descr, 255, &descr_len);
     ODBC_DISALLOW();
   }
 
@@ -731,6 +767,9 @@ PIKE_MODULE_INIT
   ADD_FUNCTION("select_db", f_select_db,tFunc(tStr,tVoid), ID_PUBLIC);
   /* function(string:int|object) */
   ADD_FUNCTION("big_query", f_big_query,tFunc(tStr,tOr(tInt,tObj)), ID_PUBLIC);
+  /* function(string:int|object) */
+  ADD_FUNCTION("big_typed_query", f_big_typed_query, tFunc(tStr,tOr(tInt,tObj)),
+	       ID_PUBLIC);
   /* function(void:int) */
   ADD_FUNCTION("affected_rows", f_affected_rows,tFunc(tVoid,tInt), ID_PUBLIC);
   /* function(void|string:object) */
@@ -767,6 +806,8 @@ PIKE_MODULE_INIT
   set_init_callback(init_odbc_struct);
   set_exit_callback(exit_odbc_struct);
  
+  init_odbc_res_programs();
+
   odbc_program = end_program();
   add_program_constant("odbc", odbc_program, 0);
  
@@ -777,8 +818,6 @@ PIKE_MODULE_INIT
   ADD_FUNCTION ("connect_lock", f_connect_lock, tFunc(tOr(tVoid,tInt),tInt01), ID_PUBLIC);
 #endif
 
-  init_odbc_res_programs();
-
 #else
   if(!TEST_COMPAT(7,6))
     HIDE_MODULE();
