@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 78ee17410fe6b8d829eb398b4dc1a8b15f8e94fc $
 */
 
 #include "global.h"
@@ -117,7 +116,7 @@ static void pdf_magic_index(INT32 args)
 
    if (args!=1) 
       Pike_error("PDF.`[]: Too few or too many arguments\n");
-   if (sp[-1].type!=T_STRING)
+   if (TYPEOF(sp[-1]) != T_STRING)
       Pike_error("PDF.`[]: Illegal type of argument\n");
 
    for (i=0; i<(int)NELEM(submagic); i++)
@@ -152,7 +151,7 @@ static void pdf_magic_index(INT32 args)
    stack_swap();
    f_arrow(2);
 
-   if (sp[-1].type==T_INT)
+   if (TYPEOF(sp[-1]) == T_INT)
    {
       pop_stack();
       stack_dup();
@@ -161,7 +160,7 @@ static void pdf_magic_index(INT32 args)
       f_add(2);
       SAFE_APPLY_MASTER("resolv",1);
    }
-   if (sp[-1].type==T_INT)
+   if (TYPEOF(sp[-1]) == T_INT)
    {
       pop_stack();
       stack_dup();
