@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 84b8999ea0a2a5732849d07c45ba3465f35fbb99 $
 */
 
 /*
@@ -129,6 +128,8 @@
 				 * times. */
 #define WORDSTART 11		/* node matching a start of a word          */
 #define WORDEND 12		/* node matching an end of a word           */
+#define KPLUS	13		/* node Match this (simple) thing 1 or more
+				 * times. */
 #define	OPEN	20		/* no	Mark this point in input as start of
 				 * #n. */
  /* OPEN+1 is number 1, etc. */
@@ -148,10 +149,10 @@
  * BACK		Normal "nxt" pointers all implicitly point forward; BACK
  *		exists to make loop structures possible.
  *
- * STAR		complex '*', are implemented as circular BRANCH structures 
+ * STAR,KPLUS	Complex cases are implemented as circular BRANCH structures
  *		using BACK.  Simple cases (one character per match) are 
- *		implemented with STAR for speed and to minimize recursive 
- *		plunges.
+ *		implemented with STAR or KPLUS for speed and to minimize
+ *		recursive plunges.
  *
  * OPEN,CLOSE	...are numbered at compile time.
  */
@@ -203,7 +204,7 @@
  * Flags to be passed up and down.
  */
 #define	HASWIDTH	01	/* Known never to match null string. */
-#define	SIMPLE		02	/* Simple enough to be STAR operand. */
+#define	SIMPLE		02	/* Simple enough to be STAR or KPLUS operand. */
 #define	SPSTART		04	/* Starts with * */
 #define	WORST		0	/* Worst case. */
 
@@ -260,7 +261,7 @@ regexp *pike_regcomp(char *exp,int excompat)
     if (exp == (char *)NULL)
 	FAIL("NULL argument");
 
-    exp2=(short*)xalloc( (strlen(exp)+1) * sizeof(short) );
+    exp2=xalloc( (strlen(exp)+1) * sizeof(short) );
     for ( scan=exp,dest=exp2;( c= UCHARAT(scan++)); ) {
 	switch (c) {
 	    case '(':
@@ -315,7 +316,7 @@ regexp *pike_regcomp(char *exp,int excompat)
 	FAIL("regexp too big");
 
     /* Allocate space. */
-    r = (regexp *) xalloc(sizeof(regexp) + (unsigned) regsize);
+    r = xalloc(sizeof(regexp) + (unsigned) regsize);
 
     /* Second pass: emit code. */
     regparse = exp2;
@@ -360,7 +361,7 @@ regexp *pike_regcomp(char *exp,int excompat)
 	    r->regmlen = len;
 	}
     }
-    free((char*)exp2);
+    free(exp2);
     return (r);
 }
 
@@ -470,7 +471,7 @@ static char  *regbranch(int *flagp)
 }
 
 /*
- - regpiece - something followed by possible [*]
+ - regpiece - something followed by possible [*] or [+]
  *
  * Note that the branching code sequence used for * is somewhat optimized:  
  * they use the same NOTHING node as both the endmarker for their branch 
@@ -515,14 +516,26 @@ static char *regpiece(int *flagp)
   }
   else if(op == PLUS)
   {
-    /*  Emit a+ as (a&) where & means "self" /Fredrik Hubinette */
-    char *tmp;
-    tmp=regnode(BACK);
-    reginsert(BRANCH, tmp);
-    regtail(ret, tmp);
-    regoptail(tmp, ret);
-    regtail(ret, regnode(BRANCH));
-    regtail(ret, regnode(NOTHING));
+    if (flags & SIMPLE)
+    {
+      reginsert(KPLUS, ret);
+    }
+    else
+    {
+      /* ret ->	1: x			nxt: 2
+       * tmp ->	2: BRANCH	op: 3	nxt: 4
+       *	3: BACK			nxt: 1
+       *	4: BRANCH	op: 5	nxt: 5
+       *	5: NOTHING
+       */
+      char *tmp;
+      tmp=regnode(BACK);
+      reginsert(BRANCH, tmp);
+      regtail(ret, tmp);
+      regoptail(tmp, ret);
+      regtail(ret, regnode(BRANCH));
+      regtail(ret, regnode(NOTHING));
+    }
   }
     
   regparse++;
@@ -784,7 +797,7 @@ int pike_regexec(regexp *prog, char *string)
     /* If there is a "must appear" string, look for it. */
     if (prog->regmust != (char *)NULL) {
 	s = string;
-	while ((s = STRCHR(s, prog->regmust[0])) != (char *)NULL) {
+	while ((s = strchr(s, prog->regmust[0])) != (char *)NULL) {
 	    if (strncmp(s, prog->regmust, prog->regmlen) == 0)
 		break;		/* Found it. */
 	    s++;
@@ -803,7 +816,7 @@ int pike_regexec(regexp *prog, char *string)
     s = string;
     if (prog->regstart != '\0')
 	/* We know what char it must start with. */
-	while ((s = STRCHR(s, prog->regstart)) != (char *)NULL) {
+	while ((s = strchr(s, prog->regstart)) != (char *)NULL) {
 	    if (regtry(prog, s))
 		return (1);
 	    s++;
@@ -940,13 +953,13 @@ char           *prog;
 	    break;
 	case ANYOF:
 	    if (*reginput == '\0' || 
-		 STRCHR(OPERAND(scan), *reginput) == (char *)NULL)
+		 strchr(OPERAND(scan), *reginput) == (char *)NULL)
 		return (0);
 	    reginput++;
 	    break;
 	case ANYBUT:
 	    if (*reginput == '\0' || 
-		 STRCHR(OPERAND(scan), *reginput) != (char *)NULL)
+		 strchr(OPERAND(scan), *reginput) != (char *)NULL)
 		return (0);
 	    reginput++;
 	    break;
@@ -973,6 +986,7 @@ char           *prog;
 		}
 	    }
 	    break;
+	case KPLUS:
 	case STAR:{
 		register char   nextch;
 		register ptrdiff_t no;
@@ -1092,13 +1106,13 @@ char           *p;
 	}
 	break;
     case ANYOF:
-	while (*scan != '\0' && STRCHR(opnd, *scan) != (char *)NULL) {
+	while (*scan != '\0' && strchr(opnd, *scan) != (char *)NULL) {
 	    count++;
 	    scan++;
 	}
 	break;
     case ANYBUT:
-	while (*scan != '\0' && STRCHR(opnd, *scan) == (char *)NULL) {
+	while (*scan != '\0' && strchr(opnd, *scan) == (char *)NULL) {
 	    count++;
 	    scan++;
 	}
@@ -1254,6 +1268,10 @@ char           *op;
 	p = "STAR";
 	break;
 
+    case KPLUS:
+	p = "KPLUS";
+	break;
+
     default:
         if(OP(op) >= OPEN && OP(op) < OPEN+NSUBEXP)
 	{
