@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 1639e7946bcb30ca2867eded812331afc1a0ac08 $
 */
 
 
@@ -119,13 +118,13 @@ static void raise_error(char *msg, DNSServiceErrorType err)
 }
 
 
-static void start_service_callback(DNSServiceRef ref,
-				   DNSServiceFlags flags,
-				   DNSServiceErrorType error,
-				   const char *name,
-				   const char *regtype,
-				   const char *domain,
-				   void *context)
+static void start_service_callback(DNSServiceRef UNUSED(ref),
+				   DNSServiceFlags UNUSED(flags),
+				   DNSServiceErrorType UNUSED(error),
+				   const char *UNUSED(name),
+				   const char *UNUSED(regtype),
+				   const char *UNUSED(domain),
+				   void *UNUSED(context))
 {
 }
 
@@ -291,7 +290,7 @@ static void * howl_thread(void *arg)
 }
 
 
-static void init_howl_module()
+static void init_howl_module(void)
 {
   if (sw_discovery_init(&service_session) == SW_OKAY) {
     th_create_small(&service_thread, howl_thread, NULL);
@@ -299,7 +298,7 @@ static void init_howl_module()
 }
 
 
-static void exit_howl_module()
+static void exit_howl_module(void)
 {
   /* Close active session */
   if (service_session)
@@ -371,13 +370,13 @@ static void f_create(INT32 args)
 }
 
 
-static void init_service_struct(struct object *o)
+static void init_service_struct(struct object *UNUSED(o))
 {
   THIS->service_ref = 0;
 }
 
 
-static void exit_service_struct(struct object *o)
+static void exit_service_struct(struct object *UNUSED(o))
 {
   /* Stop an existing service */
   stop_service(THIS);
