@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: aea5be43025ee272e22858fcd810cac69ca37036 $
 */
 
 #define NO_PIKE_SHORTHAND
@@ -13,7 +12,6 @@
 
 #include "machine.h"
 
-#include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -57,7 +55,7 @@ struct  header_buf
   int mode;
 };
 
-static void f_hp_init( struct object *o )
+static void f_hp_init( struct object *UNUSED(o) )
 {
   THP->headers = NULL;
   THP->pnt = NULL;
@@ -67,7 +65,7 @@ static void f_hp_init( struct object *o )
   THP->mode = 0;
 }
 
-static void f_hp_exit( struct object *o )
+static void f_hp_exit( struct object *UNUSED(o) )
 {
   if( THP->headers )
     free( THP->headers );
@@ -102,7 +100,7 @@ static void f_hp_feed( INT32 args )
 
   if (args != 1)
     Pike_error("Bad number of arguments to feed().\n");
-  if( Pike_sp[-1].type != PIKE_T_STRING )
+  if( TYPEOF(Pike_sp[-1]) != PIKE_T_STRING )
     Pike_error("Wrong type of argument to feed()\n");
   if( str->size_shift )
     Pike_error("Wide string headers not supported\n");
@@ -128,7 +126,7 @@ static void f_hp_feed( INT32 args )
     hp->pnt = (hp->headers + hp->hsize - hp->left);
   }
 
-  MEMCPY( hp->pnt, str->str, str_len );
+  memcpy( hp->pnt, str->str, str_len );
   pop_n_elems( args );
 
   /* FIXME: The below does not support lines terminated with just \r. */
@@ -233,7 +231,7 @@ static void f_hp_feed( INT32 args )
 
       if((tmp = low_mapping_lookup(headers, Pike_sp-2)))
       {
-	if( tmp->type == PIKE_T_ARRAY )
+	if( TYPEOF(*tmp) == PIKE_T_ARRAY )
 	{
           f_aggregate( 1 );
 	  ref_push_array(tmp->u.array);
@@ -303,12 +301,12 @@ static void f_make_http_headers( INT32 args )
   struct pike_string *res;
   int terminator = 2;
 
-  if( Pike_sp[-args].type != PIKE_T_MAPPING )
+  if( TYPEOF(Pike_sp[-args]) != PIKE_T_MAPPING )
     Pike_error("Wrong argument type to make_http_headers(mapping heads)\n");
   m = Pike_sp[-args].u.mapping;
 
   if (args > 1) {
-    if (Pike_sp[1-args].type != PIKE_T_INT)
+    if (TYPEOF(Pike_sp[1-args]) != PIKE_T_INT)
       Pike_error("Bad argument 2 to make_http_headers(). Expected int.\n");
     if (Pike_sp[1-args].u.integer)
       terminator = 0;
@@ -317,17 +315,18 @@ static void f_make_http_headers( INT32 args )
   /* loop to check len */
   NEW_MAPPING_LOOP( m->data )
   {
-    if( k->ind.type != PIKE_T_STRING || k->ind.u.string->size_shift )
+    if( TYPEOF(k->ind) != PIKE_T_STRING || k->ind.u.string->size_shift )
       Pike_error("Wrong argument type to make_http_headers("
             "mapping(string(8bit):string(8bit)|array(string(8bit))) heads)\n");
-    if( k->val.type == PIKE_T_STRING && !k->val.u.string->size_shift )
+    if( TYPEOF(k->val) == PIKE_T_STRING && !k->val.u.string->size_shift )
       total_len +=  k->val.u.string->len + 2 + k->ind.u.string->len + 2;
-    else if( k->val.type == PIKE_T_ARRAY )
+    else if( TYPEOF(k->val) == PIKE_T_ARRAY )
     {
       struct array *a = k->val.u.array;
       ptrdiff_t i, kl = k->ind.u.string->len + 2 ;
       for( i = 0; i<a->size; i++ )
-        if( a->item[i].type != PIKE_T_STRING||a->item[i].u.string->size_shift )
+        if( TYPEOF(a->item[i]) != PIKE_T_STRING ||
+	    a->item[i].u.string->size_shift )
           Pike_error("Wrong argument type to make_http_headers("
                 "mapping(string(8bit):string(8bit)|"
                 "array(string(8bit))) heads)\n");
@@ -350,7 +349,7 @@ static void f_make_http_headers( INT32 args )
   {
     unsigned char *s;
     ptrdiff_t l, c;
-    if( k->val.type == PIKE_T_STRING )
+    if( TYPEOF(k->val) == PIKE_T_STRING )
     {
       STRADD( k->ind ); *(pnt++) = ':'; *(pnt++) = ' ';
       STRADD( k->val ); *(pnt++) = '\r'; *(pnt++) = '\n';
@@ -378,97 +377,102 @@ static void f_make_http_headers( INT32 args )
 static void f_http_decode_string(INT32 args)
 /*! @decl string http_decode_string(string encoded)
  *!
- *! Decodes an http transport-encoded string.
+ *! Decodes an http transport-encoded string. Knows about %XX and
+ *! %uXXXX syntax. Treats %UXXXX as %uXXXX. It will treat '+' as '+'
+ *! and not ' ', so form decoding needs to replace that in a prior
+ *! step.
+ *!
+ *! @note
+ *!   Performs a best-effort decoding. Invalid and truncated escapes
+ *!   will still be decoded.
  */
 {
-   int proc;
-   int size_shift = 0;
-   int adjust_len = 0;
-   p_wchar0 *foo, *bar, *end;
-   struct pike_string *newstr;
+   int proc = 0;
+   int trunc = 0;
+   int size_shift;
+   PCHARP foo, end;
+   struct string_builder newstr;
 
-   if (!args || Pike_sp[-args].type != PIKE_T_STRING ||
-       Pike_sp[-args].u.string->size_shift)
-     Pike_error("Invalid argument to http_decode_string(string(8bit));\n");
+   if (!args || TYPEOF(Pike_sp[-args]) != PIKE_T_STRING)
+     Pike_error("Invalid argument to http_decode_string(string).\n");
 
-   foo = bar = STR0(Pike_sp[-args].u.string);
-   end = foo + Pike_sp[-args].u.string->len;
+   foo = MKPCHARP_STR(Pike_sp[-args].u.string);
+   end = ADD_PCHARP(foo, Pike_sp[-args].u.string->len);
 
-   /* count '%' and wide characters */
-   for (proc=0; foo<end; foo++) {
-     if (*foo=='%') {
-       proc++;
-       if (foo[1] == 'u' || foo[1] == 'U') {
-	 /* %uXXXX */
-	 if (foo[2] != '0' || foo[3] != '0') {
-	   size_shift = 1;
-	 }
-	 foo += 5;
-	 if (foo < end) {
-	   adjust_len += 5;
-	 } else {
-	   adjust_len += end - (foo - 4);
-	 }
-       } else {
-	 foo += 2;
-	 if (foo < end) {
-	   adjust_len += 2;
-	 } else {
-	   adjust_len += end - (foo - 1);
-	 }
+   size_shift = Pike_sp[-args].u.string->size_shift;
+
+   /* Count '%' and wide characters.
+    *
+    * trunc counts the number of characters that are to be removed.
+    */
+   for (; COMPARE_PCHARP(foo, <, end);) {
+     p_wchar2 c = EXTRACT_PCHARP(foo);
+     INC_PCHARP(foo, 1);
+     if (c != '%') continue;
+     proc++;
+     /* there are at least 2 more characters */
+     if (SUBTRACT_PCHARP(end, foo) < 2) {
+       trunc += SUBTRACT_PCHARP(end, foo);
+       break;
+     }
+     c = EXTRACT_PCHARP(foo);
+     if (c == 'u' || c == 'U') {
+       if (SUBTRACT_PCHARP(end, foo) < 5) {
+	 trunc += SUBTRACT_PCHARP(end, foo);
+         break;
        }
+       /* %uXXXX */
+       if (EXTRACT_PCHARP(foo) != '0' || INDEX_PCHARP(foo, 1) != '0') {
+         if (!size_shift) size_shift = 1;
+       }
+       trunc += 5;
+       INC_PCHARP(foo, 5);
+     } else {
+       trunc += 2;
+       INC_PCHARP(foo, 2);
      }
    }
 
    if (!proc) { pop_n_elems(args-1); return; }
 
-   newstr = begin_wide_shared_string(Pike_sp[-args].u.string->len - adjust_len,
-				     size_shift);
-   if (size_shift) {
-     p_wchar1 *dest = STR1(newstr);
+   init_string_builder_alloc(&newstr, Pike_sp[-args].u.string->len - trunc,
+			     size_shift);
 
-     for (proc=0; bar<end; dest++)
-       if (*bar=='%') {
-	 if (bar[1] == 'u' || bar[1] == 'U') {
-	   if (bar<end-5)
-	     *dest = (((bar[2]<'A')?(bar[2]&15):((bar[2]+9)&15))<<12)|
-	       (((bar[3]<'A')?(bar[3]&15):((bar[3]+9)&15))<<8)|
-	       (((bar[4]<'A')?(bar[4]&15):((bar[4]+9)&15))<<4)|
-	       ((bar[5]<'A')?(bar[5]&15):((bar[5]+9)&15));
-	   else
-	     *dest=0;
-	   bar+=6;
-	 } else {
-	   if (bar<end-2)
-	     *dest=(((bar[1]<'A')?(bar[1]&15):((bar[1]+9)&15))<<4)|
-	       ((bar[2]<'A')?(bar[2]&15):((bar[2]+9)&15));
-	   else
-	     *dest=0;
-	   bar+=3;
+   foo = MKPCHARP_STR(Pike_sp[-args].u.string);
+
+   for (; COMPARE_PCHARP(foo, <, end); INC_PCHARP(foo, 1)) {
+     p_wchar2 c = EXTRACT_PCHARP(foo);
+     if (c == '%') {
+       c = INDEX_PCHARP(foo, 1);
+       if (c == 'u' || c == 'U') {
+	 c = 0;
+	 if (SUBTRACT_PCHARP(end, foo) > 5) {
+	   p_wchar2 hex = INDEX_PCHARP(foo, 2);
+	   c = (((hex<'A')?hex:(hex + 9)) & 15)<<12;
+	   hex = INDEX_PCHARP(foo, 3);
+	   c |= (((hex<'A')?hex:(hex + 9)) & 15)<<8;
+	   hex = INDEX_PCHARP(foo, 4);
+	   c |= (((hex<'A')?hex:(hex + 9)) & 15)<<4;
+	   hex = INDEX_PCHARP(foo, 5);
+	   c |= ((hex<'A')?hex:(hex + 9)) & 15;
 	 }
+	 INC_PCHARP(foo, 5);
        } else {
-	 *dest=*(bar++);
-       }
-   } else {
-     foo = STR0(newstr);
-     for (proc=0; bar<end; foo++)
-       if (*bar=='%') {
-	 if (bar[1] == 'u' || bar[1] == 'U') {
-	   /* We know that the following two characters are zeros. */
-	   bar+=3;
+	 c = 0;
+	 if (SUBTRACT_PCHARP(end, foo) > 2) {
+	   p_wchar2 hex = INDEX_PCHARP(foo, 1);
+	   c = (((hex<'A')?hex:(hex + 9)) & 15)<<4;
+	   hex = INDEX_PCHARP(foo, 2);
+	   c |= ((hex<'A')?hex:(hex + 9)) & 15;
 	 }
-	 if (bar<end-2)
-	   *foo=(((bar[1]<'A')?(bar[1]&15):((bar[1]+9)&15))<<4)|
-	     ((bar[2]<'A')?(bar[2]&15):((bar[2]+9)&15));
-	 else
-	   *foo=0;
-	 bar+=3;
-       } else {
-	 *foo=*(bar++);
+	 INC_PCHARP(foo, 2);
        }
+     }
+     string_builder_putchar(&newstr, c);
    }
+
    pop_n_elems(args);
-   push_string(end_shared_string(newstr));
+   push_string(finish_string_builder(&newstr));
 }
 
 static void f_html_encode_string( INT32 args )
@@ -483,39 +487,11 @@ static void f_html_encode_string( INT32 args )
   if( args != 1 )
     Pike_error("Wrong number of arguments to html_encode_string\n" );
   
-  switch( Pike_sp[-1].type )
+  switch( TYPEOF(Pike_sp[-1]) )
   {
     void o_cast_to_string();
 
     case PIKE_T_INT:
-      /* Optimization, this is basically a inlined cast_int_to_string */
-      {
-	char buf[21], *b = buf + (sizeof (buf) - 2);
-	int neg, j=0;
-	unsigned INT_TYPE i = Pike_sp[-1].u.integer;
-	if( Pike_sp[-1].u.integer < 0 )
-	{
-	  neg = 1;
-	  i = -i; /* Always defined for unsigned - signed overflows are not. */
-	}
-	else
-	  neg = 0;
-	pop_stack();
-
-	buf[sizeof (buf) - 1] = 0;
-
-	while( i >= 10 )
-	{
-	  b[ -j++ ] = '0'+(i%10);
-	  i /= 10;
-	}
-	b[ -j++ ] = '0'+(i%10);
-	if( neg )  b[ -j++ ] = '-';
-	assert (b - j + 1 >= buf);
-	push_text( b-j+1 );
-      }
-      return;
-
     case PIKE_T_FLOAT:
       /* Optimization, no need to check the resultstring for
        * unsafe characters. 
