@@ -1,7 +1,6 @@
 #include "global.h"
 #include "stralloc.h"
 #include "global.h"
-RCSID("$Id: 773e3fb1cd199102632ced91ef7d28189278f5a9 $");
 #include "interpret.h"
 #include "program.h"
 #include "object.h"
@@ -46,7 +45,7 @@ int wf_blob_next( Blob *b )
     push_int( b->docid );
     push_longest( (LONGEST) b );
     apply_svalue( b->feed, 3 );
-    if( sp[-1].type != T_STRING )
+    if( TYPEOF(sp[-1]) != T_STRING )
     {
       b->eof = 1;
       return -1;
@@ -69,7 +68,7 @@ int wf_blob_next( Blob *b )
       push_int( b->docid );
       push_longest( (LONGEST) b );
       apply_svalue( b->feed, 3 );
-      if( sp[-1].type != T_STRING )
+      if( TYPEOF(sp[-1]) != T_STRING )
       {
 	b->eof = 1;
 	return -1;
@@ -156,8 +155,7 @@ int wf_blob_docid( Blob *b )
 
 Blob *wf_blob_new( struct svalue *feed, struct pike_string *word )
 {
-  Blob *b = malloc( sizeof( Blob ) );
-  MEMSET(b, 0, sizeof(Blob) );
+  Blob *b = calloc( 1, sizeof( Blob ) );
   b->word = word;
   if( word )
     add_ref(word);
@@ -300,7 +298,7 @@ static void f_blob_create( INT32 args )
   if( args )
   {
     struct pike_string *s = sp[-1].u.string;
-    if( sp[-1].type != PIKE_T_STRING )
+    if( TYPEOF(sp[-1]) != PIKE_T_STRING )
       Pike_error("Expected a string\n");
     _append_blob( THIS, s );
   }
@@ -311,7 +309,7 @@ static void f_blob_create( INT32 args )
 
 static void f_blob_merge( INT32 args )
 {
-  if(!args || sp[-1].type != PIKE_T_STRING )
+  if(!args || TYPEOF(sp[-1]) != PIKE_T_STRING )
     Pike_error("Expected a string\n");
   _append_blob( THIS, sp[-1].u.string );
 }
@@ -371,7 +369,7 @@ static void f_blob_remove_list( INT32 args )
     struct hash *h;
     struct hash *p = NULL;
 
-    if (docs->item[i].type != T_INT)
+    if (TYPEOF(docs->item[i]) != T_INT)
       Pike_error("Bad argument 1 to remove_list, expected array(int).\n");
 
     doc_id = docs->item[i].u.integer;
@@ -531,10 +529,10 @@ static void f_blob__cast( INT32 args )
 #ifdef HANDLES_UNALIGNED_READ
       fsort( zipp[i].b->data+5, nh, 2, (void *)cmp_hit );
 #else
-      short *data = (short *)malloc( nh * 2 );
-      MEMCPY( data,  zipp[i].b->data+5, nh * 2 );
+      short *data = malloc( nh * 2 );
+      memcpy( data,  zipp[i].b->data+5, nh * 2 );
       fsort( data, nh, 2, (void *)cmp_hit );
-      MEMCPY( zipp[i].b->data+5, data, nh * 2 );
+      memcpy( zipp[i].b->data+5, data, nh * 2 );
       free( data );
 #endif
     }
@@ -560,21 +558,22 @@ static void f_blob__cast( INT32 args )
 /*! @endmodule
  */
 
-static void init_blob_struct( )
+static void init_blob_struct(struct object * UNUSED(o))
 {
-  MEMSET( THIS, 0, sizeof( struct blob_data ) );
+  memset( THIS, 0, sizeof( struct blob_data ) );
 }
 
-static void exit_blob_struct( )
+static void exit_blob_struct(struct object * o)
 {
   int i;
   for( i = 0; i<HSIZE; i++ )
     if( THIS->hash[i] )
       free_hash( THIS->hash[i] );
-  init_blob_struct();
+  /* why? */
+  init_blob_struct(o);
 }
 
-void init_blob_program()
+void init_blob_program(void)
 {
   start_new_program();
   ADD_STORAGE( struct blob_data );
@@ -592,7 +591,7 @@ void init_blob_program()
   add_program_constant( "Blob", blob_program, 0 );
 }
 
-void exit_blob_program()
+void exit_blob_program(void)
 {
   free_program( blob_program );
 }
