@@ -1,9 +1,4 @@
-
-
-/* $Id: 70e3ec3cf51319ea1d9ec4eaa7417aeffd63b9bd $
- */
 #include "global.h"
-
 #include "config.h"
 
 #ifdef HAVE_SYS_TYPES_H
@@ -17,7 +12,6 @@
 #endif
 
 #include "stralloc.h"
-RCSID("$Id: 70e3ec3cf51319ea1d9ec4eaa7417aeffd63b9bd $");
 #include "interpret.h"
 
 #include "whitefish.h"
@@ -77,7 +71,7 @@ void wf_buffer_wint( struct buffer *b,
 {
   s = htonl(s);
   wf_buffer_make_space( b, 4 );
-  MEMCPY( b->data+b->size, &s, 4 );
+  memcpy( b->data+b->size, &s, 4 );
   b->size += 4;
 }
 
@@ -125,7 +119,7 @@ int wf_buffer_memcpy( struct buffer *d,
   if( nelems <= 0 )
     return 0;
   wf_buffer_make_space( d, nelems );
-  MEMCPY( d->data+d->size, s->data+s->rpos, nelems );
+  memcpy( d->data+d->size, s->data+s->rpos, nelems );
   s->rpos += nelems;
   d->size += nelems;
   return nelems;
@@ -156,7 +150,7 @@ void wf_buffer_seek_w( struct buffer *b, unsigned int pos )
   if( pos > b->size )
   {
     wf_buffer_make_space( b, (unsigned int)(pos-b->size) );
-    MEMSET( b->data+b->size, 0, (unsigned int)(pos-b->size) );
+    memset( b->data+b->size, 0, (unsigned int)(pos-b->size) );
   }
   b->size = pos;
 }
@@ -166,7 +160,7 @@ void wf_buffer_clear( struct buffer *b )
   if( !b->read_only && b->data )
     free( b->data );
   if( b->read_only && b->str )    free_string( b->str );
-  MEMSET(b, 0, sizeof(struct buffer));
+  memset(b, 0, sizeof(struct buffer));
 }
 
 void wf_buffer_free( struct buffer *b )
@@ -201,15 +195,13 @@ void wf_buffer_set_pike_string( struct buffer *b,
     b->size = data->len;
     b->data = malloc( b->size );
     b->allocated_size = b->size;
-    MEMCPY( b->data, data->str, b->size );
+    memcpy( b->data, data->str, b->size );
   }
 }
 
 struct buffer *wf_buffer_new( )
 {
-  struct buffer *b = xalloc( sizeof( struct buffer ) );
-  MEMSET( b, 0, sizeof(struct buffer) );
-  return b;
+  return xcalloc( 1, sizeof( struct buffer ) );
 }
 
 void wf_buffer_append( struct buffer *b,
@@ -217,6 +209,6 @@ void wf_buffer_append( struct buffer *b,
 		       int size )
 {
   wf_buffer_make_space( b, size );
-  MEMCPY( b->data+b->size, data, size );
+  memcpy( b->data+b->size, data, size );
   b->size+=size;
 }
