@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 58e671657f8abc27980fc792c2d364ac8aea97a6 $
 */
 
 /*
@@ -37,11 +36,13 @@
 #include "mapping.h"
 #include "builtin_functions.h"
 #include "constants.h"
+#include "time_stuff.h"
 #include "pike_memory.h"
 #include "pike_security.h"
 #include "bignum.h"
 #include "pike_rusage.h"
 #include "pike_netlib.h"
+#include "pike_cpulib.h"
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -689,6 +690,64 @@ void f_utime(INT32 args)
 }
 #endif
 
+#if !defined(HAVE_SYNC) && defined(__NT__)
+static void sync(void)
+{
+  /* NB: For some stupid reason FindFirstVolume()/FindNextVolume()
+   *     will only list local filesystems. So we have to fall
+   *     back to using GetLogicalDrives().
+   */
+  /* NB: \\.\ is the DOS device namespace prefix. */
+  char drive[8] = "\\\\.\\A:";
+  char *driveletter = drive + 4;
+  DWORD drives = GetLogicalDrives();
+  DWORD mask = 1;
+  /* Loop over all mounted volumes. */
+  while (mask) {
+    if (mask & drives) {
+      char device[MAX_PATH*2];
+      HANDLE volfile;
+
+      if (QueryDosDeviceA(driveletter, device, MAX_PATH*2)) {
+	volfile = CreateFileA(device, FILE_READ_DATA, FILE_SHARE_READ,
+			      NULL, OPEN_EXISTING, 0, NULL);
+	if (volfile != INVALID_HANDLE_VALUE) {
+	  CloseHandle(volfile);
+	}
+      }
+
+      /* Flush the drive. */
+      volfile = CreateFileA(drive, FILE_READ_DATA, FILE_SHARE_READ,
+			    NULL, OPEN_EXISTING, 0, NULL);
+      if (volfile != INVALID_HANDLE_VALUE) {
+	CloseHandle(volfile);
+      }
+    }
+    *driveletter++;
+    mask = mask << 1;
+  }
+}
+#define HAVE_SYNC
+#endif
+
+#ifdef HAVE_SYNC
+/*! @decl void sync()
+ *!
+ *! Flush operating system disk buffers to permanent storage.
+ *!
+ *! @note
+ *!   On some operating systems this may require
+ *!   administrative privileges.
+ */
+void f_sync(INT32 args)
+{
+  pop_n_elems(args);
+  THREADS_ALLOW_UID()
+  sync();
+  THREADS_DISALLOW_UID();
+}
+#endif
+
 #ifdef HAVE_INITGROUPS
 /*! @decl void initgroups(string name, int base_gid)
  *!
@@ -784,14 +843,14 @@ void f_setgroups(INT32 args)
 
   get_all_args("setgroups", args, "%a", &arr);
   if ((size = arr->size)) {
-    gids = (gid_t *)alloca(arr->size * sizeof(gid_t));
+    gids = alloca(arr->size * sizeof(gid_t));
     if (!gids) Pike_error("setgroups(): Too large array (%d).\n", arr->size);
   } else {
     gids = (gid_t *)safeguard;
   }
 
   for (i=0; i < size; i++) {
-    if (arr->item[i].type != T_INT) {
+    if (TYPEOF(arr->item[i]) != T_INT) {
       Pike_error("setgroups(): Bad element %d in array (expected int)\n", i);
     }
     gids[i] = arr->item[i].u.integer;
@@ -833,7 +892,7 @@ void f_getgroups(INT32 args)
     /* OS which doesn't understand this convention */
     numgrps = NGROUPS_MAX;
   }
-  gids = (gid_t *)xalloc(sizeof(gid_t) * numgrps);
+  gids = xalloc(sizeof(gid_t) * numgrps);
 
   numgrps = getgroups(numgrps, gids);
 
@@ -872,7 +931,7 @@ void f_innetgr(INT32 args)
 		 BIT_STRING|BIT_INT|BIT_VOID, BIT_STRING|BIT_INT|BIT_VOID, 0);
 
   for(i = 0; i < args; i++) {
-    if (sp[i-args].type == T_STRING) {
+    if (TYPEOF(sp[i-args]) == T_STRING) {
       if (sp[i-args].u.string->size_shift) {
 	SIMPLE_BAD_ARG_ERROR("innetgr", i+1, "string (8bit)");
       }
@@ -1097,7 +1156,7 @@ void f_getpgrp(INT32 args)
   int pgid = 0;
 
   if (args) {
-    if (sp[-args].type != T_INT) {
+    if (TYPEOF(sp[-args]) != T_INT) {
       SIMPLE_BAD_ARG_ERROR("getpgrp", 1, "int");
     }
     pid = sp[-args].u.integer;
@@ -1161,7 +1220,7 @@ void f_setpgrp(INT32 args)
 void f_getsid(INT32 args)
 {
   int pid = 0;
-  if (args >= 1 && sp[-args].type != T_INT)
+  if (args >= 1 && TYPEOF(sp[-args]) != T_INT)
     SIMPLE_BAD_ARG_ERROR("getsid", 1, "int");
   if (args >= 1)
        pid = sp[-args].u.integer;
@@ -1374,6 +1433,13 @@ f_get(f_getppid, getppid)
  *!
  *!   The second variant only works on systems that also have
  *!   the fchroot(2) system call.
+ *!
+ *! @note
+ *!   On success the current working directory will be changed to
+ *!   the new @expr{"/"@}. This behavior was added in Pike 7.9.
+ *!
+ *! @note
+ *!   This function could be interrupted by signals prior to Pike 7.9.
  */
 void f_chroot(INT32 args)
 {
@@ -1392,29 +1458,36 @@ void f_chroot(INT32 args)
 
 
 #ifdef HAVE_FCHROOT
-  if(sp[-args].type == T_STRING)
+  if(TYPEOF(sp[-args]) == T_STRING)
   {
 #endif /* HAVE_FCHROOT */
-    res = chroot((char *)sp[-args].u.string->str);
+    while (((res = chroot((char *)sp[-args].u.string->str)) == -1) &&
+	   (errno == EINTR))
+      ;
+    if (!res) {
+      while ((chdir("/") == -1) && (errno == EINTR))
+	;
+    }
     pop_n_elems(args);
     push_int(!res);
     return;
 #ifdef HAVE_FCHROOT
-  } else
-#if 0 
-    if(sp[-args].type == T_OBJECT)
-#endif /* 0 */
-      {
-	int fd;
+  } else {
+    int fd;
 
-	apply(sp[-args].u.object, "query_fd", 0);
-	fd=sp[-1].u.integer;
-	pop_stack();
-	res=fchroot(fd);
-	pop_n_elems(args);
-	push_int(!res);
-	return;
-      }
+    apply(sp[-args].u.object, "query_fd", 0);
+    fd=sp[-1].u.integer;
+    pop_stack();
+    while (((res = fchroot(fd)) == -1) && (errno == EINTR))
+      ;
+    if (!res) {
+      while ((fchdir(fd) == -1) && (errno == EINTR))
+	;
+    }
+    pop_n_elems(args);
+    push_int(!res);
+    return;
+  }
 #endif /* HAVE_FCHROOT */
 }
 #endif /* HAVE_CHROOT */
@@ -1770,7 +1843,7 @@ int my_isipv6nr(char *s)
 
 #define CALL_GETHOSTBYNAME(X) \
     THREADS_ALLOW(); \
-    MEMSET((char *)&data,0,sizeof(data)); \
+    memset(&data,0,sizeof(data)); \
     if(gethostbyname_r((X), &result, &data) < 0) { \
       ret=0; \
     }else{ \
@@ -1780,7 +1853,7 @@ int my_isipv6nr(char *s)
 
 #define CALL_GETHOSTBYADDR(X,Y,Z) \
     THREADS_ALLOW(); \
-    MEMSET((char *)&data,0,sizeof(data)); \
+    memset(&data,0,sizeof(data)); \
     if(gethostbyaddr_r((X),(Y),(Z), &result, &data) < 0) { \
       ret=0; \
     }else{ \
@@ -1813,7 +1886,7 @@ int my_isipv6nr(char *s)
 
 #define CALL_GETSERVBYNAME(X,Y) \
     THREADS_ALLOW(); \
-    MEMSET((char *)&data,0,sizeof(data)); \
+    memset(&data,0,sizeof(data)); \
     if(getservbyname_r((X), (Y), &result, &data) < 0) { \
       ret=0; \
     }else{ \
@@ -1841,18 +1914,18 @@ int my_isipv6nr(char *s)
 #endif
 #endif /* !GETSERV_DECLARE */
 
-/* this is used from modules/file, and modules/spider! */
+/* this is also used from modules/_Stdio */
 int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port,
-		  int inet_flags)
+                  int inet_flags)
 {
 #ifdef HAVE_GETADDRINFO
   struct addrinfo hints = { 0, PF_UNSPEC, 0, 0, 0, NULL, NULL, NULL, }, *res;
   char servnum_buf[200];
 #endif /* HAVE_GETADDRINFO */
-
+  int err;
   int udp = inet_flags & 1;
 
-  MEMSET((char *)addr,0,sizeof(PIKE_SOCKADDR));
+  memset(addr,0,sizeof(PIKE_SOCKADDR));
   if(name && !strcmp(name,"*"))
     name = NULL;
 
@@ -1877,24 +1950,44 @@ int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port,
 #ifdef AI_V4MAPPED
     hints.ai_flags = AI_V4MAPPED;
 #endif
+  }
+#endif
+#ifdef AI_NUMERICHOST
+  hints.ai_flags |= AI_NUMERICHOST;
+#endif
+  hints.ai_protocol = (udp? IPPROTO_UDP:IPPROTO_TCP);
+  if (!service && (port > 0) && (port & 0xffff)) {
+    /* NB: MacOS X doesn't like the combination of AI_NUMERICSERV and port #0.
+     *     cf [bug 7599] and https://bugs.python.org/issue17269 .
+     */
+#ifdef AI_NUMERICSERV
+    hints.ai_flags |= AI_NUMERICSERV;
+#endif
+    sprintf(service = servnum_buf, "%"PRINTPIKEINT"d", port & 0xffff);
+  }
+
+  if( (err=getaddrinfo(name, service, &hints, &res)) )
+  {
+    /* Try again without AI_NUMERICHOST. */
+#ifdef AI_NUMERICHOST
+    hints.ai_flags &= ~AI_NUMERICHOST;
+    err=getaddrinfo(name, service, &hints, &res);
 #endif
   }
-  hints.ai_protocol = (udp? IPPROTO_UDP:IPPROTO_TCP);
-  if(!service)
-    sprintf(servnum_buf, "%"PRINTPIKEINT"d", (port<0? 0:port));
-  if(!getaddrinfo(name, (service? service : servnum_buf), &hints, &res)) {
+  if(!err)
+  {
     struct addrinfo *p, *found = NULL;
     size_t addr_len=0;
     for(p=res; p; p=p->ai_next) {
       if(p->ai_addrlen > addr_len &&
-	 p->ai_addrlen <= sizeof(*addr) &&
-	 p->ai_addr)
-	addr_len = (found = p)->ai_addrlen;
+         p->ai_addrlen <= sizeof(*addr) &&
+         p->ai_addr)
+        addr_len = (found = p)->ai_addrlen;
     }
     if(found) {
 /*       fprintf(stderr, "Got %d bytes (family: %d (%d))\n", */
 /* 	      addr_len, found->ai_addr->sa_family, found->ai_family); */
-      MEMCPY((char *)addr, (char *)found->ai_addr, addr_len);
+      memcpy(addr, found->ai_addr, addr_len);
     }
     freeaddrinfo(res);
     if(addr_len) {
@@ -1909,7 +2002,7 @@ int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port,
   if (inet_flags & 2) {
     SOCKADDR_FAMILY(*addr) = AF_INET6;
     /* Note: This is equvivalent to :: (aka IPv6 ANY). */
-    MEMSET(&addr->ipv6.sin6_addr, 0, sizeof(addr->ipv6.sin6_addr));
+    memset(&addr->ipv6.sin6_addr, 0, sizeof(addr->ipv6.sin6_addr));
   }
 #endif
 
@@ -1936,7 +2029,7 @@ int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port,
       ipv4.sin_addr.s_addr = inet_addr(name);
       addr->ipv6.sin6_addr.s6_addr[10] = 0xff;
       addr->ipv6.sin6_addr.s6_addr[11] = 0xff;
-      MEMCPY(addr->ipv6.sin6_addr.s6_addr + 12, &ipv4.sin_addr.s_addr, 4);
+      memcpy(addr->ipv6.sin6_addr.s6_addr + 12, &ipv4.sin_addr.s_addr, 4);
     } else
 #endif
       addr->ipv4.sin_addr.s_addr = inet_addr(name);
@@ -1950,22 +2043,22 @@ int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port,
 
     if(!ret) {
       if (strlen(name) < 1024) {
-	Pike_error("Invalid address '%s'\n",name);
+        Pike_error("Invalid address '%s'\n",name);
       } else {
-	Pike_error("Invalid address\n");
+        Pike_error("Invalid address\n");
       }
     }
 
     SOCKADDR_FAMILY(*addr) = ret->h_addrtype;
 
 #ifdef HAVE_H_ADDR_LIST
-    MEMCPY((char *)SOCKADDR_IN_ADDR(*addr),
-	   (char *)ret->h_addr_list[0],
-	   ret->h_length);
+    memcpy(SOCKADDR_IN_ADDR(*addr),
+           ret->h_addr_list[0],
+           ret->h_length);
 #else
-    MEMCPY((char *)SOCKADDR_IN_ADDR(*addr),
-	   (char *)ret->h_addr,
-	   ret->h_length);
+    memcpy(SOCKADDR_IN_ADDR(*addr),
+           ret->h_addr,
+           ret->h_length);
 #endif
 #else
     if (strlen(name) < 1024) {
@@ -1984,9 +2077,9 @@ int get_inet_addr(PIKE_SOCKADDR *addr,char *name,char *service, INT_TYPE port,
 
     if(!ret) {
       if (strlen(service) < 1024) {
-	Pike_error("Invalid service '%s'\n",service);
+        Pike_error("Invalid service '%s'\n",service);
       } else {
-	Pike_error("Invalid service\n");
+        Pike_error("Invalid service\n");
       }
     }
 
@@ -2044,7 +2137,7 @@ static void describe_hostent(struct hostent *hp)
 #else
       struct in_addr in;
  
-      MEMCPY(&in.s_addr, *p, sizeof (in.s_addr));
+      memcpy(&in.s_addr, *p, sizeof (in.s_addr));
       push_text(inet_ntoa(in));
 #endif
       nelem++;
@@ -2067,7 +2160,7 @@ static void describe_hostent(struct hostent *hp)
     push_text(fd_inet_ntop(hp->h_addrtype, hp->h_addr, buffer, sizeof(buffer)));
 #else
     struct in_addr in;
-    MEMCPY(&in.s_addr, hp->h_addr, sizeof (in.s_addr));
+    memcpy(&in.s_addr, hp->h_addr, sizeof (in.s_addr));
     push_text(inet_ntoa(in));
 #endif
   }
@@ -2108,6 +2201,7 @@ void f_gethostbyaddr(INT32 args)
   pop_n_elems(args);
 
   CALL_GETHOSTBYADDR((char *)&addr, sizeof (addr), AF_INET);
+  INVALIDATE_CURRENT_TIME();
 
   if(!ret) {
     push_int(0);
@@ -2147,6 +2241,7 @@ void f_gethostbyname(INT32 args)
   get_all_args("gethostbyname", args, "%s", &name);
 
   CALL_GETHOSTBYNAME(name);
+  INVALIDATE_CURRENT_TIME();
  
   pop_n_elems(args);
   
@@ -2308,36 +2403,80 @@ static void f_system_nanosleep(INT32 args)
 #define PIKE_RLIM_T rlim_t
 #endif
 
-#if defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)
-static struct pike_string *s_cpu=NULL;
-static struct pike_string *s_fsize=NULL;
-static struct pike_string *s_data=NULL;
-static struct pike_string *s_stack=NULL;
-static struct pike_string *s_core=NULL;
-static struct pike_string *s_rss=NULL;
-static struct pike_string *s_nproc=NULL;
-static struct pike_string *s_nofile=NULL;
-static struct pike_string *s_memlock=NULL;
-static struct pike_string *s_as=NULL;
-static struct pike_string *s_vmem=NULL;
-
-static void make_rlimit_strings(void)
-{
-   MAKE_CONSTANT_SHARED_STRING(s_cpu,"cpu");
-   MAKE_CONSTANT_SHARED_STRING(s_fsize,"fsize");
-   MAKE_CONSTANT_SHARED_STRING(s_data,"data");
-   MAKE_CONSTANT_SHARED_STRING(s_stack,"stack");
-   MAKE_CONSTANT_SHARED_STRING(s_core,"core");
-   MAKE_CONSTANT_SHARED_STRING(s_rss,"rss");
-   MAKE_CONSTANT_SHARED_STRING(s_nproc,"nproc");
-   MAKE_CONSTANT_SHARED_STRING(s_nofile,"nofile");
-   MAKE_CONSTANT_SHARED_STRING(s_memlock,"memlock");
-   MAKE_CONSTANT_SHARED_STRING(s_as,"as");
-   MAKE_CONSTANT_SHARED_STRING(s_vmem,"vmem");
-}
-#endif /* HAVE_GETRLIMIT || HAVE_SETRLIMIT */
-
 #ifdef HAVE_GETRLIMIT
+static const struct {
+    const char *name;
+    const int lim;
+} __limits[] = {
+#ifdef RLIMIT_CORE
+    {"core", RLIMIT_CORE},
+#endif
+#ifdef RLIMIT_CPU
+    {"cpu", RLIMIT_CPU},
+#endif
+#ifdef RLIMIT_DATA
+    {"data", RLIMIT_DATA},
+#endif
+#ifdef RLIMIT_FSIZE
+    {"fsize", RLIMIT_FSIZE},
+#endif
+#ifdef RLIMIT_NOFILE
+    {"nofile", RLIMIT_NOFILE},
+#endif
+#ifdef RLIMIT_MEMLOCK
+    {"memlock", RLIMIT_MEMLOCK},
+#endif
+#ifdef RLIMIT_MSGQUEUE
+    {"msgqueue", RLIMIT_MSGQUEUE},
+#endif
+#ifdef RLIMIT_NICE
+    {"nice", RLIMIT_NICE},
+#endif
+#ifdef RLIMIT_RTPRIO
+    {"rtprio", RLIMIT_RTPRIO},
+#endif
+#ifdef RLIMIT_RTTIME
+    {"rttime", RLIMIT_RTTIME},
+#endif
+#ifdef RLIMIT_NPROC
+    {"nproc", RLIMIT_NPROC},
+#endif
+#ifdef RLIMIT_RSS
+    {"rss", RLIMIT_RSS},
+#endif
+#ifdef RLIMIT_SIGPENDING
+    {"sigpending", RLIMIT_SIGPENDING},
+#endif
+#ifdef RLIMIT_STACK
+    {"stack", RLIMIT_STACK},
+#endif
+#ifdef RLIMIT_VMEM
+    {"map_mem", RLIMIT_VMEM},
+    {"vmem", RLIMIT_VMEM},
+#endif
+#ifdef RLIMIT_AS
+    {"as", RLIMIT_AS},
+    {"mem", RLIMIT_AS},
+#endif
+};
+
+int get_limit_id( const char *limit )
+{
+   unsigned int i;
+   for( i=0; i<sizeof(__limits)/sizeof(__limits[0]); i++ )
+     if( !strcmp( limit, __limits[i].name ) )
+       return __limits[i].lim;
+   return -1;
+}
+
+int get_one_limit( const char *limit, struct rlimit *rl )
+{
+   int lim = get_limit_id( limit );
+   if( lim < 0 )
+       return -2;
+   return getrlimit( lim, rl );
+}
+
 /*! @decl array(int) getrlimit(string resource)
  *! Returns the current process limitation for the selected @[resource].
  *!
@@ -2388,101 +2527,38 @@ static void f_getrlimit(INT32 args)
 {
    struct rlimit rl;
    int res=-1;
-   if (!s_cpu) make_rlimit_strings();
    if (args<1)
       SIMPLE_TOO_FEW_ARGS_ERROR("getrlimit",1);
-   if (sp[-args].type!=T_STRING) 
+   if (TYPEOF(sp[-args]) != T_STRING)
       SIMPLE_BAD_ARG_ERROR("getrlimit",1,"string");
-
-#ifdef RLIMIT_CPU
-   if (sp[-args].u.string==s_cpu)
-      res=getrlimit(RLIMIT_CPU,&rl);
-#endif
-#ifdef RLIMIT_FSIZE
-   else if (sp[-args].u.string==s_fsize)
-      res=getrlimit(RLIMIT_FSIZE,&rl);
-#endif
-#ifdef RLIMIT_DATA
-   else if (sp[-args].u.string==s_data)
-      res=getrlimit(RLIMIT_DATA,&rl);
-#endif
-#ifdef RLIMIT_STACK
-   else if (sp[-args].u.string==s_stack)
-      res=getrlimit(RLIMIT_STACK,&rl);
-#endif
-#ifdef RLIMIT_CORE
-   else if (sp[-args].u.string==s_core)
-      res=getrlimit(RLIMIT_CORE,&rl);
-#endif
-#ifdef RLIMIT_RSS
-   else if (sp[-args].u.string==s_rss)
-      res=getrlimit(RLIMIT_RSS,&rl);
-#endif
-#ifdef RLIMIT_NPROC
-   else if (sp[-args].u.string==s_nproc)
-      res=getrlimit(RLIMIT_NPROC,&rl);
-#endif
-#ifdef RLIMIT_NOFILE
-   else if (sp[-args].u.string==s_nofile)
-      res=getrlimit(RLIMIT_NOFILE,&rl);
-#endif
-#ifdef RLIMIT_MEMLOCK
-   else if (sp[-args].u.string==s_memlock)
-      res=getrlimit(RLIMIT_MEMLOCK,&rl);
-#endif
-#ifdef RLIMIT_AS
-   else if (sp[-args].u.string==s_as)
-      res=getrlimit(RLIMIT_AS,&rl);
-#endif
-#ifdef RLIMIT_VMEM
-   else if (sp[-args].u.string==s_vmem)
-      res=getrlimit(RLIMIT_VMEM,&rl);
-#endif
-/* NOFILE is called OFILE on some systems */
-#ifdef RLIMIT_OFILE
-   else if (sp[-args].u.string==s_nofile)
-      res=getrlimit(RLIMIT_OFILE,&rl);
-#endif
-   else if (sp[-args].u.string==s_cpu      ||
-	    sp[-args].u.string==s_fsize    ||
-	    sp[-args].u.string==s_data     ||
-	    sp[-args].u.string==s_stack    ||
-	    sp[-args].u.string==s_core     ||
-	    sp[-args].u.string==s_rss      ||
-	    sp[-args].u.string==s_nproc    ||
-	    sp[-args].u.string==s_nofile   ||
-	    sp[-args].u.string==s_memlock  ||
-	    sp[-args].u.string==s_vmem     ||
-	    sp[-args].u.string==s_as)
+   res = get_one_limit( Pike_sp[-args].u.string->str, &rl );
+   if( res == -2 )
    {
 /* no such resource on this system */
       rl.rlim_cur=(PIKE_RLIM_T)0;
       rl.rlim_max=(PIKE_RLIM_T)0;
       res=0;
    }
-   else
-      Pike_error("getrlimit: no such resource\n");
 
    if (res==-1)
    {
 /* this shouldn't happen */
       Pike_error("getrlimit: error; errno=%d\n",errno);
    }
-
    pop_n_elems(args);
 #ifdef RLIM_INFINITY
    if (rl.rlim_cur==RLIM_INFINITY)
       push_int(-1);
    else
 #endif
-      push_int( (INT_TYPE)rl.rlim_cur );
+      push_int64( (INT_TYPE)rl.rlim_cur );
 
 #ifdef RLIM_INFINITY
    if (rl.rlim_max==RLIM_INFINITY)
       push_int(-1);
    else
 #endif
-      push_int( (INT_TYPE)rl.rlim_max );
+      push_int64( (INT_TYPE)rl.rlim_max );
 
    f_aggregate(2);
 }
@@ -2495,104 +2571,45 @@ static void f_getrlimit(INT32 args)
  */
 static void f_getrlimits(INT32 args)
 {
+   unsigned int i;
    int n=0;
    pop_n_elems(args); /* no args */
-
-   if (!s_cpu) make_rlimit_strings();
-
-#ifdef RLIMIT_CPU
-   ref_push_string(s_cpu);
-   ref_push_string(s_cpu);
-   f_getrlimit(1);
-   n+=2;
-#endif
-
-#ifdef RLIMIT_FSIZE
-   ref_push_string(s_fsize);
-   ref_push_string(s_fsize);
-   f_getrlimit(1);
-   n+=2;
-#endif
-
-#ifdef RLIMIT_DATA
-   ref_push_string(s_data);
-   ref_push_string(s_data);
-   f_getrlimit(1);
-   n+=2;
-#endif
-
-#ifdef RLIMIT_STACK
-   ref_push_string(s_stack);
-   ref_push_string(s_stack);
-   f_getrlimit(1);
-   n+=2;
-#endif
-
-#ifdef RLIMIT_CORE
-   ref_push_string(s_core);
-   ref_push_string(s_core);
-   f_getrlimit(1);
-   n+=2;
-#endif
-
-#ifdef RLIMIT_RSS
-   ref_push_string(s_rss);
-   ref_push_string(s_rss);
-   f_getrlimit(1);
-   n+=2;
-#endif
-
-#ifdef RLIMIT_NPROC
-   ref_push_string(s_nproc);
-   ref_push_string(s_nproc);
-   f_getrlimit(1);
-   n+=2;
-#endif
-
-#ifdef RLIMIT_NOFILE 
-   ref_push_string(s_nofile);
-   ref_push_string(s_nofile);
-   f_getrlimit(1);
-   n+=2;
-#else
-#ifdef RLIMIT_OFILE 
-   ref_push_string(s_nofile);
-   ref_push_string(s_nofile);
-   f_getrlimit(1);
-   n+=2;
-#endif
-#endif
-
-#ifdef RLIMIT_MEMLOCK
-   ref_push_string(s_memlock);
-   ref_push_string(s_memlock);
-   f_getrlimit(1);
-   n+=2;
-#endif
-
-#ifdef RLIMIT_AS
-   ref_push_string(s_as);
-   ref_push_string(s_as);
-   f_getrlimit(1);
-   n+=2;
-#endif
-   
-#ifdef RLIMIT_VMEM
-#if RLIMIT_VMEM != RLIMIT_AS
-/* they are alias on some systems, OSF1 for one */
-   ref_push_string(s_vmem);
-   ref_push_string(s_vmem);
-   f_getrlimit(1);
-   n+=2;
-#endif
-#endif
-   
+   for( i=0; i<sizeof(__limits)/sizeof(__limits[0]); i++ )
+   {
+       push_text( __limits[i].name );
+       push_text( __limits[i].name );
+       f_getrlimit(1);
+       n += 2;
+   }
    f_aggregate_mapping(n);
 }
 
 #endif
 
 #ifdef HAVE_SETRLIMIT
+int set_one_limit( const char *limit, INT64 soft, INT64 hard )
+{
+   struct rlimit rl;
+   int lim;
+#ifdef RLIM_INFINITY
+   if( soft == -1 )
+       rl.rlim_cur = RLIM_INFINITY;
+   else
+       rl.rlim_cur = soft;
+   if( hard == -1 )
+       rl.rlim_max = RLIM_INFINITY;
+   else
+       rl.rlim_max = hard;
+#else
+   rl.rlim_cur = soft;
+   rl.rlim_max = hard;
+#endif
+   lim = get_limit_id( limit );
+   if( lim < 0 )
+       return -2;
+   return setrlimit( lim, &rl );
+}
+
 /*! @decl int(0..1) setrlimit(string resource, int soft, int hard)
  *! Sets the @[soft] and the @[hard] process limit on a @[resource].
  *! @seealso
@@ -2600,105 +2617,32 @@ static void f_getrlimits(INT32 args)
  */
 static void f_setrlimit(INT32 args)
 {
-   struct rlimit rl;
-   int res=-1;
-   if (!s_cpu) make_rlimit_strings();
    if (args<3)
       SIMPLE_TOO_FEW_ARGS_ERROR("setrlimit",3);
-   if (sp[-args].type!=T_STRING) 
+   if (TYPEOF(sp[-args]) != T_STRING)
       SIMPLE_BAD_ARG_ERROR("setrlimit",1,"string");
-   if (sp[1-args].type!=T_INT ||
-       sp[1-args].u.integer<-1) 
+   if (TYPEOF(sp[1-args]) != T_INT ||
+       sp[1-args].u.integer<-1)
       SIMPLE_BAD_ARG_ERROR("setrlimit",2,"int(-1..)");
-   if (sp[2-args].type!=T_INT ||
-       sp[2-args].u.integer<-1) 
+   if (TYPEOF(sp[2-args]) != T_INT ||
+       sp[2-args].u.integer<-1)
       SIMPLE_BAD_ARG_ERROR("setrlimit",3,"int(-1..)");
 
-#ifdef RLIM_INFINITY
-   if (sp[1-args].u.integer==-1)
-      rl.rlim_cur=RLIM_INFINITY;
-   else
-#endif
-      rl.rlim_cur=(PIKE_RLIM_T)sp[1-args].u.integer;
-
-#ifdef RLIM_INFINITY
-   if (sp[2-args].u.integer==-1)
-      rl.rlim_max=RLIM_INFINITY;
-   else
-#endif
-      rl.rlim_max=(PIKE_RLIM_T)sp[2-args].u.integer;
-
-
-#ifdef RLIMIT_CPU
-   if (sp[-args].u.string==s_cpu)
-      res=setrlimit(RLIMIT_CPU,&rl);
-#endif
-#ifdef RLIMIT_FSIZE
-   else if (sp[-args].u.string==s_fsize)
-      res=setrlimit(RLIMIT_FSIZE,&rl);
-#endif
-#ifdef RLIMIT_DATA
-   else if (sp[-args].u.string==s_data)
-      res=setrlimit(RLIMIT_DATA,&rl);
-#endif
-#ifdef RLIMIT_STACK
-   else if (sp[-args].u.string==s_stack)
-      res=setrlimit(RLIMIT_STACK,&rl);
-#endif
-#ifdef RLIMIT_CORE
-   else if (sp[-args].u.string==s_core)
-      res=setrlimit(RLIMIT_CORE,&rl);
-#endif
-#ifdef RLIMIT_RSS
-   else if (sp[-args].u.string==s_rss)
-      res=setrlimit(RLIMIT_RSS,&rl);
-#endif
-#ifdef RLIMIT_NPROC
-   else if (sp[-args].u.string==s_nproc)
-      res=setrlimit(RLIMIT_NPROC,&rl);
-#endif
-#ifdef RLIMIT_NOFILE
-   else if (sp[-args].u.string==s_nofile)
-      res=setrlimit(RLIMIT_NOFILE,&rl);
-#endif
-#ifdef RLIMIT_MEMLOCK
-   else if (sp[-args].u.string==s_memlock)
-      res=setrlimit(RLIMIT_MEMLOCK,&rl);
-#endif
-#ifdef RLIMIT_AS
-   else if (sp[-args].u.string==s_as)
-      res=setrlimit(RLIMIT_AS,&rl);
-#endif
-#ifdef RLIMIT_VMEM
-   else if (sp[-args].u.string==s_vmem)
-      res=setrlimit(RLIMIT_VMEM,&rl);
-#endif
-/* NOFILE is called OFILE on some systems */
-#ifdef RLIMIT_OFILE
-   else if (sp[-args].u.string==s_nofile)
-      res=setrlimit(RLIMIT_OFILE,&rl);
-#endif
-   else if (sp[-args].u.string==s_cpu      ||
-	    sp[-args].u.string==s_fsize    ||
-	    sp[-args].u.string==s_data     ||
-	    sp[-args].u.string==s_stack    ||
-	    sp[-args].u.string==s_core     ||
-	    sp[-args].u.string==s_rss      ||
-	    sp[-args].u.string==s_nproc    ||
-	    sp[-args].u.string==s_nofile   ||
-	    sp[-args].u.string==s_memlock  ||
-	    sp[-args].u.string==s_vmem     ||
-	    sp[-args].u.string==s_as)
-      Pike_error("setrlimit: no %s resource on this system\n",
-		 sp[-args].u.string->str);
-   else
-      Pike_error("setrlimit: no such resource\n");
-
-   pop_n_elems(args);
-   if (res==-1)
-      push_int(0);
-   else
-      push_int(1);
+   switch( set_one_limit( Pike_sp[-args].u.string->str, Pike_sp[1-args].u.integer, Pike_sp[2-args].u.integer ) )
+   {
+    case -2:
+        Pike_error("setrlimit: no %s resource on this system\n",
+        sp[-args].u.string->str);
+        break;
+    case -1:
+        pop_n_elems(args);
+        push_int(0);
+        break;
+    default:
+        pop_n_elems(args);
+        push_int(1);
+        break;
+   }
 }
 #endif
 
@@ -2937,30 +2881,19 @@ void f_system_getloadavg(INT32 args)
 }
 #endif // HAVE_GETLOADAVG
 
-#ifdef HAVE_RDTSC
+#ifdef RDTSC
 
 /*! @decl int rdtsc()
  *! Executes the rdtsc (clock pulse counter) instruction
  *! and returns the result.
  */
 
-#define RDTSC(l,h)							\
-   __asm__ __volatile__ (  "rdtsc"					\
-			   :"=a" (l),					\
-			   "=d" (h))
-
-static INLINE long long rdtsc(void)
-{
-   long long now;
-   unsigned long nl,nh;
-   RDTSC(nl,nh);
-   return (((long long)nh)<<32)|nl;
-}
-
 static void f_rdtsc(INT32 args)
 {
+   INT64 tsc;
    pop_n_elems(args);
-   push_int64(rdtsc());
+   RDTSC (tsc);
+   push_int64(tsc);
 }
 
 #endif
@@ -3207,6 +3140,18 @@ static void f_getrusage(INT32 args)
    f_aggregate_mapping(n*2);
 }
 
+#ifdef HAVE_DAEMON
+/*! @decl int daemon(int nochdir, int noclose)
+ *! Low level system daemon() function, see also @[Process.daemon()]
+ */
+void f_daemon(INT32 args)
+{
+   INT_TYPE a, b;
+   get_all_args("daemon", args, "%i%i", &a, &b);
+   push_int( daemon( a, b) );
+}
+#endif /* HAVE_DAEMON */
+
 /*! @endmodule
  */
 
@@ -3269,6 +3214,10 @@ PIKE_MODULE_INIT
 		0, OPT_SIDE_EFFECT);
 #endif
 
+#ifdef HAVE_SYNC
+  ADD_FUNCTION2("sync", f_sync, tFunc(tNone, tVoid), 0, OPT_SIDE_EFFECT);
+#endif
+
 #ifdef HAVE_INITGROUPS
   
 /* function(string, int:void) */
@@ -3517,7 +3466,7 @@ PIKE_MODULE_INIT
 #endif
 #endif
 
-#ifdef HAVE_RDTSC
+#ifdef RDTSC
   ADD_FUNCTION("rdtsc",f_rdtsc,
 	       tFunc(tNone,tInt),0);
 #endif
@@ -3555,6 +3504,10 @@ PIKE_MODULE_INIT
 				      cleanup_after_fork, 0, 0));
 #endif
 
+#ifdef HAVE_DAEMON
+  ADD_FUNCTION2("daemon", f_daemon, tFunc(tInt tInt, tInt),
+                0, OPT_SIDE_EFFECT | OPT_EXTERNAL_DEPEND);
+#endif /* HAVE_DAEMON */
 
 #ifdef __NT__
   {
@@ -3578,50 +3531,4 @@ PIKE_MODULE_EXIT
 #ifdef GETHOSTBYNAME_MUTEX_EXISTS
   mt_destroy(&gethostbyname_mutex);
 #endif
-#if defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)
-  if (s_cpu) {
-    free_string(s_cpu);
-    s_cpu=NULL;
-  }
-  if (s_fsize) {
-    free_string(s_fsize);
-    s_fsize=NULL;
-  }
-  if (s_data) {
-    free_string(s_data);
-    s_data=NULL;
-  }
-  if (s_stack) {
-    free_string(s_stack);
-    s_stack=NULL;
-  }
-  if (s_core) {
-    free_string(s_core);
-    s_core=NULL;
-  }
-  if (s_rss) {
-    free_string(s_rss);
-    s_rss=NULL;
-  }
-  if (s_nproc) {
-    free_string(s_nproc);
-    s_nproc=NULL;
-  }
-  if (s_nofile) {
-    free_string(s_nofile);
-    s_nofile=NULL;
-  }
-  if (s_memlock) {
-    free_string(s_memlock);
-    s_memlock=NULL;
-  }
-  if (s_as) {
-    free_string(s_as);
-    s_as=NULL;
-  }
-  if (s_vmem) {
-    free_string(s_vmem);
-    s_vmem=NULL;
-  }
-#endif /* HAVE_GETRLIMIT || HAVE_SETRLIMIT */
 }
