@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 0dd47b2fed1334a56d2e908cf84ca6e743423ab3 $
 */
 
 #include "global.h"
@@ -98,7 +97,7 @@ ptrdiff_t insert_opcode(p_instr *opcode)
 ptrdiff_t insert_opcode2(unsigned int f,
 			 INT32 b,
 			 INT32 c,
-			 INT32 current_line,
+			 INT_TYPE current_line,
 			 struct pike_string *current_file)
 {
   p_instr *p;
@@ -129,7 +128,7 @@ ptrdiff_t insert_opcode2(unsigned int f,
 
 ptrdiff_t insert_opcode1(unsigned int f,
 			 INT32 b,
-			 INT32 current_line,
+			 INT_TYPE current_line,
 			 struct pike_string *current_file)
 {
 #ifdef PIKE_DEBUG
@@ -140,7 +139,8 @@ ptrdiff_t insert_opcode1(unsigned int f,
   return insert_opcode2(f,b,0,current_line,current_file);
 }
 
-ptrdiff_t insert_opcode0(int f,int current_line, struct pike_string *current_file)
+ptrdiff_t insert_opcode0(int f, INT_TYPE current_line,
+			 struct pike_string *current_file)
 {
 #ifdef PIKE_DEBUG
   if(hasarg(f))
@@ -193,7 +193,7 @@ INT32 assemble(int store_linenumbers)
   {
     for (e = 0; e < length; e++) {
       if (c[e].opcode == F_POP_SYNCH_MARK) synch_depth--;
-      fprintf(stderr, "~~~%4d %4lx %*s", c[e].line,
+      fprintf(stderr, "~~~%4ld %4lx %*s", (long)c[e].line,
 	      DO_NOT_WARN((unsigned long)e), synch_depth, "");
       dump_instr(c+e);
       fprintf(stderr,"\n");
@@ -210,7 +210,7 @@ INT32 assemble(int store_linenumbers)
   if (store_linenumbers) {
     p_wchar2 *current_tripple;
     struct pike_string *previous_file = NULL;
-    int previous_line = 0;
+    INT_TYPE previous_line = 0;
     ptrdiff_t num_linedirectives = 0;
 
     /* Count the number of F_FILENAME/F_LINE pseudo-ops we need to add. */
@@ -253,7 +253,11 @@ INT32 assemble(int store_linenumbers)
       if (c[e].line != previous_line) {
 	current_tripple[0] = F_LINE;
 	current_tripple[1] = c[e].line;
-	current_tripple[2] = 0;
+#if SIZEOF_INT_TYPE > 4
+	current_tripple[2] = c[e].line>>32;
+#else
+        current_tripple[2] = 0;
+#endif
 	current_tripple += 3;
 	previous_line = c[e].line;
       }
@@ -297,8 +301,8 @@ INT32 assemble(int store_linenumbers)
     c=(p_instr *)instrbuf.s.str;
     for(e=0;e<length;e++,c++) {
       if (c->opcode == F_POP_SYNCH_MARK) synch_depth--;
-      fprintf(stderr, " * %4d %4lx ",
-	      c->line, DO_NOT_WARN((unsigned long)e));
+      fprintf(stderr, " * %4ld %4lx ",
+	      (long)c->line, DO_NOT_WARN((unsigned long)e));
       dump_instr(c);
       if ((instrs[c->opcode - F_OFFSET].flags & I_POINTER) &&
 	  (c->arg > max_label)) {
@@ -320,11 +324,10 @@ INT32 assemble(int store_linenumbers)
     if (c->opcode == F_ENTRY) c->opcode = F_NOP;
 #endif
 
-  labels=(INT32 *)xalloc(sizeof(INT32) * 4 * (max_label+2));
+  labels=xalloc(sizeof(INT32) * 4 * (max_label+2));
   jumps = labels + max_label + 2;
   uses = jumps + max_label + 2;
   aliases = uses + max_label + 2;
-
   while(relabel)
   {
     /* First do the relabel pass. */
@@ -460,24 +463,33 @@ INT32 assemble(int store_linenumbers)
   }
 
   /* Time to create the actual bytecode. */
-
   c=(p_instr *)instrbuf.s.str;
   length=instrbuf.s.len / sizeof(p_instr);
 
   for(e=0;e<=max_label;e++) labels[e]=jumps[e]=-1;
-  
-#ifdef ALIGN_PIKE_FUNCTION_BEGINNINGS
-  while( ( (((INT32) PIKE_PC)+2) & (ALIGN_PIKE_JUMPS-1)))
-    ins_byte(0);
+
+
+#ifdef START_NEW_FUNCTION
+  START_NEW_FUNCTION(store_linenumbers);
 #endif
 
 #ifdef PIKE_PORTABLE_BYTECODE
+#ifdef ALIGN_PIKE_FUNCTION_BEGINNINGS
+  while( ( (((INT32) PIKE_PC)+4) & (ALIGN_PIKE_JUMPS-1)))
+    ins_byte(0);
+#endif
+
   if (store_linenumbers) {
     ins_data(store_prog_string(tripples));
     free_string(tripples);
   } else {
     ins_data(0);
   }
+#else
+#ifdef ALIGN_PIKE_FUNCTION_BEGINNINGS
+  while( ( ((INT32) PIKE_PC) & (ALIGN_PIKE_JUMPS-1)))
+    ins_byte(0);
+#endif
 #endif /* PIKE_PORTABLE_BYTECODE */
 
   entry_point = PIKE_PC;
@@ -497,7 +509,7 @@ INT32 assemble(int store_linenumbers)
     if((a_flag > 2 && store_linenumbers) || a_flag > 3)
     {
       if (c->opcode == F_POP_SYNCH_MARK) synch_depth--;
-      fprintf(stderr, "===%4d %4lx %*s", c->line,
+      fprintf(stderr, "===%4ld %4lx %*s", (long)c->line,
 	      DO_NOT_WARN((unsigned long)PIKE_PC), synch_depth, "");
       dump_instr(c);
       fprintf(stderr,"\n");
@@ -561,6 +573,7 @@ INT32 assemble(int store_linenumbers)
       if ((e+1 < length) &&
 	  (c[1].opcode != F_LABEL) &&
 	  (c[1].opcode != F_BYTE) &&
+	  (c[1].opcode != F_ENTRY) &&
 	  (c[1].opcode != F_DATA)) {
 	/* Don't add redundant code before labels or raw data. */
 	UPDATE_PC();
@@ -757,7 +770,11 @@ INT32 assemble(int store_linenumbers)
     }
   }
 
-  free((char *)labels);
+  free(labels);
+
+#ifdef END_FUNCTION
+  END_FUNCTION(store_linenumbers);
+#endif
 
 #ifdef PIKE_DEBUG
   if (a_flag > 6) {
@@ -806,7 +823,7 @@ static p_instr *instructions;
 /* insopt{0,1,2} push an instruction on instrstack. */
 
 static INLINE p_instr *insopt2(int f, INT32 a, INT32 b,
-			       int cl, struct pike_string *cf)
+			       INT_TYPE cl, struct pike_string *cf)
 {
   p_instr *p;
 
@@ -832,7 +849,8 @@ static INLINE p_instr *insopt2(int f, INT32 a, INT32 b,
   return p;
 }
 
-static INLINE p_instr *insopt1(int f, INT32 a, int cl, struct pike_string *cf)
+static INLINE p_instr *insopt1(int f, INT32 a, INT_TYPE cl,
+			       struct pike_string *cf)
 {
 #ifdef PIKE_DEBUG
   if(!hasarg(f) && a)
@@ -842,7 +860,7 @@ static INLINE p_instr *insopt1(int f, INT32 a, int cl, struct pike_string *cf)
   return insopt2(f,a,0,cl, cf);
 }
 
-static INLINE p_instr *insopt0(int f, int cl, struct pike_string *cf)
+static INLINE p_instr *insopt0(int f, INT_TYPE cl, struct pike_string *cf)
 {
 #ifdef PIKE_DEBUG
   if(hasarg(f))
@@ -946,13 +964,13 @@ static void do_optimization(int topop, int topush, ...)
   int q=0;
   int oplen;
   struct pike_string *cf;
-  INT32 cl=instr(0)->line;
+  INT_TYPE cl=instr(0)->line;
 
 #ifdef PIKE_DEBUG
   if(a_flag>5)
   {
     int e;
-    fprintf(stderr,"PEEP at %d:",cl);
+    fprintf(stderr, "PEEP at %ld:", cl);
     for(e = topop; e--;)
     {
       fprintf(stderr," ");
@@ -1065,7 +1083,7 @@ static int asm_opt(void)
     for(e=0;e<length;e++,c++)
     {
       if (c->opcode == F_POP_SYNCH_MARK) synch_depth--;
-      fprintf(stderr,"<<<%4d: %*s",c->line,synch_depth,"");
+      fprintf(stderr,"<<<%4ld: %*s",(long)c->line,synch_depth,"");
       dump_instr(c);
       fprintf(stderr,"\n");
       if (c->opcode == F_SYNCH_MARK) synch_depth++;
@@ -1098,7 +1116,7 @@ static int asm_opt(void)
     relabel |= low_asm_opt();
   }
 
-  free((char *)instructions);
+  free(instructions);
 
 #ifdef PIKE_DEBUG
   if(a_flag > 4)
@@ -1114,7 +1132,7 @@ static int asm_opt(void)
     for(e=0;e<length;e++,c++)
     {
       if (c->opcode == F_POP_SYNCH_MARK) synch_depth--;
-      fprintf(stderr,">>>%4d: %*s",c->line,synch_depth,"");
+      fprintf(stderr,">>>%4ld: %*s",(long)c->line,synch_depth,"");
       dump_instr(c);
       fprintf(stderr,"\n");
       if (c->opcode == F_SYNCH_MARK) synch_depth++;
