@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: f72567fb3d329eef6ba170b623942800f070d2bf $
 */
 
 #include "global.h"
@@ -12,6 +11,7 @@
 #include "gc.h"
 #include "fd_control.h"
 #include "dmalloc.h"
+#include "block_allocator.h"
 
 #ifdef HAVE_SYS_MMAN_H
 #include <sys/mman.h>
@@ -25,6 +25,8 @@
 #include <fcntl.h>
 #endif
 
+#include <stdlib.h>
+
 #include <errno.h>
 
 int page_size;
@@ -39,7 +41,7 @@ char *strdup(const char *str)
     int len = strlen(str)+1;
 
     res = xalloc(len);
-    MEMCPY(res, str, len);
+    memcpy(res, str, len);
   }
   return(res);
 }
@@ -71,16 +73,16 @@ p_wchar2 *MEMCHR2(p_wchar2 *p, p_wchar2 c, ptrdiff_t e)
   return (p_wchar2 *)NULL;
 }
 
-void swap(char *a, char *b, size_t size)
+static void swap(char *a, char *b, size_t size)
 {
   size_t tmp;
   char tmpbuf[1024];
   while(size)
   {
     tmp = MINIMUM((size_t)sizeof(tmpbuf), size);
-    MEMCPY(tmpbuf,a,tmp);
-    MEMCPY(a,b,tmp);
-    MEMCPY(b,tmpbuf,tmp);
+    memcpy(tmpbuf,a,tmp);
+    memcpy(a,b,tmp);
+    memcpy(b,tmpbuf,tmp);
     size-=tmp;
     a+=tmp;
     b+=tmp;
@@ -178,106 +180,199 @@ void reorder(char *memory, INT32 nitems, INT32 size,INT32 *order)
 #endif
 
   default:
-    for(e=0;e<nitems;e++) MEMCPY(tmp+e*size, memory+order[e]*size, size);
+    for(e=0;e<nitems;e++) memcpy(tmp+e*size, memory+order[e]*size, size);
   }
 
-  MEMCPY(memory, tmp, size * nitems);
+  memcpy(memory, tmp, size * nitems);
   free(tmp);
 }
 
-size_t hashmem(const unsigned char *a_, size_t len_, size_t mlen_)
+#include "siphash24.c"
+
+#if (defined(__i386__) || defined(__amd64__)) && defined(__GNUC__)
+/*
+ * This would look much better using the compiler intrinsics, or even the
+ * assembler instructions directly.
+ *
+ * However, that requires a (at least in 2011) somewhat modern gcc (4.5.*).
+ *
+ * For reference:
+ *  #define CRC32SI(H,P) H=__builtin_ia32_crc32si(H,P)
+ *  #define CRC32SQ(H,P) H=__builtin_ia32_crc32qi(H,P)
+ *  #define __cpuid(level,a,b,c,d) __get_cpuid(level, &a,&b,&c,&d)
+ *
+ * The value for the SSE4_2 is also available in cpuid.h in modern
+ * gcc:s.
+ */
+#ifdef HAVE_CRC32_INTRINSICS
+#define CRC32SI(H,P) H=__builtin_ia32_crc32si(H,*(P))
+#define CRC32SQ(H,P) H=__builtin_ia32_crc32qi(H,*(P))
+#else
+
+/* GCC versions without __builtin_ia32_crc32* also lacks the support
+ * in the assembler, given that the binutils are the same version.
+ *
+ * Adding a second test for that seems like overkill.
+ */
+
+#define CRC32SI(H,P)                                                  \
+    __asm__ __volatile__(                                             \
+        ".byte 0xf2, 0xf, 0x38, 0xf1, 0xf1;"                          \
+        :"=S"(H) :"0"(H), "c"(*(P)))
+
+#define CRC32SQ(H,P)                                                  \
+    __asm__ __volatile__(                                             \
+        ".byte 0xf2, 0xf, 0x38, 0xf0, 0xf1"                           \
+        :"=S"(H) :"0"(H), "c"(*(P)))
+#endif
+
+#if SIZEOF_CHAR_P == 4
+#define __cpuid(level, a, b, c, d)                      \
+    __asm__ ("pushl %%ebx      \n\t"                    \
+             "cpuid \n\t"                               \
+             "movl %%ebx, %1   \n\t"                    \
+             "popl %%ebx       \n\t"                    \
+             : "=a" (a), "=r" (b), "=c" (c), "=d" (d)   \
+             : "a" (level)                              \
+             : "cc")
+#else
+#define __cpuid(level, a, b, c, d)                      \
+    __asm__ ("push %%rbx      \n\t"			\
+             "cpuid \n\t"                               \
+             "movl %%ebx, %1   \n\t"                    \
+             "pop %%rbx       \n\t"			\
+             : "=a" (a), "=r" (b), "=c" (c), "=d" (d)   \
+             : "a" (level)                              \
+             : "cc")
+#endif
+
+#define bit_SSE4_2 (1<<20)
+
+__attribute__((const)) static inline int supports_sse42( ) 
 {
-  size_t ret_;
-
-  DO_HASHMEM(ret_, a_, len_, mlen_);
-
-  return ret_;
+    unsigned int ignore, cpuid_ecx;
+    __cpuid( 0x1, ignore, ignore, cpuid_ecx, ignore );
+    return cpuid_ecx & bit_SSE4_2;
 }
 
-size_t hashstr(const unsigned char *str, ptrdiff_t maxn)
+#ifdef __i386__
+__attribute__((fastcall))
+#endif
+#ifdef HAVE_CRC32_INTRINSICS
+/*
+The intrinsics are only available if -msse4 is specified.
+However, specifying that option on the command-line makes the whole runtime-test here
+pointless, since gcc will use other sse4 instructions when suitable.
+*/
+__attribute__((target("sse4")))
+#endif
+__attribute__((hot))
+static inline size_t low_hashmem_ia32_crc32( const void *s, size_t len,
+					     size_t nbytes, size_t key )
 {
-  size_t ret,c;
-  
-  if(!(ret=str++[0]))
-    return ret;
-  for(; maxn>=0; maxn--)
+  unsigned int h = len;
+  const unsigned int *p = s;
+
+  if( key )
+      return low_hashmem_siphash24(s,len,nbytes,key);
+
+  if( nbytes >= len )
   {
-    c=str++[0];
-    if(!c) break;
-    ret ^= ( ret << 4 ) + c ;
-    ret &= 0x7fffffff;
-  }
+    /* Hash the whole memory area */
+    const unsigned int *e = p + (len>>2);
+    const unsigned char *c = (const unsigned char*)e;
 
-  return ret;
-}
-
-#define MK_HASHMEM(NAME, TYPE)					\
-  size_t NAME(const TYPE *str, ptrdiff_t len, ptrdiff_t maxn)	\
-  {								\
-    size_t ret,c;						\
-    								\
-    ret = len*92873743;						\
-  								\
-    len = MINIMUM(maxn,len);					\
-    for(; len>=0; len--)					\
-    {								\
-      c=str++[0];						\
-      ret ^= ( ret << 4 ) + c ;					\
-      ret &= 0x7fffffff;					\
-    }								\
-  								\
-    return ret;							\
-  }
-
-MK_HASHMEM(simple_hashmem, unsigned char)
-MK_HASHMEM(simple_hashmem1, p_wchar1)
-MK_HASHMEM(simple_hashmem2, p_wchar2)
-
-void memfill(char *to,
-	     INT32 tolen,
-	     char *from,
-	     INT32 fromlen,
-	     INT32 offset)
-{
-  if(fromlen==1)
-  {
-    MEMSET(to, *from, tolen);
-  }
-  else if(tolen>0)
-  {
-    INT32 tmp=MINIMUM(tolen, fromlen - offset);
-    MEMCPY(to, from + offset, tmp);
-    to+=tmp;
-    tolen-=tmp;
-
-    if(tolen > 0)
-    {
-      tmp=MINIMUM(tolen, fromlen);
-      MEMCPY(to, from, tmp);
-      from=to;
-      to+=tmp;
-      tolen-=tmp;
-      
-      while(tolen>0)
-      {
-	tmp=MINIMUM(tolen, fromlen);
-	MEMCPY(to, from, tmp);
-	fromlen+=tmp;
-	tolen-=tmp;
-	to+=tmp;
-      }
+    /* .. all full integers .. */
+    while( p<e ) {
+      CRC32SI(h, p++ );
     }
+
+    len &= 3;
+
+    /* any remaining bytes. */
+    while( len-- )
+      CRC32SQ( h, c++ );
+    return h;
   }
+  else
+  {
+    const unsigned int *e = p+(nbytes>>2);
+#ifdef PIKE_DEBUG
+    /*
+      This code makes assumputions that is not true if nbytes & 3 is true.
+
+      Specifically, it will not read enough (up to 3 bytes too
+      little) in the first loop.
+
+      Also, if nbytes < 32 the unroll assumptions do not hold
+
+      This could easily be fixed, but all calls to hashmem tend
+      to use either power-of-two values or the length of the
+      whole memory area.
+    */
+    if( nbytes & 3 )
+      Pike_fatal("do_hash_ia32_crc32: nbytes & 3 should be 0.\n");
+    if( nbytes < 32 )
+      Pike_fatal("do_hash_ia32_crc32: nbytes is less than 32.\n");
+#endif
+    while( p+7 < e )
+    {
+      CRC32SI(h,&p[0]);
+      CRC32SI(h,&p[1]);
+      CRC32SI(h,&p[2]);
+      CRC32SI(h,&p[3]);
+      CRC32SI(h,&p[4]);
+      CRC32SI(h,&p[5]);
+      CRC32SI(h,&p[6]);
+      CRC32SI(h,&p[7]);
+      p+=8;
+    }
+    /* include 8 bytes from the end. Note that this might be a
+     * duplicate of the previous bytes.
+     *
+     * Also note that this means we are rather likely to read
+     * unaligned memory.  That is OK, however.
+     */
+    e = (const unsigned int *)((const unsigned char *)s+len-8);
+    CRC32SI(h,e++);
+    CRC32SI(h,e);
+  }
+#if SIZEOF_CHAR_P > 4
+    /* FIXME: We could use the long long crc32 instructions that work on 64 bit values.
+     * however, those are only available when compiling to amd64.
+     */
+  return (((size_t)h)<<32) | h;
+#endif
+  return h;
 }
 
-#if 0
-#if defined(HAVE_GETRLIMIT) && defined(HAVE_SETRLIMIT) && defined(RLIMIT_DATA)
-#define SOFTLIM 
-
-static long softlim_should_be=0;
+#ifdef __i386__
+__attribute__((fastcall))
 #endif
+  size_t (*low_hashmem)(const void *, size_t, size_t, size_t);
+
+static void init_hashmem()
+{
+  if( supports_sse42() )
+    low_hashmem = low_hashmem_ia32_crc32;
+  else
+    low_hashmem = low_hashmem_siphash24;
+}
+#else
+static void init_hashmem(){}
+
+ATTRIBUTE((hot))
+  size_t low_hashmem(const void *a, size_t len_, size_t mlen_, size_t key_)
+{
+    return low_hashmem_siphash24(a, len_, mlen_, key_);
+}
 #endif
 
+ATTRIBUTE((hot))
+  size_t hashmem(const void *a, size_t len_, size_t mlen_)
+{
+  return low_hashmem(a, len_, mlen_, 0);
+}
 
 PMOD_EXPORT void *debug_xalloc(size_t size)
 {
@@ -320,13 +415,31 @@ PMOD_EXPORT void *debug_xcalloc(size_t n, size_t s)
   return 0;
 }
 
+PMOD_EXPORT void *xalloc_aligned(size_t size, size_t alignment) {
+    void * ret;
+
+    if (!size) return 0;
+
+#ifdef HAVE_POSIX_MEMALIGN
+    if (posix_memalign(&ret, alignment, size)) {
+	Pike_error(msg_out_of_mem_2, size);
+    }
+#else
+    if ((ret = memalign(alignment, size)) == NULL) {
+	Pike_error(msg_out_of_mem_2, size);
+    }
+#endif
+
+    return ret;
+}
+
 PMOD_EXPORT char *debug_xstrdup(const char *src)
 {
   char *dst = NULL;
   if (src) {
     int len = strlen (src) + 1;
     dst = malloc (len);
-    MEMCPY (dst, src, len);
+    memcpy (dst, src, len);
   }
   return dst;
 }
@@ -442,9 +555,6 @@ static INLINE void *mexec_do_alloc (void *start, size_t length)
 static size_t sep_allocs = 0, grow_allocs = 0, total_size = 0;
 #endif
 
-#if 0
-#define MEXEC_MAGIC	0xdeadfeedf00dfaddLL
-#endif /* 0 */
 struct mexec_block {
   struct mexec_hdr *hdr;
   ptrdiff_t size;
@@ -465,41 +575,41 @@ static struct mexec_hdr {
 
 #ifdef PIKE_DEBUG
 static void low_verify_mexec_hdr(struct mexec_hdr *hdr,
-				 const char *file, int line)
+				 const char *file, INT_TYPE line)
 {
   struct mexec_free_block *ptr;
   char *blk_ptr;
   if (!hdr) return;
   if (d_flag) {
     if (hdr->bottom > ((char *)hdr) + hdr->size) {
-      Pike_fatal("%s:%d:Bad bottom %p > %p\n",
-		 file, line,
+      Pike_fatal("%s:%ld:Bad bottom %p > %p\n",
+		 file, (long)line,
 		 hdr->bottom, ((char *)hdr) + hdr->size);
     }
     for (blk_ptr = (char *)(hdr+1); blk_ptr < hdr->bottom;) {
       struct mexec_free_block *blk = (struct mexec_free_block *)blk_ptr;
       if (blk->size <= 0) {
-	Pike_fatal("%s:%d:Bad block size: %p\n",
-		   file, line,
+	Pike_fatal("%s:%ld:Bad block size: %p\n",
+		   file, (long)line,
 		   (void *)blk->size);
       }
       blk_ptr += blk->size;
     }
     if (blk_ptr != hdr->bottom) {
-      Pike_fatal("%s:%d:Block reaches past bottom! %p > %p\n",
-		 file, line,
+      Pike_fatal("%s:%ld:Block reaches past bottom! %p > %p\n",
+		 file, (long)line,
 		 blk_ptr, hdr->bottom);
     }
     if (d_flag > 1) {
       for (ptr = hdr->free; ptr; ptr = ptr->next) {
 	if (ptr < (struct mexec_free_block *)(hdr+1)) {
-	  Pike_fatal("%s:%d:Free block before start of header. %p < %p\n",
-		     file, line,
+	  Pike_fatal("%s:%ld:Free block before start of header. %p < %p\n",
+		     file, (long)line,
 		     ptr, hdr+1);
 	}
 	if (((char *)ptr) >= hdr->bottom) {
-	  Pike_fatal("%s:%d:Free block past bottom. %p >= %p\n",
-		     file, line,
+	  Pike_fatal("%s:%ld:Free block past bottom. %p >= %p\n",
+		     file, (long)line,
 		     ptr, hdr->bottom);
 	}
       }
@@ -690,13 +800,6 @@ PMOD_EXPORT void mexec_free(void *ptr)
       /* Join with bottom. */
       hdr->bottom = (char *)blk;
       hdr->free = blk->next;
-#if 0
-      if (hdr->bottom == (char *)(hdr + 1)) {
-	/* The entire mmapped block is free.
-	 * FIXME: Consider unmapping it.
-	 */
-      }
-#endif /* 0 */
     } else {
       hdr->free = blk;
     }
@@ -704,7 +807,7 @@ PMOD_EXPORT void mexec_free(void *ptr)
   verify_mexec_hdr(hdr);
 }
 
-PMOD_EXPORT void *mexec_alloc(size_t sz)
+void *mexec_alloc(size_t sz)
 {
   struct mexec_hdr *hdr;
   struct mexec_block *res;
@@ -747,7 +850,7 @@ PMOD_EXPORT void *mexec_alloc(size_t sz)
   return res + 1;
 }
 
-PMOD_EXPORT void *mexec_realloc(void *ptr, size_t sz)
+void *mexec_realloc(void *ptr, size_t sz)
 {
   struct mexec_hdr *hdr;
   struct mexec_hdr *old_hdr = NULL;
@@ -863,7 +966,7 @@ PMOD_EXPORT void *mexec_realloc(void *ptr, size_t sz)
 
 #elif defined (VALGRIND_DISCARD_TRANSLATIONS)
 
-PMOD_EXPORT void *mexec_alloc (size_t sz)
+void *mexec_alloc (size_t sz)
 {
   size_t *blk = malloc (sz + sizeof (size_t));
   if (!blk) return NULL;
@@ -871,7 +974,7 @@ PMOD_EXPORT void *mexec_alloc (size_t sz)
   return blk + 1;
 }
 
-PMOD_EXPORT void *mexec_realloc (void *ptr, size_t sz)
+void *mexec_realloc (void *ptr, size_t sz)
 {
   if (ptr) {
     size_t *oldblk = ptr;
@@ -888,7 +991,7 @@ PMOD_EXPORT void *mexec_realloc (void *ptr, size_t sz)
   return mexec_malloc (sz);
 }
 
-PMOD_EXPORT void mexec_free (void *ptr)
+void mexec_free (void *ptr)
 {
   size_t *blk = ptr;
   VALGRIND_DISCARD_TRANSLATIONS (blk, blk[-1]);
@@ -897,16 +1000,16 @@ PMOD_EXPORT void mexec_free (void *ptr)
 
 #else  /* !USE_MY_MEXEC_ALLOC && !VALGRIND_DISCARD_TRANSLATIONS */
 
-PMOD_EXPORT void *mexec_alloc(size_t sz)
+void *mexec_alloc(size_t sz)
 {
   return malloc(sz);
 }
-PMOD_EXPORT void *mexec_realloc(void *ptr, size_t sz)
+void *mexec_realloc(void *ptr, size_t sz)
 {
   if (ptr) return realloc(ptr, sz);
   return malloc(sz);
 }
-PMOD_EXPORT void mexec_free(void *ptr)
+void mexec_free(void *ptr)
 {
   free(ptr);
 }
@@ -1184,7 +1287,7 @@ PMOD_EXPORT void *realloc(void *x,size_t y)
     if(old_size >= y) return x;
     ret=malloc(y);
     if(!ret) return 0;
-    MEMCPY(ret, x, old_size);
+    memcpy(ret, x, old_size);
     if(x) free(x);
   }else{
     ret=debug_realloc(x, y, DMALLOC_LOCATION());
@@ -1211,7 +1314,7 @@ void *fake_realloc(void *x,size_t y)
     if(old_size >= y) return x;
     ret=malloc(y);
     if(!ret) return 0;
-    MEMCPY(ret, x, old_size);
+    memcpy(ret, x, old_size);
     if(x) free(x);
   }else{
     ret=real_realloc(x,y);
@@ -1229,7 +1332,7 @@ void *calloc(size_t x, size_t y)
   LOWDEBUG3("calloc x",x);
   LOWDEBUG3("calloc y",y);
   ret=malloc(x*y);
-  if(ret) MEMSET(ret,0,x*y);
+  if(ret) memset(ret,0,x*y);
 #ifndef REPORT_ENCAPSULATED_MALLOC
   dmalloc_accept_leak(ret);
 #endif
@@ -1241,7 +1344,7 @@ void *fake_calloc(size_t x, size_t y)
 {
   void *ret;
   ret=fake_malloc(x*y);
-  if(ret) MEMSET(ret,0,x*y);
+  if(ret) memset(ret,0,x*y);
 #ifndef REPORT_ENCAPSULATED_MALLOC
   dmalloc_accept_leak(ret);
 #endif
@@ -1288,13 +1391,6 @@ void *fake_calloc(size_t x, size_t y)
 static struct memhdr *my_find_memhdr(void *, int);
 static void dump_location_bt (LOCATION l, int indent, const char *prefix);
 
-#include "block_alloc_h.h"
-
-BLOCK_ALLOC_FILL_PAGES(memloc, n/a);
-BLOCK_ALLOC_FILL_PAGES(memory_map, n/a);
-BLOCK_ALLOC_FILL_PAGES(memory_map_entry, n/a);
-
-#include "block_alloc.h"
 
 int verbose_debug_malloc = 0;
 int debug_malloc_check_all = 0;
@@ -1366,7 +1462,19 @@ struct memloc
 #define MEM_TRACE			64
 #define MEM_SCANNED			128
 
-BLOCK_ALLOC_FILL_PAGES(memloc, 64)
+static struct block_allocator memloc_allocator = BA_INIT_PAGES(sizeof(struct memloc), 64);
+
+static struct memloc * alloc_memloc() {
+    return ba_alloc(&memloc_allocator);
+}
+
+static void really_free_memloc(struct memloc * ml) {
+    ba_free(&memloc_allocator, ml);
+}
+
+void count_memory_in_memlocs(size_t * n, size_t * s) {
+    ba_count_all(&memloc_allocator, n, s);
+}
 
 struct memhdr
 {
@@ -1423,8 +1531,8 @@ char *do_pad(char *mem, long size)
     /*  fprintf(stderr,"Padding  %p(%d) %ld\n",mem, size, q); */
 #if 1
     q%=RNDSIZE;
-    MEMCPY(mem - DEBUG_MALLOC_PAD, rndbuf+q, DEBUG_MALLOC_PAD);
-    MEMCPY(mem + size, rndbuf+q, DEBUG_MALLOC_PAD);
+    memcpy(mem - DEBUG_MALLOC_PAD, rndbuf+q, DEBUG_MALLOC_PAD);
+    memcpy(mem + size, rndbuf+q, DEBUG_MALLOC_PAD);
 #else
     for(e=0;e< DEBUG_MALLOC_PAD; e+=4)
     {
@@ -1477,12 +1585,12 @@ void check_pad(struct memhdr *mh, int freeok)
 /*  fprintf(stderr,"Checking %p(%d) %ld\n",mem, size, q);  */
 #if 1
   /* optimization? */
-  if(MEMCMP(mem - DEBUG_MALLOC_PAD, mem+size, DEBUG_MALLOC_PAD))
+  if(memcmp(mem - DEBUG_MALLOC_PAD, mem+size, DEBUG_MALLOC_PAD))
   {
     q= (((long)mem) ^ 0x555555) + (size * 9248339);
     
     q%=RNDSIZE;
-    if(MEMCMP(mem - DEBUG_MALLOC_PAD, rndbuf+q, DEBUG_MALLOC_PAD))
+    if(memcmp(mem - DEBUG_MALLOC_PAD, rndbuf+q, DEBUG_MALLOC_PAD))
     {
       out_biking=1;
       fprintf(stderr,"Pre-padding overwritten for "
@@ -1491,7 +1599,7 @@ void check_pad(struct memhdr *mh, int freeok)
       abort();
     }
     
-    if(MEMCMP(mem + size, rndbuf+q, DEBUG_MALLOC_PAD))
+    if(memcmp(mem + size, rndbuf+q, DEBUG_MALLOC_PAD))
     {
       out_biking=1;
       fprintf(stderr,"Post-padding overwritten for "
@@ -1563,6 +1671,8 @@ static INLINE unsigned long lhash(struct memhdr *m, LOCATION location)
   return l;
 }
 
+#include "block_alloc.h"
+
 #undef INIT_BLOCK
 #undef EXIT_BLOCK
 
@@ -1616,6 +1726,7 @@ static INLINE unsigned long lhash(struct memhdr *m, LOCATION location)
 
 #undef BLOCK_ALLOC_HSIZE_SHIFT
 #define BLOCK_ALLOC_HSIZE_SHIFT 1
+
 PTR_HASH_ALLOC_FILL_PAGES(memhdr, 128)
 
 #undef INIT_BLOCK
@@ -1750,58 +1861,6 @@ static void add_location(struct memhdr *mh,
   return;
 }
 
-static void remove_location(struct memhdr *mh, LOCATION location)
-{
-  struct memloc *ml,**prev;
-  unsigned long l;
-
-#if defined(DMALLOC_VERIFY_INTERNALS) && !defined(__NT__) && defined(PIKE_THREADS)
-  if(!mt_trylock(& debug_malloc_mutex))
-    Pike_fatal("remove_location running unlocked!\n");
-#endif
-  
-#if DEBUG_MALLOC - 0 < 2
-  if(find_location(&no_leak_memlocs, location)) return;
-#endif
-
-  if (!(ml = find_location(mh, location))) {
-    return;
-  }
-
-  /* Unlink from the hashtable. */
-  if ((ml->hash_prev[0] = ml->hash_next)) {
-    ml->hash_next->hash_prev = ml->hash_prev;
-  }
-
-  prev=&mh->locations;
-  while((ml=*prev))
-  {
-#ifdef DMALLOC_VERIFY_INTERNALS
-    if (ml->mh != mh) {
-      Pike_fatal("Non-owned memloc in location list!\n");
-    }
-#endif
-
-    if(ml->location==location)
-    {
-      *prev=ml->next;
-#ifdef DMALLOC_VERIFY_INTERNALS
-      mh->times -= ml->times;
-#endif
-#ifdef DMALLOC_TRACE_MEMLOC
-      if (ml == DMALLOC_TRACE_MEMLOC) {
-        fprintf(stderr, "rem_loc: Freeing memloc %p location %s memhdr: %p data: %p\n",
-		ml, ml->location, ml->mh, ml->mh->data);
-      }
-#endif /* DMALLOC_TRACE_MEMLOC */
-      really_free_memloc(ml);
-      return;
-    }else{
-      prev=&ml->next;
-    }
-  }
-}
-
 LOCATION dmalloc_default_location=0;
 
 static struct memhdr *low_make_memhdr(void *p, int s, LOCATION location
@@ -1831,7 +1890,7 @@ static struct memhdr *low_make_memhdr(void *p, int s, LOCATION location
   mh->gc_generation=gc_generation * 1000 + Pike_in_gc;
 #ifdef DMALLOC_C_STACK_TRACE
   if (bt_len > 0) {
-    MEMCPY (mh->alloc_bt, bt, bt_len * sizeof (c_stack_frame));
+    memcpy (mh->alloc_bt, bt, bt_len * sizeof (c_stack_frame));
     mh->alloc_bt_len = bt_len;
   }
   else
@@ -1947,7 +2006,7 @@ PMOD_EXPORT int dmalloc_unregister(void *p, int already_gone)
   return ret;
 }
 
-static int low_dmalloc_mark_as_free(void *p, int already_gone)
+static int low_dmalloc_mark_as_free(void *p, int UNUSED(already_gone))
 {
   struct memhdr *mh=find_memhdr(p);
   if(mh)
@@ -2005,6 +2064,15 @@ static void flush_blocks_to_free(void)
   }
 }
 
+MALLOC_FUNCTION
+PMOD_EXPORT void * system_malloc(size_t n) {
+    return real_malloc(n);
+}
+
+PMOD_EXPORT void system_free(void * p) {
+    real_free(p);
+}
+
 PMOD_EXPORT void *debug_malloc(size_t s, LOCATION location)
 {
   char *m;
@@ -2044,7 +2112,7 @@ PMOD_EXPORT void *debug_calloc(size_t a, size_t b, LOCATION location)
 {
   void *m=debug_malloc(a*b,location);
   if(m)
-    MEMSET(m, 0, a*b);
+    memset(m, 0, a*b);
 
   if(verbose_debug_malloc)
     fprintf(stderr, "calloc(%ld, %ld) => %p  (%s)\n",
@@ -2117,7 +2185,7 @@ PMOD_EXPORT void debug_free(void *p, LOCATION location, int mustfind)
     if (PIKE_MEM_CHECKER())
       PIKE_MEM_NA_RANGE(p, mh->size);
     else
-      MEMSET(p, 0x55, mh->size);
+      memset(p, 0x55, mh->size);
     if(mh->size < MAX_UNFREE_MEM/FREE_DELAY)
     {
       add_location(mh, location);
@@ -2175,32 +2243,6 @@ PMOD_EXPORT int dmalloc_check_allocated (void *p, int must_be_freed)
   return res;
 }
 
-#if 0
-/* Disabled since it isn't used. */
-void dmalloc_check_block_free(void *p, LOCATION location,
-			      char *struct_name, describe_block_fn *describer)
-{
-  struct memhdr *mh;
-  mt_lock(&debug_malloc_mutex);
-  mh=my_find_memhdr(p,0);
-
-  if(mh && mh->size>=0)
-  {
-    if(!(mh->flags & MEM_IGNORE_LEAK))
-    {
-      fprintf(stderr, "Freeing %s still in use %p at %s.\n",
-	      struct_name ? struct_name : "small block", p, LOCATION_NAME(location));
-      if (describer) describer (p);
-      debug_malloc_dump_references(p,0,2,0);
-    }
-    mh->flags |= MEM_FREE | MEM_IGNORE_LEAK;
-    mh->size = ~mh->size;
-  }
-
-  mt_unlock(&debug_malloc_mutex);
-}
-#endif
-
 PMOD_EXPORT void dmalloc_free(void *p)
 {
   debug_free(p, DMALLOC_LOCATION(), 0);
@@ -2212,7 +2254,7 @@ PMOD_EXPORT char *debug_strdup(const char *s, LOCATION location)
   long length;
   length=strlen(s);
   m=(char *)debug_malloc(length+1,location);
-  MEMCPY(m,s,length+1);
+  memcpy(m,s,length+1);
 
   if(verbose_debug_malloc)
     fprintf(stderr, "strdup(\"%s\") => %p  (%s)\n", s, m, LOCATION_NAME(location));
@@ -2251,7 +2293,7 @@ struct parsed_location
 {
   const char *file;
   size_t file_len;
-  int line;
+  INT_TYPE line;
   const char *extra;
 };
 
@@ -2260,17 +2302,17 @@ static void parse_location (struct memloc *l, struct parsed_location *pl)
   const char *p;
   pl->file = LOCATION_NAME (l->location);
 
-  p = STRCHR (pl->file, ' ');
+  p = strchr (pl->file, ' ');
   if (p)
     pl->extra = p;
   else
-    pl->extra = STRCHR (pl->file, 0);
+    pl->extra = strchr (pl->file, 0);
 
-  p = STRCHR (pl->file, ':');
+  p = strchr (pl->file, ':');
   if (p && p < pl->extra) {
     const char *pp;
-    while ((pp = STRCHR (p + 1, ':')) && pp < pl->extra) p = pp;
-    pl->line = atoi (p + 1);
+    while ((pp = strchr (p + 1, ':')) && pp < pl->extra) p = pp;
+    pl->line = strtol (p + 1, NULL, 10);
     pl->file_len = p - pl->file;
   }
   else {
@@ -2575,57 +2617,6 @@ static void low_search_all_memheaders_for_references(void)
       }
       if( ! ((sizeof(void *)-1) & (size_t)p ))
       {
-#if 0
-	if(m->size > 0)
-	{
-#if defined(__NT__) && !defined(__GNUC__)
-	  __try {
-#endif
-#ifdef PIKE_EXTRA_DEBUG
-	    fprintf(stderr, "PIKE_EXTRA_DEBUG:\n"
-		    "  Scanning memory block at %p, %ld bytes, "
-		    "generation %d, flags: 0x%08x\n",
-		    m->data, m->size, m->gc_generation, m->flags);
-#endif /* PIKE_EXTRA_DEBUG */
-	    for(e=0;e<m->size/sizeof(void *);e++) {
-	      void *addr = p[e];
-	      if (!addr || ((sizeof(void *)-1) & (size_t)addr)) {
-		/* No need to hunt for memhdrs for NULL or
-		 * unaligned addresses.
-		 * This also filters out memory areas that have
-		 * been cleared by block_alloc.h:really_free_*() et al.
-		 *	/grubba 2003-03-15
-		 */
-		continue;
-	      }
-	      /* NOTE: We must not use find_memhdr() here,
-	       *       since it might alter the data-structure
-	       *       we're looping over...
-	       *
-	       *       A specific case is a bucket containing two
-	       *       self-referring memhdrs, using find_memhdr()
-	       *       will then cause an infinite loop.
-	       *	/grubba 2003-03-16
-	       */
-	      if((tmp=just_find_memhdr(addr)))
-		tmp->flags |= MEM_REFERENCED;
-	    }
-#if defined(__NT__) && !defined(__GNUC__)
-	  }
-	  __except( 1 ) {
-	    fprintf(stderr,"*** DMALLOC memory access error ***\n");
-	    fprintf(stderr,"Failed to access this memory block:\n");
-	    fprintf(stderr,"Block: %p, size=%ld, gc_generation=%d, flags=%d\n",
-		    m->data,
-		    m->size,
-		    m->gc_generation,
-		    m->flags);
-	    dump_memhdr_locations(m, 0, 0);
-	    fprintf(stderr,"-----------------------------------\n");
-	  }
-#endif
-	}
-#endif
       }
     }
   }
@@ -2765,7 +2756,6 @@ static void cleanup_memhdrs(void)
 
   }
   mt_unlock(&debug_malloc_mutex);
-  mt_destroy(&debug_malloc_mutex);
 }
 
 #ifdef _REENTRANT
@@ -2790,11 +2780,8 @@ static void initialize_dmalloc(void)
 #ifdef DMALLOC_REMEMBER_LAST_LOCATION
     th_key_create(&dmalloc_last_seen_location, 0);
 #endif
-    init_memloc_blocks();
-    init_memory_map_blocks();
-    init_memory_map_entry_blocks();
     init_memhdr_hash();
-    MEMSET(mlhash, 0, sizeof(mlhash));
+    memset(mlhash, 0, sizeof(mlhash));
 
     for(e=0;e<(long)NELEM(rndbuf);e++) rndbuf[e]= (rand() % 511) | 1;
     
@@ -2810,16 +2797,6 @@ static void initialize_dmalloc(void)
     mt_init(&debug_malloc_mutex);
 #endif
 
-    /* NOTE: th_atfork() may be a simulated function, which
-     *       utilizes callbacks. We thus need to initialize
-     *       the callback blocks before we perform the call
-     *       to th_atfork().
-     */
-    {
-      extern void init_callback_blocks(void);
-      init_callback_blocks();
-    }
-
     th_atfork(lock_da_lock, unlock_da_lock,  unlock_da_lock);
 #endif
 #ifdef DMALLOC_USING_DLOPEN
@@ -2870,7 +2847,8 @@ struct dmalloc_string
 
 static struct dmalloc_string *dstrhash[DSTRHSIZE];
 
-static LOCATION low_dynamic_location(char type, const char *file, int line,
+static LOCATION low_dynamic_location(char type, const char *file,
+				     INT_TYPE line,
 				     const char *name,
 				     const unsigned char *bin_data,
 				     unsigned int bin_data_len)
@@ -2893,14 +2871,14 @@ static LOCATION low_dynamic_location(char type, const char *file, int line,
   for(prev = dstrhash + h; (str=*prev); prev = &str->next)
   {
     if(hval == str->hval &&
-       !STRNCMP(str->str+1, file, len) &&
+       !strncmp(str->str+1, file, len) &&
        str->str[len+1]==':' &&
        LOCATION_TYPE (str->str) == type &&
-       atoi(str->str+len+2) == line)
+       strtol(str->str+len+2, NULL, 10) == line)
     {
 
       if (name) {
-	char *s = STRCHR (str->str + len + 2, ' ');
+	char *s = strchr (str->str + len + 2, ' ');
 	if (!s) continue;
 	s++;
 	if (strcmp (s, name)) continue;
@@ -2914,7 +2892,7 @@ static LOCATION low_dynamic_location(char type, const char *file, int line,
 	unsigned int str_bin_len = EXTRACT_UWORD (str_bin_base);
 	str_bin_base += 2;
 	if (str_bin_len != bin_data_len ||
-	    MEMCMP (bin_data, str_bin_base, str_bin_len))
+	    memcmp (bin_data, str_bin_base, str_bin_len))
 	  continue;
       }
 
@@ -2930,7 +2908,7 @@ static LOCATION low_dynamic_location(char type, const char *file, int line,
     char line_str[30];
     size_t l;
 
-    sprintf (line_str, "%d", line);
+    sprintf (line_str, "%ld", (long)line);
     l = len + strlen (line_str) + 2;
     if (name) l += name_len + 1;
     str=malloc (sizeof (struct dmalloc_string) + l +
@@ -2947,7 +2925,7 @@ static LOCATION low_dynamic_location(char type, const char *file, int line,
 	Pike_fatal ("Too long bin_data blob: %u\n", bin_data_len);
       ((unsigned char *) str->str)[l + 1] = ((unsigned char *) &bl)[0];
       ((unsigned char *) str->str)[l + 2] = ((unsigned char *) &bl)[1];
-      MEMCPY (str->str + l + 3, bin_data, bin_data_len);
+      memcpy (str->str + l + 3, bin_data, bin_data_len);
     }
 
     str->hval=hval;
@@ -2960,13 +2938,13 @@ static LOCATION low_dynamic_location(char type, const char *file, int line,
   return str->str;
 }
 
-LOCATION dynamic_location(const char *file, int line)
+LOCATION dynamic_location(const char *file, INT_TYPE line)
 {
   return low_dynamic_location('D',file,line, NULL, NULL, 0);
 }
 
 
-PMOD_EXPORT void * debug_malloc_name(void *p,const char *file, int line)
+PMOD_EXPORT void * debug_malloc_name(void *p,const char *file, INT_TYPE line)
 {
   if(p)
   {
@@ -3014,9 +2992,9 @@ PMOD_EXPORT int debug_malloc_copy_names(void *p, void *p2)
   return names;
 }
 
-char *dmalloc_find_name(void *p)
+const char *dmalloc_find_name(void *p)
 {
-  char *name=0;
+  const char *name=0;
   if(p)
   {
     struct memhdr *mh;
@@ -3041,7 +3019,8 @@ char *dmalloc_find_name(void *p)
 }
 
 PMOD_EXPORT void *debug_malloc_update_location_bt (void *p, const char *file,
-						   int line, const char *name)
+						   INT_TYPE line,
+						   const char *name)
 {
   LOCATION l;
 #ifdef DMALLOC_C_STACK_TRACE
@@ -3058,16 +3037,16 @@ PMOD_EXPORT void *debug_malloc_update_location_bt (void *p, const char *file,
   return debug_malloc_update_location (p, l);
 }
 
+#ifdef DMALLOC_C_STACK_TRACE
 static void dump_location_bt (LOCATION location, int indent, const char *prefix)
 {
-#ifdef DMALLOC_C_STACK_TRACE
   if (LOCATION_TYPE (location) == 'B') {
     c_stack_frame bt[BT_MAX_FRAMES];
     int i, frames;
     unsigned char *bin_base =
       (unsigned char *) location + strlen (location) + 1;
     unsigned int bin_len = EXTRACT_UWORD (bin_base);
-    MEMCPY ((unsigned char *) bt, bin_base + 2, bin_len);
+    memcpy (bt, bin_base + 2, bin_len);
     frames = bin_len / sizeof (c_stack_frame);
 
     for (i = 0; i < frames; i++) {
@@ -3075,8 +3054,12 @@ static void dump_location_bt (LOCATION location, int indent, const char *prefix)
       backtrace_symbols_fd (bt + i, 1, 2);
     }
   }
-#endif
 }
+#else
+static void dump_location_bt (LOCATION UNUSED(location), int UNUSED(indent),
+                              const char *UNUSED(prefix))
+{ }
+#endif
 
 PMOD_EXPORT int debug_malloc_touch_fd(int fd, LOCATION location)
 {
@@ -3098,7 +3081,7 @@ PMOD_EXPORT void debug_malloc_accept_leak_fd(int fd)
   dmalloc_accept_leak(FD2PTR(fd));
 }
 
-PMOD_EXPORT int debug_malloc_close_fd(int fd, LOCATION location)
+PMOD_EXPORT int debug_malloc_close_fd(int fd, LOCATION UNUSED(location))
 {
   if(fd==-1) return fd;
 #ifdef DMALLOC_TRACK_FREE
@@ -3152,8 +3135,26 @@ struct memory_map_entry
   struct memory_map *recur;
 };
 
-BLOCK_ALLOC_FILL_PAGES(memory_map, 8)
-BLOCK_ALLOC_FILL_PAGES(memory_map_entry, 16)
+static struct block_allocator memory_map_allocator = BA_INIT_PAGES(sizeof(struct memory_map), 8);
+
+static struct memory_map * alloc_memory_map() {
+    return ba_alloc(&memory_map_allocator);
+}
+
+void count_memory_in_memory_maps(size_t * n, size_t * s) {
+    ba_count_all(&memory_map_allocator, n, s);
+}
+
+static struct block_allocator memory_map_entry_allocator
+    = BA_INIT_PAGES(sizeof(struct memory_map_entry), 16);
+
+static struct memory_map_entry * alloc_memory_map_entry() {
+    return ba_alloc(&memory_map_entry_allocator);
+}
+
+void count_memory_in_memory_map_entrys(size_t * n, size_t * s) {
+    ba_count_all(&memory_map_entry_allocator, n, s);
+}
 
 void dmalloc_set_mmap(void *ptr, struct memory_map *m)
 {
@@ -3246,7 +3247,7 @@ void dmalloc_describe_location(void *p, int offset, int indent)
   }
 }
 
-struct memory_map *dmalloc_alloc_mmap(char *name, int line)
+struct memory_map *dmalloc_alloc_mmap(char *name, INT_TYPE line)
 {
   struct memory_map *m;
   mt_lock(&debug_malloc_mutex);
@@ -3258,7 +3259,7 @@ struct memory_map *dmalloc_alloc_mmap(char *name, int line)
   m->refs=0;
 
   if(strlen(m->name)+12<sizeof(m->name))
-    sprintf(m->name+strlen(m->name),":%d",line);
+    sprintf(m->name+strlen(m->name), ":%ld", (long)line);
 
   m->entries=0;
   mt_unlock(&debug_malloc_mutex);
@@ -3300,10 +3301,9 @@ static void cleanup_debug_malloc(void)
 {
   size_t i;
 
-  free_all_memloc_blocks();
+  ba_destroy(&memloc_allocator);
+  ba_destroy(&memory_map_allocator);
   exit_memhdr_hash();
-  free_all_memory_map_blocks();
-  free_all_memory_map_entry_blocks();
 
   for (i = 0; i < DSTRHSIZE; i++) {
     struct dmalloc_string *str = dstrhash[i], *next;
@@ -3313,12 +3313,15 @@ static void cleanup_debug_malloc(void)
       str = next;
     }
   }
+
+  mt_destroy(&debug_malloc_mutex);
 }
 
 #endif	/* DEBUG_MALLOC */
 
 void init_pike_memory (void)
 {
+  init_hashmem();
 #if defined (HAVE_GETPAGESIZE)
   page_size = getpagesize();
 #elif defined (HAVE_SYSCONF) && defined (_SC_PAGESIZE)
