@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 06270c41364381b1c7d9a4e41e863d2f64cd552a $
 */
 
 #include <version.h>
@@ -70,14 +69,14 @@ void pgtk2_index_stack(char *what) {
   push_text(what);
   f_index(2);
 #ifdef PIKE_DEBUG
-  if (Pike_sp[-1].type==PIKE_T_INT)
+  if (TYPEOF(Pike_sp[-1]) == PIKE_T_INT)
     Pike_error("Internal indexing error.\n");
 #endif
 }
 
 int get_color_from_pikecolor(struct object *o, INT_TYPE *r, INT_TYPE *g, INT_TYPE *b) {
   struct color_struct *col;
-  col=(struct color_struct *)get_storage(o,image_color_program);
+  col=get_storage(o,image_color_program);
   if (!col)
     return 0;
   *r=col->rgbl.r/(COLORLMAX/65535);
@@ -95,7 +94,7 @@ GdkImage *gdkimage_from_pikeimage(struct object *img, int fast, GObject **pi) {
 
   TIMER_INIT("Getting extents");
 
-  img_data=(struct image*)get_storage(img, image_program);
+  img_data=get_storage(img, image_program);
 
   /* 1a: create the actual image... */
   x = img_data->xsize;
@@ -163,7 +162,7 @@ GdkImage *gdkimage_from_pikeimage(struct object *img, int fast, GObject **pi) {
       int j,i,r,g,b;
       PFTIME("Creating colormap");
       colors_allocated=1;
-      MEMSET(allocated,0,sizeof(allocated));
+      memset(allocated,0,sizeof(allocated));
       for (r=0; r<3; r++) for (g=0; g<4; g++) for (b=0; b<3; b++) {
 	GdkColor color;
 	color.red = (guint16)(r * (65535/2.0));
@@ -243,12 +242,12 @@ GdkImage *gdkimage_from_pikeimage(struct object *img, int fast, GObject **pi) {
       /*             6       5    4  3       2     1 */
       PFTIME("Dithering image");
       apply_svalue(Pike_sp-6,5);
-      if (Pike_sp[-1].type!=PIKE_T_STRING) {
+      if (TYPEOF(Pike_sp[-1]) != PIKE_T_STRING) {
 	gdk_image_destroy((void *)i);
 	Pike_error("Failed to convert image\n");
       }
       PFTIME("Converting image");
-      MEMCPY(i->mem,Pike_sp[-1].u.string->str,Pike_sp[-1].u.string->len);
+      memcpy(i->mem,Pike_sp[-1].u.string->str,Pike_sp[-1].u.string->len);
       pop_stack(); /* string */
       pop_stack(); /* function */
     }
@@ -277,7 +276,7 @@ void push_gobjectclass(void *obj, struct program *def) {
   pgtk2__init_object(o);
 
   /* Extra ref already added in pgtk2__init_object */
-  push_object(o); 
+  push_object(o);
   return;
 }
 
@@ -292,7 +291,7 @@ void push_pgdk2object(void *obj, struct program *def, int owned) {
   call_c_initializers(o);
   ((struct object_wrapper *)o->storage)->obj=obj;
   ((struct object_wrapper *)o->storage)->owned = owned;
-  push_object(o); 
+  push_object(o);
   return;
 }
 
@@ -300,7 +299,7 @@ GObject *get_pg2object(struct object *from, struct program *type) {
   struct object_wrapper * o;
   if (!from)
     return NULL;
-  o=(struct object_wrapper *)get_storage(from,type);
+  o=get_storage(from,type);
   if (!o)
     return 0;
   return o->obj;
@@ -308,7 +307,7 @@ GObject *get_pg2object(struct object *from, struct program *type) {
 
 void *get_pgdk2object(struct object *from, struct program *type) {
   void *f;
-  if (!from) 
+  if (!from)
     return NULL;
   if (type)
     f=get_storage( from, type );
@@ -320,8 +319,7 @@ void *get_pgdk2object(struct object *from, struct program *type) {
 }
 
 void pgtk2_destruct(struct object *o) {
-  struct object_wrapper *ow=
-	   (struct object_wrapper *)get_storage(o,pg2_object_program);
+  struct object_wrapper *ow=get_storage(o,pg2_object_program);
   if (ow) /* This should always be true. But let's add a check anyway. */
     ow->obj=NULL;
   if (o->refs>1)
@@ -343,28 +341,28 @@ void pgtk2_get_mapping_arg(struct mapping *map,
                           void *dest, long *mask, int len) {
   struct svalue *s;
   if ((s=simple_mapping_string_lookup(map,name))) {
-    if (s->type==type) {
+    if (TYPEOF(*s) == type) {
       switch(type) {
        case PIKE_T_STRING:
 #ifdef PIKE_DEBUG
          if (len!=sizeof(char *))
            Pike_fatal("oddities detected\n");
 #endif
-         MEMCPY(((char **)dest),&s->u.string->str,sizeof(char *));
+         memcpy(dest,&s->u.string->str,sizeof(char *));
          break;
        case PIKE_T_INT:
          if (len==2) {
            short i=(short)s->u.integer;
-           MEMCPY(((short *)dest),&i,2);
+           memcpy(dest,&i,2);
          } else if (len==4)
-           MEMCPY(((int *)dest),&s->u.integer,len);
+           memcpy(dest,&s->u.integer,len);
          break;
        case PIKE_T_FLOAT:
          if (len==sizeof(FLOAT_TYPE))
-           MEMCPY(((FLOAT_TYPE *)dest),&s->u.float_number,len);
+           memcpy(dest,&s->u.float_number,len);
          else if (len==sizeof(double)) {
            double d=s->u.float_number;
-           MEMCPY(((double *)dest),&d,len);
+           memcpy(dest,&d,len);
          }
          break;
       }
@@ -393,7 +391,7 @@ struct my_pixel pgtk2_pixel_from_xpixel(unsigned int pix, GdkImage *i) {
   GdkColor * c;
   struct my_pixel res;
   int l;
-  if (!col) 
+  if (!col)
     col=gdk_colormap_get_system();
   *((int  *)&res)=0;
   switch(i->visual->type) {
@@ -438,7 +436,7 @@ void push_atom(GdkAtom a) {
 void push_Xpseudo32bitstring(void *f, int nelems) {
   if (sizeof(long)!=4) {
     long *q=(long *)f;
-    int *res=(int *)xalloc(nelems*4),i;
+    int *res=xalloc(nelems*4),i;
     for (i=0; i<nelems; i++)
       res[i]=q[i];
     push_string(make_shared_binary_string2((const p_wchar2 *)res,nelems));
@@ -461,7 +459,7 @@ gint pgtk2_buttonfuncwrapper(GObject *obj, struct signal_data *d, void *foo) {
 
 void push_gdk_event(GdkEvent *e) {
   if (e) {
-    GdkEvent *f=g_malloc(sizeof(GdkEvent)); 
+    GdkEvent *f=g_malloc(sizeof(GdkEvent));
     if (f==NULL) {
       push_int(0);
       return;
@@ -529,31 +527,40 @@ static int pgtk2_push_int_param(const GValue *a) {
       retval=(LONGEST)g_value_get_uint64(a);
       break;
     case G_TYPE_INT:
-      retval=(LONGEST)g_value_get_int(a); 
-      break;
-    case G_TYPE_ENUM:
-      retval=(LONGEST)g_value_get_enum(a); 
+      retval=(LONGEST)g_value_get_int(a);
       break;
     case G_TYPE_FLAGS:
-      retval=(LONGEST)g_value_get_flags(a); 
+      retval=(LONGEST)g_value_get_flags(a);
       break;
     case G_TYPE_BOOLEAN:
-      retval=(LONGEST)g_value_get_boolean(a); 
+      retval=(LONGEST)g_value_get_boolean(a);
       break;
     case G_TYPE_LONG:
-      retval=(LONGEST)g_value_get_long(a); 
+      retval=(LONGEST)g_value_get_long(a);
       break;
     case G_TYPE_CHAR:
-      retval=(LONGEST)g_value_get_char(a); 
+#ifdef HAVE_G_VALUE_GET_SCHAR
+      retval=(LONGEST)g_value_get_schar(a);
+#else
+      retval=(LONGEST)g_value_get_char(a);
+#endif
       break;
     default:
-      retval=(LONGEST)g_value_get_uint(a); 
+      retval=(LONGEST)g_value_get_uint(a);
       break;
   }
   push_int64(retval);
   return PUSHED_VALUE;
 }
 
+static int pgtk2_push_enum_param(const GValue *a) {
+  /* This can't be handled by push_int_param as the type of
+  an enumeration is some subclass of G_TYPE_ENUM, rather than
+  actually being G_TYPE_ENUM exactly. */
+  push_int64((LONGEST)g_value_get_enum(a));
+  return PUSHED_VALUE;
+}
+
 static int pgtk2_push_float_param(const GValue *a) {
   FLOAT_TYPE retval;
   if (G_VALUE_TYPE(a)==G_TYPE_FLOAT)
@@ -586,6 +593,9 @@ static int pgtk2_push_object_param(const GValue *a) {
       push_gdkobject(gp,rectangle,0);
     } else if (G_VALUE_HOLDS(a,g_type_from_name("GdkRegion"))) {
       push_gdkobject(gp,region,0);
+    } else {
+      /* Don't know how to push this sort of object, so push its name */
+      PGTK_PUSH_GCHAR(G_VALUE_TYPE_NAME(a));
     }
   } else {
     obj=g_value_get_object(a);
@@ -596,9 +606,7 @@ static int pgtk2_push_object_param(const GValue *a) {
 }
 
 static int pgtk2_push_pike_object_param(const GValue *a) {
-  struct object *o=g_value_get_pointer(a);
-  if (o)
-    ref_push_object(o);
+  push_int64((LONGEST)g_value_get_pointer(a));
   return PUSHED_VALUE;
 }
 
@@ -633,7 +641,7 @@ static void build_push_callbacks() {
   CB(GTK_TYPE_TREE_ITER,	pgtk2_push_object_param);
   CB(GTK_TYPE_TREE_MODEL,	pgtk2_push_object_param);
   CB(PANGO_TYPE_ATTR_LIST,	pgtk2_push_object_param);
-  CB(GTK_TYPE_TREE_PATH,	pgtk2_push_object_param); 
+  CB(GTK_TYPE_TREE_PATH,	pgtk2_push_object_param);
   CB(PANGO_TYPE_FONT_DESCRIPTION,	pgtk2_push_object_param);
   CB(PANGO_TYPE_CONTEXT,	pgtk2_push_object_param);
   CB(PANGO_TYPE_LAYOUT,		pgtk2_push_object_param);
@@ -656,7 +664,7 @@ static void build_push_callbacks() {
   CB( G_TYPE_INT,              pgtk2_push_int_param );
   CB( G_TYPE_INT64,              pgtk2_push_int_param );
   CB( G_TYPE_UINT64,              pgtk2_push_int_param );
-  CB( G_TYPE_ENUM,             pgtk2_push_int_param );
+  CB( G_TYPE_ENUM,             pgtk2_push_enum_param );
   CB( G_TYPE_FLAGS,            pgtk2_push_int_param );
   CB( G_TYPE_BOOLEAN,          pgtk2_push_int_param );
   CB( G_TYPE_UINT,             pgtk2_push_int_param );
@@ -664,8 +672,8 @@ static void build_push_callbacks() {
   CB( G_TYPE_ULONG,            pgtk2_push_int_param );
   CB( G_TYPE_CHAR,             pgtk2_push_int_param );
 
-  CB( G_TYPE_NONE,    NULL ); 
-  
+  CB( G_TYPE_NONE,    NULL );
+
   CB( G_TYPE_POINTER,  pgtk2_push_pike_object_param );
 
   CB( G_TYPE_PARAM, pgtk2_push_gparamspec_param );
@@ -787,7 +795,7 @@ void pgtk2_free_signal_data(struct signal_data *s, GClosure *gcl) {
 
 void pgtk2_push_gchar(const gchar *s) {
   if (s) {
-    push_text(s); 
+    push_text(s);
     push_int(1);
     f_utf8_to_string(2);
   } else
@@ -821,7 +829,7 @@ void pgtk2_get_string_arg_with_sprintf( INT32 args )
   if( args < 1 )
     Pike_error("Too few arguments, %d required, got %d\n", 1, args);
 
-  if( Pike_sp[-args].type != PIKE_T_STRING )
+  if( TYPEOF(Pike_sp[-args]) != PIKE_T_STRING )
     Pike_error("Illegal argument %d, expected string\n", 0);
 
   if( args > 1 )
@@ -832,7 +840,7 @@ void pgtk2_get_string_arg_with_sprintf( INT32 args )
 
 void pgtk2_default__sprintf(int args, int offset, int len) {
   int mode = 0;
-  if (args>0 && Pike_sp[-args].type==PIKE_T_INT)
+  if (args>0 && TYPEOF(Pike_sp[-args]) == PIKE_T_INT)
     mode=Pike_sp[-args].u.integer;
   pgtk2_pop_n_elems(args);
   if (mode!='O') {
@@ -843,7 +851,7 @@ void pgtk2_default__sprintf(int args, int offset, int len) {
 }
 
 void pgtk2_clear_obj_struct(struct object *o) {
-  MEMSET(Pike_fp->current_storage,0,sizeof(struct object_wrapper));
+  memset(Pike_fp->current_storage,0,sizeof(struct object_wrapper));
 }
 
 void pgtk2_setup_mixin(struct object *o, struct program *p) {
@@ -855,7 +863,7 @@ void pgtk2_setup_mixin(struct object *o, struct program *p) {
 }
 
 LONGEST pgtk2_get_int(struct svalue *s) {
-  if (s->type==PIKE_T_INT)
+  if (TYPEOF(*s) == PIKE_T_INT)
     return s->u.integer;
 #ifdef AUTO_BIGNUM
   if (is_bignum_object_in_svalue(s)) {
@@ -864,14 +872,14 @@ LONGEST pgtk2_get_int(struct svalue *s) {
     return res;
   }
 #endif
-  if (s->type==PIKE_T_FLOAT)
+  if (TYPEOF(*s) == PIKE_T_FLOAT)
     return (LONGEST)s->u.float_number;
   return 0;
 }
 
 int pgtk2_is_int(struct svalue *s) {
-  return ((s->type==PIKE_T_INT) ||
-          (s->type==PIKE_T_FLOAT)
+  return ((TYPEOF(*s) == PIKE_T_INT) ||
+          (TYPEOF(*s) == PIKE_T_FLOAT)
 #ifdef AUTO_BIGNUM
           || is_bignum_object_in_svalue(s)
 #endif
@@ -880,15 +888,16 @@ int pgtk2_is_int(struct svalue *s) {
 
 /* double should be enough */
 double pgtk2_get_float(struct svalue *s) {
-  if (s->type==PIKE_T_FLOAT)
+  if (TYPEOF(*s) == PIKE_T_FLOAT)
     return s->u.float_number;
-  if (s->type==PIKE_T_INT)
+  if (TYPEOF(*s) == PIKE_T_INT)
     return (double)s->u.integer;
 #ifdef AUTO_BIGNUM
   if (is_bignum_object_in_svalue(s)) {
     FLOAT_TYPE f;
-    push_text("float");
-    apply(s->u.object,"cast",1);
+    ref_push_type_value(float_type_string);
+    stack_swap();
+    f_cast();
     f=Pike_sp[-1].u.float_number;
     pop_stack();
     return (double)f;
@@ -902,8 +911,8 @@ void pgtk2_free_object(struct object *o) {
 }
 
 int pgtk2_is_float(struct svalue *s) {
-  return ((s->type==PIKE_T_FLOAT) ||
-          (s->type==PIKE_T_INT)
+  return ((TYPEOF(*s) == PIKE_T_FLOAT) ||
+          (TYPEOF(*s) == PIKE_T_INT)
 #ifdef AUTO_BIGNUM
           || is_bignum_object_in_svalue(s)
 #endif
@@ -919,7 +928,7 @@ void pgtk2_set_property(GObject *g, char *prop, struct svalue *sv) {
   if (!(gps->flags & G_PARAM_WRITABLE))
     Pike_error("This property is not writable.\n");
 /*
-  if (gps->value_type==PANGO_TYPE_STYLE || 
+  if (gps->value_type==PANGO_TYPE_STYLE ||
 	gps->value_type==GTK_TYPE_WRAP_MODE ||
 	gps->value_type==GTK_TYPE_JUSTIFICATION ||
 	gps->value_type==PANGO_TYPE_UNDERLINE ||
@@ -928,7 +937,7 @@ void pgtk2_set_property(GObject *g, char *prop, struct svalue *sv) {
     return;
   }
 */
-  if (sv->type==PIKE_T_OBJECT) {
+  if (TYPEOF(*sv) == PIKE_T_OBJECT) {
     GObject *go=get_gobject(sv->u.object);
     if (go && G_IS_OBJECT(go)) {
       if (gps->value_type==GDK_TYPE_PIXMAP || gps->value_type==GTK_TYPE_WIDGET)
@@ -1176,7 +1185,7 @@ void pgtk2_set_gvalue(GValue *gv, GType gt, struct svalue *sv) {
       gt==GDK_TYPE_PIXBUF || gt==GDK_TYPE_PIXMAP || gt==GDK_TYPE_IMAGE ||
       gt==GDK_TYPE_WINDOW || gt==GDK_TYPE_VISUAL ||
       gt==GDK_TYPE_DRAWABLE || gt==GDK_TYPE_GC) {
-    if (sv->type==PIKE_T_OBJECT) {
+    if (TYPEOF(*sv) == PIKE_T_OBJECT) {
       GObject *go;
       go=get_gobject(sv->u.object);
       if (go && G_IS_OBJECT(go))
@@ -1185,12 +1194,12 @@ void pgtk2_set_gvalue(GValue *gv, GType gt, struct svalue *sv) {
     }
   }
   if (gt==GDK_TYPE_COLOR) {
-    if (sv->type==PIKE_T_OBJECT && get_gdkobject(sv->u.object,color))
+    if (TYPEOF(*sv) == PIKE_T_OBJECT && get_gdkobject(sv->u.object,color))
       g_value_set_boxed(gv,get_gdkobject(sv->u.object,color));
     return;
   }
   if (gt==GDK_TYPE_RECTANGLE) {
-    if (sv->type==PIKE_T_OBJECT && get_gdkobject(sv->u.object,rectangle))
+    if (TYPEOF(*sv) == PIKE_T_OBJECT && get_gdkobject(sv->u.object,rectangle))
       g_value_set_boxed(gv,get_gdkobject(sv->u.object,rectangle));
     return;
   }
@@ -1202,7 +1211,11 @@ void pgtk2_set_gvalue(GValue *gv, GType gt, struct svalue *sv) {
       g_value_set_uint(gv,(guint)PGTK_GETINT(sv));
       break;
     case G_TYPE_CHAR:
+#ifdef HAVE_G_VALUE_SET_SCHAR
+      g_value_set_schar(gv,(gchar)PGTK_GETINT(sv));
+#else
       g_value_set_char(gv,(gchar)PGTK_GETINT(sv));
+#endif
       break;
     case G_TYPE_UCHAR:
       g_value_set_uchar(gv,(guchar)PGTK_GETINT(sv));
@@ -1235,7 +1248,7 @@ void pgtk2_set_gvalue(GValue *gv, GType gt, struct svalue *sv) {
       g_value_set_double(gv,(gdouble)pgtk2_get_float(sv));
       break;
     case G_TYPE_STRING:
-      if (sv->type==PIKE_T_STRING) 
+      if (TYPEOF(*sv) == PIKE_T_STRING)
       {
 	  push_svalue( sv );
 	  f_string_to_utf8(1);
@@ -1245,7 +1258,7 @@ void pgtk2_set_gvalue(GValue *gv, GType gt, struct svalue *sv) {
 	g_value_set_string(gv,"");
       break;
     case G_TYPE_OBJECT:
-      if (sv->type==PIKE_T_OBJECT) {
+      if (TYPEOF(*sv) == PIKE_T_OBJECT) {
 	GObject *go=get_gobject(sv->u.object);
 	if (go && G_IS_OBJECT(go))
 	  g_value_set_object(gv,go);
@@ -1255,9 +1268,9 @@ void pgtk2_set_gvalue(GValue *gv, GType gt, struct svalue *sv) {
 	g_value_set_object(gv,NULL);
       break;
     case G_TYPE_POINTER:
-      if (sv->type==PIKE_T_OBJECT) {
+      if (TYPEOF(*sv) == PIKE_T_OBJECT) {
 	g_value_set_pointer(gv,sv->u.object);
- 	add_ref(sv->u.object); 
+ 	add_ref(sv->u.object);
       } else
 	g_value_set_pointer(gv,NULL);
       break;
@@ -1296,7 +1309,7 @@ GObject *pgtk2_create_new_obj_with_properties(GType type, struct mapping *m) {
     Pike_error("Could not get a reference to type %s.\n",g_type_name(type));
   params=g_new0(GParameter,m_sizeof(m));
   NEW_MAPPING_LOOP(m->data) {
-    if (k->ind.type==PIKE_T_STRING) {
+    if (TYPEOF(k->ind) == PIKE_T_STRING) {
       gchar *s=PGTK_GETSTR(&k->ind);
       pspec=g_object_class_find_property(class,s);
       if (!pspec) {
@@ -1326,7 +1339,7 @@ void pgtk2_marshaller(GClosure *closure,
 		      gpointer marshal_data) {
   typedef void (*pgtk2_marshal_func)(gpointer data1,
 				     gpointer data2,
-				     guint n_params, 
+				     guint n_params,
 				     const GValue *param_values,
 				     GValue *return_value);
   register pgtk2_marshal_func callback;
@@ -1489,7 +1502,7 @@ void pgtk2_get_doc(GObject *o, struct svalue *dest) {
   push_string(make_shared_binary_string(str->str,str->len));
   g_string_free(str,TRUE);
   if (dest) {
-    assign_svalue_no_free(dest, &Pike_sp[-1]);
+    assign_svalue_no_free(dest, Pike_sp - 1);
     pop_stack();
   }
 }
