@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 5120f610ca35e1304d301a8d191aa4dd9ea3a735 $
 */
 
 #include "global.h"
@@ -58,13 +57,13 @@ struct source *source_pikestring_make( struct svalue *s,
 {
   struct ps_source *res;
 
-  if( s->type != PIKE_T_STRING )   return 0;
+  if( TYPEOF(*s) != PIKE_T_STRING )   return 0;
   if( s->u.string->size_shift )    return 0;
 
-  res = malloc( sizeof( struct ps_source ) );
+  res = calloc( 1, sizeof( struct ps_source ) );
+  if( !res ) return NULL;
   debug_malloc_touch( res );
   debug_malloc_touch( s );
-  MEMSET( res, 0, sizeof( struct ps_source ) );
 
   res->s.free_source = free_source;
   res->s.get_data = get_data;
