@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: c01896599e3327a625487d529aa5210474b39539 $
 */
 
 #include "global.h"
@@ -18,6 +17,7 @@
 #include "pike_float.h"
 #include "pike_types.h"
 #include "sscanf.h"
+#include "bitvector.h"
 
 #define sp Pike_sp
 
@@ -128,9 +128,7 @@ static void *pcharp_to_svalue_percent_o(struct svalue *res,
 					PCHARP str,
 					ptrdiff_t len)
 {
-  res->type = T_INT;
-  res->subtype = NUMBER_UNDEFINED;
-  res->u.integer = 0;
+  SET_SVAL(*res, T_INT, NUMBER_UNDEFINED, integer, 0);
 
   for (;len>0; INC_PCHARP(str, 1), len--)
   {
@@ -173,9 +171,7 @@ static void *pcharp_to_svalue_percent_o(struct svalue *res,
 	    INC_PCHARP(str, 1);	/* Skip the quote. */
 
 	    if (cnt) string_builder_append(&tmp, start, cnt);
-	    res->type=T_STRING;
-	    res->subtype=0;
-	    res->u.string=finish_string_builder(&tmp);
+	    SET_SVAL(*res, T_STRING, 0, string, finish_string_builder(&tmp));
 	    return str.ptr;
 
 	  case '\\':
@@ -219,9 +215,7 @@ static void *pcharp_to_svalue_percent_o(struct svalue *res,
       {
 	INC_PCHARP(str, 1);	/* Skip the quote. */
 
-	res->subtype=0;
-	res->type=T_INT;
-	res->u.integer = EXTRACT_PCHARP(str);
+	SET_SVAL(*res, T_INT, NUMBER_NUMBER, integer, EXTRACT_PCHARP(str));
 	INC_PCHARP(str, 1);
 
 	len -= 2;
@@ -290,9 +284,9 @@ static ptrdiff_t PIKE_CONCAT(read_set,SIZE) (			\
   MATCH_IS_WIDE( int set_size=0; )				\
 								\
   if(cnt>=match_len)						\
-    Pike_error("Error in sscanf format string.\n");		\
+    Pike_error("Unterminated sscanf set.\n");			\
 								\
-  MEMSET(set->c, 0, sizeof(set->c));				\
+  memset(set->c, 0, sizeof(set->c));				\
   set->a=0;							\
 								\
   if(match[cnt]=='^' &&						\
@@ -302,7 +296,7 @@ static ptrdiff_t PIKE_CONCAT(read_set,SIZE) (			\
     set->neg=1;							\
     cnt++;							\
     if(cnt>=match_len)						\
-      Pike_error("Error in sscanf format string.\n");		\
+      Pike_error("Unterminated negated sscanf set.\n");	\
   }else{							\
     set->neg=0;							\
   }								\
@@ -312,7 +306,7 @@ static ptrdiff_t PIKE_CONCAT(read_set,SIZE) (			\
     set->c[last=match[cnt]]=1;					\
     cnt++;							\
     if(cnt>=match_len)						\
-      Pike_error("Error in sscanf format string.\n");		\
+      Pike_error("Empty sscanf range.\n");			\
   }								\
 								\
   for(;match[cnt]!=']';)					\
@@ -321,7 +315,7 @@ static ptrdiff_t PIKE_CONCAT(read_set,SIZE) (			\
     {								\
       cnt++;							\
       if(cnt>=match_len)					\
-	Pike_error("Error in sscanf format string.\n");		\
+	Pike_error("Unterminated sscanf range.\n");		\
 								\
       if(match[cnt]==']')					\
       {								\
@@ -330,7 +324,8 @@ static ptrdiff_t PIKE_CONCAT(read_set,SIZE) (			\
       }								\
 								\
       if(last > match[cnt])					\
-	Pike_error("Error in sscanf format string.\n");		\
+	Pike_error("Inverted sscanf range [%c-%c].\n",		\
+		   last, match[cnt]);				\
 								\
 MATCH_IS_WIDE(							\
       if(last < (p_wchar2)sizeof(set->c) && last >= 0)		\
@@ -378,7 +373,7 @@ MATCH_IS_WIDE(							\
     }								\
     cnt++;							\
     if(cnt>=match_len)						\
-      Pike_error("Error in sscanf format string.\n");		\
+      Pike_error("Unterminated sscanf set.\n");			\
   }								\
 								\
 MATCH_IS_WIDE(							\
@@ -393,13 +388,13 @@ MATCH_IS_WIDE(							\
          order[e+1]+1 != order[e]) {				\
         free_array(set->a);					\
         set->a=0;						\
-        free((char *)order);					\
+        free(order);                                            \
 	Pike_error("Overlapping ranges in sscanf not supported.\n"); \
       }								\
     }								\
 								\
     order_array(set->a,order);					\
-    free((char *)order);					\
+    free(order);                                                \
   }								\
 )								\
   return cnt;							\
@@ -465,108 +460,101 @@ static INLINE FLOAT_TYPE low_parse_IEEE_float(char *b, int sz)
   r = (double)f;
   if(extra_f)
     r += ((double)extra_f)/4294967296.0;
-  return (FLOAT_TYPE)(s? -LDEXP(r, e):LDEXP(r, e));
+  return (FLOAT_TYPE)(s? -ldexp(r, e):ldexp(r, e));
 }
 
 #endif
 
-#ifdef PIKE_DEBUG
-#define DO_IF_DEBUG(X)		X
-#else /* !PIKE_DEBUG */
-#define DO_IF_DEBUG(X)
-#endif /* PIKE_DEBUG */
-
-#ifdef AUTO_BIGNUM
-#define DO_IF_BIGNUM(X)		X
-#else /* !AUTO_BIGNUM */
-#define DO_IF_BIGNUM(X)
-#endif /* AUTO_BIGNUM */
-
-#ifdef __CHECKER__
-#define DO_IF_CHECKER(X)	X
-#else /* !__CHECKER__ */
-#define DO_IF_CHECKER(X)
-#endif /* __CHECKER__ */
+static float extract_float_be(const char * x) {
+    float f;
 
 #ifdef FLOAT_IS_IEEE_BIG
-#define EXTRACT_FLOAT(SVAL, INPUT, SHIFT)		\
-	    do {					\
-	      float f;					\
-	      ((char *)&f)[0] = *((INPUT));		\
-	      ((char *)&f)[1] = *((INPUT)+1);		\
-	      ((char *)&f)[2] = *((INPUT)+2);		\
-	      ((char *)&f)[3] = *((INPUT)+3);		\
-	      (SVAL).u.float_number = f;		\
-	    } while(0)
+    memcpy(&f, x, sizeof(f));
+#elif FLOAT_IS_IEEE_LITTLE
+    unsigned INT32 tmp = get_unaligned32(x);
+    tmp = bswap32(tmp);
+    memcpy(&f, &tmp, sizeof(f));
 #else
-#ifdef FLOAT_IS_IEEE_LITTLE
-#define EXTRACT_FLOAT(SVAL, INPUT, SHIFT)		\
-	    do {					\
-	      float f;					\
-	      ((char *)&f)[3] = *((INPUT));		\
-	      ((char *)&f)[2] = *((INPUT)+1);		\
-	      ((char *)&f)[1] = *((INPUT)+2);		\
-	      ((char *)&f)[0] = *((INPUT)+3);		\
-	      (SVAL).u.float_number = f;		\
-	    } while(0)
-#else
-#define EXTRACT_FLOAT(SVAL, INPUT, SHIFT)				\
-	    do {							\
-	      char x[4];						\
-	      x[0] = (INPUT)[0];					\
-	      x[1] = (INPUT)[1];					\
-	      x[2] = (INPUT)[2];					\
-	      x[3] = (INPUT)[3];					\
-	      (SVAL).u.float_number = low_parse_IEEE_float(x, 4);	\
-            } while(0)
-#endif
+    f = low_parse_IEEE_float(x, 4);
 #endif
+    return f;
+}
 
-#ifdef DOUBLE_IS_IEEE_BIG
-#define EXTRACT_DOUBLE(SVAL, INPUT, SHIFT)		\
-	    do {					\
-	      double d;					\
-	      ((char *)&d)[0] = *((INPUT));		\
-	      ((char *)&d)[1] = *((INPUT)+1);		\
-	      ((char *)&d)[2] = *((INPUT)+2);		\
-	      ((char *)&d)[3] = *((INPUT)+3);		\
-	      ((char *)&d)[4] = *((INPUT)+4);		\
-	      ((char *)&d)[5] = *((INPUT)+5);		\
-	      ((char *)&d)[6] = *((INPUT)+6);		\
-	      ((char *)&d)[7] = *((INPUT)+7);		\
-	      (SVAL).u.float_number = (FLOAT_TYPE)d;	\
-	    } while(0)
+static double extract_double_be(const char * x) {
+    double f;
+
+#ifdef FLOAT_IS_IEEE_BIG
+    memcpy(&f, x, sizeof(f));
+#elif FLOAT_IS_IEEE_LITTLE
+    unsigned INT64 tmp = get_unaligned64(x);
+    tmp = bswap64(tmp);
+    memcpy(&f, &tmp, sizeof(f));
 #else
-#ifdef DOUBLE_IS_IEEE_LITTLE
-#define EXTRACT_DOUBLE(SVAL, INPUT, SHIFT)		\
-	    do {					\
-	      double d;					\
-	      ((char *)&d)[7] = *((INPUT));		\
-	      ((char *)&d)[6] = *((INPUT)+1);		\
-	      ((char *)&d)[5] = *((INPUT)+2);		\
-	      ((char *)&d)[4] = *((INPUT)+3);		\
-	      ((char *)&d)[3] = *((INPUT)+4);		\
-	      ((char *)&d)[2] = *((INPUT)+5);		\
-	      ((char *)&d)[1] = *((INPUT)+6);		\
-	      ((char *)&d)[0] = *((INPUT)+7);		\
-	      (SVAL).u.float_number = (FLOAT_TYPE)d;	\
-	    } while(0)
+    f = low_parse_IEEE_float(x, 8);
+#endif
+    return f;
+}
+
+static float extract_float_le(const char * x) {
+    float f;
+
+#ifdef FLOAT_IS_IEEE_LITTLE
+    memcpy(&f, x, sizeof(f));
+#elif FLOAT_IS_IEEE_BIG
+    unsigned INT32 tmp = get_unaligned32(x);
+    tmp = bswap32(tmp);
+    memcpy(&f, &tmp, sizeof(f));
 #else
-#define EXTRACT_DOUBLE(SVAL, INPUT, SHIFT)				\
-	    do {							\
-	      char x[8];						\
-	      x[0] = (INPUT)[0];					\
-	      x[1] = (INPUT)[1];					\
-	      x[2] = (INPUT)[2];					\
-	      x[3] = (INPUT)[3];					\
-	      x[4] = (INPUT)[4];					\
-	      x[5] = (INPUT)[5];					\
-	      x[6] = (INPUT)[6];					\
-	      x[7] = (INPUT)[7];					\
-	      (SVAL).u.float_number = low_parse_IEEE_float(x, 8);	\
-	    } while(0)
+    unsigned INT32 tmp = get_unaligned32(x);
+    tmp = bswap32(tmp);
+    f = low_parse_IEEE_float((char*)&tmp, sizeof(tmp));
 #endif
+    return f;
+}
+
+static double extract_double_le(const char * x) {
+    double f;
+
+#ifdef FLOAT_IS_IEEE_LITTLE
+    memcpy(&f, x, sizeof(f));
+#elif FLOAT_IS_IEEE_BIG
+    unsigned INT64 tmp = get_unaligned64(x);
+    tmp = bswap64(tmp);
+    memcpy(&f, &tmp, sizeof(f));
+#else
+    unsigned INT64 tmp = get_unaligned64(x);
+    tmp = bswap64(tmp);
+    f = low_parse_IEEE_float((char*)&tmp, sizeof(tmp));
 #endif
+    return f;
+}
+
+#define EXTRACT_FLOAT(SVAL, input, shift, fun)    do {      \
+    char x[4];                                              \
+    if (shift == 0) {                                       \
+        memcpy(x, input, sizeof(x));                        \
+    } else {                                                \
+        PCHARP tmp = MKPCHARP(input, shift);                \
+        size_t i;                                           \
+        for (i = 0; i < sizeof(x); INC_PCHARP(tmp, 1), i++) \
+            x[i] = EXTRACT_PCHARP(tmp);                     \
+    }                                                       \
+    (SVAL).u.float_number = fun(x);                         \
+} while (0)
+
+#define EXTRACT_DOUBLE(SVAL, input, shift, fun)   do {      \
+    char x[8];                                              \
+    if (shift == 0) {                                       \
+        memcpy(x, input, sizeof(x));                        \
+    } else {                                                \
+        PCHARP tmp = MKPCHARP(input, shift);                \
+        size_t i;                                           \
+        for (i = 0; i < sizeof(x); INC_PCHARP(tmp, 1), i++) \
+            x[i] = EXTRACT_PCHARP(tmp);                     \
+    }                                                       \
+    (SVAL).u.float_number = fun(x);                         \
+} while (0)
+
 
 /* Avoid some warnings about loss of precision */
 #ifdef __ECL
@@ -597,8 +585,8 @@ static INLINE INT32 TO_INT32(ptrdiff_t x)
  *   Pushes non-ignored matches on the Pike stack in the order they
  *   were matched.
  *
- * FIXME: chars_matched and success are only used internally, and
- *        should probably be gotten rid of.
+ * FIXME: success is only used internally, and should probably be
+ * gotten rid of.
  */
 #define MK_VERY_LOW_SSCANF(INPUT_SHIFT, MATCH_SHIFT)			 \
 static INT32 PIKE_CONCAT4(very_low_sscanf_,INPUT_SHIFT,_,MATCH_SHIFT)(	 \
@@ -650,7 +638,7 @@ static INT32 PIKE_CONCAT4(very_low_sscanf_,INPUT_SHIFT,_,MATCH_SHIFT)(	 \
     DO_IF_DEBUG(							 \
     if(match[cnt]!='%' || match[cnt+1]=='%')				 \
     {									 \
-      Pike_fatal("Error in sscanf.\n");					 \
+      Pike_fatal("Failed to escape in sscanf.\n");			 \
     }									 \
     );									 \
 									 \
@@ -663,7 +651,7 @@ static INT32 PIKE_CONCAT4(very_low_sscanf_,INPUT_SHIFT,_,MATCH_SHIFT)(	 \
 									 \
     cnt++;								 \
     if(cnt>=match_len)							 \
-      Pike_error("Error in sscanf format string.\n");			 \
+      Pike_error("Missing format specifier in sscanf format string.\n"); \
 									 \
     while(1)								 \
     {									 \
@@ -673,7 +661,8 @@ static INT32 PIKE_CONCAT4(very_low_sscanf_,INPUT_SHIFT,_,MATCH_SHIFT)(	 \
 	  no_assign=1;							 \
 	  cnt++;							 \
 	  if(cnt>=match_len)						 \
-	    Pike_error("Error in sscanf format string.\n");		 \
+	    Pike_error("Missing format specifier in ignored sscanf "	 \
+		       "format string.\n");				 \
 	  continue;							 \
 									 \
 	case '0': case '1': case '2': case '3': case '4':		 \
@@ -722,8 +711,7 @@ static INT32 PIKE_CONCAT4(very_low_sscanf_,INPUT_SHIFT,_,MATCH_SHIFT)(	 \
 	      }								 \
 	    }								 \
 	  }								 \
-	  sval.type=T_ARRAY;						 \
-	  sval.u.array=allocate_array(0);				 \
+	  SET_SVAL(sval, T_ARRAY, 0, array, allocate_array(0));		 \
 	  SET_ONERROR(err, do_free_array, sval.u.array);		 \
 									 \
 	  while(input_len-eye)						 \
@@ -758,8 +746,7 @@ static INT32 PIKE_CONCAT4(very_low_sscanf_,INPUT_SHIFT,_,MATCH_SHIFT)(	 \
 INPUT_IS_WIDE(								 \
           int e;							 \
 )									 \
-	  sval.type=T_INT;						 \
-	  sval.subtype=NUMBER_NUMBER;					 \
+          SET_SVAL(sval, T_INT, NUMBER_NUMBER, integer, 0);		 \
           if(field_length == -1)					 \
           { 								 \
 	    if(eye+1 > input_len)					 \
@@ -798,7 +785,6 @@ INPUT_IS_WIDE(								 \
 	     }								 \
 	     while(--field_length >= 0)					 \
 	     {								 \
-               DO_IF_BIGNUM(						 \
 	       if(INT_TYPE_LSH_OVERFLOW(sval.u.integer, 8))		 \
 	       {							 \
 		 push_int(sval.u.integer);				 \
@@ -815,7 +801,6 @@ INPUT_IS_WIDE(								 \
 		 sval=*--sp;						 \
 		 break;							 \
 	       }							 \
-	       );							 \
 	       sval.u.integer<<=8;					 \
 	       sval.u.integer |= input[--pos];				 \
 	     }								 \
@@ -825,7 +810,6 @@ INPUT_IS_WIDE(								 \
 	     }								 \
 	     while(--field_length >= 0)					 \
 	     {								 \
-               DO_IF_BIGNUM(						 \
 	       if(INT_TYPE_LSH_OVERFLOW(sval.u.integer, 8))		 \
 	       {							 \
 		 push_int(sval.u.integer);				 \
@@ -843,7 +827,6 @@ INPUT_IS_WIDE(								 \
 		 sval=*--sp;						 \
 		 break;							 \
 	       }							 \
-	       );							 \
 	       sval.u.integer<<=8;					 \
 	       sval.u.integer |= input[eye];				 \
 	       eye++;							 \
@@ -854,7 +837,7 @@ INPUT_IS_WIDE(								 \
  								         \
       case 'H':								\
 	{								\
-          int len=0;							\
+          unsigned long len=0;                                           \
           if(field_length == -1)					\
 	    field_length=1;						\
           if(field_length == 0)                                         \
@@ -891,7 +874,7 @@ INPUT_IS_WIDE(								 \
 	      eye++;							\
 	    }								\
 	  }								\
-	  if(eye+len > input_len)					\
+	  if(len > (unsigned long)(input_len-eye))                      \
 	  {								\
 	    chars_matched[0]=eye-field_length;				\
 	    return matches;						\
@@ -899,11 +882,9 @@ INPUT_IS_WIDE(								 \
 	  if (no_assign) {						\
 	    no_assign = 2;						\
 	  } else {							\
-	    sval.type=T_STRING;						\
-	    DO_IF_CHECKER(sval.subtype=0);				\
-	    sval.u.string=PIKE_CONCAT(make_shared_binary_string,	\
-				      INPUT_SHIFT)(input+eye,		\
-						   len);		\
+	    SET_SVAL(sval, T_STRING, 0, string,				\
+		     PIKE_CONCAT(make_shared_binary_string,		\
+				 INPUT_SHIFT)(input+eye, len));		\
 	  }								\
 	  eye+=len;							\
 	  break;							\
@@ -950,15 +931,15 @@ INPUT_IS_WIDE(								 \
 	{								 \
 	  PIKE_CONCAT(p_wchar, INPUT_SHIFT) *t;				 \
 	  PCHARP t2;							 \
+	  FLOAT_TYPE f;							 \
 									 \
 	  if(eye>=input_len)						 \
 	  {								 \
 	    chars_matched[0]=eye;					 \
 	    return matches;						 \
 	  }								 \
-	  sval.u.float_number =						 \
-	    (FLOAT_TYPE)STRTOD_PCHARP(MKPCHARP(input+eye,		 \
-					       INPUT_SHIFT),&t2);	 \
+	  f = (FLOAT_TYPE)STRTOD_PCHARP(MKPCHARP(input+eye,		 \
+						 INPUT_SHIFT),&t2);	 \
 	  t = (PIKE_CONCAT(p_wchar, INPUT_SHIFT) *)(t2.ptr);		 \
 	  if(input + eye == t)						 \
 	  {								 \
@@ -966,8 +947,7 @@ INPUT_IS_WIDE(								 \
 	    return matches;						 \
 	  }								 \
 	  eye=t-input;							 \
-	  sval.type=T_FLOAT;						 \
-	  DO_IF_CHECKER(sval.subtype=0);				 \
+	  SET_SVAL(sval, T_FLOAT, 0, float_number, f);			 \
 	  break;							 \
 	}								 \
 									 \
@@ -981,15 +961,16 @@ INPUT_IS_WIDE(								 \
 	    chars_matched[0]=eye;					 \
 	    return matches;						 \
 	  }								 \
-	  sval.type=T_FLOAT;						 \
-	  DO_IF_CHECKER(sval.subtype=0);				 \
+	  SET_SVAL(sval, T_FLOAT, 0, float_number, 0.0);		 \
 	  switch(field_length) {					 \
 	    case 4:							 \
-	      EXTRACT_FLOAT(sval, input+eye, INPUT_SHIFT);		 \
+              if (minus_flag) EXTRACT_FLOAT(sval, input+eye, INPUT_SHIFT, extract_float_le);    \
+              else EXTRACT_FLOAT(sval, input+eye, INPUT_SHIFT, extract_float_be);               \
 	      eye += 4;							 \
 	      break;							 \
 	    case 8:							 \
-	      EXTRACT_DOUBLE(sval, input+eye, INPUT_SHIFT);		 \
+              if (minus_flag) EXTRACT_DOUBLE(sval, input+eye, INPUT_SHIFT, extract_double_le);    \
+              else EXTRACT_DOUBLE(sval, input+eye, INPUT_SHIFT, extract_double_be);               \
 	      eye += 8;							 \
 	      break;							 \
 	  }								 \
@@ -1007,11 +988,10 @@ INPUT_IS_WIDE(								 \
 	    if (no_assign) {						 \
 	      no_assign = 2;						 \
 	    } else {							 \
-	      sval.type=T_STRING;					 \
-	      DO_IF_CHECKER(sval.subtype=0);				 \
-	      sval.u.string=PIKE_CONCAT(make_shared_binary_string,	 \
-					INPUT_SHIFT)(input+eye,		 \
-						     field_length);	 \
+	      SET_SVAL(sval, T_STRING, 0, string,			 \
+		       PIKE_CONCAT(make_shared_binary_string,		 \
+				   INPUT_SHIFT)(input+eye,		 \
+						field_length));		 \
 	    }								 \
 	    eye+=field_length;						 \
 	    break;							 \
@@ -1022,11 +1002,10 @@ INPUT_IS_WIDE(								 \
 	    if (no_assign) {						 \
 	      no_assign = 2;						 \
 	    } else {							 \
-	      sval.type=T_STRING;					 \
-	      DO_IF_CHECKER(sval.subtype=0);				 \
-	      sval.u.string=PIKE_CONCAT(make_shared_binary_string,	 \
-					INPUT_SHIFT)(input+eye,		 \
-						     input_len-eye);	 \
+	      SET_SVAL(sval, T_STRING, 0, string,			 \
+		       PIKE_CONCAT(make_shared_binary_string,		 \
+				   INPUT_SHIFT)(input+eye,		 \
+						input_len-eye));	 \
 	    }								 \
 	    eye=input_len;						 \
 	    break;							 \
@@ -1069,30 +1048,30 @@ INPUT_IS_WIDE(								 \
 									 \
 	  /* sscanf("foo-bar","%s%d",a,b) might not work as expected */	 \
 		case 'd':						 \
-		  MEMSET(set.c, 1, sizeof(set.c));			 \
+		  memset(set.c, 1, sizeof(set.c));			 \
 		  for(e='0';e<='9';e++) set.c[e]=0;			 \
 		  set.c['-']=0;						 \
 		  goto match_set;					 \
 									 \
 		case 'o':						 \
-		  MEMSET(set.c, 1, sizeof(set.c));			 \
+		  memset(set.c, 1, sizeof(set.c));			 \
 		  for(e='0';e<='7';e++) set.c[e]=0;			 \
 		  goto match_set;					 \
 									 \
 		case 'x':						 \
-		  MEMSET(set.c, 1, sizeof(set.c));			 \
+		  memset(set.c, 1, sizeof(set.c));			 \
 		  for(e='0';e<='9';e++) set.c[e]=0;			 \
 		  for(e='a';e<='f';e++) set.c[e]=0;			 \
 		  goto match_set;					 \
 									 \
 		case 'D':						 \
-		  MEMSET(set.c, 1, sizeof(set.c));			 \
+		  memset(set.c, 1, sizeof(set.c));			 \
 		  for(e='0';e<='9';e++) set.c[e]=0;			 \
 		  set.c['-']=0;						 \
 		  goto match_set;					 \
 									 \
 		case 'f':						 \
-		  MEMSET(set.c, 1, sizeof(set.c));			 \
+		  memset(set.c, 1, sizeof(set.c));			 \
 		  for(e='0';e<='9';e++) set.c[e]=0;			 \
 		  set.c['.']=set.c['-']=0;				 \
 		  goto match_set;					 \
@@ -1129,11 +1108,10 @@ INPUT_IS_WIDE(								 \
 	      if (no_assign) {						 \
 		no_assign = 2;						 \
 	      } else {							 \
-		sval.type=T_STRING;					 \
-		DO_IF_CHECKER(sval.subtype=0);				 \
-		sval.u.string=PIKE_CONCAT(make_shared_binary_string,	 \
-					  INPUT_SHIFT)(input+eye,	 \
-						       input_len-eye);	 \
+		SET_SVAL(sval, T_STRING, 0, string,			 \
+			 PIKE_CONCAT(make_shared_binary_string,		 \
+				     INPUT_SHIFT)(input+eye,		 \
+						  input_len-eye));	 \
 	      }								 \
 	      eye=input_len;						 \
 	      break;							 \
@@ -1178,11 +1156,10 @@ INPUT_IS_WIDE(								 \
 	    if (no_assign) {						 \
 	      no_assign = 2;						 \
 	    } else {							 \
-	      sval.type=T_STRING;					 \
-	      DO_IF_CHECKER(sval.subtype=0);				 \
-	      sval.u.string=PIKE_CONCAT(make_shared_binary_string,	 \
-					INPUT_SHIFT)(input+start,	 \
-						     eye-start);	 \
+	      SET_SVAL(sval, T_STRING, 0, string,			 \
+		       PIKE_CONCAT(make_shared_binary_string,		 \
+				   INPUT_SHIFT)(input+start,		 \
+						eye-start));		 \
 	    }								 \
 									 \
 	    cnt=end_str_end-match-1;					 \
@@ -1219,9 +1196,8 @@ INPUT_IS_WIDE(								 \
 	        {							 \
 		  INT32 x;						 \
 		  struct svalue tmp;					 \
-		  tmp.type=T_INT;					 \
-		  tmp.subtype = NUMBER_NUMBER;				\
-		  tmp.u.integer=input[eye];				 \
+		  SET_SVAL(tmp, T_INT, NUMBER_NUMBER,			 \
+			   integer, input[eye]);			 \
 		  x=switch_lookup(set.a, &tmp);				 \
 		  if( set.neg != (x<0 && (x&1)) ) break;		 \
 	        }else{							 \
@@ -1240,10 +1216,9 @@ INPUT_IS_WIDE(								 \
 	  if (no_assign) {						 \
 	    no_assign = 2;						 \
 	  } else {							 \
-	    sval.type=T_STRING;						 \
-	    DO_IF_CHECKER(sval.subtype=0);				 \
-	    sval.u.string=PIKE_CONCAT(make_shared_binary_string,	 \
-				      INPUT_SHIFT)(input+e,eye-e);	 \
+	    SET_SVAL(sval, T_STRING, 0, string,				 \
+		     PIKE_CONCAT(make_shared_binary_string,		 \
+				 INPUT_SHIFT)(input+e,eye-e));		 \
 	  }								 \
 	  break;							 \
 									 \
@@ -1274,9 +1249,8 @@ INPUT_IS_WIDE(								 \
         }								 \
 									 \
 	case 'n':							 \
-	  sval.type=T_INT;						 \
-	  sval.subtype=NUMBER_NUMBER;					 \
-	  sval.u.integer=TO_INT32(eye - truncated);			 \
+	  SET_SVAL(sval, T_INT, NUMBER_NUMBER, integer,			 \
+		   TO_INT32(eye - truncated));				 \
 	  break;							 \
 									 \
 	default:							 \
@@ -1345,6 +1319,49 @@ MK_VERY_LOW_SSCANF(2,1)
 MK_VERY_LOW_SSCANF(1,2)
 MK_VERY_LOW_SSCANF(2,2)
 
+
+/* */
+INT32 low_sscanf_pcharp(PCHARP input, ptrdiff_t len,
+                        PCHARP format, ptrdiff_t format_len,
+                        ptrdiff_t *chars_matched,
+                        int flags )
+{
+  int ok;
+  check_c_stack(sizeof(struct sscanf_set)*2 + 512);
+  switch( input.shift*3 + format.shift )
+  {
+    case 0:
+      return very_low_sscanf_0_0(input.ptr, len,format.ptr, format_len,
+                                 chars_matched, &ok, flags  );
+    case 1:
+      return very_low_sscanf_0_1(input.ptr, len, format.ptr, format_len,
+                                 chars_matched, &ok, flags  );
+    case 2:
+      return very_low_sscanf_0_2(input.ptr, len, format.ptr, format_len,
+                                 chars_matched, &ok, flags  );
+    case 3:
+      return very_low_sscanf_1_0(input.ptr, len, format.ptr, format_len,
+                                 chars_matched, &ok, flags  );
+    case 4:
+      return very_low_sscanf_1_1(input.ptr, len, format.ptr, format_len,
+                                 chars_matched, &ok, flags  );
+    case 5:
+      return very_low_sscanf_1_2(input.ptr, len, format.ptr, format_len,
+                                 chars_matched, &ok, flags  );
+    case 6:
+      return very_low_sscanf_2_0(input.ptr, len, format.ptr, format_len,
+                                 chars_matched, &ok, flags  );
+    case 7:
+      return very_low_sscanf_2_1(input.ptr, len, format.ptr, format_len,
+                                 chars_matched, &ok, flags  );
+    case 8:
+      return very_low_sscanf_2_2(input.ptr, len, format.ptr, format_len,
+                                 chars_matched, &ok, flags );
+    default:
+      Pike_error("impossible");
+  }
+}
+
 /* Simplified interface to very_low_sscanf_{0,1,2}_{0,1,2}(). */
 INT32 low_sscanf(struct pike_string *data, struct pike_string *format, INT32 flags)
 {
@@ -1589,10 +1606,10 @@ void o_sscanf(INT32 args, INT32 flags)
   int x;
   struct svalue *save_sp=sp;
 
-  if(sp[-args].type != T_STRING)
+  if(TYPEOF(sp[-args]) != T_STRING)
     SIMPLE_BAD_ARG_ERROR("sscanf", 1, "string");
 
-  if(sp[1-args].type != T_STRING)
+  if(TYPEOF(sp[1-args]) != T_STRING)
     SIMPLE_BAD_ARG_ERROR("sscanf", 2, "string");
 
   i = low_sscanf(sp[-args].u.string, sp[1-args].u.string, flags);
@@ -1855,13 +1872,13 @@ void f___handle_sscanf_format(INT32 args)
 #endif /* 0 */
   if (args != 4)
     SIMPLE_WRONG_NUM_ARGS_ERROR("__handle_sscanf_format", 4);
-  if (Pike_sp[-4].type != PIKE_T_STRING)
+  if (TYPEOF(Pike_sp[-4]) != PIKE_T_STRING)
     SIMPLE_ARG_TYPE_ERROR("__handle_sscanf_format", 1, "string");
-  if (Pike_sp[-3].type != PIKE_T_STRING)
+  if (TYPEOF(Pike_sp[-3]) != PIKE_T_STRING)
     SIMPLE_ARG_TYPE_ERROR("__handle_sscanf_format", 2, "string");
-  if (Pike_sp[-2].type != PIKE_T_TYPE)
+  if (TYPEOF(Pike_sp[-2]) != PIKE_T_TYPE)
     SIMPLE_ARG_TYPE_ERROR("__handle_sscanf_format", 3, "type");
-  if (Pike_sp[-1].type != PIKE_T_TYPE)
+  if (TYPEOF(Pike_sp[-1]) != PIKE_T_TYPE)
     SIMPLE_ARG_TYPE_ERROR("__handle_sscanf_format", 4, "type");
 
   tmp = Pike_sp[-1].u.type;
