@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 75bd012d30f21f73adf9c25f5656cecfb6c920ca $
 */
 
 #ifndef SVALUE_H
@@ -91,59 +90,133 @@ union anything
 /* Note: At least multisets overlays the type field and uses the top 4
  * bits in it internally. */
 
+#if !defined(HAVE_UNION_INIT) && !defined(NO_COMBINED_TYPE_SUBTYPE)
+/* We can't use the tu union in this case, since we need to
+ * support static initialization of svalues.
+ */
+#define NO_COMBINED_TYPE_SUBTYPE
+#endif
+
 /**
  */
 struct svalue
 {
-  unsigned INT16 type; /**< the data type, see PIKE_T_... */
-  unsigned INT16 subtype; /**< used to store the zero type, among others */
+#ifndef NO_COMBINED_TYPE_SUBTYPE
+  union {
+    struct {
+      unsigned short type; /**< the data type, see PIKE_T_... */
+      unsigned short subtype; /**< used to store the zero type, among others */
+    } t;
+#if PIKE_BYTEORDER == 1234
+    ptrdiff_t type_subtype;
+#else
+    /* For big-endian 64-bit architectures we don't want to require
+     * 64-bit constants when setting the type_subtype field.
+     */
+    INT32 type_subtype;
+#if SIZEOF_CHAR_P == 8
+    INT32 pad__;
+#endif
+#endif
+  } tu;
+#else /* NO_COMBINED_TYPE_SUBTYPE */
+  /* Syntax-compatible with the above, but only structs,
+   * and thus no combined type_subtype field.
+   */
+  struct {
+    struct {
+      unsigned short type; /**< the data type, see PIKE_T_... */
+      unsigned short subtype; /**< used to store the zero type, among others */
+    } t;
+  } tu;
+#endif /* NO_COMBINED_TYPE_SUBTYPE */
   union anything u; /**< contains the value */
 };
 
-#define TYPEOF(SVAL)	((SVAL).type)
-#define SUBTYPEOF(SVAL)	((SVAL).subtype)
+#define TYPEOF(SVAL)	((SVAL).tu.t.type)
+#define SUBTYPEOF(SVAL)	((SVAL).tu.t.subtype)
+
 #define SET_SVAL_TYPE(SVAL, TYPE)	(TYPEOF(SVAL) = (TYPE))
 #define SET_SVAL_SUBTYPE(SVAL, TYPE)	(SUBTYPEOF(SVAL) = (TYPE))
-#define SET_SVAL(SVAL, TYPE, SUBTYPE, FIELD, EXPR) do {	\
+
+#ifdef NO_COMBINED_TYPE_SUBTYPE
+/* Some compilers have aliasing problems with unions,
+ * so skip the optimization of setting both fields
+ * with one write with them.
+ */
+#define SET_SVAL_TYPE_SUBTYPE(SVAL, TYPE, SUBTYPE)	\
+  ((TYPEOF(SVAL) = (TYPE)),				\
+   (SUBTYPEOF(SVAL) = (SUBTYPE)))
+/* Used when we don't care if the subtype gets set or not. */
+#define SET_SVAL_TYPE_DC(SVAL, TYPE)	SET_SVAL_TYPE(SVAL, TYPE)
+#else
+#if PIKE_BYTEORDER == 1234
+#define TYPE_SUBTYPE(X,Y) ((unsigned int)(X)|((unsigned int)(Y)<<16))
+#else
+#define TYPE_SUBTYPE(X,Y) ((unsigned int)(Y)|((unsigned int)(X)<<16))
+#endif
+
+#define SET_SVAL_TYPE_SUBTYPE(SVAL, TYPE, SUBTYPE) \
+  ((SVAL).tu.type_subtype = TYPE_SUBTYPE(TYPE,SUBTYPE))
+/* Setting the entire type_subtype field is probably faster than
+ * just setting the type field.
+ */
+#define SET_SVAL_TYPE_DC(SVAL, TYPE)	SET_SVAL_TYPE_SUBTYPE(SVAL, TYPE, 0)
+#endif
+
+#define SET_SVAL(SVAL, TYPE, SUBTYPE, FIELD, EXPR) do { \
     /* Set the type afterwards to avoid a clobbered	\
      * svalue in case EXPR throws. */			\
-    (SVAL).u.FIELD = (EXPR);				\
-    SET_SVAL_TYPE((SVAL), (TYPE));			\
-    SET_SVAL_SUBTYPE((SVAL), (SUBTYPE));		\
+    struct svalue * __sv_ptr = &( SVAL );		\
+    __sv_ptr->u.FIELD = (EXPR);				\
+    SET_SVAL_TYPE_SUBTYPE(*__sv_ptr, TYPE, SUBTYPE);	\
   } while(0)
-#define INVALIDATE_SVAL(SVAL) SET_SVAL_TYPE(SVAL, 99) /* an invalid type */
 
-#define PIKE_T_ARRAY 0
-#define PIKE_T_MAPPING 1
-#define PIKE_T_MULTISET 2
-#define PIKE_T_OBJECT 3
-#define PIKE_T_FUNCTION 4
-#define PIKE_T_PROGRAM 5
-#define PIKE_T_STRING 6
-#define PIKE_T_TYPE 7
-#define PIKE_T_INT 8
-#define PIKE_T_FLOAT 9
+/*
+*/
+#define INVALIDATE_SVAL(SVAL) SET_SVAL_TYPE_DC(SVAL, 99) /* an invalid type */
+
+/* The native types.
+ *
+ * Note that PIKE_T_INT is zero so that cleared memory
+ * is filled with zeroes.
+ */
+#define PIKE_T_INT 0
+#define PIKE_T_FLOAT 1
+
+/* NB: The reference counted types all have bit 4 set. */
+#define PIKE_T_ARRAY 8
+#define PIKE_T_MAPPING 9
+#define PIKE_T_MULTISET 10
+#define PIKE_T_OBJECT 11
+#define PIKE_T_FUNCTION 12
+#define PIKE_T_PROGRAM 13
+#define PIKE_T_STRING 14
+#define PIKE_T_TYPE 15
 
 /* The types above are valid types in svalues.
  * The following are only used by the internal systems.
  */
 
-#define PIKE_T_ZERO  14	/**< Can return 0, but nothing else */
+/* NB: 6 & 7 below are selected for easy backward compat with Pike 7.8. */
+#define PIKE_T_ZERO  6	/**< Can return 0, but nothing else */
 
 
-#define T_UNFINISHED 15
+#define T_UNFINISHED 7
+
+/*
+ * NOTE: The following types may show up on the stack, but are NOT
+ *       reference counted there. There they are mainly used for
+ *       lvalues. They MUST NOT have a value that has bit 3 (8) set.
+ */
 
 #define T_VOID       16 /**< Can't return any value. Also used on stack to fill out the second
  * svalue on an lvalue when it isn't used. */
 
-
 #define T_MANY       17
 
 #define PIKE_T_INT_UNTYPED  18 /* Optimization of int type size */
 
-#define PIKE_T_GET_SET 32	/* Getter setter.
-				 * Only valid in struct identifier */
-
 /* Type to put in freed svalues. Only the type field in such svalues
  * is defined. Freeing a PIKE_T_FREE svalue is allowed and does
  * nothing. mark_free_svalue() is preferably used to set this type.
@@ -154,7 +227,30 @@ struct svalue
  *
  * PIKE_T_FREE svalues are recorded as BIT_INT in type hint fields.
  */
-#define PIKE_T_FREE 237
+#define PIKE_T_FREE 19
+
+/** svalue.u.lval points to an svalue. Primarily used in lvalues on
+ * stack, but can also occur in arrays containing lvalue pairs.
+ */
+#define T_SVALUE_PTR 20
+
+/** svalue.u.identifer is an identifier index in an object. Primarily
+ * used in lvalues on stack, but can also occur in arrays containing
+ * lvalue pairs.
+ */
+#define T_OBJ_INDEX 21
+#define T_ARRAY_LVALUE 22
+
+/* No types above this value should appear on the stack. */
+#define PIKE_T_STACK_MAX	T_ARRAY_LVALUE
+
+/*
+ * The following types are only used in compile-time types and
+ * as markers in struct identifier.
+ */
+
+#define PIKE_T_GET_SET 32	/* Getter setter.
+				 * Only valid in struct identifier */
 
 #define PIKE_T_ATTRIBUTE 238	/* Attribute node. */
 #define PIKE_T_NSTRING 239	/* Narrow string. Only for serialization. */
@@ -166,21 +262,20 @@ struct svalue
 #define T_DELETED 246
 #define PIKE_T_UNKNOWN 247
 
-/** svalue.u.identifer is an identifier index in an object. Primarily
- * used in lvalues on stack, but can also occur in arrays containing
- * lvalue pairs. */
-#define T_OBJ_INDEX 248
-
-/** svalue.u.lval points to an svalue. Primarily used in lvalues on
- * stack, but can also occur in arrays containing lvalue pairs. */
-#define T_SVALUE_PTR 249
-
-#define T_ARRAY_LVALUE 250
 #define PIKE_T_MIXED 251
 #define T_NOT 253
 #define T_AND 254
 #define T_OR 255
 
+/* This flag is only valid in struct reference, and corresponds
+ * to struct identifier identifier_flags IDENTIFIER_NO_THIS_REF. */
+#define PIKE_T_NO_REF_FLAG	256
+#define PIKE_T_NO_REF_OBJECT	(PIKE_T_NO_REF_FLAG|PIKE_T_OBJECT)
+#define PIKE_T_NO_REF_FUNCTION	(PIKE_T_NO_REF_FLAG|PIKE_T_FUNCTION)
+#define PIKE_T_NO_REF_MIXED	(PIKE_T_NO_REF_FLAG|PIKE_T_MIXED)
+#define PIKE_T_NO_REF_INT	(PIKE_T_NO_REF_FLAG|PIKE_T_INT)
+#define PIKE_T_NO_REF_FLOAT	(PIKE_T_NO_REF_FLAG|PIKE_T_FLOAT)
+
 /* These are only used together with describe() and friends. */
 #define T_STORAGE 10000
 #define T_MAPPING_DATA 10001
@@ -188,6 +283,10 @@ struct svalue
 #define T_MULTISET_DATA 10003
 #define T_STRUCT_CALLABLE 10004
 
+/* NOTE: The t* macros below currently use the old type encoding
+ *       to be compatible with __parse_pike_type() in older
+ *       versions of Pike.
+ */
 #define tArr(VAL) "\000" VAL
 #define tArray tArr(tMix)
 #define tMap(IND,VAL) "\001" IND VAL
@@ -266,20 +365,20 @@ struct svalue
 #define tRef tOr(tString,tComplex)
 #define tIfnot(X,Y) tAnd(tNot(X),Y)
 #define tAny tOr(tVoid,tMix)
-#define tAttr(X,Y) "\356\0"X"\0"Y
-#define tName(X,Y) "\361\0"X"\0"Y
+#define tAttr(X,Y) "\356\0" X "\0" Y
+#define tName(X,Y) "\361\0" X "\0" Y
 #if PIKE_BYTEORDER == 1234
 /* Little endian */
-#define tAttr1(X,Y) "\356\5"X"\0\0"Y
-#define tAttr2(X,Y) "\356\6"X"\0\0\0\0"Y
-#define tName1(X,Y) "\361\5"X"\0\0"Y
-#define tName2(X,Y) "\361\6"X"\0\0\0\0"Y
+#define tAttr1(X,Y) "\356\5" X "\0\0" Y
+#define tAttr2(X,Y) "\356\6" X "\0\0\0\0" Y
+#define tName1(X,Y) "\361\5" X "\0\0" Y
+#define tName2(X,Y) "\361\6" X "\0\0\0\0" Y
 #else /* PIKE_BYTEORDER != 1234 */
 /* Big endian */
-#define tAttr1(X,Y) "\356\1"X"\0\0"Y
-#define tAttr2(X,Y) "\356\2"X"\0\0\0\0"Y
-#define tName1(X,Y) "\361\1"X"\0\0"Y
-#define tName2(X,Y) "\361\2"X"\0\0\0\0"Y
+#define tAttr1(X,Y) "\356\1" X "\0\0" Y
+#define tAttr2(X,Y) "\356\2" X "\0\0\0\0" Y
+#define tName1(X,Y) "\361\1" X "\0\0" Y
+#define tName2(X,Y) "\361\2" X "\0\0\0\0" Y
 #endif /* PIKE_BYTEORDER == 1234 */
 
 /* Some convenience macros for common attributes. */
@@ -313,12 +412,12 @@ struct svalue
 #define BIT_VOID (1 << T_VOID)
 
 /** This is used in typechecking to signify that the rest of the
- * arguments has to be of this type.
+ * arguments have to be of this type.
  */
 #define BIT_MANY (1 << T_MANY)
 
 #define BIT_NOTHING 0
-#define BIT_MIXED 0x7fff
+#define BIT_MIXED 0xff7f
 #define BIT_BASIC (BIT_INT|BIT_FLOAT|BIT_STRING|BIT_TYPE)
 #define BIT_COMPLEX (BIT_ARRAY|BIT_MULTISET|BIT_OBJECT|BIT_PROGRAM|BIT_MAPPING|BIT_FUNCTION)
 #define BIT_CALLABLE (BIT_FUNCTION|BIT_PROGRAM|BIT_ARRAY|BIT_OBJECT)
@@ -326,10 +425,12 @@ struct svalue
 
 /* Max type which contains svalues */
 #define MAX_COMPLEX PIKE_T_PROGRAM
-/* Max type with ref count */
-#define MAX_REF_TYPE PIKE_T_TYPE
+/* Min type with ref count */
+#define MIN_REF_TYPE PIKE_T_ARRAY
 /* Max type handled by svalue primitives */
-#define MAX_TYPE PIKE_T_FLOAT
+#define MAX_TYPE PIKE_T_TYPE
+
+#define REFCOUNTED_TYPE(T)	(((T) & ~(MIN_REF_TYPE - 1)) == MIN_REF_TYPE)
 
 #define NUMBER_NUMBER 0
 #define NUMBER_UNDEFINED 1
@@ -337,8 +438,14 @@ struct svalue
 
 #define FUNCTION_BUILTIN USHRT_MAX
 
-extern PMOD_EXPORT struct svalue svalue_undefined,
-  svalue_int_zero, svalue_int_one;
+extern PMOD_EXPORT const  struct svalue svalue_undefined, svalue_int_zero;
+#ifdef HAVE_UNION_INIT
+extern PMOD_EXPORT const  struct svalue svalue_int_one;
+#else
+/* If union initializers are unavailable,
+   svalue_int_one needs to be assignable. */
+extern PMOD_EXPORT struct svalue svalue_int_one;
+#endif
 
 #define is_gt(a,b) is_lt(b,a)
 #define is_ge(a,b) is_le(b,a)
@@ -346,31 +453,39 @@ extern PMOD_EXPORT struct svalue svalue_undefined,
 /* SAFE_IS_ZERO is compatible with the old IS_ZERO, but you should
  * consider using UNSAFE_IS_ZERO instead, since exceptions thrown from
  * `! functions will be propagated correctly then. */
-#define UNSAFE_IS_ZERO(X) ((X)->type==PIKE_T_INT?(X)->u.integer==0:(1<<(X)->type)&(BIT_OBJECT|BIT_FUNCTION)?!svalue_is_true(X):0)
-#define SAFE_IS_ZERO(X) ((X)->type==PIKE_T_INT?(X)->u.integer==0:(1<<(X)->type)&(BIT_OBJECT|BIT_FUNCTION)?!safe_svalue_is_true(X):0)
+#define UNSAFE_IS_ZERO(X) (TYPEOF(*(X))==PIKE_T_INT?(X)->u.integer==0:(1<<TYPEOF(*(X)))&(BIT_OBJECT|BIT_FUNCTION)?!complex_svalue_is_true(X):0)
+#define SAFE_IS_ZERO(X) (TYPEOF(*(X))==PIKE_T_INT?(X)->u.integer==0:(1<<TYPEOF(*(X)))&(BIT_OBJECT|BIT_FUNCTION)?!safe_svalue_is_true(X):0)
 
-#define IS_UNDEFINED(X) (check_svalue (X), (X)->type==PIKE_T_INT&&(X)->subtype==NUMBER_UNDEFINED)
+#define IS_UNDEFINED(X) (check_svalue (X), TYPEOF(*(X))==PIKE_T_INT&&SUBTYPEOF(*(X))==NUMBER_UNDEFINED)
 
-#define IS_DESTRUCTED(X) \
-  (((X)->type == PIKE_T_OBJECT || (X)->type==PIKE_T_FUNCTION) && !(X)->u.object->prog)
+#define IS_DESTRUCTED(X)                                                \
+((TYPEOF(*(X)) == PIKE_T_OBJECT && !(X)->u.object->prog) ||             \
+ (TYPEOF(*(X)) == PIKE_T_FUNCTION && SUBTYPEOF(*(X)) != FUNCTION_BUILTIN\
+  && (!(X)->u.object->prog                                              \
+      || ((X)->u.object->prog == pike_trampoline_program                \
+          && !((struct pike_trampoline *)(X)->u.object->storage)        \
+          ->frame->current_object->prog                                 \
+          && SUBTYPEOF(*(X)) == QUICK_FIND_LFUN(pike_trampoline_program,\
+                                                LFUN_CALL)))))
 
-#define check_destructed(S) \
-do{ \
-  struct svalue *_s=(S); \
-  if(IS_DESTRUCTED(_s)) { \
-    free_object(_s->u.object); \
-    _s->type = PIKE_T_INT; \
-    _s->subtype = NUMBER_DESTRUCTED ; \
-    _s->u.integer = 0; \
-  } \
-}while(0)
+#define check_destructed(S)			\
+  do{						\
+    struct svalue *_s=(S);			\
+    if(IS_DESTRUCTED(_s)) {			\
+      free_object(_s->u.object);		\
+      SET_SVAL(*_s, PIKE_T_INT,			\
+	       NUMBER_DESTRUCTED,		\
+	       integer, 0);			\
+    }						\
+  }while(0)
 
 /* var MUST be a variable!!! */
 #define safe_check_destructed(var) do{ \
-  if((var->type == PIKE_T_OBJECT || var->type==PIKE_T_FUNCTION) && !var->u.object->prog) \
-    var=&svalue_int_zero; \
-}while(0)
+    if(IS_DESTRUCTED(var))	       \
+      var=&svalue_int_zero;	       \
+  }while(0)
 
+/* FIXME: Is this actually used for functions? */
 #define check_short_destructed(U,T) \
 do{ \
   union anything *_u=(U); \
@@ -382,38 +497,15 @@ do{ \
 }while(0)
 
 
-#ifdef PIKE_RUN_UNLOCKED
-#define add_ref(X) pike_atomic_inc32(&(X)->refs)
-#define sub_ref(X) pike_atomic_dec_and_test32(&(X)->refs)
-
-#if 0
-#define IF_LOCAL_MUTEX(X) X
-#define USE_LOCAL_MUTEX
-#define pike_lock_data(X) mt_lock(&(X)->mutex)
-#define pike_unlock_data(X) mt_unlock(&(X)->mutex)
-#else
-#define IF_LOCAL_MUTEX(X)
-#define pike_lock_data(X) pike_lockmem((X))
-#define pike_unlock_data(X) pike_unlockmem((X))
-#endif
-
-#else
-#define IF_LOCAL_MUTEX(X)
-#define add_ref(X) (void)((X)->refs++)
+#define add_ref(X) ((void)((X)->refs++))
 #define sub_ref(X) (--(X)->refs > 0)
-#define pike_lock_data(X) (void)(X)
-#define pike_unlock_data(X) (void)(X)
-#endif
-
 
 #ifdef PIKE_DEBUG
 PMOD_EXPORT extern void describe(void *); /* defined in gc.c */
 PMOD_EXPORT extern const char msg_type_error[];
 PMOD_EXPORT extern const char msg_assign_svalue_error[];
 
-#define IS_INVALID_TYPE(T)						\
-  ((T > MAX_TYPE && T < T_OBJ_INDEX && T != T_VOID && T != PIKE_T_FREE) || \
-   T > T_ARRAY_LVALUE)
+#define IS_INVALID_TYPE(T) is_invalid_stack_type(T)
 
 #define check_type(T) do {						\
     TYPE_T typ_ = (T);							\
@@ -422,7 +514,7 @@ PMOD_EXPORT extern const char msg_assign_svalue_error[];
 
 #define check_svalue_type(S) do {					\
     const struct svalue *sval_ = (S);					\
-    TYPE_T typ_ = sval_->type;						\
+    TYPE_T typ_ = TYPEOF(*sval_);					\
     if (IS_INVALID_TYPE (typ_)) debug_svalue_type_error (sval_);	\
   } while (0)
 
@@ -434,19 +526,20 @@ void low_thorough_check_short_svalue (const union anything *u, TYPE_T type);
     TYPE_T typ_ = (T);							\
     check_short_svalue (anyth_, typ_);					\
     if (d_flag <= 50) /* Done directly by check_svalue otherwise. */	\
-      if (typ_ <= MAX_REF_TYPE)						\
+      if (REFCOUNTED_TYPE(typ_))					\
 	low_thorough_check_short_svalue (anyth_, typ_);			\
   } while (0)
 #define thorough_check_svalue(S) do {					\
     struct svalue *sval_ = (S);						\
     check_svalue (sval_);						\
     if (d_flag <= 50) /* Done directly by check_svalue otherwise. */	\
-      if (sval_->type <= MAX_REF_TYPE)					\
-	low_thorough_check_short_svalue (&sval_->u, sval_->type);	\
+      if (REFCOUNTED_TYPE(TYPEOF(*sval_)))				\
+	low_thorough_check_short_svalue (&sval_->u, TYPEOF(*sval_));	\
   } while (0)
 
 void check_short_svalue(const union anything *u, TYPE_T type);
 PMOD_EXPORT void debug_svalue_type_error (const struct svalue *s);
+PMOD_EXPORT int is_invalid_stack_type(unsigned int t);
 PMOD_EXPORT void debug_check_svalue(const struct svalue *s);
 void debug_check_type_hint (const struct svalue *svals, size_t num, TYPE_FIELD type_hint);
 PMOD_EXPORT void real_gc_mark_external_svalues(const struct svalue *s, ptrdiff_t num,
@@ -454,7 +547,7 @@ PMOD_EXPORT void real_gc_mark_external_svalues(const struct svalue *s, ptrdiff_t
 
 PMOD_EXPORT extern const char msg_sval_obj_wo_refs[];
 #define check_refs(S) do {\
- if((S)->type <= MAX_REF_TYPE && (!(S)->u.refs || (S)->u.refs[0] < 0)) { \
+ if(REFCOUNTED_TYPE(TYPEOF(*(S))) && (!(S)->u.refs || (S)->u.refs[0] < 0)) { \
    fprintf (stderr, "%s", msg_sval_obj_wo_refs);			\
    describe((S)->u.refs);						\
    Pike_fatal("%s", msg_sval_obj_wo_refs);				\
@@ -462,7 +555,7 @@ PMOD_EXPORT extern const char msg_sval_obj_wo_refs[];
 
 PMOD_EXPORT extern const char msg_ssval_obj_wo_refs[];
 #define check_refs2(S,T) do { \
-if((T) <= MAX_REF_TYPE && (S)->refs && (S)->refs[0] <= 0) {\
+if(REFCOUNTED_TYPE(T) && (S)->refs && (S)->refs[0] <= 0) {\
   fprintf (stderr, "%s", msg_ssval_obj_wo_refs);	   \
   describe((S)->refs);					   \
   Pike_fatal("%s", msg_ssval_obj_wo_refs);		   \
@@ -472,65 +565,46 @@ if((T) <= MAX_REF_TYPE && (S)->refs && (S)->refs[0] <= 0) {\
   debug_check_type_hint ((SVALS), (NUM), (TYPE_HINT))
 
 #ifdef DEBUG_MALLOC
-static INLINE struct svalue *dmalloc_check_svalue(struct svalue *s, char *l)
+static INLINE struct svalue ATTRIBUTE((unused)) *dmalloc_check_svalue(struct svalue *s, char *l)
 {
-#if 0
-  /* What's this supposed to accomplish? Dmalloc tracks memory blocks,
-   * not single svalues that point to them. /mast */
-  debug_malloc_update_location(s,l);
-#endif
-#if 1
-  if(s && s->type <= MAX_REF_TYPE)
+  if(s && REFCOUNTED_TYPE(TYPEOF(*s)))
     debug_malloc_update_location(s->u.refs,l);
-#endif
   return s;
 }
 
-static INLINE struct svalue *dmalloc_check_svalues(struct svalue *s, size_t num, char *l)
+static INLINE struct svalue ATTRIBUTE((unused)) *dmalloc_check_svalues(struct svalue *s, size_t num, char *l)
 {
   while (num--) dmalloc_check_svalue (s + num, l);
   return s;
 }
 
-static INLINE union anything *dmalloc_check_union(union anything *u,int type, char * l)
+static INLINE union anything ATTRIBUTE((unused)) *dmalloc_check_union(union anything *u,int type, char * l)
 {
-#if 0
-  debug_malloc_update_location(u,l);
-#endif
-#if 1
-  if(u && type <= MAX_REF_TYPE)
+  if(u && REFCOUNTED_TYPE(type))
     debug_malloc_update_location(u->refs,l);
-#endif
   return u;
 }
 
 #undef add_ref
-#undef sub_ref
-
-#ifdef PIKE_RUN_UNLOCKED
-#define add_ref(X) pike_atomic_inc32((INT32 *)debug_malloc_update_location( &((X)->refs), DMALLOC_NAMED_LOCATION(" add_ref")))
-#define sub_ref(X) pike_atomic_dec_and_test32((INT32 *)debug_malloc_update_location( &((X)->refs), DMALLOC_NAMED_LOCATION(" sub_ref")))
-#else
 #define add_ref(X) (((INT32 *)debug_malloc_update_location( &((X)->refs), DMALLOC_NAMED_LOCATION(" add_ref")))[0]++)
+#undef sub_ref
 #define sub_ref(X) (--((INT32 *)debug_malloc_update_location( &((X)->refs), DMALLOC_NAMED_LOCATION(" sub_ref")))[0] > 0)
-#endif
 
 #else  /* !DEBUG_MALLOC */
 #define dmalloc_check_svalue(S,L) (S)
 #define dmalloc_check_svalues(S,L,N) (S)
 #define dmalloc_check_union(U,T,L) (U)
-
 #endif	/* !DEBUG_MALLOC */
 
 /* To be used for type checking in macros. */
-static INLINE struct array *pass_array (struct array *a) {return a;}
-static INLINE struct mapping *pass_mapping (struct mapping *m) {return m;}
-static INLINE struct multiset *pass_multiset (struct multiset *l) {return l;}
-static INLINE struct object *pass_object (struct object *o) {return o;}
-static INLINE struct program *pass_program (struct program *p) {return p;}
-static INLINE struct pike_string *pass_string (struct pike_string *s) {return s;}
-static INLINE struct pike_type *pass_type (struct pike_type *t) {return t;}
-static INLINE struct callable *pass_callable (struct callable *c) {return c;}
+static INLINE struct array ATTRIBUTE((unused)) *pass_array (struct array *a) {return a;}
+static INLINE struct mapping ATTRIBUTE((unused)) *pass_mapping (struct mapping *m) {return m;}
+static INLINE struct multiset ATTRIBUTE((unused)) *pass_multiset (struct multiset *l) {return l;}
+static INLINE struct object ATTRIBUTE((unused)) *pass_object (struct object *o) {return o;}
+static INLINE struct program ATTRIBUTE((unused)) *pass_program (struct program *p) {return p;}
+static INLINE struct pike_string ATTRIBUTE((unused)) *pass_string (struct pike_string *s) {return s;}
+static INLINE struct pike_type ATTRIBUTE((unused)) *pass_type (struct pike_type *t) {return t;}
+static INLINE struct callable ATTRIBUTE((unused)) *pass_callable (struct callable *c) {return c;}
 
 #else  /* !PIKE_DEBUG */
 
@@ -566,7 +640,7 @@ static INLINE struct callable *pass_callable (struct callable *c) {return c;}
       _X__->u.loc++; /* Attempt to achieve an odd address. */		\
     );									\
     PIKE_MEM_WO(*_X__);							\
-    _X__->type = PIKE_T_FREE;						\
+    SET_SVAL_TYPE(*_X__, PIKE_T_FREE);					\
   } while (0)
 
 /* This is a debug macro to assert that an svalue is free and
@@ -576,7 +650,7 @@ static INLINE struct callable *pass_callable (struct callable *c) {return c;}
 #define assert_free_svalue(X) do {					\
     DO_IF_DEBUG (							\
       struct svalue *_X__ = (X);					\
-      _X__->type = PIKE_T_UNKNOWN;					\
+      SET_SVAL_TYPE(*_X__, PIKE_T_UNKNOWN);				\
       DO_IF_DMALLOC (							\
 	_X__->u.loc = " " __FILE__ ":" DEFINETOSTR (__LINE__);		\
 	_X__->u.loc++; /* Attempt to achieve an odd address. */		\
@@ -586,43 +660,31 @@ static INLINE struct callable *pass_callable (struct callable *c) {return c;}
     );									\
   } while (0)
 
-/* This define
- * should check that the svalue address (X) is on the local stack,
- * the processor stack or in a locked memory object
- *
- * Or, it could just try to make sure it's not in an unlocked memory
- * object...
- */
-#define assert_svalue_locked(X)
-
-
-#define swap_svalues_unlocked(X,Y)  do {		\
+#define swap_svalues(X,Y)  do {		\
   struct svalue *_a=(X);				\
   struct svalue *_b=(Y);				\
   struct svalue _tmp;					\
-  assert_svalue_locked(_a); assert_svalue_locked(_b);	\
   dmalloc_touch_svalue(_a);				\
   dmalloc_touch_svalue(_b);				\
   _tmp=*_a; *_a=*_b; *_b=_tmp;				\
 }while(0)
 
 /* Handles PIKE_T_FREE. */
-#define free_svalue_unlocked(X) do {				\
+#define free_svalue(X) do {				\
   struct svalue *_s=(X);					\
-  assert_svalue_locked(_s);					\
   DO_IF_DEBUG (							\
-    if (_s->type != PIKE_T_FREE) {				\
+    if (TYPEOF(*_s) != PIKE_T_FREE) {				\
       check_svalue_type(_s);					\
       check_refs(_s);						\
     }								\
   );								\
-  if (_s->type > MAX_REF_TYPE)					\
+  if (!REFCOUNTED_TYPE(TYPEOF(*_s)))				\
     assert_free_svalue (_s);					\
   else {							\
     DO_IF_DEBUG (						\
       DO_IF_PIKE_CLEANUP (					\
 	if (gc_external_refs_zapped)				\
-	  gc_check_zapped (_s->u.ptr, _s->type, __FILE__, __LINE__))); \
+	  gc_check_zapped (_s->u.ptr, TYPEOF(*_s), __FILE__, __LINE__))); \
     if (sub_ref(_s->u.dummy) <=0)				\
       really_free_svalue(_s);					\
     else							\
@@ -630,11 +692,10 @@ static INLINE struct callable *pass_callable (struct callable *c) {return c;}
   }								\
 }while(0)
 
-#define free_short_svalue_unlocked(X,T) do {				\
+#define free_short_svalue(X,T) do {				\
   union anything *_s=(X); TYPE_T _t=(T);				\
   check_type(_t); check_refs2(_s,_t);					\
-  assert_svalue_locked(_s);						\
-  if(_t<=MAX_REF_TYPE && _s->refs) {					\
+  if(REFCOUNTED_TYPE(_t) && _s->refs) {					\
     DO_IF_DEBUG (							\
       DO_IF_PIKE_CLEANUP (						\
 	if (gc_external_refs_zapped)					\
@@ -646,23 +707,23 @@ static INLINE struct callable *pass_callable (struct callable *c) {return c;}
 }while(0)
 
 /* Handles PIKE_T_FREE. */
-#define add_ref_svalue_unlocked(X) do {				\
+#define add_ref_svalue(X) do {				\
   struct svalue *_tmp=(X);					\
   DO_IF_DEBUG (							\
-    if (_tmp->type != PIKE_T_FREE) {				\
+    if (TYPEOF(*_tmp) != PIKE_T_FREE) {				\
       check_svalue_type(_tmp);					\
       check_refs(_tmp);						\
     }								\
   );								\
-  if(_tmp->type <= MAX_REF_TYPE) add_ref(_tmp->u.dummy);	\
+  if(REFCOUNTED_TYPE(TYPEOF(*_tmp))) add_ref(_tmp->u.dummy);	\
 }while(0)
 
 /* Handles PIKE_T_FREE. */
-#define assign_svalue_no_free_unlocked(X,Y) do {	\
+#define assign_svalue_no_free(X,Y) do {	\
   struct svalue *_to=(X);				\
   const struct svalue *_from=(Y);			\
   DO_IF_DEBUG (						\
-    if (_from->type != PIKE_T_FREE) {			\
+    if (TYPEOF(*_from) != PIKE_T_FREE) {		\
       check_svalue_type(_from);				\
       check_refs(_from);				\
     }							\
@@ -670,11 +731,11 @@ static INLINE struct callable *pass_callable (struct callable *c) {return c;}
       Pike_fatal(msg_assign_svalue_error, _to);		\
   );							\
   *_to=*_from;						\
-  if(_to->type <= MAX_REF_TYPE) add_ref(_to->u.dummy);	\
+  if(REFCOUNTED_TYPE(TYPEOF(*_to))) add_ref(_to->u.dummy); \
 }while(0)
 
 /* Handles PIKE_T_FREE. */
-#define assign_svalue_unlocked(X,Y) do {	\
+#define assign_svalue(X,Y) do {	\
   struct svalue *_to2=(X);			\
   const struct svalue *_from2=(Y);		\
   if (_to2 != _from2) {				\
@@ -758,6 +819,7 @@ PMOD_EXPORT void assign_short_svalue(union anything *to,
 			 const union anything *from,
 			 TYPE_T type);
 PMOD_EXPORT unsigned INT32 hash_svalue(const struct svalue *s);
+PMOD_EXPORT int complex_svalue_is_true(const struct svalue *s); /* only handles object + function */
 PMOD_EXPORT int svalue_is_true(const struct svalue *s);
 PMOD_EXPORT int safe_svalue_is_true(const struct svalue *s);
 PMOD_EXPORT int is_identical(const struct svalue *a, const struct svalue *b);
@@ -806,12 +868,20 @@ PMOD_EXPORT int real_gc_cycle_check_short_svalue(union anything *u, TYPE_T type)
 int gc_cycle_check_weak_short_svalue(union anything *u, TYPE_T type);
 void real_gc_free_svalue(struct svalue *s);
 void real_gc_free_short_svalue(union anything *u, TYPE_T type);
-PMOD_EXPORT INT32 pike_sizeof(const struct svalue *s);
+PMOD_EXPORT INT_TYPE pike_sizeof(const struct svalue *s);
 int svalues_are_constant(const struct svalue *s,
 			 INT32 num,
 			 TYPE_FIELD hint,
 			 struct processing *p);
 
+static INLINE TYPE_FIELD ATTRIBUTE((unused)) BITOF(struct svalue sv) {
+    if (TYPEOF(sv) >= sizeof(TYPE_FIELD) * 8) {
+        return BIT_MIXED | BIT_UNFINISHED;
+    }
+
+    return 1 << TYPEOF(sv);
+}
+
 #define gc_cycle_check_without_recurse gc_mark_without_recurse
 #define gc_cycle_check_weak_without_recurse gc_mark_without_recurse
 
@@ -827,10 +897,10 @@ int svalues_are_constant(const struct svalue *s,
   } while (0)
 
 #ifdef DEBUG_MALLOC
-static INLINE TYPE_FIELD dmalloc_gc_mark_svalues (struct svalue *s, size_t num, char *l)
+static INLINE TYPE_FIELD ATTRIBUTE((unused)) dmalloc_gc_mark_svalues (struct svalue *s, size_t num, char *l)
   {return real_gc_mark_svalues (dmalloc_check_svalues (s, num, l), num);}
 #define gc_mark_svalues(S, NUM) dmalloc_gc_mark_svalues ((S), (NUM), DMALLOC_LOCATION())
-static INLINE TYPE_FIELD dmalloc_gc_cycle_check_svalues (struct svalue *s, size_t num, char *l)
+static INLINE TYPE_FIELD ATTRIBUTE((unused)) dmalloc_gc_cycle_check_svalues (struct svalue *s, size_t num, char *l)
   {return real_gc_cycle_check_svalues (dmalloc_check_svalues (s, num, l), num);}
 #define gc_cycle_check_svalues(S, NUM) dmalloc_gc_cycle_check_svalues ((S), (NUM), DMALLOC_LOCATION())
 #else
@@ -865,158 +935,28 @@ static INLINE TYPE_FIELD dmalloc_gc_cycle_check_svalues (struct svalue *s, size_
 
 #endif /* !NO_PIKE_SHORTHAND */
 
-#if 0 /* PIKE_RUN_UNLOCKED */
-
-#include "pike_error.h"
-
-/*#define swap_svalues swap_svalues*/
-/*#define free_svalue free_svalue_unlocked*/
-/*#define free_short_svalue free_short_svalue_unlocked */
-/*#define add_ref_svalue add_ref_svalue_unlocked*/
-
-/*
- * These don't work right now - Hubbe
- */
-#define assign_svalue_no_free assign_svalue_no_free_unlocked
-#define assign_svalue assign_svalue_unlocked
-
-/* FIXME:
- * These routines assumes that pointers are 32 bit
- * and svalues 64 bit!!!!! - Hubbe
- */
-
-#ifndef swap_svalues 
-#define swap_svalues swap_svalues_unlocked
-#endif
-
-#ifndef free_svalue
-static INLINE void free_svalue(struct svalue *s)
-{
-  INT64 tmp;
-  struct svalue zero;
-  SET_SVAL(zero, PIKE_T_INT, NUMBER_NUMBER, integer, 0);
-  tmp=pike_atomic_swap64((INT64 *)s, *(INT64 *)&zero);
-  free_svalue_unlocked((struct svalue *)&tmp);
-}
-#endif
-
-#ifndef free_short_svalue
-static INLINE void free_short_svalue(union anything *s, int t)
-{
-  if(t <= MAX_REF_TYPE)
-  {
-    INT32 tmp;
-    tmp=pike_atomic_swap32((INT32 *)s, 0);
-    free_short_svalue_unlocked((union anything *)&tmp, t);
-  }
-}
-#endif
-
-#ifndef add_ref_svalue
-static INLINE void add_ref_svalue(struct svalue *s)
-{
-  INT64 sv;
-  sv=pike_atomic_get64((INT64 *)s);
-  add_ref_svalue_unlocked((struct svalue *)&sv);
-}
-#endif
-
-#ifndef assign_svalue_no_free
-void assign_svalue_no_free(struct svalue *to, const struct svalue *from)
-{
-  INT64 tmp, sv;
-  sv=pike_atomic_get64((INT64 *)from);
-#ifdef PIKE_DEBUG
-  if(sv != *(INT64*)from)
-  {
-    fprintf(stderr,"pike_atomic_get64() is broken %llx != %llx (%08x%08x)!\n",
-	    sv,
-	    *(INT64*)from,
-	    ((INT32*)from)[1], ((INT32*)from)[0]);
-    abort();
-  }
-#endif
-  add_ref_svalue_unlocked((struct svalue *)&sv);
-  pike_atomic_set64((INT64 *)to, sv);
-#ifdef PIKE_DEBUG
-  if(*(INT64*)to != *(INT64*)from)
-  {
-    fprintf(stderr,"pike_atomic_set64() is broken!\n");
-    abort();
-  }
-#endif
-}
-#endif
-
-#ifndef assign_svalue
-static INLINE void assign_svalue(struct svalue *to, const struct svalue *from)
-{
-  INT64 tmp, sv;
-  if(to != from)
-  {
-    sv=pike_atomic_get64((INT64 *)from);
-    add_ref_svalue_unlocked((struct svalue *)&sv);
-    tmp=pike_atomic_swap64((INT64 *)to, sv);
-    free_svalue_unlocked((struct svalue *)&tmp);
-  }
-}
-#endif
-
-#else /* FOO_PIKE_RUN_UNLOCKED */
-#define swap_svalues swap_svalues
-#define free_svalue free_svalue_unlocked
-#define free_short_svalue free_short_svalue_unlocked 
-#define add_ref_svalue add_ref_svalue_unlocked
-#define assign_svalue_no_free assign_svalue_no_free_unlocked
-#define assign_svalue assign_svalue_unlocked
-#endif /* FOO_PIKE_RUN_UNLOCKED */
-
-#ifdef PIKE_RUN_UNLOCKED
-#include "pike_threadlib.h"
-#endif
-
-/* 
- * Note to self:
- * It might be better to use a static array of mutexes instead
- * and just lock mutex ptr % array_size instead.
- * That way I wouldn't need a mutex in each memory object,
- * but it would cost a couple of cycles in every lock/unlock
- * operation instead.
- */
-#define PIKE_MEMORY_OBJECT_MEMBERS \
-  INT32 refs \
+#define PIKE_MEMORY_OBJECT_MEMBERS	\
+  INT32 refs				\
   DO_IF_SECURITY(; struct object *prot) \
-  IF_LOCAL_MUTEX(; PIKE_MUTEX_T mutex)
 
 #ifdef PIKE_SECURITY
-#ifdef USE_LOCAL_MUTEX
-#define PIKE_CONSTANT_MEMOBJ_INIT(refs) refs, 0, PTHREAD_MUTEX_INITIALIZER
+#define PIKE_CONSTANT_MEMOBJ_INIT(refs, type) refs, 0
 #else
-#define PIKE_CONSTANT_MEMOBJ_INIT(refs) refs, 0
-#endif
-#else
-#ifdef USE_LOCAL_MUTEX
-#define PIKE_CONSTANT_MEMOBJ_INIT(refs) refs, PTHREAD_MUTEX_INITIALIZER
-#else
-#define PIKE_CONSTANT_MEMOBJ_INIT(refs) refs
-#endif
+#define PIKE_CONSTANT_MEMOBJ_INIT(refs, type) refs
 #endif
 
-#define INIT_PIKE_MEMOBJ(X) do {			\
+#define INIT_PIKE_MEMOBJ(X, TYPE) do {			\
   struct ref_dummy *v_=(struct ref_dummy *)(X);		\
   v_->refs=0;						\
   add_ref(v_); /* For DMALLOC... */			\
   DO_IF_SECURITY( INITIALIZE_PROT(v_) );		\
-  IF_LOCAL_MUTEX(mt_init_recursive(&(v_->mutex)));	\
 }while(0)
 
 #define EXIT_PIKE_MEMOBJ(X) do {		\
   struct ref_dummy *v_=(struct ref_dummy *)(X);		\
   DO_IF_SECURITY( FREE_PROT(v_) );		\
-  IF_LOCAL_MUTEX(mt_destroy(&(v_->mutex)));	\
 }while(0)
 
-
 struct ref_dummy
 {
   PIKE_MEMORY_OBJECT_MEMBERS;
@@ -1025,13 +965,13 @@ struct ref_dummy
 /* The following macro is useful to initialize static svalues. Note
  * that the value isn't always set. */
 #ifdef HAVE_UNION_INIT
-#define SVALUE_INIT(TYPE, SUBTYPE, VAL) {TYPE, SUBTYPE, {VAL}}
-#define SVALUE_INIT_INT(VAL) {T_INT, NUMBER_NUMBER, {VAL}}
-#define SVALUE_INIT_FREE {PIKE_T_FREE, NUMBER_NUMBER, {0}}
+#define SVALUE_INIT(TYPE, SUBTYPE, VAL) {{{TYPE, SUBTYPE}}, {VAL}}
+#define SVALUE_INIT_INT(VAL) {{{T_INT, NUMBER_NUMBER}}, {VAL}}
+#define SVALUE_INIT_FREE {{{PIKE_T_FREE, NUMBER_NUMBER}}, {0}}
 #else
-#define SVALUE_INIT(TYPE, SUBTYPE, VAL) {TYPE, SUBTYPE}
-#define SVALUE_INIT_INT(VAL) {T_INT, NUMBER_NUMBER}
-#define SVALUE_INIT_FREE {PIKE_T_FREE, NUMBER_NUMBER}
+#define SVALUE_INIT(TYPE, SUBTYPE, VAL) {{{TYPE, SUBTYPE}}}
+#define SVALUE_INIT_INT(VAL) {{{T_INT, NUMBER_NUMBER}}}
+#define SVALUE_INIT_FREE {{{PIKE_T_FREE, NUMBER_NUMBER}}}
 #endif
 
 #endif /* !SVALUE_H */
